using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Components.Agenda.Management;
using System.Drawing.Drawing2D;
using Components.Agenda.Management.StructsEnums;
using Components.Agenda.GUI.EventArgsObj;

namespace Components.Agenda.GUI
{
    #region Delegate
    /// <summary>
    /// Handler d'vnement de demande de rendez-vous
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void MeetingAskedHandler(object sender, MeetingEventArgs e);
    /// <summary>
    /// Handler d'vnement pour une demande de modification ou de suppression de rendez-vous
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void MeetingModificationAskedHandler(object sender, MeetingModificationEventArgs e);
    #endregion

    /// <summary>
    /// Composant d'affichage de jour
    /// </summary>
    public partial class DayAgenda : UserControl
    {
        #region Variables
        /// <summary>
        /// Contient toutes les informations pour un jour
        /// </summary>
        private Hours vHours = new Hours();
        /// <summary>
        /// Informations pour la cration d'un rendez-vous
        /// </summary>
        private MeetingInfo vMeetingInfo;
        /// <summary>
        /// Donne la couleur pour le dgrad
        /// </summary>
        private Color vFadeColor = Color.Silver;
        /// <summary>
        /// Position of the autoscroll
        /// </summary>
        private Point vAutoScrollPosition;
        /// <summary>
        /// Indicates whether we middle button of the mouse was used
        /// </summary>
        private bool vScrolled = false;
        /// <summary>
        /// Last scroll direction
        /// </summary>
        private ScrollDirection vScrollDirection;
        /// <summary>
        /// Gives the type of scroll
        /// </summary>
        private ScrollType vScrollType;
        /// <summary>
        /// Indicates the first hour to display in the DayAgenda
        /// </summary>
        private DateTime vFirstHourToDisplay = new DateTime(1, 1, 1, 8, 30, 0);
        /// <summary>
        /// Indicates the number of lines the scroll needs
        /// </summary>
        private int vLinesToScroll = 1;

        #region Focus
        /// <summary>
        /// Indique si on utilise la souris
        /// </summary>
        Boolean vHaveMouse;
        /// <summary>
        /// Coordonnes du dpart de la souris
        /// </summary>
        Point vStartingFocusePoint = new Point();
        /// <summary>
        /// Coordonne de fin de la souris
        /// </summary>
        Point vPreviousFocusePoint = new Point();
        #endregion
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public DayAgenda()
        {
            InitializeComponent();

            vAutoScrollPosition = this.AutoScrollPosition;
            vHours.SizeOfComponent = pb_Day.Size;
            pb_Day.Height = vHours.Height;
            this.AutoScroll = true;

            //Aucun rendez-vous en cours de cration
            vMeetingInfo.Creating = false;

            this.MeetingList = new List<MeetingRow>();
        }
        #endregion

        #region Event
        /// <summary>
        /// Event Paint
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            if (vScrolled)
            {
                switch (vScrollType)
                {
                    #region ScrollType.MouseScroll
                    case ScrollType.MouseScroll:
                        switch (vScrollDirection)
                        {
                            case ScrollDirection.Maximum:
                                break;
                            case ScrollDirection.Down:
                                g.TranslateClip(0, vLinesToScroll * Hours.HEADER_HEIGHT);
                                break;
                            case ScrollDirection.Up:
                                g.TranslateClip(0, -(vLinesToScroll) * Hours.HEADER_HEIGHT);
                                break;
                        }
                        AutoScrollPosition = vAutoScrollPosition;
                        break;
                    #endregion

                    case ScrollType.ProgrammingScroll:
                        AutoScrollPosition = vAutoScrollPosition;
                        break;
                }
                vScrolled = false;
            }            
            base.OnPaint(e);
        }
        /// <summary>
        /// Evnement Paint
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            RefreshComponent(g);
        }
        /// <summary>
        /// Evnement de changement de taille
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_SizeChanged(object sender, EventArgs e)
        {
            this.pb_Day.Visible = false;
            vHours.SizeOfComponent = pb_Day.Size;

            Graphics g = pb_Day.CreateGraphics();

            foreach (DayRectangleInfo ri in vHours.Recalculate())
            {
                #region Dessinage
                LinearGradientBrush lgb =
                    new LinearGradientBrush(ri.Rectangle, ri.BackColor, ri.BackColor2, LinearGradientMode.Horizontal);
                g.FillRectangle(lgb, ri.Rectangle);


                switch (ri.Position)
                {
                    case MeetingPosition.StartingMeeting:
                        #region StartingMeeting
                        //Ligne suprieure
                        g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + vHours.SizeOfComponent.Width, ri.Rectangle.Location.Y));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion

                    case MeetingPosition.Meeting:
                        #region Meeting

                        break;
                        #endregion

                    case MeetingPosition.EndingMeeting:
                        #region EndingMeeting
                        //Ligne infrieure
                        g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));
                        break;
                        #endregion

                    case MeetingPosition.OutOfMeeting:
                        #region OutOfMeeting
                        g.FillRectangle(new SolidBrush(ri.BackColor), ri.Rectangle);

                        //Ligne suprieure
                        g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                        //Ligne infrieure
                        g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                }
                #endregion
            }
            this.pb_Day.Visible = true;
        }
        /// <summary>
        /// Evnement sur Clique Down de la souris
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                #region Prparation du rendez-vous
                vMeetingInfo.Creating = true;

                foreach (DayRectangleInfo ri in vHours.ListToDraw)
                {
                    if (ri.Rectangle.Contains(e.Location))
                    {
                        vMeetingInfo.Start = vHours.GetDate(ri.Rectangle.Y / vHours.HeaderHeight);
                        vStartingFocusePoint = ri.Rectangle.Location;
                        break;
                    }
                }
                #endregion

                #region Start focus
                this.vStartingFocusePoint = new Point(vHours.HeaderWidth, e.Y);
                this.vStartingFocusePoint.Offset(pb_Day.Left, pb_Day.Top);
                this.vPreviousFocusePoint = vStartingFocusePoint;
                this.vHaveMouse = true;
                #endregion
            }
            else
                vMeetingInfo.Creating = false;
        }
        /// <summary>
        /// Evnement sur Clique Up de la souris
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseUp(object sender, MouseEventArgs e)
        {
            //Identifiant du rendez-vous
            int IDMtg = -1;

            if (e.Button == MouseButtons.Left && vMeetingInfo.Creating)
            {

                #region Prparation du rendez-vous
                foreach (DayRectangleInfo ri in vHours.ListToDraw)
                {
                    if (ri.Rectangle.Contains(e.Location))
                    {
                        vMeetingInfo.End = vHours.GetDate(ri.Rectangle.Y / vHours.HeaderHeight);
                        vPreviousFocusePoint = ri.Rectangle.Location;
                        IDMtg = ri.IDMeeting;
                        break;
                    }
                }
                #endregion

                #region Stop focus
                vHaveMouse = false;
                #endregion
            }
            #region Effacage du rectangle
            Point pCurrent = new Point(vHours.HeaderWidth, e.Y);
            pCurrent.Offset(pb_Day.Left, pb_Day.Top);

            //Prparation du nouveau rectangle
            Rectangle NewRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                new Size(pb_Day.Width - vHours.HeaderWidth, pCurrent.Y - vStartingFocusePoint.Y)));

            //Dessinage                            
            ControlPaint.FillReversibleRectangle(NewRec, Color.Black);
            #endregion

            #region Prparation d'un vnement de demande ou de modification de rendez-vous
            if (vMeetingInfo.Creating)
            {
                if (vMeetingInfo.End < vMeetingInfo.Start)
                    vMeetingInfo.Creating = false;

                //Demande de cration de rendez-vous
                else if (vMeetingInfo.Start < vMeetingInfo.End)
                    OnMeetingAsked(new MeetingEventArgs(vMeetingInfo));

                //Vrification si c'est un rdv  supprimer ou  modifier
                else if (IDMtg != -1)
                {
                    MeetingInfo mi = new MeetingInfo();
                    bool Continue = false;
                    foreach (MeetingRow mr in vHours.Hour)
                    {
                        if (mr.IDMeeting == IDMtg)
                        {
                            mi.Start = mr.StartDate;
                            mi.End = mr.EndDate;
                            mi.Identifiant = IDMtg;
                            Continue = true;
                        }

                    }
                    if (Continue)
                        OnMeetingModificationAsked(new MeetingModificationEventArgs(mi, IDMtg));
                }
            }
            #endregion

            vMeetingInfo.Creating = false;
        }
        /// <summary>
        /// Evnement sur un mouvement de la souris
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseMove(object sender, MouseEventArgs e)
        {
            if (vHaveMouse)
            {
                Point pCurrent = new Point(vHours.HeaderWidth, e.Y);
                pCurrent.Offset(pb_Day.Left, pb_Day.Top);

                //Prparation sur l'ancien rectangle
                Rectangle OldRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                    new Size(pb_Day.Width - vHours.HeaderWidth, vPreviousFocusePoint.Y - vStartingFocusePoint.Y)));

                //Prparation du nouveau rectangle
                Rectangle NewRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                    new Size(pb_Day.Width - vHours.HeaderWidth, pCurrent.Y - vStartingFocusePoint.Y)));

                //Dessinage                
                ControlPaint.FillReversibleRectangle(OldRec, Color.Black);
                ControlPaint.FillReversibleRectangle(NewRec, Color.Black);

                vPreviousFocusePoint = pCurrent;
            }
        }
        /// <summary>
        /// Notifie une demande de modification ou de suppression de renez-vous
        /// </summary>
        /// <param name="args">Informations sur le rendez-vous</param>
        protected virtual void OnMeetingModificationAsked(MeetingModificationEventArgs args)
        {
            if (MeetingModificationAsked != null)
                MeetingModificationAsked(this, args);

        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Rafraichi le composant
        /// </summary>        
        public void RefreshComponent()
        {
            RefreshComponent(pb_Day.CreateGraphics());
        }
        /// <summary>
        /// Rafraichi le composant
        /// </summary>
        /// <param name="g">Objet Graphics pour le dessinage</param>
        protected void RefreshComponent(Graphics g)
        {
            foreach (DayRectangleInfo ri in vHours.Recalculate())
            {
                #region Dessinage
                //Remplissage du rectangle
                LinearGradientBrush lgb =
                    new LinearGradientBrush(ri.Rectangle, ri.BackColor, ri.BackColor2, LinearGradientMode.Horizontal);
                g.FillRectangle(lgb, ri.Rectangle);

                switch (ri.Position)
                {
                    case MeetingPosition.StartingMeeting:
                        #region StartingMeeting
                        //Ligne suprieure
                        g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + vHours.SizeOfComponent.Width, ri.Rectangle.Location.Y));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                    case MeetingPosition.Meeting:
                        #region Meeting

                        break;
                        #endregion
                    case MeetingPosition.EndingMeeting:
                        #region EndingMeeting
                        //Ligne infrieure
                        g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));
                        break;
                        #endregion
                    case MeetingPosition.OutOfMeeting:
                        #region OutOfMeeting
                        g.FillRectangle(new SolidBrush(ri.BackColor), ri.Rectangle);

                        //Ligne suprieure
                        g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                        //Ligne infrieure
                        g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                }
                #endregion
            }
        }
        /// <summary>
        /// Ask Scrol up or down the Day_Agenda
        /// </summary>
        /// <param name="_MoveLines">Number of lines we'll move up (if positive) or move down (if negative)</param>
        public void Wheel(int _MoveLines)
        {
            vLinesToScroll = Math.Abs(_MoveLines);
            int X = Math.Abs(this.AutoScrollPosition.X);
            int Y = Math.Abs(this.AutoScrollPosition.Y);
            int LastY = pb_Day.Size.Height - this.Size.Height + _MoveLines + 1;

            #region Move Up or Down
            if (_MoveLines > 0) //UP
            {
                Y -= (vLinesToScroll * Hours.HEADER_HEIGHT);
                vScrollDirection = Y < 0 ? ScrollDirection.Maximum : ScrollDirection.Up;
                //Y = Y < 0 ? 0 : Y;
            }
            else //DOWN
            {
                Y += (vLinesToScroll * Hours.HEADER_HEIGHT);
                vScrollDirection = Y > LastY ? ScrollDirection.Maximum : ScrollDirection.Down;
                //Y = Y > LastY ? LastY + 10 : Y;
            }
            #endregion

            vAutoScrollPosition = new Point(X, Y);
            vScrolled = true;
            vScrollType = ScrollType.MouseScroll;
            this.Refresh();
        }
        /// <summary>
        /// Shows the first hour of the day
        /// </summary>
        public void DisplayFirstHour()
        {
            int Minutes = (vFirstHourToDisplay.Hour * 60) + vFirstHourToDisplay.Minute;
            int Line = Minutes / TimeOfMeeting;
            if (Line <= Hours.DAYS_PARTS)
            {
                this.vAutoScrollPosition = new Point(vAutoScrollPosition.X, Line * Hours.HEADER_HEIGHT);
                this.vScrolled = true;
                this.vScrollType = ScrollType.ProgrammingScroll;
                this.Refresh();
            }
        }
        #endregion

        #region Personal Event
        /// <summary>
        /// Evnement sur demande de rendez-vous
        /// </summary>
        [Category("DayAgenda-Events")]
        [Description("Se produit lorsque on selectionne une plage d'heures pour demander un rendez-vous")]
        public event MeetingAskedHandler MeetingAsked;
        /// <summary>
        /// Notifie un vnement de demande de rendez-vous
        /// </summary>
        /// <param name="args">Informations sur le rendez-vous</param>
        protected virtual void OnMeetingAsked(MeetingEventArgs args)
        {
            if (MeetingAsked != null)
                MeetingAsked(this, args);
        }
        /// <summary>
        /// Evnement sur une demande de modification ou de suppression de rendez-vous
        /// </summary>
        [Category("DayAgenda-Events")]
        [Description("Se produit lorsque on fait une demande de modification ou de suppression d'un rendez-vous")]
        public event MeetingModificationAskedHandler MeetingModificationAsked;

        #endregion

        #region Accesseurs
        /// <summary>
        /// Initialise la liste des rendez-vous de la journe
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get
            {
                if (vHours.MeetingList == null)
                    return new List<MeetingRow>();
                return vHours.MeetingList;
            }
            set
            {
                vHours.MeetingList = value;
                vHours.Recalculate();
            }
        }
        /// <summary>
        /// Initialise la date  afficher dans le DayAgenda
        /// </summary>
        [Browsable(false)]
        public DateTime DateToDisplay
        {
            get { return vHours.DateToDisplay; }
            set { vHours.DateToDisplay = value; }
        }
        /// <summary>
        /// Gets the laps of time (in minutes) of a rendez-vous
        /// </summary>
        [Browsable(false)]
        public static int TimeOfMeeting
        {
            get
            {
                float ret = 24;
                ret /= Hours.DAYS_PARTS;
                ret *= 60;
                if (ret == 0) ret = 1;
                return (int)ret;
            }
        }
        /// <summary>
        /// Gets or sets the first hour to display in the DayAgenda
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("Initialise la premire heure  afficher dans le DayAgenda")]
        public DateTime FirstHourToDisplay
        {
            get { return vFirstHourToDisplay; }
            set { vFirstHourToDisplay = value; }
        }

        #region Colors Options
        /// <summary>
        /// Retourne ou initialise le couleur de fond des heures sans rendez-vous
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("Initialise la couleur de fond d'un jour sans rendez-vous")]
        public Color NonMeetingBackColor
        {
            get { return vHours.NonMeetingBackColor; }
            set
            {
                vHours.NonMeetingBackColor = value;
                vHours.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond d'un jour avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Initialise la couleur de fond d'un jour avec rendez-vous")]
        public Color MeetingFadeColorTwo
        {
            get { return vHours.SecondMeetingBackcolor; }
            set
            {
                vHours.SecondMeetingBackcolor = value;
                vHours.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur du texte d'une heure avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Initialise la couleur du texte d'une heure avec rendez-vous")]
        public Color MeetingTextColor
        {
            get { return vHours.RdvTextColor; }
            set
            {
                vHours.RdvTextColor = value;
                vHours.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture d'une heure avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("Initialise la police d'criture d'une heure avec rendez-vous")]
        public Font MeetingTextFont
        {
            get { return vHours.RdvTextFont; }
            set
            {
                vHours.RdvTextFont = value;
                vHours.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur pour le dgrad de couleur d'un rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Initialise la couleur pour le dgrad de couleur d'un rendez-vous")]
        public Color MeetingFadeColorOne
        {
            get { return vHours.FirstMeetingFadeColor; }
            set
            {
                vHours.FirstMeetingFadeColor = value;
                vHours.Recalculate();
            }

        }
        #endregion
        #endregion
    }

    #region struct
    /// <summary>
    /// Contient les informations pour la cration d'un rendez-vous
    /// </summary>
    public struct MeetingInfo
    {
        /// <summary>
        /// Est-ce un rendez-vous en cours de cration
        /// </summary>
        public bool Creating;

        /// <summary>
        /// Date de dpart du rendez-vous
        /// </summary>
        public DateTime Start;

        /// <summary>
        /// Date de fin du rendez-vous
        /// </summary>
        public DateTime End;

        /// <summary>
        /// Identifiant du rendez-vous
        /// </summary>
        public int Identifiant;
    }
    /// <summary>
    /// Scroll direction
    /// </summary>
    internal enum ScrollDirection
    {
        /// <summary>
        /// Scroll up
        /// </summary>
        Up = 1,
        /// <summary>
        /// Scroll down
        /// </summary>
        Down = -1,
        /// <summary>
        /// Scrolled to maximum (MaxUp or MaxDown)
        /// </summary>
        Maximum = 0,
    }
    /// <summary>
    /// Gives the type of scroll
    /// </summary>
    internal enum ScrollType
    {
        /// <summary>
        /// Scroll from the mouse
        /// </summary>
        MouseScroll,
        /// <summary>
        /// Programmatic asking of scrolling
        /// </summary>
        ProgrammingScroll,
    }
    #endregion
}
