using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Components.Agenda.Management;
using System.Drawing.Design;
using Utils.Misc;
using Components.TypeEditors;
using System.IO;
using Components.Agenda.Management.StructsEnums;

namespace Components.Agenda.GUI
{
    #region Delegate
    /// <summary>
    /// Handler d'vnement de mois chang
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    /// <returns></returns>
    public delegate void MonthChangedHandler(object sender, MonthEventArgs e);

    /// <summary>
    /// Handler pour l'vnement de clique sur un jour
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void ClickedOnDayHandler(object sender, DayEventArgs e);
    #endregion

    #region Class
    /// <summary>
    /// Composant agenda
    /// </summary>
    public partial class MonthAgenda : UserControl
    {
        #region Variables
        /// <summary>
        /// Langue du callandrier
        /// </summary>
        private Language vLanguage;

        /// <summary>
        /// Contient toutes les cases de jours du calendrier
        /// </summary>
        private Days vDays;

        /// <summary>
        /// Date  afficher
        /// </summary>
        private DateTime vDateToDisplay = DateTime.Now;

        /// <summary>
        /// Indique s'il faut faire Next ou Previous month ou rien
        /// </summary>
        private int vWhereClicked = 0; //0 Previous Month; 1 Nothing; 2 Next Month    
        private const int PREVIOUS_MONTH = 0;
        private const int NOTHING = 1;
        private const int NEXT_MONTH = 2;

        #region Header
        /// <summary>
        /// Hauteur du header
        /// </summary>
        private int vHeaderHeight = 40;

        /// <summary>
        /// Info de dessinage du header
        /// </summary>
        private MonthRectangleInfo[] vHeader = new MonthRectangleInfo[3];
        #endregion

        #region HeaderDay
        /// <summary>
        /// Hauteur du header de jour
        /// </summary>
        private int vHeaderDayHeight = 30;

        /// <summary>
        /// Header contenant le nom des jours
        /// </summary>
        private MonthRectangleInfo[] vHeaderDays = new MonthRectangleInfo[7];
        #endregion

        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public MonthAgenda()
        {
            InitializeComponent();

            for (int i = 0; i < 3; i++)
                vHeader[i] = new MonthRectangleInfo();

            string[] month;
            string[] day;

            vLanguage = new Language(ApplicationPath.CommandPath + @"\OptionsFiles\language.xml");

            #region Days
            vDays = new Days(pb_Agenda.Size, vHeaderHeight + vHeaderDayHeight);
            vDays.Resize();
            #endregion

            #region Nom jour et mois
            if (File.Exists(ApplicationPath.CommandPath + @"\OptionsFiles\language.xml"))
            {
                month = vLanguage.GetMonthData();
                vHeader[1].DayNumber = "" + vDateToDisplay.Day + " " + month[vDateToDisplay.Month]
                    + " " + vDateToDisplay.Year;

                day = vLanguage.GetDayData();

            }
            else
            {
                vLanguage = new Language();
                month = vLanguage.GetMonthData();
                day = vLanguage.GetDayData();

                vHeader[1].DayNumber = month[vDateToDisplay.Month - 1] + " " + vDateToDisplay.Year;
            }
            #endregion

            #region Header
            SetHeader();
            #endregion

            #region HeaderDay
            //Informations sur le header de jours
            Rectangle r = new Rectangle(new Point(0, vHeaderHeight),
                new Size(((pb_Agenda.Width) / 7) + (pb_Agenda.Width % 7) - 1, this.vHeaderDayHeight));
            Color CBack = Color.White;
            Color CText = Color.Black;
            Font FText = new Font("Times new Roman", 12);
            Pen PSur = new Pen(Brushes.Black, 1);


            for (int i = 0; i < 7; i++)
            {
                if (vHeaderDays[i] == null)
                    vHeaderDays[i] = new MonthRectangleInfo();

                vHeaderDays[i].Rectangle = r;
                r = new Rectangle(new Point(r.X + r.Width, vHeaderHeight),
                    new Size((pb_Agenda.Width) / 7, this.vHeaderDayHeight));
                //vHeaderDays[i].BackColor = CBack;
                //vHeaderDays[i].TextColor = CText;
                //vHeaderDays[i].TextFont = FText;
                //vHeaderDays[i].SurroundPen = PSur;
                vHeaderDays[i].DayNumber = day[i];
            }
            #endregion
        }
        #endregion

        #region Mthodes prives
        /// <summary>
        /// Dessine le composant
        /// </summary>
        /// <param name="g">Surface de dessin</param>
        protected void Redraw(Graphics g)
        {
            #region Header
            SetHeader();

            //Cration de la date du header
            string[] month = vLanguage.GetMonthData();
            string[] day = vLanguage.GetDayData();
            vHeader[1].DayNumber = month[vDateToDisplay.Month - 1] + " " + vDateToDisplay.Year;
            SizeF CoordString;

            for (int i = 0; i < 3; i++)
            {
                g.FillRectangle(new SolidBrush(vHeader[i].BackColor), vHeader[i].Rectangle);
                g.DrawRectangle(vHeader[i].BorderPen, vHeader[i].Rectangle);

                CoordString = g.MeasureString(vHeader[i].DayNumber, vHeader[i].TextFont);

                CoordString.Width = ((vHeader[i].Rectangle.Width) / 2) - (CoordString.Width / 2);
                CoordString.Height = ((vHeader[i].Rectangle.Height) / 2) - (CoordString.Height / 2);

                if (i == 2)
                    CoordString.Width += pb_Agenda.Width - 51;

                g.DrawString(vHeader[i].DayNumber, vHeader[i].TextFont, new SolidBrush(vHeader[i].TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));
            }
            #endregion

            #region Header Days
            for (int i = 0; i < 7; i++)
            {
                vHeaderDays[i].DayNumber = day[i];
                g.FillRectangle(new SolidBrush(vHeaderDays[i].BackColor), vHeaderDays[i].Rectangle);
                g.DrawRectangle(vHeaderDays[i].BorderPen, vHeaderDays[i].Rectangle);

                CoordString = g.MeasureString(vHeaderDays[i].DayNumber, vHeaderDays[i].TextFont);
                CoordString.Width = vHeaderDays[i].Rectangle.X
                    + (vHeaderDays[i].Rectangle.Width / 2) - (CoordString.Width / 2);

                CoordString.Height = vHeaderDays[i].Rectangle.Y
                    + (vHeaderDays[i].Rectangle.Height / 2) - (CoordString.Height / 2);

                g.DrawString(vHeaderDays[i].DayNumber, vHeaderDays[i].TextFont, new SolidBrush(vHeaderDays[i].TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));
            }
            #endregion

            #region Days
            vDays.Date = vDateToDisplay;
            vDays.Resize();

            //Dessinage des jours dans la calendrier            
            foreach (MonthRectangleInfo r in vDays)
            {
                //Affichage d'aujourd'hui
                if (vDays.TodayIndex.ToString() == r.DayNumber && vDays.IsActualMonth)
                    g.FillRectangle(new SolidBrush(vDays.TodayBackColor), r.Rectangle);
                else
                    g.FillRectangle(new SolidBrush(r.BackColor), r.Rectangle);
                g.DrawRectangle(r.BorderPen, r.Rectangle);

                //On dessine les jours en haut  droite de la case
                CoordString = g.MeasureString(r.DayNumber, r.TextFont);
                CoordString.Width = r.Rectangle.X + (r.Rectangle.Width - CoordString.Width);
                //+ (r.Rectangle.Width / 2) - (CoordString.Width / 2);

                CoordString.Height = r.Rectangle.Y;
                //+ (r.Rectangle.Height / 2) - (CoordString.Height / 2);

                Font nf = new Font(r.TextFont, FontStyle.Bold);
                g.DrawString(r.DayNumber, r.TextFont, new SolidBrush(r.TextColor),
                    new Point((int)CoordString.Width, (int)CoordString.Height));

                //Affichage des nombre de rdv
                g.DrawString(r.Text, nf, new SolidBrush(r.TextColor),
                    new Point(r.Rectangle.X, (r.Rectangle.Y + r.Rectangle.Height)
                    - (int)g.MeasureString(r.Text, r.TextFont).Height));

                r.Text = "";
            }
            #endregion
        }
        /// <summary>
        /// Demande d'un redessinage de la fentre
        /// </summary>
        protected void Redraw()
        {
            Graphics g = pb_Agenda.CreateGraphics();
            Redraw(g);
        }
        /// <summary>
        /// Demande d'un redessinage de la fentre
        /// </summary>
        /// <param name="_SizeChanged">La taille de la fentre  change?</param>
        protected void Redraw(bool _SizeChanged)
        {
            #region Resize de la fentre
            if (_SizeChanged)
            {

                vHeader[1].Rectangle = new Rectangle(new Point(0, 0), new Size(pb_Agenda.Width - 1, this.vHeaderHeight));

                #region HeaderDays
                Rectangle r = new Rectangle(new Point(0, vHeaderHeight),
                    new Size(((pb_Agenda.Width) / 7) + (pb_Agenda.Width % 7) - 1, this.vHeaderDayHeight));

                for (int i = 0; i < 7; i++)
                {
                    vHeaderDays[i].Rectangle = r;
                    r = new Rectangle(new Point(r.X + r.Width, vHeaderHeight),
                        new Size((pb_Agenda.Width) / 7, this.vHeaderDayHeight));
                }
                #endregion

                #region Days
                vDays.ComponentSize = pb_Agenda.Size;
                #endregion
            }
            #endregion

            Graphics g = pb_Agenda.CreateGraphics();
            Redraw(g);
        }
        /// <summary>
        /// Prpare la taille des trois cellules du header de base
        /// </summary>
        protected void SetHeader()
        {
            vHeader[0].Rectangle = new Rectangle(new Point(0, 0),
                new Size(51, this.vHeaderHeight));

            //Next et Previous son 10 ton plus bas que le reste
            Color MiddleColor = vHeader[2].BackColor;
            int _R = MiddleColor.R;
            int _G = MiddleColor.R;
            int _B = MiddleColor.B;

            Color TColor;
            if (_R * _G * _B > 385)
                TColor = Color.Black;
            else
                TColor = Color.White;

            vHeader[0].BackColor = vHeader[2].BackColor;
            vHeader[0].TextColor = TColor;
            vHeader[0].TextFont = new Font("Times new Roman", 15);
            vHeader[0].BorderPen = new Pen(Brushes.White, 1);
            vHeader[0].DayNumber = "<<";

            vHeader[1].Rectangle = new Rectangle(new Point(51, 0),
                new Size(pb_Agenda.Width - 2 * 51, this.vHeaderHeight));
            //vHeader[1].BackColor = vHeader[2].BackColor;
            //vHeader[1].TextColor= this.
            //vHeader[1].TextFont = new Font("Times new Roman", 26);
            vHeader[1].BorderPen = new Pen(vHeader[1].BackColor, 1);

            vHeader[2].Rectangle = new Rectangle(new Point(pb_Agenda.Width - 51, 0),
                new Size(51, this.vHeaderHeight));
            vHeader[2].BackColor = vHeader[1].BackColor;
            vHeader[2].TextColor = TColor;
            vHeader[2].TextFont = new Font("Times new Roman", 15);
            vHeader[2].BorderPen = new Pen(vHeader[2].BackColor, 1);
            vHeader[2].DayNumber = ">>";
        }
        /// <summary>
        /// Teste si le bouton de la souris se situe sur un des boutons de naviguation
        /// des mois
        /// </summary>
        /// <param name="e">Informations sur le souris sous forme de MouseEventArgs</param>
        /// <returns>True si le curseur est sur un boutn de navigation false dans le cas contraire</returns>
        protected bool IsCursorOnMonth(MouseEventArgs e)
        {
            int zWidth = vHeader[0].Rectangle.Width;
            int zHeight = vHeader[0].Rectangle.Height;
            int zX = vHeader[0].Rectangle.X;
            int zY = vHeader[0].Rectangle.Y;

            if ((e.X > zX && e.X < zWidth + zX) && (e.Y > zY && e.Y < zHeight + zY))
            {
                Cursor.Current = Cursors.Hand;
                vWhereClicked = 0;
                return true;
            }
            else
                Cursor.Current = Cursors.Default;

            zWidth = vHeader[2].Rectangle.Width;
            zHeight = vHeader[2].Rectangle.Height;
            zX = vHeader[2].Rectangle.X;
            zY = vHeader[2].Rectangle.Y;

            if ((e.X > zX && e.X < zWidth + zX) && (e.Y > zY && e.Y < zHeight + zY))
            {
                Cursor.Current = Cursors.Hand;
                vWhereClicked = 2;
                return true;
            }
            else
            {
                Cursor.Current = Cursors.Default;
                return false;
            }
        }
        /// <summary>
        /// Vrifie si la souris se trouve sur une case de jour
        /// </summary>
        /// <param name="e">Informations sur l'emplacement de la souris</param>       
        /// <returns>Le numro du jour ou 0 si ce n'est pas un jour</returns>
        protected MonthRectangleInfo IsCursorOnDay(MouseEventArgs e)
        {
            foreach (MonthRectangleInfo r in vDays)
            {
                if (r.Rectangle.Contains(e.Location))
                    return r;
            }
            return null;
        }
        /// <summary>
        /// Indique si le curseur est sur le header
        /// </summary>
        /// <param name="e">Informations sur le curseur de la souris</param>
        /// <returns>True si le curseur se trouve sur le header.</returns>
        protected bool IsCursorOnHeader(MouseEventArgs e)
        {
            if (vHeader[1].Rectangle.Contains(e.Location))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region Mthodes publique
        /// <summary>
        /// Dplace le mois afficher du nombre de mois entr en arguments
        /// </summary>
        /// <param name="_Month">Nombre de mois a ajouter pour le dplacement (un ngatif = retour en arrire)</param>
        public void MoveMonth(int _Month)
        {
            this.DateToDisplay = vDateToDisplay.AddMonths(_Month);
            OnMonthChanged(new MonthEventArgs(vDateToDisplay, vDateToDisplay.AddMonths(_Month)));
        }
        /// <summary>
        /// Displays the current month
        /// </summary>
        public void DisplayToday()
        {
            DateTime dtBuf = vDateToDisplay;
            this.DateToDisplay = DateTime.Today;
            OnMonthChanged(new MonthEventArgs(vDateToDisplay, dtBuf));
        }
        #endregion

        #region Events
        /// <summary>
        /// Evnement paint de redessinage du calendrier
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Paint(object sender, PaintEventArgs e)
        {
            Redraw(e.Graphics);
        }
        /// <summary>
        /// Evnement sur changement de taille
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_SizeChanged(object sender, EventArgs e)
        {
            Redraw(true);
        }
        /// <summary>
        /// Event lorsque la souris passe sur le composant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseMove(object sender, MouseEventArgs e)
        {
            IsCursorOnMonth(e);
        }
        /// <summary>
        /// Event lorsque il y a un relachement du bouton de la souris
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseDown(object sender, MouseEventArgs e)
        {
            #region Clique sur mois-anne
            if (IsCursorOnHeader(e))
            {
                DateSelector ds = new DateSelector();
                ds.MonthNames = vLanguage.GetMonthData();
                ds.Language = vLanguage.SelectedLanguage;
                ds.Date = vDateToDisplay;

                Graphics g = pb_Agenda.CreateGraphics();
                SizeF CoordString = g.MeasureString(vHeader[1].DayNumber, vHeader[1].TextFont);
                g.Dispose();
                Point point = this.PointToScreen(vHeader[1].Rectangle.Location);
                ds.Location = new Point((int)(point.X + CoordString.Width), (int)(point.Y + CoordString.Height / 2));

                ds.ShowDialog();
                try { this.DateToDisplay = new DateTime(ds.Year, ds.Month, 1); }
                catch (ArgumentOutOfRangeException ex) { MessageBox.Show(ex.Message); }
                finally { ds.Dispose(); }

            }
            #endregion

            #region Clique sur Next - Previous month
            if (e.Button == MouseButtons.Left && IsCursorOnMonth(e))
            {
                if (vWhereClicked == PREVIOUS_MONTH)
                {
                    this.DateToDisplay = vDateToDisplay.AddMonths(-1);
                    vWhereClicked = NOTHING;
                    OnMonthChanged(new MonthEventArgs(vDateToDisplay, vDateToDisplay.AddMonths(-1)));
                    //MessageBox.Show("Sub Month");
                }
                if (vWhereClicked == NEXT_MONTH)
                {
                    this.DateToDisplay = vDateToDisplay.AddMonths(1);
                    vWhereClicked = NOTHING;
                    OnMonthChanged(new MonthEventArgs(vDateToDisplay, vDateToDisplay.AddMonths(1)));
                    //MessageBox.Show("Add Month");
                }

            }
            #endregion

            #region Clique sur un jour
            MonthRectangleInfo r = IsCursorOnDay(e);
            if (e.Button == MouseButtons.Left && r != null)
            {
                if (r.Position == CasePositionInMonth.InMonth)
                {
                    OnClickedOnDay(new DayEventArgs(new DateTime(vDateToDisplay.Year,
                        vDateToDisplay.Month, Int32.Parse(r.DayNumber))));
                    //MessageBox.Show("Clique sur la date: "
                    //     + new DateTime(vDateToDisplay.Year, vDateToDisplay.Month, Int32.Parse(r.DayNumber)).ToString());
                }
                else if (r.Position == CasePositionInMonth.After)
                {
                    this.DateToDisplay = vDateToDisplay.AddMonths(1);
                    vWhereClicked = 1;
                    OnMonthChanged(new MonthEventArgs(vDateToDisplay, vDateToDisplay.AddMonths(1)));
                }
                else
                {
                    this.DateToDisplay = vDateToDisplay.AddMonths(-1);
                    vWhereClicked = 1;
                    OnMonthChanged(new MonthEventArgs(vDateToDisplay, vDateToDisplay.AddMonths(-1)));
                }
            }
            #endregion
        }
        #endregion

        #region Personal Event
        /// <summary>
        /// Evnement pour le changement de mois
        /// </summary>
        [Category("MonthAgenda-Events")]
        [Description("Se produit lorsque l'on clique sur un bouton de changement de mois")]
        public event MonthChangedHandler MonthChanged;

        /// <summary>
        /// Notifie l'vnement de changement de mois
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnMonthChanged(MonthEventArgs args)
        {
            if (MonthChanged != null)
                MonthChanged(this, args);
        }

        /// <summary>
        /// Evnement de clique sur une journe
        /// </summary>
        [Category("MonthAgenda-Events")]
        [Description("Se produite lorsque on clique sur une case de jour de l'agenda")]
        public event ClickedOnDayHandler ClickedOnDay;

        /// <summary>
        /// Notify d'un clique sur une case de jour
        /// </summary>
        /// <param name="args">Informations sur l'vnement</param>
        public virtual void OnClickedOnDay(DayEventArgs args)
        {
            if (ClickedOnDay != null)
                ClickedOnDay(this, args);
        }
        #endregion

        #region Accesseur
        /// <summary>
        /// Retourne ou initialise la langue du Calmendrier
        /// </summary>        
        [Browsable(true)]
        [Description("Initialise la langue du calendrier")]
        [Category("Options")]
        public string Language
        {
            set
            {
                vLanguage.SelectedLanguage = value;
                vLanguage.Load();
                Redraw();
            }
            get { return vLanguage.SelectedLanguage; }
        }
        /// <summary>
        /// Retourne ou initialise la path du fichier de langues
        /// </summary>
        [Browsable(true)]
        [Description("Initialise la path du fichier de langue")]
        [Category("Options")]
        [Editor(typeof(UIPathEditor), typeof(UITypeEditor))]
        public string LanguageFile
        {
            get { return vLanguage.Path; }
            set
            {
                vLanguage.Path = value;
                Redraw();
            }
        }
        /// <summary>
        /// Retourne ou initialise la date a afficher
        /// </summary>
        [Category("Options")]
        [Description("Initialise la date a affichier")]
        [Browsable(false)]
        public DateTime DateToDisplay
        {
            get { return vDateToDisplay; }
            set
            {
                vDateToDisplay = value;
                Redraw();
            }
        }

        #region Header
        /// <summary>
        /// Couleur du texte du header
        /// </summary>
        [Description("Initialise la couleur du texte du header")]
        [Category("Options - Header")]
        public Color HeaderTextColor
        {
            get { return vHeader[1].TextColor; }
            set
            {
                vHeader[1].TextColor = value;
                Redraw();
            }
        }

        /// <summary>
        /// Police du header
        /// </summary>
        [Category("Options - Header")]
        [Description("Initialise la police du texte")]
        public Font HeaderTextFont
        {
            get { return vHeader[1].TextFont; }
            set
            {
                if (value.Size > 32)
                    value = new Font(value.Name, 32);
                vHeader[1].TextFont = value;
                Redraw();
            }
        }

        /// <summary>
        /// Couleur de fond du header
        /// </summary>
        [Category("Options - Header")]
        [Description("Initialise la couleur de fond du header")]
        public Color HeaderBackColor
        {
            get { return vHeader[1].BackColor; }
            set
            {
                for (int i = 0; i < vHeader.Length; i++)
                    vHeader[i].BackColor = value;

                Redraw();
            }
        }

        /// <summary>
        /// Couleur de contour du header
        /// </summary>
        [Category("Options - Header")]
        [Browsable(false)]
        public Pen HeaderBorder
        {
            get { return vHeader[1].BorderPen; }
            set
            {
                for (int i = 0; i < vHeader.Length; i++)
                    vHeader[i].BorderPen = value;
                Redraw();
            }
        }
        #endregion

        #region Header Day
        /// <summary>
        /// Couleur du texte du header de jour
        /// </summary>
        [Description("Initialise la couleur du texte du header")]
        [Category("Options - HeaderDay")]
        public Color HeaderDayTextColor
        {
            get { return vHeaderDays[1].TextColor; }
            set
            {
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].TextColor = value;
                Redraw();
            }
        }

        /// <summary>
        /// Police du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Description("Initialise la police du texte")]
        public Font HeaderDayTextFont
        {
            get { return vHeaderDays[1].TextFont; }
            set
            {
                if (value.Size > 20)
                    value = new Font(value.Name, 20);

                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].TextFont = value;
                Redraw();
            }
        }

        /// <summary>
        /// Couleur de fond du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Description("Initialise la couleur de fond du header")]
        public Color HeaderDayBackColor
        {
            get { return vHeaderDays[1].BackColor; }
            set
            {
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BackColor = value;
                Redraw();
            }
        }

        /// <summary>
        /// Couleur de contour du header de jour
        /// </summary>
        [Category("Options - HeaderDay")]
        [Browsable(false)]
        public Pen HeaderDaySurround
        {
            get { return vHeaderDays[1].BorderPen; }
            set
            {
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BorderPen = value;
                Redraw();
            }
        }
        #endregion

        #region Days
        /// <summary>
        /// Retourne ou initialise la couleur d'criture des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Initialise le couleur d'criture des jours")]
        public Color DaysTextColor
        {
            get { return vDays.ColorText; }
            set
            {
                vDays.ColorText = value;
                Redraw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la police d'affichage des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Iitialise la police d'affichage des jours")]
        public Font DaysTextFont
        {
            get { return vDays.Font; }
            set
            {
                vDays.Font = value;
                Redraw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Initialise la couleur de fond des jours")]
        public Color DaysBackColor
        {
            get { return vDays.ColorBack; }
            set
            {
                vDays.ColorBack = value;
                Redraw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours hors du mois
        /// </summary>
        [Category("Options-Days")]
        [Description("Initialise la couleur de fond des jours hors du mois")]
        public Color DaysNonInMonthColor
        {
            get { return vDays.ColorNonMonthBack; }
            set
            {
                vDays.ColorNonMonthBack = value;
                Redraw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la couleur des colonnes du tableau des jours
        /// </summary>
        [Category("Options-Days")]
        [Description("Initialise la couleur des colonnes du tableau des jours")]
        public Color DaysBorderColor
        {
            get { return vDays.ColorSurround; }
            set
            {
                vDays.ColorSurround = value;
                for (int i = 0; i < vHeaderDays.Length; i++)
                    vHeaderDays[i].BorderPen = new Pen(value);
                Redraw();
            }
        }

        /// <summary>
        /// Retourne ou initialise le couleur de fond d'aujourd'hui
        /// </summary>
        [Category("Options-Today")]
        [Description("Initialise la couleur de fond d'aujourd'hui")]
        public Color TodayBackColor
        {
            get { return vDays.TodayBackColor; }
            set
            {
                vDays.TodayBackColor = value;
                Redraw();
            }
        }

        /// <summary>
        /// Donne la liste des remarques des rendez-vous
        /// </summary>
        [Browsable(false)]
        public List<MeetingRow> MeetingList
        {
            get { return vDays.MeetingList; }
            set
            {
                vDays.MeetingList = value;
                Redraw();
            }
        }
        #endregion
        #endregion
    }
    #endregion

    #region EventArgs
    /// <summary>
    /// Classe d'argument de changement de mois dans le callandrier
    /// </summary>
    public class MonthEventArgs
    {
        #region Variables
        /// <summary>
        /// Date avant l'vnement
        /// </summary>
        private DateTime vCurrentDate;

        /// <summary>
        /// Date aprs le changement de mois
        /// </summary>
        private DateTime vNewDate;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_CurrentDate">Date avant le changement de date</param>
        /// <param name="_NewDate">Date aprs le changement de date</param>
        public MonthEventArgs(DateTime _CurrentDate, DateTime _NewDate)
        {
            vCurrentDate = _CurrentDate;
            vNewDate = _NewDate;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne le mois avant le changement de mois
        /// </summary>
        public int CurrentMonth { get { return vCurrentDate.Month; } }

        /// <summary>
        /// Retourne l'anne avant le changement de mois
        /// </summary>
        public int CurrentYear { get { return vCurrentDate.Year; } }

        /// <summary>
        /// Retourne la date avant le changement de mois
        /// </summary>
        public DateTime CurrentDate { get { return vCurrentDate; } }

        /// <summary>
        /// Retourne le mois aprs le changement de mois
        /// </summary>
        public int NewMonth { get { return vNewDate.Month; } }

        /// <summary>
        /// Retourne l'anne aprs le changement de mois
        /// </summary>
        public int NewYear { get { return vNewDate.Year; } }

        /// <summary>
        /// Retourne la date aprs le changement de mois
        /// </summary>
        public DateTime NewDate { get { return vNewDate; } }

        #endregion
    }

    /// <summary>
    /// Information sur un vnement de clique sur un jour
    /// </summary>
    public class DayEventArgs
    {
        #region Variables
        /// <summary>
        /// Date de la case sur laquelle on a cliquer
        /// </summary>
        private DateTime vDateKey;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_DateKey">Date de la case clique</param>
        public DayEventArgs(DateTime _DateKey) { vDateKey = _DateKey; }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne la date de la case sur laquelle on a cliqu
        /// </summary>
        public DateTime Date { get { return vDateKey; } }
        #endregion
    }
    #endregion
}
