using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Components.Agenda.GUI
{
    /// <summary>
    /// Fentre de selection de date et de jour
    /// </summary>
    public partial class DateSelector : Form
    {
        #region Variables
        /// <summary>
        /// Mois selectionn
        /// </summary>
        private int vMonth;
        /// <summary>
        /// Anne selectionne
        /// </summary>
        private int vYear;
        /// <summary>
        /// Nom des 12 mois de l'anne
        /// </summary>
        private string[] vMonthNames = new string[12];
        /// <summary>
        /// Langue du callendrier
        /// </summary>
        private string vLanguage = "";
        /// <summary>
        /// Date selectionne
        /// </summary>
        private DateTime vDate = DateTime.Now;
        /// <summary>
        /// Inqique si l'vnement de changement d'index peut tre lanc
        /// </summary> 
        private bool IsEventable = false;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public DateSelector()
        {
            InitializeComponent();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le nom des 12 mois de l'anne
        /// </summary>
        public string[] MonthNames
        {
            get { return vMonthNames; }
            set
            {
                vMonthNames = value;
                foreach (string month in vMonthNames)
                    cb_Month.Items.Add(month);
            }
        }

        /// <summary>
        /// Retourne ou initialise la langue du calendrier
        /// </summary>
        public string Language
        {
            get { return vLanguage; }
            set { vLanguage = value; }
        }

        /// <summary>
        /// Retourne ou initialise la date courrante du calendrier
        /// </summary>
        public DateTime Date
        {
            get { return vDate; }
            set
            {
                vDate = value;
                cb_Month.SelectedIndex = vDate.Month - 1;

                for (int i = vDate.Year - 10; i < vDate.Year + 10; i++)
                    cb_Year.Items.Add(i);
                cb_Year.SelectedIndex = 10;
                IsEventable = true;
            }
        }

        /// <summary>
        /// Retourne le mois selectionn
        /// </summary>
        public int Month
        {
            get
            {
                if (vMonth > 1 || vMonth <= 12)
                    return vMonth + 1;
                else
                    return DateTime.Today.Month;
            }
        }

        /// <summary>
        /// Retourne l'anne selectionne
        /// </summary>
        public int Year { get { return vYear; } }
        #endregion

        #region Event
        /// <summary>
        /// Evnement sur une touche appuye
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cb_Month_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                vMonth = vDate.Month -1;
                vYear = vDate.Year;
                this.Close();
            }
                        
        }
        /// <summary>
        /// Le mois a chang
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (IsEventable)
            {
                vMonth = cb_Month.SelectedIndex;
                vYear = (int)cb_Year.SelectedItem;
                this.Close();
            }
        }
        #endregion
    }
}