using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Components.Agenda.Management;
using Components.Agenda.Management.StructsEnums;

namespace Components.Agenda.GUI
{
    /// <summary>
    /// Composant de selection d'heure
    /// </summary>
    public partial class HourSelector : UserControl
    {
        #region Variables
        /// <summary>
        /// Contient la valeur des heures
        /// </summary>
        protected HourSelectorRectangleInfo[] vHour = new HourSelectorRectangleInfo[24];
        /// <summary>
        /// Contient la valeur des minutes
        /// </summary>
        protected HourSelectorRectangleInfo[] vMinutes = new HourSelectorRectangleInfo[60];
        /// <summary>
        /// Taille du composant
        /// </summary>
        protected Size vHourSize;
        /// <summary>
        /// Affichage des minutes
        /// </summary>
        protected MinutesSelection vMinutesSelection = MinutesSelection.Quarter;
        /// <summary>
        /// Donne l'paisseur du trait qui spare les heures des minutes
        /// </summary>
        protected const int vBorderWidth = 4;
        /// <summary>
        /// Indique le nombre de colonnes  afficher
        /// </summary>
        protected int vColumns = 1;
        /// <summary>
        /// Date selectionne
        /// </summary>
        protected DateTime vSelectedHour = new DateTime();
        /// <summary>
        /// Indique si le retour heure selectionne est visible
        /// </summary>
        protected bool vHourVisible = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public HourSelector()
        {
            InitializeComponent();

            Recalculate();
            DrawHour(pb_Hour.CreateGraphics());
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Mthodes de redessinage des heures du composant
        /// </summary>
        /// <param name="g">Objet Graphis dans lequel on dessine</param>
        protected void DrawHour(Graphics g)
        {
            int i = 0;
            foreach (HourSelectorRectangleInfo r in vHour)
            {
                if (i % 2 == 0)
                    g.FillRectangle(new SolidBrush(r.BackColor2), r.Rectangle);
                else
                    g.FillRectangle(new SolidBrush(r.BackColor), r.Rectangle);
                i++;
                g.DrawRectangle(r.BorderPen, r.Rectangle);
                Point p = GetMiddlePosition(r, g);
                g.DrawString(r.Text, r.TextFont, new SolidBrush(r.TextColor), p);
            }
            g.DrawLine(new Pen(Color.Black, vBorderWidth), new Point(0, vHour[vHour.Length - 1].Rectangle.Y * 2),
                new Point(pb_Hour.Width, vHour[vHour.Length - 1].Rectangle.Y * 2));
        }

        /// <summary>
        /// Mthodes de redessinage des minutes du composant
        /// </summary>
        /// <param name="g">Objet Graphis dans lequel on dessine</param>
        protected void DrawMinute(Graphics g)
        {
            int i = 0;
            foreach (HourSelectorRectangleInfo r in vMinutes)
            {
                if (i % 2 == 0)
                    g.FillRectangle(new SolidBrush(r.BackColor2), r.Rectangle);
                else
                    g.FillRectangle(new SolidBrush(r.BackColor), r.Rectangle);
                i++;
                g.DrawRectangle(r.BorderPen, r.Rectangle);
                Point p = GetMiddlePosition(r, g);
                g.DrawString(r.Text, r.TextFont, new SolidBrush(r.TextColor), p);
            }
        }

        /// <summary>
        /// Fait un redessinage complet du composant
        /// </summary>
        /// <param name="gMin">Objet Graphics des minutes</param>
        /// <param name="gHour">Objet Graphics des heures</param>
        protected void Redraw(Graphics gMin, Graphics gHour)
        {
            DrawHour(gHour);
            DrawMinute(gMin);
        }

        /// <summary>
        /// Recalcule les composants
        /// </summary>
        protected void Recalculate()
        {
            #region Cration des heures
            Point point = new Point(0, 0);
            vHourSize = pb_Hour.Size;

            for (int i = 0; i < 24; i++)
            {
                if (i != 0)
                    point.X += (vHourSize.Width - 1) / 12;
                if (i == 12)
                {
                    point.X = 0;
                    point.Y += (vHourSize.Height - 1) / 2;
                }

                if (vHour[i] == null)
                    vHour[i] = new HourSelectorRectangleInfo();
                vHour[i].BorderPen = new Pen(Color.Black);
                vHour[i].Rectangle = new Rectangle(point, new Size(vHourSize.Width / 12, vHourSize.Height / 2));
                vHour[i].Text = "" + i;
                vHour[i].TextFont = new Font("Times New Roman", 10);
            }
            #endregion

            #region creation des minutes
            point = new Point(0, 0);
            int mul = 1;

            #region Size of Array
            switch (vMinutesSelection)
            {
                case MinutesSelection.FiveMinutes:
                    vMinutes = new HourSelectorRectangleInfo[12];
                    vColumns = 1;
                    mul = 60 / 12;
                    break;
                case MinutesSelection.HalfAnHour:
                    vMinutes = new HourSelectorRectangleInfo[2];
                    vColumns = 1;
                    mul = 60 / 2;
                    break;
                case MinutesSelection.Minute:
                    vMinutes = new HourSelectorRectangleInfo[60];
                    vColumns = 12;
                    mul = 60 / 60;
                    break;
                case MinutesSelection.Quarter:
                    vMinutes = new HourSelectorRectangleInfo[4];
                    vColumns = 1;
                    mul = 60 / 4;
                    break;
            }
            #endregion

            for (int i = 0; i < vMinutes.Length; i++)
            {
                if (i != 0)
                    point.X += (vHourSize.Width - 1) / vColumns;

                if (i % vColumns == 0 && i != 0)
                {
                    point.X = 0;
                    point.Y += (pb_Minute.Height - 1) / (vMinutes.Length / vColumns);
                }

                if (vMinutes[i] == null)
                    vMinutes[i] = new HourSelectorRectangleInfo();

                vMinutes[i].BorderPen = new Pen(Color.Black);
                vMinutes[i].Rectangle = new Rectangle(point,
                    new Size((vHourSize.Width - 1) / vColumns, pb_Minute.Height / (vMinutes.Length / vColumns)));

                vMinutes[i].Text = ": " + (i * mul).ToString();
                vMinutes[i].TextFont = new Font("Times New Roman", 10);
            }
            #endregion
        }

        /// <summary>
        /// Retourne une SizeF donnant le position pour mettre un
        /// </summary>
        /// <param name="_ri">Le DayRectangleInfo contenant toutes les informations</param>
        /// <param name="_g">Graphics dans lequel on fait les mesures</param>
        /// <returns>Un point contenant les coordonnes du texte  afficher</returns>
        protected Point GetMiddlePosition(HourSelectorRectangleInfo _ri, Graphics _g)
        {
            SizeF CoordString = _g.MeasureString(_ri.Text, _ri.TextFont);

            CoordString.Width = ((_ri.Rectangle.Width) / 2) - (CoordString.Width / 2);
            CoordString.Height = ((_ri.Rectangle.Height) / 2) - (CoordString.Height / 2);

            return new Point((int)CoordString.Width + _ri.Rectangle.X, (int)CoordString.Height + _ri.Rectangle.Y);
        }
        #endregion

        #region Event
        /// <summary>
        /// Evnement Paint de la partie des heures
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Hour_Paint(object sender, PaintEventArgs e) { DrawHour(e.Graphics); }

        /// <summary>
        /// Evnement sur changement de taille du composant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Hour_Resize(object sender, EventArgs e)
        {
            Recalculate();
            DrawHour(pb_Hour.CreateGraphics());
        }

        /// <summary>
        /// Evnement sur un clique de souris dans la zone des heures
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Hour_MouseClick(object sender, MouseEventArgs e)
        {
            foreach (HourSelectorRectangleInfo r in vHour)
            {
                if (r.Rectangle.Contains(e.Location))
                {
                    vSelectedHour = new DateTime(vSelectedHour.Year, vSelectedHour.Month, vSelectedHour.Day,
                        Int16.Parse(r.Text), vSelectedHour.Minute, vSelectedHour.Second);

                    if (vSelectedHour.Minute < 10)
                        lbl_SelectedHour.Text = vSelectedHour.Hour + "h0" + vSelectedHour.Minute;
                    else
                        lbl_SelectedHour.Text = vSelectedHour.Hour + "h" + vSelectedHour.Minute;

                    break;
                }
            }
        }

        /// <summary>
        /// Evnement Paint sur la partie des minutes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Minute_Paint(object sender, PaintEventArgs e) { DrawMinute(e.Graphics); }

        /// <summary>
        /// Evnement Resize sur la partie des minutes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Minute_Resize(object sender, EventArgs e)
        {
            Recalculate();
            DrawMinute(pb_Minute.CreateGraphics());
        }

        /// <summary>
        /// Evnement de clique sur les minutes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Minute_MouseClick(object sender, MouseEventArgs e)
        {
            foreach (HourSelectorRectangleInfo r in vMinutes)
            {
                if (r.Rectangle.Contains(e.Location))
                {
                    string minute = r.Text.Replace(':', ' ');
                    vSelectedHour = new DateTime(vSelectedHour.Year, vSelectedHour.Month, vSelectedHour.Day,
                        vSelectedHour.Hour, Int16.Parse(minute), vSelectedHour.Second);

                    if (vSelectedHour.Minute < 10)
                        lbl_SelectedHour.Text = vSelectedHour.Hour + "h0" + vSelectedHour.Minute;
                    else
                        lbl_SelectedHour.Text = vSelectedHour.Hour + "h" + vSelectedHour.Minute;
                }
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise les minutes  afficher
        /// </summary>
        [Category("Options")]
        [Description("Initialise l'affiche de selection des minutes")]
        public MinutesSelection MinutesSelection
        {
            get { return vMinutesSelection; }
            set
            {
                vMinutesSelection = value;
                Recalculate();
                Redraw(pb_Minute.CreateGraphics(), pb_Hour.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond du composant "heure"
        /// </summary>
        [Category("Options-Color")]
        [Description("Initialise la couleur de fond du composant \"heure\"")]
        public Color HourBackColor
        {
            get { return vHour[1].BackColor; }
            set
            {
                for (int i = 0; i < vHour.Length; i++)
                {
                    vHour[i].BackColor = value;
                }
                DrawHour(pb_Hour.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond alternative du composant "heure"
        /// </summary>
        [Category("Options-Color")]
        [Description("Initialise la couleur de fond alternative du composant \"heure\"")]
        public Color HourBackColor2
        {
            get { return vHour[1].BackColor2; }
            set
            {
                for (int i = 0; i < vHour.Length; i++)
                {
                    vHour[i].BackColor2 = value;
                }
                DrawHour(pb_Hour.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond du composant "minute"
        /// </summary>
        [Category("Options-Color")]
        [Description("Initialise la couleur de fond du composant \"minute\"")]
        public Color MinuteBackColor
        {
            get { return vMinutes[1].BackColor; }
            set
            {
                for (int i = 0; i < vMinutes.Length; i++)
                {
                    vMinutes[i].BackColor = value;
                }
                DrawMinute(pb_Minute.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond alternative du composant "minute"
        /// </summary>
        [Category("Options-Color")]
        [Description("Initialise la couleur de fond alternative du composant \"minute\"")]
        public Color MinuteBackColor2
        {
            get { return vMinutes[1].BackColor2; }
            set
            {
                for (int i = 0; i < vMinutes.Length; i++)
                {
                    vMinutes[i].BackColor2 = value;
                }
                DrawMinute(pb_Minute.CreateGraphics());
            }
        }
        /// <summary>
        /// Retourne ou initialise l'heure selectionne
        /// </summary>
        [Browsable(false)]
        public DateTime SelectedHour
        {
            get { return vSelectedHour; }
            set
            {
                vSelectedHour = value;

                if (vSelectedHour.Hour < 10)
                    lbl_SelectedHour.Text = "0" + vSelectedHour.Hour;
                else
                    lbl_SelectedHour.Text = "" + vSelectedHour.Hour;

                if (vSelectedHour.Minute < 10)
                    lbl_SelectedHour.Text += "h0" + vSelectedHour.Minute;
                else
                    lbl_SelectedHour.Text += "h" + vSelectedHour.Minute;

            }
        }
        #endregion
    }
}
