using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Drawing;
using Utils.Misc;
using Components.Agenda.Management.StructsEnums;

namespace Components.Agenda.Management
{
    /// <summary>
    /// Classe de gestion des jours du mois pour l'affichage
    /// </summary>
    public class Days : IEnumerator
    {
        #region Variables
        /// <summary>
        /// Variable d'numaration pour this[]
        /// </summary>
        protected int vEnumerator = -1;
        /// <summary>
        /// Tableau de jours 
        /// </summary>
        protected MonthRectangleInfo[] vDays = new MonthRectangleInfo[42];
        /// <summary>
        /// Police du texte de la cellule
        /// </summary>
        protected Font vFont = new Font("Times new Roman", 12);
        /// <summary>
        /// Couleur du texte
        /// </summary>
        protected Color vColorText = Color.Black;
        /// <summary>
        /// Couleur de fond
        /// </summary>
        protected Color vColorBack = Color.White;
        /// <summary>
        /// Couleur des case de jour hors du mois courrant
        /// </summary>
        protected Color vColorNonMonthBack = Color.Silver;
        /// <summary>
        /// Couleur de la case du jour d'aujourd'hui
        /// </summary>
        protected Color vTodayBackColor = Color.Red;
        /// <summary>
        /// Couleur du contour du tableau
        /// </summary>
        protected Pen vSurround = new Pen(Brushes.Black, 1);
        /// <summary>
        /// Point de coord (0,0) pour l'affichage des jours
        /// </summary>
        protected int vCoorY;
        /// <summary>
        /// Taille du PictureBox dans lequel on dessine
        /// </summary>
        protected Size vSizeOfComponent;
        /// <summary>
        /// Date  afficher
        /// </summary>
        protected DateTime vDate = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
        /// <summary>
        /// Liste des rendez-vous du mois courrant
        /// </summary>
        protected List<MeetingRow> vMeetingList;
        ///<summary>
        ///Index de la date d'aujourd'hui
        ///</summary>
        protected int vTodayIndex = -1;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_CoodY">Coordonne Y pour la coord (0,0) du premier point 
        /// (X + hauteur des headers; Y)</param>
        /// <param name="_SizeOfComponent">Taille du PictureBox dans lequel on dessine</param>
        public Days(Size _SizeOfComponent, int _CoodY)
        {
            vCoorY = _CoodY;
            vSizeOfComponent = _SizeOfComponent;

            for (int i = 0; i < vDays.Length; i++)
                vDays[i] = new MonthRectangleInfo();
            Resize();
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Recalcule la taille des cellules des jours
        /// </summary>
        public void Resize()
        {
            SetDate();
            Size SizeCard = new Size(vSizeOfComponent.Width / 7, ((vSizeOfComponent.Height - 1) - vCoorY) / 6);
            int Y = vCoorY;

            Rectangle r = new Rectangle(new Point(0, vCoorY),
                new Size((SizeCard.Width + (vSizeOfComponent.Width % 7)) - 1, SizeCard.Height));

            for (int semaine = 0; semaine < 6; semaine++)
            {
                for (int jour = 0; jour < 7; jour++)
                {
                    if (jour == 0)
                        r = new Rectangle(new Point(r.X, Y),
                            new Size(SizeCard.Width + (vSizeOfComponent.Width % 7) - 1, SizeCard.Height));
                    else
                        r = new Rectangle(new Point(r.X + r.Width, Y), SizeCard);

                    if (vDays[jour + (semaine * 7)] == null)
                        vDays[jour + (semaine * 7)] = new MonthRectangleInfo();

                    vDays[jour + (semaine * 7)].Rectangle = r;
                    vDays[jour + (semaine * 7)].TextColor = vColorText;
                    vDays[jour + (semaine * 7)].TextFont = vFont;
                }
                Y += SizeCard.Height;
                r.X = 0;
            }
        }
        /// <summary>
        /// Recalcule le tableau pour que le mois courrant soit correct
        /// </summary>
        protected void SetDate()
        {
            DateTime dt = new DateTime(vDate.Year, vDate.Month, 1);
            int FirstDay = (int)dt.DayOfWeek - 1;
            if (FirstDay == -1)
                FirstDay = 6;

            int DaysInMonth = Month.GetDaysInMonth(vDate.Year, vDate.Month) - 1;
            int jour = 0;

            for (int i = 0; i < 42; i++)
            {
                if (vDays[i] == null)
                    vDays[i] = new MonthRectangleInfo();

                //Jours avant le mois
                if (i < FirstDay)
                {
                    vDays[i].DayNumber = "";
                    vDays[i].BackColor = vColorNonMonthBack;
                    vDays[i].Position = CasePositionInMonth.Before;
                } 
                //Jours aprs le mois
                else if (i - FirstDay > DaysInMonth)
                {
                    vDays[i].DayNumber = "";
                    vDays[i].BackColor = vColorNonMonthBack;
                    vDays[i].Position = CasePositionInMonth.After;
                }
                //Jour du mois
                else
                {
                    vDays[i].DayNumber = ((i + 1) - FirstDay).ToString();
                    vDays[i].BackColor = vColorBack;
                    vDays[i].Position = CasePositionInMonth.InMonth;

                    if (vMeetingList != null)
                    {
                        int count = 0;
                        foreach (MeetingRow mr in vMeetingList)
                        {
                            if (mr.StartDate.Month == vDate.Month
                                && mr.StartDate.Day == jour + 1
                                && mr.StartDate.Year == vDate.Year)
                            {
                                count++;
                            }
                        }
                        if (count != 0)
                            vDays[i].Text = count + " rdv";
                        jour++;
                    }
                }
                //Si nous sommes aujourd'hui
                if (vDate.Month == DateTime.Today.Month
                    && vDate.Year == DateTime.Today.Year && (i - FirstDay) + 1 == DateTime.Today.Day)
                {
                    vDays[i].BackColor = vTodayBackColor;
                    vTodayIndex = i;
                    vDays[i].Position = CasePositionInMonth.InMonth;
                }
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la police du texte de la cellule de jour
        /// </summary>
        public Font Font
        {
            get { return vFont; }
            set { vFont = value; }
        }
        /// <summary>
        /// Retourne ou initialise le couleur du texte de la cellule de jour
        /// </summary>
        public Color ColorText
        {
            get { return vColorText; }
            set { vColorText = value; }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fonc de la cellule de jour
        /// </summary>
        public Color ColorBack
        {
            get { return vColorBack; }
            set { vColorBack = value; }
        }
        /// <summary>
        /// Retourne ou initialise les couleurs des jours hors du mois courrant
        /// </summary>
        public Color ColorNonMonthBack
        {
            get { return vColorNonMonthBack; }
            set { vColorNonMonthBack = value; }
        }
        /// <summary>
        /// Retourne ou initialise la date
        /// </summary>
        public DateTime Date
        {
            get { return vDate; }
            set
            {
                vDate = value;
                SetDate();
            }
        }
        /// <summary>
        /// Accesseurs afin d'atteindre les jours  afficher dans la calendrier
        /// </summary>
        /// <param name="_Index">Index pour accder au jour voulu</param>
        /// <returns>RectangleInfo contenant les information pour l'afichage d'un jour</returns>
        public MonthRectangleInfo this[int _Index]
        {
            get { return vDays[_Index]; }
        }
        /// <summary>
        /// Donne la taille du composant
        /// </summary>
        public Size ComponentSize
        {
            get { return vSizeOfComponent; }
            set
            {
                if (value != vSizeOfComponent)
                {
                    vSizeOfComponent = value;
                    Resize();
                }
            }
        }
        /// <summary>
        /// Retourne le nombre d'lmentss dans la liste des jours du mois
        /// </summary>
        public int Count
        {
            get { return this.vDays.Length; }
        }
        /// <summary>
        /// Retourne ou initialise la couleur des colones du tableau des jours
        /// </summary>
        public Color ColorSurround
        {
            get { return this.vSurround.Color; }
            set
            {
                vSurround = new Pen(value);
                for (int i = 0; i < vDays.Length; i++)
                    vDays[i].BorderPen = new Pen(value);
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur d'aujourd'hui
        /// </summary>
        public Color TodayBackColor
        {
            get { return vTodayBackColor; }
            set
            {
                this.SetDate();
                vTodayBackColor = value;
                if (this.vTodayIndex != -1)
                    vDays[vTodayIndex].BackColor = value;
            }
        }
        /// <summary>
        /// Retourne l'index du jour d'aujourd'hui ou -1 si ce n'est pas le bon mois ou anne
        /// </summary>
        public int TodayIndex
        { get { return vTodayIndex; } }
        /// <summary>
        /// Retourne true si c'est le mois actuel
        /// </summary>
        public bool IsActualMonth
        {
            get
            {
                if (vTodayIndex != -1) return false;
                else return true;
            }
        }
        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous pour le mois donn
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return vMeetingList; }
            set { vMeetingList = value; }
        }
        #endregion

        #region IEnumerator Membres
        /// <summary>
        /// Retourne une rfrence sur cet objet et met l'index  0
        /// </summary>
        /// <returns>Une rfrence sur this</returns>
        public IEnumerator GetEnumerator() { vEnumerator = -1; return this; }

        /// <summary>
        /// Obtient l'lment en cours dans la collection.
        /// </summary>
        object IEnumerator.Current
        {
            get { return vDays[vEnumerator]; }
        }

        /// <summary>
        /// Avance l'numrateur  l'lment suivant de la collection.       
        /// </summary>
        /// <returns>true si l'numrateur a pu avancer jusqu' l'lment suivant; false si l'numrateur
        /// a dpass la fin de la collection.</returns>
        public bool MoveNext() { return ++vEnumerator < vDays.Length; }

        /// <summary>
        /// Rtablit l'numrateur  sa position initiale, qui prcde le premier lment
        /// de la collection.
        /// </summary>
        public void Reset() { vEnumerator = -1; }
        #endregion
    }
}
