using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Components.Agenda.Management.StructsEnums;

namespace Components.Agenda.Management
{
    /// <summary>
    /// Classe de gestion des affichages des heures d'une journe
    /// </summary>
    public class Hours
    {
        #region Variables
        /// <summary>
        /// Indique le nombre de colonnes pour un moment partag
        /// </summary>
        protected int vColumns = 0;

        #region Const
        /// <summary>
        /// Largeur de la colonne des heures
        /// </summary>
        public const int COLUMN_WIDTH = 50;
        /// <summary>
        /// Hauteur du header
        /// </summary>
        public const int HEADER_HEIGHT = 25;
        /// <summary>
        /// Nombre de partie contenue dans un jour
        /// </summary>
        public const int DAYS_PARTS = 24 * 2;
        #endregion

        #region Options
        /// <summary>
        /// Couleur de fond d'une heure qui n'est pas un rdv
        /// </summary>
        protected Color vNonRdvBackColor = Color.White;
        /// <summary>
        /// Couleur du texte d'une heure qui n'est pas un rdv
        /// </summary>
        protected Color vNonRdvTextColor = Color.Black;
        /// <summary>
        /// Police d'criture d'une heure qui n'est pas un rendez-vous
        /// </summary>
        protected Font vNonRdvTextFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur de fond d'une heure qui est un rendez-vous
        /// </summary>
        protected Color vFirstMeetingFadeColor = SystemColors.ActiveCaption;//Color.Tomato;
        /// <summary>
        /// Deuxime couleur pour le dgrad de fond d'un rendez-vous
        /// </summary>
        protected Color vSecondMeetingFadeColor = SystemColors.ActiveBorder;//Color.Silver;
        /// <summary>
        /// Couleur d'criture d'une heure qui est un rendez-vous
        /// </summary>
        protected Color vRdvTextColor = Color.Black;
        /// <summary>
        /// Police d'criture d'une heure qui est un rendez-vous
        /// </summary>
        protected Font vRdvTextFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur de fond du header
        /// </summary>
        protected Color vHeaderColorBack = Color.White;
        /// <summary>
        /// Police d'criture du header
        /// </summary>
        protected Font vHeaderFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur du texte du header
        /// </summary>
        protected Color vHeaderTextColor = Color.Black;
        #endregion

        #region Misc
        /// <summary>
        /// Conient l'heure que reprsente cette case dans la DayAgenda
        /// </summary>
        protected DayRectangleInfo[] vHourHeader = new DayRectangleInfo[DAYS_PARTS];
        /// <summary>
        /// Liste qui contient tous les rendez-vous du jours courrant
        /// </summary>
        protected List<MeetingRow> vList = new List<MeetingRow>();
        /// <summary>
        /// Taille du composant dans lequel on doit crire
        /// </summary>
        protected Size vSizeOfComponent = new Size(10, 10);
        /// <summary>
        /// Date  afficher
        /// </summary>
        protected DateTime vDate;
        /// <summary>
        /// Liste des rectangles  dessiner
        /// </summary>
        protected List<DayRectangleInfo> vListToDraw;
        #endregion

        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>        
        public Hours()
        {
            vList = new List<MeetingRow>();
            ListToDraw = Recalculate();
        }
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_List">Initialise la liste de rendez-vous de la journe</param>
        public Hours(List<MeetingRow> _List)
        {
            vList = _List;
            ListToDraw = Recalculate();
        }
        #endregion

        #region Mthodes publique
        /// <summary>
        /// Prpare les rectangles qu'il faudra dessiner dans le PictureBox reprsentant le DayAgenda
        /// </summary>
        /// <returns>Une liste contenant tous les rectangles  dessiner</returns>
        public List<DayRectangleInfo> Recalculate()
        {
            int FirstStep = 0 - Hours.COLUMN_WIDTH;
            //TODO lorsque le rdv suivant ne se chevauche pas. 
            //vrifier tous les prcdents en partant du deuxime avant.

            List<DayRectangleInfo> ListToDraw = new List<DayRectangleInfo>();

            #region Il y a des rendez-vous aujourd'hui
            if (vList.Count > 0)
            {
                //Index de la liste des rendez-vous
                int MeetingIdx = 0;

                //Nombre de colonnes
                int Columns = 0;

                //Passage en revue de toutes les parties de la journe
                for (int i = 0; i < DAYS_PARTS; i++)
                {
                    #region C'est un rendez-vous
                    if (MeetingIdx < vList.Count && GetIndex(vList[MeetingIdx].StartDate) == i)
                    {
                        DateTime StartDate = vList[MeetingIdx].StartDate;
                        DateTime EndDate = vList[MeetingIdx].EndDate;
                        Columns = 0;

                        #region Comptage des rendez-vous parallles
                        bool bExit = false;
                        for (int idx = MeetingIdx; idx < vList.Count - 1 && !bExit; idx++)
                        {
                            //Le rendez-vous suivant est en paralle
                            if (vList[idx + 1].StartDate < vList[idx].EndDate)
                            {
                                Columns++;
                                if (EndDate < vList[idx + 1].EndDate)
                                    EndDate = vList[idx + 1].EndDate;
                            }
                            else //Sinon on vrifie que ce rdv ne chevauche aucun rdv prcdent
                            {
                                int ret = 0;
                                for (int prec = idx; prec >= 0; prec--)
                                {
                                    if (vList[prec].EndDate > vList[idx + 1].StartDate)
                                    {
                                        Columns++;
                                        ret++;
                                        if (EndDate < vList[idx + 1].EndDate)
                                            EndDate = vList[idx + 1].EndDate;
                                        break;
                                    }
                                }
                                if (ret == 0)
                                {
                                    bExit = true;
                                    break;
                                }
                            }
                        }
                        #endregion

                        #region Plusieurs colonnes
                        if (++Columns > 1)// && Columns != 0)
                        {
                            //passe en revue les colonnes
                            for (int col = 0; col < Columns; col++, MeetingIdx++)
                            {
                                bool IsTitled = false;

                                //On dessine le rectangle
                                for (int TempIdx = GetIndex(StartDate); TempIdx < GetIndex(EndDate); TempIdx++)
                                {
                                    DayRectangleInfo ri = new DayRectangleInfo();
                                    ri.Rectangle = new Rectangle(
                                        new Point(COLUMN_WIDTH + (col * ((vSizeOfComponent.Width - COLUMN_WIDTH) / Columns)),
                                        TempIdx * HEADER_HEIGHT),
                                        new Size((vSizeOfComponent.Width - COLUMN_WIDTH) / Columns, HEADER_HEIGHT));

                                    //Si index dans le rendez-vous
                                    if (GetIndex(vList[MeetingIdx].StartDate) <= TempIdx
                                        && GetIndex(vList[MeetingIdx].EndDate) > TempIdx)
                                    {
                                        ri.TextFont = vRdvTextFont;
                                        ri.TextColor = vRdvTextColor;
                                        ri.BackColor = vFirstMeetingFadeColor;
                                        ri.BackColor2 = vSecondMeetingFadeColor;
                                        ri.IDMeeting = vList[MeetingIdx].IDMeeting;

                                        //Est-ce un dbut de rendez-vous?
                                        if (!IsTitled)
                                        {
                                            if (Columns > 2)
                                                ri.Text = GetShorName(vList[MeetingIdx].Title);
                                            else
                                                ri.Text = vList[MeetingIdx].Title;
                                            IsTitled = true;
                                            ri.Position = MeetingPosition.StartingMeeting;
                                        }
                                        //Est-ce une fin de rendez-vous?
                                        else if (TempIdx + 1 >= GetIndex(EndDate))
                                            ri.Position = MeetingPosition.EndingMeeting;

                                        else //C'est dans un rendez-vous
                                            ri.Position = MeetingPosition.Meeting;

                                        ri.BorderPen = new Pen(vFirstMeetingFadeColor);
                                    }
                                    //Ce n'est pas un rendez-vous?
                                    else
                                    {
                                        ri.TextFont = vNonRdvTextFont;
                                        ri.TextColor = vRdvTextColor;
                                        ri.BackColor = vNonRdvBackColor;
                                        ri.Position = MeetingPosition.OutOfMeeting;
                                        //ri.BorderPen = new Pen(vNonRdvBackColor);                                        
                                    }
                                    ListToDraw.Add(ri);
                                }
                            }
                            i = GetIndex(EndDate) - 1;
                        }
                        #endregion

                        #region Une seule colonne
                        else
                        {
                            int one = GetIndex(vList[MeetingIdx].StartDate);

                            bool IsTitled = false;

                            for (; one < GetIndex(vList[MeetingIdx].EndDate); one++)
                            {
                                DayRectangleInfo ri = new DayRectangleInfo();
                                ri.TextFont = vRdvTextFont;
                                ri.TextColor = vRdvTextColor;
                                ri.BackColor = vFirstMeetingFadeColor;
                                ri.BackColor2 = vSecondMeetingFadeColor;
                                ri.Rectangle = new Rectangle(new Point(COLUMN_WIDTH, one * HEADER_HEIGHT),
                                    new Size(vSizeOfComponent.Width - vColumns, HEADER_HEIGHT));
                                ri.IDMeeting = vList[MeetingIdx].IDMeeting;

                                //Est-ce le dbut d'un rendez-vous
                                if (!IsTitled)
                                {
                                    ri.Text = vList[MeetingIdx].Title;
                                    ri.Position = MeetingPosition.StartingMeeting;
                                    IsTitled = true;
                                }

                                //Est-ce la fin d'un rendez-vous
                                else if (one + 1 >= GetIndex(vList[MeetingIdx].EndDate))
                                    ri.Position = MeetingPosition.EndingMeeting;

                                else //C'est dans un rendez-vous
                                    ri.Position = MeetingPosition.Meeting;

                                ri.BorderPen = new Pen(vFirstMeetingFadeColor);
                                ListToDraw.Add(ri);
                            }
                            i = one - 1;
                            MeetingIdx++;
                        }
                        #endregion
                    }
                    #endregion

                    #region Ce n'est pas un rendez-vous
                    else
                    {
                        DayRectangleInfo ri = new DayRectangleInfo();
                        ri.BackColor = vNonRdvBackColor;
                        ri.TextColor = vNonRdvTextColor;
                        ri.TextFont = vNonRdvTextFont;
                        ri.Text = "";
                        ri.Position = MeetingPosition.OutOfMeeting;

                        ri.Rectangle = new Rectangle(new Point(COLUMN_WIDTH, i * HEADER_HEIGHT),
                            new Size(vSizeOfComponent.Width - COLUMN_WIDTH, HEADER_HEIGHT));

                        ListToDraw.Add(ri);
                    }
                    #endregion
                }
            }
            #endregion

            #region Il n'y a aucun rendez-vous aujourd'hui
            else
            {
                for (int i = 0; i < DAYS_PARTS; i++)
                {
                    DayRectangleInfo ri = new DayRectangleInfo();
                    ri.BackColor = vNonRdvBackColor;
                    ri.TextColor = vNonRdvTextColor;
                    ri.TextFont = vNonRdvTextFont;
                    ri.Position = MeetingPosition.OutOfMeeting;

                    ri.Rectangle = new Rectangle(new Point(FirstStep + COLUMN_WIDTH, i * HEADER_HEIGHT),
                        new Size(vSizeOfComponent.Width, HEADER_HEIGHT));
                    ListToDraw.Add(ri);
                }
            }
            #endregion

            ListToDraw.AddRange(SetHeaders());
            vListToDraw = ListToDraw;
            return ListToDraw;
        }
        /// <summary>
        /// Retourne une SizeF donnant le position pour mettre un
        /// </summary>
        /// <param name="_ri">Le DayRectangleInfo contenant toutes les informations</param>
        /// <param name="_g">Graphics dans lequel on fait les mesures</param>
        /// <returns>Un point contenant les coordonnes du texte  afficher</returns>
        public Point GetMiddlePosition(DayRectangleInfo _ri, Graphics _g)
        {
            SizeF CoordString = _g.MeasureString(_ri.Text, _ri.TextFont);

            CoordString.Width = ((_ri.Rectangle.Width) / 2) - (CoordString.Width / 2);
            CoordString.Height = ((_ri.Rectangle.Height) / 2) - (CoordString.Height / 2);

            return new Point((int)CoordString.Width + _ri.Rectangle.X, (int)CoordString.Height + _ri.Rectangle.Y);
        }
        /// <summary>
        /// Donne la date en rapport avec l'index de sa position dans le tableau
        /// </summary>
        /// <param name="_index">Position de l'lment duquel il faut dduire la date</param>
        /// <returns>La date de l'lment</returns>
        public DateTime GetDate(int _index)
        {
            int div = 60 / (DAYS_PARTS / 24);

            int minute = _index * div;
            int hour = minute / 60;
            minute = minute % 60;

            if (hour == 24) hour = 0;

            return new DateTime(vDate.Year, vDate.Month, vDate.Day, hour, minute, 0);
        }
        #endregion

        #region Mthodes prives
        /// <summary>
        /// Initialise le header avec les valeurs par dfaut
        /// </summary>        
        protected DayRectangleInfo[] SetHeaders()
        {
            string h = "";
            int Col = GetColumnsNumber();
            string sHeure;
            string sMin;

            for (int i = 0; i < vHourHeader.Length; i++)
            {
                int heure = (int)(i / (DAYS_PARTS / 24));
                sHeure = "" + heure;
                int min = (i % (DAYS_PARTS / 24)) * (60 / (DAYS_PARTS / 24));
                sMin = "" + min;

                if (heure < 10)
                    sHeure = "0" + heure;
                if (min == 0)
                    sMin = "00";
                h = sHeure + "h" + sMin;

                #region Mise du texte et Affichage
                if (i < DAYS_PARTS)
                {
                    if (vHourHeader[i] == null)
                        vHourHeader[i] = new DayRectangleInfo();

                    vHourHeader[i].Text = h;
                    if (vHourHeader[i].BorderPen == null)
                        vHourHeader[i].BorderPen = new Pen(Color.Black);
                    if (vHourHeader[i].TextFont == null)
                        vHourHeader[i].TextFont = new Font("Times New Roman", 12);

                    vHourHeader[i].Rectangle =
                        new Rectangle(new Point(0, i * HEADER_HEIGHT),
                        new Size(COLUMN_WIDTH, HEADER_HEIGHT));
                }
                #endregion
            }
            return vHourHeader;
        }
        /// <summary>
        /// Donne l'index dans le tableau en rapport  l'heure donne
        /// </summary>
        /// <param name="_dt">L'heure pour laquelle il faut donner l'index</param>
        /// <returns>L'index dans le tableau des rendez-vous</returns>
        protected int GetIndex(DateTime _dt)
        {
            int min = 60 / (DAYS_PARTS / 24);
            int ret = _dt.Hour * (DAYS_PARTS / 24);
            ret += (int)(_dt.Minute / min);
            // ret--;
            return ret;
        }
        /// <summary>
        /// Prpare la liste des rendez-vous
        /// </summary>
        /// <returns>Le nombre de colonnes pour l'affichage</returns>
        protected int GetColumnsNumber()
        {
            int Col = 1;

            for (int i = 0; i < vList.Count - 1; i++)
            {
                if (vList[i].StartDate <= vList[i + 1].StartDate && vList[i].EndDate >= vList[i + 1].StartDate
                    || vList[i].StartDate <= vList[i + 1].EndDate && vList[i].EndDate >= vList[i + 1].EndDate)
                {
                    Col++;
                }
            }
            return Col;
        }
        /// <summary>
        /// Trouve l'index ou se situe l'lment du rendez-vous
        /// </summary>
        /// <param name="_Hour">Heure du rendez-vous</param>
        /// <returns>L'index dans le tableau</returns>
        protected int IndexOf(DateTime _Hour)
        {
            int hour = _Hour.Hour;
            int min = _Hour.Minute;

            return (hour * 4) + (min / 15);
        }
        /// <summary>
        /// Retourne les initiales du noms entr en argument
        /// </summary>
        /// <param name="_Name">Nom duquel il faut extraire les initiales</param>
        /// <returns>Les initiales du nom</returns>
        protected string GetShorName(string _Name)
        {
            string[] buf ={ "", "" };
            buf = _Name.Split(' ');
            string ret = "";
            if (buf[1].Length < 11)
                return buf[1];
            else
            {
                foreach (string s in buf)
                {
                    string d = s.ToUpper();
                    ret += d[0];
                }
                return ret;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne la hauteur du composant
        /// </summary>
        public int Height
        {
            get { return vHourHeader.Length * HEADER_HEIGHT; }
        }
        /// <summary>
        /// Donne la hauteur d'une entre
        /// </summary>
        public int HeaderHeight { get { return HEADER_HEIGHT; } }
        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous de la journe
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return vList; }
            set
            {
                if (value != null)
                {
                    //vList = value;
                    #region Tri des dates pour afficher seulement la date donne
                    List<MeetingRow> buf = new List<MeetingRow>();
                    foreach (MeetingRow mr in value)
                    {
                        if (mr.StartDate.Year == vDate.Year && mr.StartDate.Month == vDate.Month
                            && mr.StartDate.Day == vDate.Day)
                        {
                            buf.Add(mr);
                        }
                    }
                    Utils.Misc.Sort.BubbleSort<MeetingRow>(buf);
                    vList = buf;
                    this.Recalculate();
                    #endregion
                }
            }
        }
        /// <summary>
        /// Initialise la taille du composant dans lequel on dessine
        /// </summary>
        public Size SizeOfComponent
        {
            set { vSizeOfComponent = value; }
            get { return vSizeOfComponent; }
        }
        /// <summary>
        /// Retourne ou initialise la date  afficher
        /// </summary>
        public DateTime DateToDisplay
        {
            get { return vDate; }
            set { vDate = value; }
        }
        /// <summary>
        /// Retourne le nombre de cases contenues dans un jour
        /// </summary>
        public int Length { get { return DAYS_PARTS * this.GetColumnsNumber(); } }
        /// <summary>
        /// Retourne la liste des rectangle  dessiner
        /// </summary>
        public List<DayRectangleInfo> ListToDraw
        {
            get { return vListToDraw; }
            protected set { vListToDraw = value; }
        }
        /// <summary>
        /// Indique le nombre de parties que contiennent un jour
        /// </summary>
        public int DayParts { get { return DAYS_PARTS; } }
        /// <summary>
        /// Retourne la liste des headers
        /// </summary>
        public DayRectangleInfo[] Header { get { return vHourHeader; } }
        /// <summary>
        /// Retourne la liste des heures prises
        /// </summary>
        public List<MeetingRow> Hour { get { return vList; } }
        /// <summary>
        /// Retourne la largeur du header contenant les heures de la journe
        /// </summary>
        public int HeaderWidth { get { return COLUMN_WIDTH; } }
        /// <summary>
        /// Retourne le nombre de colonnes contenue dans le jour courrant
        /// </summary>
        public int Columns { get { return vColumns; } }
        #endregion

        #region Colors Options
        /// <summary>
        /// Retourne ou initialise la couleur de fond d'une heure sans rendez-vous
        /// </summary>
        public Color NonMeetingBackColor
        {
            get { return vNonRdvBackColor; }
            set
            {
                vNonRdvBackColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond des heures avec rendez-vous
        /// </summary>
        public Color FirstMeetingFadeColor
        {
            get { return vFirstMeetingFadeColor; }
            set
            {
                vFirstMeetingFadeColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la deuxime couleur de fond pour l'affichage d'un rendez-vous
        /// </summary>
        public Color SecondMeetingBackcolor
        {
            get { return this.vSecondMeetingFadeColor; }
            set
            {
                vSecondMeetingFadeColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur du texte des heures  rendez-vous
        /// </summary>
        public Color RdvTextColor
        {
            get { return vRdvTextColor; }
            set
            {
                vRdvTextColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture des heures avec rendez-vous
        /// </summary>
        public Font RdvTextFont
        {
            get { return vRdvTextFont; }
            set
            {
                vRdvTextFont = value;
                this.Recalculate();
            }
        }
        #endregion
    }
}
