using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using Components.Agenda.Management.StructsEnums;

namespace Components.Agenda.Management
{
    /// <summary>
    /// Version de la classe Hours avec un buffer Image pour l'optimisation de l'affichage
    /// </summary>
    public class ImgHours : Hours
    {
        #region Variables
        /// <summary>
        /// Image buffer contenant l'image  dessiner
        /// </summary>
        protected Bitmap vImage;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        public ImgHours() : base() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_List">Initialise la liste de rendez-vous de la journe</param>
        public ImgHours(List<MeetingRow> _List) : base(_List) { }
        #endregion

        #region Mthodes
        /// <summary>
        /// Prpare les rectangles dans une image pour un dessinage ultrieur
        /// </summary>        
        public void Draw()
        {
            vImage = new Bitmap(base.SizeOfComponent.Width, base.Height);
            Graphics g = Graphics.FromImage(vImage);

            foreach (DayRectangleInfo ri in base.Recalculate())
            {
                #region Dessinage
                LinearGradientBrush lgb =
                    new LinearGradientBrush(ri.Rectangle, ri.BackColor, ri.BackColor2, LinearGradientMode.Horizontal);
                g.FillRectangle(lgb, ri.Rectangle);


                switch (ri.Position)
                {
                    case MeetingPosition.StartingMeeting:
                        #region StartingMeeting
                        //Ligne suprieure
                        g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + SizeOfComponent.Width, ri.Rectangle.Location.Y));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion

                    case MeetingPosition.Meeting:
                        #region Meeting

                        break;
                        #endregion

                    case MeetingPosition.EndingMeeting:
                        #region EndingMeeting
                        //Ligne infrieure
                        g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + base.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + base.HeaderHeight));
                        break;
                        #endregion

                    case MeetingPosition.OutOfMeeting:
                        #region OutOfMeeting
                        g.FillRectangle(new SolidBrush(ri.BackColor), ri.Rectangle);

                        //Ligne suprieure
                        g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                        //Ligne infrieure
                        g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + base.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + base.HeaderHeight));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                }
                #endregion
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne une image correspondant  l'affichage du jour
        /// </summary>
        public Bitmap Image
        {
            get { return vImage; }
            protected set { vImage = value; }
        }

        /// <summary>
        /// Retourne ou initialise la taille du composant parent dans lequel on dessine le jour
        /// </summary>
        public new Size SizeOfComponent
        {
            get { return base.SizeOfComponent; }
            set
            {
                base.SizeOfComponent = value;
                this.Draw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous de la journe
        /// </summary>
        public new List<MeetingRow> MeetingList
        {
            get { return base.vList; }
            set
            {
                base.MeetingList = value;
                this.Draw();
            }
        }
        #endregion
    }
}
