using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Xml;

namespace Components.Agenda.Management
{
    /// <summary>
    /// Classe contenant les information pour la langue du calendrier
    /// </summary>
    public class Language
    {
        #region Variables
        /// <summary>
        /// Path du fichier XML
        /// </summary>
        private string vPath = null;

        /// <summary>
        /// Nom des jours
        /// </summary>
        private string[] vNameOfDays = { "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche" };

        /// <summary>
        /// Nom des mois
        /// </summary>
        private string[] vNameOfMonths = { "Janvier", "Fvrier", "Mars", "Avril", "Mai",
            "Juin", "Juillet", "Aot", "Septembre", "Octobre", "Novembre", "Dcembre" };

        /// <summary>
        /// Les diffrents type de rendez-vous
        /// </summary>
        private string[] vMeetingType = { "Important", "Travail", "Personnel" };

        /// <summary>
        /// Nom de toutes les langues contenue dans le fichier XML
        /// </summary>
        private string[] vLanguages = { "franais" };

        /// <summary>
        /// Informations du fichier XML
        /// </summary>
        private XmlDocument vDoc = null;

        /// <summary>
        /// Liste des noeuds de langues
        /// </summary>
        private XmlNodeList vLanguagesNodes = null;

        /// <summary>
        /// Selected Language
        /// </summary>
        private string vSelectedLanguage = "franais";
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        public Language() 
        {
            this.Load();
        }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Path">Path du fichier XML d'options</param>
        public Language(string _Path)
        {
            if (File.Exists(_Path))
            {
                vPath = _Path;
                Load();
            }
            else
            {
                string[] d = { "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche" };
                string[] m = { "Janvier", "Fvrier", "Mars", "Avril", "Mai", "Juin", "Juillet", "Aot", "Septembre", "Octobre", "Novembre", "Dcembre" };
                string[] l = { "franais" };
                vNameOfDays = d;
                vNameOfMonths = m;
                vLanguages = l;
            }
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Charge le fichier XML en mmoire
        /// </summary>
        /// <exception cref="XmlException">En cas d'erreur XML dans le fichier</exception>
        /// <exception cref="Exception">Si le fichier XML n'a pas t indiqu</exception>
        public void Load()
        {
            try
            {
                if (vPath != null)
                {
                    vDoc = new XmlDocument();
                    vDoc.Load(vPath);

                    vLanguagesNodes = vDoc.ChildNodes[1].ChildNodes;
                    vLanguages = new string[vLanguagesNodes.Count];

                    for (int i = 0; i < vLanguages.Length; i++)
                    {
                        vLanguages[i] = vLanguagesNodes[i].Attributes[0].Value;
                    }
                }
            }
            catch (XmlException) { throw; }
        }

        /// <summary>
        /// Donne le nom des jours de la semaine pour la langue donne
        /// </summary>        
        /// <returns>Un tableau avec le nom des jours (Le premier jour est Lundi) ou les valeurs
        /// franaise si la langue n'existe pas</returns>
        public string[] GetDayData()
        {
            if (vLanguages == null) return vNameOfDays;

            int Index = -1;

            for (int i = 0; i < vLanguages.Length; i++)
            {
                if (vLanguages[i] == vSelectedLanguage)
                    Index = i;
            }


            if (File.Exists(vPath) && Index != -1)
            {

                string Buf = vLanguagesNodes[Index].ChildNodes[0].InnerText;
                string[] sep = { "," };
                return Buf.Split(sep, StringSplitOptions.RemoveEmptyEntries);
            }
            return vNameOfDays;

        }

        /// <summary>
        /// Donne le nom des mois pour la langue donne
        /// </summary>        
        /// <returns>Un tableau avec le nom des mois ou les valeurs franaise si la langue n'existe pas pas</returns>
        public string[] GetMonthData()
        {
            if (vLanguages == null) return vNameOfMonths;
            int Index = -1;

            for (int i = 0; i < vLanguages.Length; i++)
            {
                if (vLanguages[i] == vSelectedLanguage)
                    Index = i;
            }

            if (File.Exists(vPath) && Index != -1)
            {
                string Buf = vLanguagesNodes[Index].ChildNodes[1].InnerText;
                string[] sep = { "," };
                Buf = Buf.Trim();
                return Buf.Split(sep, StringSplitOptions.RemoveEmptyEntries);
            }
            return vNameOfMonths;
        }

        /// <summary>
        /// Donne, dans la langue donne les diffrents type de rendez-vous
        /// </summary>        
        /// <returns>Un tabeau avec le nom des type de rendez-vous
        /// ou les valeurs francaise si la langue n'existe pas</returns>
        public string[] GetMeetingType()
        {
            if (vLanguages == null) return vNameOfMonths;
            int Index = -1;

            for (int i = 0; i < vLanguages.Length; i++)
            {
                if (vLanguages[i] == vSelectedLanguage)
                    Index = i;
            }

            if (File.Exists(vPath) && Index != -1)
            {
                string Buf = vLanguagesNodes[Index].ChildNodes[2].InnerText;
                string[] sep = { "," };
                Buf = Buf.Trim();
                return Buf.Split(sep, StringSplitOptions.RemoveEmptyEntries);
            }
            return vNameOfMonths;
        }

        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le Path du fichier XML
        /// </summary>
        public string Path
        {
            get { return vPath; }
            set
            {
                vPath = value;
                Load();
            }
        }

        /// <summary>
        /// Retourne le nom de toutes les langues contenue dans le fichier XML
        /// </summary>
        public string[] Languages { get { return vLanguages; } }

        /// <summary>
        /// Retourne ou initialise la langue selectionne
        /// </summary>
        public string SelectedLanguage
        {
            get { return vSelectedLanguage; }
            set
            {
                this.Load();
                bool IsLanguage = false;
                foreach (string s in vLanguages)
                {
                    if (s == value)
                    {
                        IsLanguage = true;
                        break;
                    }
                }
                if (IsLanguage)
                    vSelectedLanguage = value;
            }
        }
        #endregion
    }
}
