using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Components.Agenda.GUI;
using System.Drawing;
using Utils.Misc;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;
using Components.Agenda.Management;
using System.ComponentModel;

namespace Components.Agenda.Management.StructsEnums
{   
    /// <summary>
    /// Structure contenant les informations d'un rendez-vous
    /// </summary>
    [Serializable]
    public struct MeetingRow : IComparable<MeetingRow>
    {
        #region Variables
        /// <summary>
        /// Titre du rendez-vous
        /// </summary>        
        private string vTitle;
        /// <summary>
        /// Message du rendez-vous
        /// </summary>
        private string vMessage;
        /// <summary>
        /// Date de dbut du rendez-vous
        /// </summary>
        private DateTime vStartDate;
        /// <summary>
        /// Date de fin du rendez-vous
        /// </summary>
        private DateTime vEndDate;
        /// <summary>
        /// Identifiant du client
        /// </summary>
        private int vIDPatient;
        /// <summary>
        /// Identifiant du rendez-vous
        /// </summary>
        private int vIDMeeting;
        /// <summary>
        /// Prix d'un honoraire
        /// </summary>
        private int vHonoraire;
        #endregion

        #region Mthodes
        /// <summary>
        /// Compare l'instance en cours  un objet du mme type.
        /// </summary>
        /// <param name="other">MeetingRow  comparer  cette instance.</param>
        /// <returns> Entier sign 32bits qui indique l'ordre relatif des objets compars. La
        /// valeur de retour a les significations suivantes: Valeur Signification Infrieure
        ///  zro Cette instance est infrieure  obj. Zro Cette instance est gale
        ///  obj. Suprieure  zro Cette instance est suprieure  obj.</returns> 
        ///<exception cref="ArgumentException">obj n'est pas du mme type que cette instance.</exception>
        public int CompareTo(MeetingRow other)
        {
            if (other.StartDate > this.StartDate)
                return -1;
            else if (other.StartDate < this.StartDate)
                return 1;
            else
                return 0;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le titre du rendez-vous
        /// </summary>                
        public string Title { get { return vTitle; } set { vTitle = value; } }
        /// <summary>
        /// Retourne ou initialise le message du rendez-vous
        /// </summary>        
        public string Message { get { return vMessage; } set { vMessage = value; } }
        /// <summary>
        /// Retourne ou initialise la date de dbut du rendez-vous
        /// </summary>
        public DateTime StartDate { get { return vStartDate; } set { vStartDate = value; } }
        /// <summary>
        /// Retourne ou initialise la date de fin du rendez-vous
        /// </summary>
        public DateTime EndDate { get { return vEndDate; } set { vEndDate = value; } }
        /// <summary>
        /// Retourne ou initialise l'identifiant du client
        /// </summary>
        public int IDPatient { get { return vIDPatient; } set { vIDPatient = value; } }
        /// <summary>
        /// Retourne ou initialise l'identifiant du rendez-vous
        /// </summary>
        public int IDMeeting { get { return vIDMeeting; } set { vIDMeeting = value; } }
        /// <summary>
        /// Retourne ou initialise le prix d'un honoraire
        /// </summary>
        public int Honoraire { get { return vHonoraire; } set { vHonoraire = value; } }
        #endregion

        #region Overriden
        /// <summary>
        /// Retourne un rendez-vous sous forme d'un string
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return TextManager.FirstUpper(this.StartDate.ToLongDateString()) + " de " + StartDate.ToShortTimeString()
                + "  " + EndDate.ToShortTimeString();
        }
        #endregion
    }
    /// <summary>
    /// MeetingRow rsultant d'une recherche
    /// </summary>
    public struct ResultMeetingRow
    {
        /// <summary>
        /// Rsultat de la recherche
        /// </summary>
        public MeetingRow Result;
        /// <summary>
        /// True si le MeetingRow est le rsultat ou false si la recherche n'a rien donner
        /// </summary>
        public bool IsResult;
    }
    /// <summary>
    /// Enumration qui donne la position de la case dans le mois
    /// </summary>
    public enum CasePositionInMonth
    {
        /// <summary>
        /// La case se trouve avant le mois
        /// </summary>
        Before,
        /// <summary>
        /// La case est dans le mois
        /// </summary>
        InMonth,
        /// <summary>
        /// La case est aprs le mois
        /// </summary>
        After,
    }
    /// <summary>
    /// Indique la position de la case dans le rendez-vous
    /// </summary>
    public enum MeetingPosition
    {
        /// <summary>
        /// Case hors du rendez-vous
        /// </summary>
        OutOfMeeting,
        /// <summary>
        /// Premire case d'un rendez-vous
        /// </summary>
        StartingMeeting,
        /// <summary>
        /// Est dans un rendez-vous
        /// </summary>
        Meeting,
        /// <summary>
        /// Est le fin d'un rendez-vous
        /// </summary>
        EndingMeeting,
    }
    /// <summary>
    /// Donne l'affichage des minutes dans le HourSelector
    /// </summary>
    public enum MinutesSelection
    {
        /// <summary>
        /// Affiche par minutes
        /// </summary>
        Minute,
        /// <summary>
        /// Affiche par 5 minutes
        /// </summary>
        FiveMinutes,
        /// <summary>
        /// Affiche par 1/4 d'heure
        /// </summary>
        Quarter,
        /// <summary>
        /// Affiche par demi heure
        /// </summary>
        HalfAnHour,
    }    
}
