using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.Misc;

namespace Components.ErrorCatching
{
    /// <summary>
    /// Fentre de notification d'erreur non catche
    /// </summary>
    public partial class fr_ErrorCatching : Form
    {
        #region Variables
        /// <summary>
        /// Exception  afficher
        /// </summary>
        protected Exception vException;

        /// <summary>
        /// Message  envoyer
        /// </summary>
        protected string vMessage = "";

        /// <summary>
        /// Donne le type d'utilisation de la frame
        /// </summary>
        protected TypeFrame vTypeFrame = TypeFrame.ErrorReporter;

        /// <summary>
        /// Adresse du serveur SMTP
        /// </summary>
        protected string vSMTPServeur = "";

        /// <summary>
        /// Mot de passe du serveur SMTP
        /// </summary>
        protected string vSMTPPassword = "";

        /// <summary>
        /// Port de communication du serveur SMTP
        /// </summary>
        protected int vSMTPPort;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_ErrorCatching()
        {
            InitializeComponent();
            lbl_rmq.Text = "";
        }
        #endregion

        #region Events
        /// <summary>
        /// Evnement de chargement de fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            switch (vTypeFrame)
            {
                case TypeFrame.ErrorReporter:
                    tb_Error.ReadOnly = true;
                    btn_Send.Text = "Envoyer";
                    btn_Cancel.Text = "Annuler";
                    break;
                case TypeFrame.InformationAdder:
                    tb_Error.Text = "";
                    tb_Error.ReadOnly = false;
                    btn_Cancel.Text = "Non";
                    btn_Send.Text = "Oui";
                    break;
            }
        }

        /// <summary>
        /// Boutons d'envois par mail
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Send(object sender, EventArgs e)
        {
            if (vTypeFrame == TypeFrame.ErrorReporter)
            {
                fr_ErrorCatching fr = new fr_ErrorCatching();
                fr.ExceptionToDisplay = vException;
                fr.Remark = "Voulez-vous ajouter un message  destination du dveloppeur?";
                fr.TypeFrame = TypeFrame.InformationAdder;
                fr.ShowDialog();
                this.Close();
            }
            else
            {
                if (tb_Error.Text != "")
                {
                    tb_Error.Text = vMessage += "\r\n\r\nAjout utilisateur:\r\n==========\r\n" + tb_Error.Text;
                    tb_Error.ReadOnly = true;
                }

                Mailer mail = new Mailer(vSMTPServeur, vSMTPPassword, vSMTPPort, vSMTPServeur);
                if (!mail.SendMail("information.software@gmail.com", "Error Reporting", vMessage))
                    MessageBox.Show("Erreur d'envoi de mail:\r\n" + mail.SendingError, "Erreur",
                         MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
        }

        /// <summary>
        /// Clique sur annuler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Cancel(object sender, EventArgs e)
        {
            if (vTypeFrame == TypeFrame.InformationAdder)
            {
                Mailer mail = new Mailer(vSMTPServeur, vSMTPPassword, vSMTPPort, vSMTPServeur);
                if (!mail.SendMail("information.software@gmail.com", "Error Reporting", vMessage))
                    MessageBox.Show("Erreur d'envoi de mail:\r\n" + mail.SendingError, "Erreur",
                         MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            this.Close();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'utilisation de la fentre
        /// </summary>
        protected TypeFrame TypeFrame
        {
            get { return vTypeFrame; }
            set { vTypeFrame = value; }
        }

        /// <summary>
        /// Initialise l'exception  afficher
        /// </summary>
        public Exception ExceptionToDisplay
        {
            set
            {
                vException = value;
                vMessage = tb_Error.Text = "Application ou objet cause de l'erreur:"
                    + "\r\n==========\r\n" + vException.Source + " (" + vException.TargetSite.Name + ")\r\n\r\n"
                    + "Message d'erreur:\r\n==========\r\n" + vException.Message + "\r\n\r\n"
                    + "Stack trace:\r\n==========\r\n" + vException.StackTrace;
            }
        }

        /// <summary>
        /// Initialise la remarque  indiquer dans la fentre
        /// </summary>
        public string Remark { set { lbl_rmq.Text = value; } }

        /// <summary>
        /// Adresse du serveur SMTP
        /// </summary>
        public string SMTPServer
        {
            get { return vSMTPServeur; }
            set { vSMTPServeur = value; }
        }

        /// <summary>
        /// Port de communication du serveur SMTP
        /// </summary>
        public int SMTPPort
        {
            get { return vSMTPPort; }
            set { vSMTPPort = value; }
        }

        /// <summary>
        /// Retourne ou initialise la mot de passe du serveur SMTP
        /// </summary>
        public string SMTPPassword
        {
            get { return vSMTPPassword; }
            set { vSMTPPassword = value; }
        }

        /// <summary>
        /// Indique si la fentre peut envoyer des mails.
        /// </summary>
        public bool IsMailable
        {
            get { return btn_Send.Enabled; }
            set { btn_Send.Enabled = value; }
        }
        #endregion
    }

    /// <summary>
    /// Classe d'arguments pour l'vnement d'affichage d'une exception
    /// </summary>
    public class ExceptionAskEventArgs : EventArgs
    {
        #region Variables
        /// <summary>
        /// Exception  afficher dans la fentre
        /// </summary>
        protected Exception vException;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="e">Exception  afficher dans la fentre</param>
        public ExceptionAskEventArgs(Exception e) { vException = e; }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'exception  afficher dans le fentre
        /// </summary>
        public Exception ExceptionToDisplay
        {
            get { return vException; }
            set { vException = value; }
        }
        #endregion
    }

    /// <summary>
    /// Utilis pour indiquer l'utilisation de la fentre
    /// </summary>
    public enum TypeFrame
    {
        /// <summary>
        /// Indique que le fentre est un reporteur d'erreur
        /// </summary>
        ErrorReporter,

        /// <summary>
        /// Indique que cette fentre est utilise pour ajouter des remarques
        /// </summary>
        InformationAdder,
    }
}