using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Utils.Log;

namespace Components.SplashBar
{
    /// <summary>
    /// Splash ProgressBar qui indique l'volution d'un traitement lourd
    /// </summary>
    public partial class SplashBar : Form
    {
        #region Variables
        /// <summary>
        /// Lanceur de la fentre
        /// </summary>
        private static Thread ThreadFrameLauncher;

        /// <summary>
        /// Rfrence sur lui mme
        /// </summary>
        private static SplashBar Ref = null;

        /// <summary>
        /// Handler de ma mise  jour de la ProgressBar
        /// </summary>        
        private delegate void DelegatePerformStepHandler();

        /// <summary>
        /// Handler de mise  jour de la ProgressBar
        /// </summary>
        private static DelegatePerformStepHandler DelegatePerformStep;

        /// <summary>
        /// Handler de la fonction qui rend la ProgressBar invisible
        /// </summary>
        /// <param name="_IsVisible"></param>
        private delegate void DelegateSetVisibleHandler(bool _IsVisible);

        /// <summary>
        /// Handler de la fonction qui rend la ProgressBar invisible
        /// </summary>
        private static DelegateSetVisibleHandler DelegateSetVisible;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        private SplashBar(int _Maximum, int _Minimum, int _Step)
        {
            InitializeComponent();

            pb_ProgessBar.Maximum = _Maximum;
            pb_ProgessBar.Minimum = _Minimum;
            pb_ProgessBar.Step = _Step;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Rend la fentre visible ou non
        /// </summary>
        /// <param name="_IsVisible">True rend la fentre visible sinon invisible</param>
        protected void SetVisible(bool _IsVisible) { this.Visible = _IsVisible; }

        /// <summary>
        /// Lance le SplashProgressBar
        /// </summary>
        public static void ShowSplash(int _Maximum, int _Minimum, int _Step)
        {
            try
            {
                if (Ref == null)
                    Ref = new SplashBar(_Maximum, _Minimum, _Step);

                ThreadFrameLauncher = new Thread(new ThreadStart(SplashBar.ThreadMethod));
                ThreadFrameLauncher.Start();
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "SpashBar.ShowSplash()", LogType.Erreur);
            }
        }

        /// <summary>
        /// Ferme le SplashProgressBar
        /// </summary>
        public static void CloseSplash()
        {
            try
            {
                DelegateSetVisible = new DelegateSetVisibleHandler(Ref.SetVisible);
                Ref.Invoke(DelegateSetVisible, false);
                DelegatePerformStep = new DelegatePerformStepHandler(Ref.Dispose);
                Ref.Invoke(DelegatePerformStep);
            }
            catch (Exception ex) { Log.GetInstance().Write(ex.Message, "SplashBar.CloseSplash()", LogType.Erreur); }
            finally
            {
                if (ThreadFrameLauncher != null)
                    ThreadFrameLauncher.Abort();
            }
        }

        /// <summary>
        /// Fait faire une tatpe  la ProgressBar
        /// </summary>
        public static void PerformStep()
        {
            try
            {
                if (Ref == null) return;
                DelegatePerformStep = new DelegatePerformStepHandler(Ref.pb_ProgessBar.PerformStep);
                Ref.pb_ProgessBar.Invoke(DelegatePerformStep);
                //Ref.pb_ProgessBar.PerformStep();
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "SpashBar.PerformStep()", LogType.Erreur);
            }
        }

        /// <summary>
        /// Mthode du thread qui lance la SplashProgressBar
        /// </summary>
        private static void ThreadMethod()
        {
            try
            {
                if (Ref == null) return;
                Application.Run(Ref);
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "SpashBar.ThreadMethod()", LogType.Erreur);
            }
        }

        /// <summary>
        /// Fait un pas dans le ProgressBar
        /// </summary>
        private void vPerformStep() { pb_ProgessBar.PerformStep(); }

        /// <summary>
        /// Rend le SplashBar invisible
        /// </summary>
        private void SetInvisible()
        {
            DelegatePerformStep = new DelegatePerformStepHandler(Ref.pb_ProgessBar.PerformStep);
            Ref.Invoke(DelegatePerformStep, false);
        }
        #endregion
    }
}