using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Components.TypeEditors.GUI
{
    /// <summary>
    /// Dialogue pour la selection d'un texte relativement long
    /// </summary>
    public partial class fr_LongTextEditor : Form
    {
        #region Variables
        /// <summary>
        /// Police de caractre pour le texte slectionn
        /// </summary>
        private Font vFont = new Font("Times New Roman", 12);
        #endregion

        #region Constructor
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_LongTextEditor()
        {
            InitializeComponent();
        }
        #endregion

        #region Methods
        /// <summary>
        /// Initialise le texte de la RichTextBox avec la police en argument
        /// </summary>
        /// <param name="_Font">Police de caractre dans la quelle doit se trouver le texte</param>
        private void SetFont(Font _Font)
        {
            rtb_Text.SelectAll();
            rtb_Text.SelectionFont = _Font;
            vFont = _Font;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Retourne ou initialise le texte  slectionner
        /// </summary>
        public string SelectedText { get { return rtb_Text.Text; } set { rtb_Text.Text = value; } }
        /// <summary>
        /// Retourne ou initialise la police de caractre du texte  selectionner
        /// </summary>
        public Font SelectedFont
        {
            get { return vFont; }
            set
            {
                vFont = value;
                SetFont(value);
            }
        }
        /// <summary>
        /// Indique si la boite de dialogue permet de slectionner la police de dialogue
        /// </summary>
        public bool IsFontSelectable { get { return btn_Font.Visible; } set { btn_Font.Visible = value; } }
        #endregion

        #region Events
        /// <summary>
        /// Demande un changement de police de caractre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SelectFont(object sender, EventArgs e)
        {
            DialogResult dr = fd_FontDialog.ShowDialog();
            if (dr != DialogResult.OK) return;

            SetFont(fd_FontDialog.Font);
        }        
        /// <summary>
        /// Clique sur "OK"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_OK(object sender, EventArgs e)
        {
            base.DialogResult = DialogResult.OK;
            this.Close();
        }        
        /// <summary>
        /// Clique sur "Annuler"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Cancel(object sender, EventArgs e)
        {
            base.DialogResult = DialogResult.Cancel;
            this.Close();
        }
        #endregion
    }
}