using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Utils.Log;
using System.Runtime.InteropServices;
using Components.Webcam;

namespace Components.Webcam
{
    /// <summary>
    /// Composant de gestion de la webcam
    /// </summary>
    [Designer("Sytem.Windows.Forms.Design.ParentControlDesigner,System.Design", 
        typeof(System.ComponentModel.Design.IDesigner))] // make composite
    public partial class WebcamControl : UserControl
        
    {
        #region Variables
        /// <summary>
        /// A true les exceptions sont catches ici sinon 
        /// c'est le programmeur qui doit les catcher lui mme
        /// </summary>
        private bool vCatchException = true;

        #region Variables de proprits
        /// <summary>
        /// Temps de capture en milisecondes
        /// </summary>
        private int vTimeToCapture = 100;

        /// <summary>
        /// Largeur de la frame de capture
        /// </summary>        
        private int vWidth = 320;

        /// <summary>
        /// Hauteur de la frame de capture
        /// </summary>            
        private int vHeight = 240;

        /// <summary>
        /// Capture
        /// </summary>
        private int vCapHwnd;

        /// <summary>
        /// Numro de la frame
        /// </summary>        
        private ulong vFrameNumber = 0;
        #endregion

        #region Optimisation de la capture
        /// <summary>
        /// Variable pour le transfert de donnes
        /// </summary>
        private IDataObject vIDataObject;

        /// <summary>
        /// Image temporaire de la webcam
        /// </summary>
        private Image vTempImg;

        /// <summary>
        /// Indique si la capture est en cours
        /// </summary>
        private bool vIsStopped = true;
        #endregion

        #region Delegates
        /// <summary>
        /// Delege pour l'event de capture
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        public delegate void WebCamEventHandler(object source, WebcamEventArgs e);

        /// <summary>
        ///  Evnement lanc lors d'une nouvelle capture de la webcam
        /// </summary>
        [Description("Lanc lorsqu'une image a t captur")]
        [Category("Events")]
        public event WebCamEventHandler ImageCaptured;
        #endregion

        #region API Declarations
        /// <summary>
        /// Envoi du message
        /// </summary>
        /// <param name="hWnd">Integer</param>
        /// <param name="Msg">Integer</param>
        /// <param name="wParam">Integer</param>
        /// <param name="lParam">Integer</param>
        /// <returns>Integer</returns>
        [DllImport("user32", EntryPoint = "SendMessage")]
        public static extern int SendMessage(int hWnd, uint Msg, int wParam, int lParam);

        /// <summary>
        /// Create a Capture Window
        /// </summary>
        /// <param name="lpszWindowName">string</param>
        /// <param name="dwStyle">Integer</param>
        /// <param name="X">Integer</param>
        /// <param name="Y">Integer</param>
        /// <param name="nWidth">Integer</param>
        /// <param name="nHeight">Integer</param>
        /// <param name="hwndParent">Integer</param>
        /// <param name="nID">Integer</param>
        /// <returns>Integer</returns>
        [DllImport("avicap32.dll", EntryPoint = "capCreateCaptureWindowA")]
        public static extern int capCreateCaptureWindowA(string lpszWindowName, int dwStyle, int X, int Y, int nWidth, int nHeight, int hwndParent, int nID);

        /// <summary>
        /// Ouvre le clipboard
        /// </summary>
        /// <param name="hWnd">Integer</param>
        /// <returns>Integer</returns>
        [DllImport("user32", EntryPoint = "OpenClipboard")]
        public static extern int OpenClipboard(int hWnd);

        /// <summary>
        /// Vide le clipboard
        /// </summary>
        /// <returns>Integer</returns>
        [DllImport("user32", EntryPoint = "EmptyClipboard")]
        public static extern int EmptyClipboard();

        /// <summary>
        /// Ferme le Clipboard
        /// </summary>
        /// <returns>Integer</returns>
        [DllImport("user32", EntryPoint = "CloseClipboard")]
        public static extern int CloseClipboard();

        #endregion

        #region API Constants
        /// <summary>
        /// 
        /// </summary>
        public const int WM_USER = 1024;

        /// <summary>
        /// 
        /// </summary>        
        public const int WM_CAP_CONNECT = 1034;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_DISCONNECT = 1035;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_GET_FRAME = 1084;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_COPY = 1054;

        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_START = WM_USER;

        /// <summary>
        /// 
        /// </summary>        
        public const int WM_CAP_DLG_VIDEOFORMAT = WM_CAP_START + 41;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_DLG_VIDEOSOURCE = WM_CAP_START + 42;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_DLG_VIDEODISPLAY = WM_CAP_START + 43;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_GET_VIDEOFORMAT = WM_CAP_START + 44;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_SET_VIDEOFORMAT = WM_CAP_START + 45;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_DLG_VIDEOCOMPRESSION = WM_CAP_START + 46;
        /// <summary>
        /// 
        /// </summary>
        public const int WM_CAP_SET_PREVIEW = WM_CAP_START + 50;

        #endregion

        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public WebcamControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Libre les ressources
        /// </summary>
        ~WebcamControl() { this.Stop(); }
        #endregion

        #region Mthodes
        /// <summary>
        /// Dmarre la capture du flux vido
        /// </summary>
        /// <param name="_FrameNum">Numro de la frame</param>
        /// <exception cref="Exception"></exception>
        public void Start(ulong _FrameNum)
        {
            try
            {
                if (!vIsStopped)
                    this.Stop();

                //Setup de la fentre de capture
                vCapHwnd = capCreateCaptureWindowA("Webcap", 0, 0, 0,
                    vWidth, vHeight, this.Handle.ToInt32(), 0);

                //Connection de la webcam
                Application.DoEvents();
                SendMessage(vCapHwnd, WM_CAP_CONNECT, 0, 0);
                SendMessage(vCapHwnd, WM_CAP_SET_PREVIEW, 0, 0);

                //Mise du frame number
                vFrameNumber = _FrameNum;

                //Setup du timer
                vTimer.Interval = vTimeToCapture;
                vIsStopped = false;
                vTimer.Start();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "WebcamControl.Start()", LogType.Erreur);

                if (vCatchException)
                {
                    MessageBox.Show("Une erreur est survenue lors de la capture vido!"
                        + "\n\r" + ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                    throw;
            }
        }

        /// <summary>
        /// Arrt de la lecture du flux vido
        /// </summary>
        /// <exception cref="Exception"></exception>
        public void Stop()
        {
            try
            {
                if (!vIsStopped)
                {
                    vTimer.Stop();

                    //Dconnexion de la source vido

                    Application.DoEvents();
                    SendMessage(vCapHwnd, WM_CAP_DISCONNECT, 0, 0);
                }
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "Webcam.Control()", LogType.Erreur);
                if (vCatchException)
                {
                    MessageBox.Show("Erreur lors de l'arrt de la capture vido\r\n" + ex.Message,
                         "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                    throw;

            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le temps en milliseconde entre deux captures
        /// </summary>
        [Category("Options")]
        [Description("Initialise le temps (en milliseconde) entre deux capture d'image")]
        public int TimeToCapture
        {
            get { return vTimeToCapture; }
            set { vTimeToCapture = value; }
        }

        /// <summary>
        /// Retourne ou initialise la hauteur de la capture de l'image
        /// </summary>
        [Category("Options")]
        [Description("Initialise la hauteur de capture de l'image")]
        public int CaptureHeight
        {
            get { return vHeight; }
            set { vHeight = value; }
        }

        /// <summary>
        /// Retourne ou initialise la largeur de la capture de l'image
        /// </summary>
        [Category("Options")]
        [Description("Initialise la largeur de la capture de l'image")]
        public int CaptureWidth
        {
            get { return vWidth; }
            set { vWidth = value; }
        }

        /// <summary>
        /// Retourne ou initialise le numro de la frame de capture
        /// </summary>
        [Category("Options")]
        [Description("Initialise le numro de la frame de capture")]
        public ulong FrameNumber
        {
            get { return vFrameNumber; }
            set { vFrameNumber = value; }
        }
        #endregion

        #region Events
        /// <summary>
        /// Evnement sur un tick du timer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <exception cref="Exception"></exception>
        private void Event_Tick(object sender, EventArgs e)
        {
            WebcamEventArgs args = new WebcamEventArgs();

            try
            {
                if (!vIsStopped)
                {
                    //Pause du timer
                    vTimer.Stop();

                    //Prendre l'image suivante
                    SendMessage(vCapHwnd, WM_CAP_GET_FRAME, 0, 0);

                    //copie de la frame dans le clipboard
                    SendMessage(vCapHwnd, WM_CAP_COPY, 0, 0);


                    //Copie de l'image dans l'EventArgs de l'image
                    if (ImageCaptured != null)
                    {
                        //On prend du clipboard
                        vIDataObject = Clipboard.GetDataObject();
                        vTempImg = (Bitmap)vIDataObject.GetData(DataFormats.Bitmap);

                        //L'API ne resize pas l'image. On doit le faire nous mme
                        
                        args.WebCamImage = vTempImg.GetThumbnailImage(vWidth, vHeight, null, IntPtr.Zero);

                        //On lance l'vnement
                        ImageCaptured(this, args);
                    }

                    //Relance du Timer
                    Application.DoEvents();
                    if (!vIsStopped)
                    {
                        vTimer.Start();
                        vIsStopped = false;
                    }
                }
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "WebcamControl.Event_Tick()", LogType.Erreur);
                if (vCatchException)
                {
                    MessageBox.Show("Une erreur est survenue pendant la capture de la vido!\r\n" + ex.Message,
                        "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                    throw;
            }
        }
        #endregion
    }
}
