using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Components.Agenda.GUI;
using System.Data.OleDb;
using System.Threading;
using Components.Agenda.Management;
using Utils.Log;
using Components.Agenda.Frame;
using PhoenixDLL.Class;
using Utils.DataBase;
using Components.Agenda.Management.StructsEnums;

namespace PhoenixAgenda.GUI
{
    /// <summary>
    /// Fentre pour l'ajout de rendez-vous
    /// </summary>
    public partial class fr_AddMeeting : Form
    {
        #region Delegate
        /// <summary>
        /// Thread de traitement pour le recherche en paralle
        /// </summary>
        private Thread th_Filler;
        #endregion

        #region Variables
        /// <summary>
        /// Information sur le rendez-vous
        /// </summary>
        protected MeetingInfo vMeetingInfo;
        /// <summary>
        /// Vrifie si le texte des TextBoxes a chang
        /// </summary>
        private bool IsTextChanged = true;
        /// <summary>
        /// Notifie si cette frame est pour un update
        /// </summary>
        private bool vIsUpdate = false;
        /// <summary>
        /// MeetingRow ajout dans la base de donnes
        /// </summary>
        private MeetingRow vMeetingRow;
        /// <summary>
        /// MeetingSet contenant les rendez-vous
        /// </summary>
        private MeetingSet vMeetingSet = new MeetingSet();
        /// <summary>
        /// Identifiant du patient
        /// </summary>
        private int vIDPatient = 0;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_AddMeeting()
        {
            InitializeComponent();
            base.DialogResult = DialogResult.Cancel;
            if (Thread.CurrentThread.Name != null)
                Thread.CurrentThread.Name = "Gui Thread";
            lbl_HourSart.Text = "";
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialisatie les information sur les rendez-vous
        /// </summary>
        public MeetingInfo MeetingInfo
        {
            get { return vMeetingInfo; }
            set
            {
                vMeetingInfo = value;
                string Hour = vMeetingInfo.Start.Hour.ToString();
                if (vMeetingInfo.Start.Hour < 10)
                    Hour = "0" + Hour;

                string Minute = vMeetingInfo.Start.Minute.ToString();

                if (vMeetingInfo.Start.Minute < 10)
                    Minute = "0" + Minute;

                lbl_HourSart.Text = "de " + Hour + "h" + Minute;

                Hour = vMeetingInfo.End.Hour.ToString();

                if (vMeetingInfo.End.Hour < 10)
                    Hour = "0" + Hour;

                Minute = vMeetingInfo.End.Minute.ToString();

                if (vMeetingInfo.End.Minute < 10)
                    Minute = "0" + Minute;

                lbl_HourSart.Text += "  " + Hour + "h" + Minute;
            }
        }
        /// <summary>
        /// Retourne le MeetingRow ajout dans la base de donnes
        /// </summary>
        public MeetingRow AddedMeetingRow { get { return vMeetingRow; } }
        /// <summary>
        /// Retourne la liste des rendez-vous mis  jour
        /// </summary>
        public MeetingSet MeetingSet { get { return vMeetingSet; } set { vMeetingSet = value; } }
        /// <summary>
        /// Initialise le prnom du patient
        /// </summary>
        public int IdPatient
        {
            set
            {
                vIDPatient = value;
                tsPB_ProgressBar.Style = ProgressBarStyle.Marquee;

                if (th_Filler != null && !th_Filler.IsAlive)
                {
                    ThreadHandle th = new ThreadHandle(value, this);
                    th_Filler = new Thread(new ThreadStart(th.ProcessOnID));
                    th_Filler.Name = "Data Base Thread";
                    th_Filler.Start();
                }
                else
                {
                    if (th_Filler != null)
                        th_Filler.Abort();

                    ThreadHandle th = new ThreadHandle(value, this);
                    th_Filler = new Thread(new ThreadStart(th.ProcessOnID));
                    th_Filler.Name = "Data Base Thread";
                    th_Filler.Start();
                }
            }
        }
        /// <summary>
        /// Initialise le titre du rendez-vous
        /// </summary>
        public string Title
        { set { tb_Titre.Text = value; } }
        /// <summary>
        /// Initialise le Message du rendez-vous
        /// </summary>
        public string Message
        { set { tb_Message.Text = value; } }
        /// <summary>
        /// Retourne ou initalise une valeur dsignant si la frame est un update ou un ajour de rendez-vous
        /// </summary>
        public bool IsUpdate
        {
            get { return vIsUpdate; }
            set
            {
                vIsUpdate = value;
                if (IsUpdate)
                {
                    tb_Nom.ReadOnly = true;
                    tb_Prenom.ReadOnly = true;
                    dgv_Patient.ReadOnly = true;
                    btn_Add.Text = "Mettre  jour";
                    btn_Cancel.Enabled = true;
                    btn_Cancel.Text = "Supprimer";
                }
                else
                {
                    tb_Nom.ReadOnly = false;
                    tb_Prenom.ReadOnly = false;
                    dgv_Patient.ReadOnly = false;
                    btn_Add.Text = "Ajouter";
                    btn_Cancel.Enabled = false;
                }
            }
        }
        #endregion

        #region Events
        /// <summary>
        /// Demande d'un ajout de rendez-vous
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_AddMeeting(object sender, EventArgs e)
        {
            #region Ajout de rendez-vous
            if (!IsUpdate)
            {
                bool IsAdd = true;
                string ErrorMsg = "";

                if (dgv_Patient.Rows.Count != 1)
                {
                    ErrorMsg += "Veuillez choisir un patient pour le rendez-vous.\n";
                    IsAdd = false;
                }
                if (tb_Titre.Text == "")
                {
                    ErrorMsg += "Veuillez donner un titre au rendez-vous.\n";
                    IsAdd = false;
                }

                if (IsAdd)
                {
                    vMeetingSet.Path = Options.GetInstance().Path;

                    vMeetingRow.EndDate = vMeetingInfo.End;
                    vMeetingRow.StartDate = vMeetingInfo.Start;
                    vMeetingRow.IDPatient = (int)dgv_Patient.SelectedRows[0].Cells["ID_Patient"].Value;
                    vMeetingRow.Message = tb_Message.Text;
                    vMeetingRow.Title = tb_Titre.Text;
                    vMeetingRow.Honoraire = (int)((object)dgv_Patient.SelectedRows[0].Cells["Honoraire"].Value);
                    vMeetingSet.Add(vMeetingRow);

                    DialogResult = DialogResult.OK;
                    this.Close();
                }
                else
                    MessageBox.Show(ErrorMsg, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            #endregion

            #region Modification de rendez-vous
            else
            {
                bool IsAdd = true;
                string ErrorMsg = "";

                if (tb_Titre.Text == "")
                {
                    ErrorMsg += "Veuillez donner un titre au rendez-vous.\n";
                    IsAdd = false;
                }

                if (IsAdd)
                {
                    MeetingRow mr = new MeetingRow();
                    if (dgv_Patient.SelectedRows.Count > 0)
                    {
                        mr.IDPatient = (int)dgv_Patient.SelectedRows[0].Cells["ID_Patient"].Value;

                        vMeetingSet.Path = Options.GetInstance().Path;

                        mr.EndDate = vMeetingInfo.End;
                        mr.StartDate = vMeetingInfo.Start;

                        mr.Message = tb_Message.Text;
                        mr.Title = tb_Titre.Text;
                        mr.IDMeeting = vMeetingInfo.Identifiant;                        
                        vMeetingSet.Modify(mr);

                        DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    else
                    {
                        tb_Nom.ReadOnly = false;
                        tb_Prenom.ReadOnly = false;
                    }
                }
                else
                    MessageBox.Show(ErrorMsg, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            #endregion

            base.DialogResult = DialogResult.OK;
        }
        /// <summary>
        /// Annulation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_RemoveMeeting(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Voulez-vous supprimer ce rendez-vous?", "Attention", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes)
            {
                MeetingRow mr = new MeetingRow();
                mr.IDMeeting = vMeetingInfo.Identifiant;
                mr.IDPatient = vIDPatient;
                try { vMeetingSet.Remove(mr); }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, "fr_AddMeeting.Click_RemoveMeeting()", LogType.Erreur);
                    MessageBox.Show("Impossible de supprimer ce rendez-vous!",
                        "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    base.DialogResult = DialogResult.Cancel;
                    this.Close();
                    return;
                }
                base.DialogResult = DialogResult.OK;
                this.Close();
            }

        }
        /// <summary>
        /// Clique sur un changement d'heure de dpart
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_StartHour(object sender, EventArgs e)
        {
            fr_HourSelector fr = new fr_HourSelector();
            fr.SelectedHour = vMeetingInfo.Start;
            DialogResult dr = fr.ShowDialog();
            DateTime DTNew = new DateTime(1, 1, 1, fr.SelectedHour.Hour, fr.SelectedHour.Minute, 0);
            DateTime DTOld = new DateTime(1, 1, 1, vMeetingInfo.End.Hour, vMeetingInfo.End.Minute, 0);

            if (DTNew > DTOld)
            {
                MessageBox.Show("Veuillez selectionner une heure de dbut antrieure  l'heure de fin",
                    "Attention", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else if (dr == DialogResult.OK)
            {
                vMeetingInfo.Start = fr.SelectedHour;
                lbl_HourSart.Text = "De " + fr.HourToText(vMeetingInfo.Start) + "  " + fr.HourToText(vMeetingInfo.End);
            }

        }
        /// <summary>
        /// Clique sur un changement d'heure de fin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_EndHour(object sender, EventArgs e)
        {
            fr_HourSelector fr = new fr_HourSelector();
            fr.SelectedHour = vMeetingInfo.End;
            DialogResult dr = fr.ShowDialog();

            DateTime DTNew = new DateTime(1, 1, 1, fr.SelectedHour.Hour, fr.SelectedHour.Minute, 0);
            DateTime DTOld = new DateTime(1, 1, 1, vMeetingInfo.Start.Hour, vMeetingInfo.Start.Minute, 0);
            if (DTOld > DTNew)
            {
                MessageBox.Show("Veuillez selectionner une heure de dbut antrieure  l'heure de fin",
                    "Attention", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            else if (dr == DialogResult.OK)
            {
                vMeetingInfo.End = fr.SelectedHour;
                lbl_HourSart.Text = "De " + fr.HourToText(vMeetingInfo.Start) + "  " + fr.HourToText(vMeetingInfo.End);
            }
        }
        /// <summary>
        /// Le texte  chang
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_TextChanged(object sender, EventArgs e)
        {
            if (IsTextChanged)
            {
                tsPB_ProgressBar.Style = ProgressBarStyle.Marquee;

                //Cration et lancement du thread
                if (th_Filler != null && !th_Filler.IsAlive)
                {
                    ThreadHandle th = new ThreadHandle(tb_Nom.Text, tb_Prenom.Text, this);
                    th_Filler = new Thread(new ThreadStart(th.ProcessOnName));
                    th_Filler.Name = "DataBase Thread";
                    th_Filler.Start();
                }
                else
                {
                    if (th_Filler != null)
                        th_Filler.Abort();

                    ThreadHandle th = new ThreadHandle(tb_Nom.Text, tb_Prenom.Text, this);
                    th_Filler = new Thread(new ThreadStart(th.ProcessOnName));
                    th_Filler.Name = "DataBase Thread";
                    th_Filler.Start();
                }
            }

        }
        /// <summary>
        /// Evnement sur fermeture de la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (th_Filler != null && th_Filler.IsAlive)
                th_Filler.Abort();
        }
        /// <summary>
        /// Evnement sur double clique sur une ligne du DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Dgv_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                tb_Nom.Text = (string)dgv_Patient.SelectedRows[0].Cells["Nom"].Value;
                tb_Prenom.Text = (string)dgv_Patient.SelectedRows[0].Cells["Prenom"].Value;
                string NomPrenom = tb_Prenom.Text + " " + tb_Nom.Text;

                if (NomPrenom.Length < 50) //On insre le nom du client
                    tb_Titre.Text = NomPrenom;
                else if ((tb_Nom.Text.Length + 3) < 50)
                    tb_Titre.Text = tb_Prenom.Text[0] + ". " + tb_Nom.Text;
                else
                    tb_Titre.Text = tb_Prenom.Text[0].ToString().ToUpper() + ". "
                        + tb_Nom.Text[0].ToString().ToUpper() + ".";
            }
            catch (ArgumentOutOfRangeException) { }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion
    }

    /// <summary>
    /// Classe du thread
    /// </summary>
    internal class ThreadHandle
    {
        #region Variables
        /// <summary>
        /// Identifiant du patient
        /// </summary>
        protected int vID;

        /// <summary>
        /// Nom du patient
        /// </summary>
        protected string vNom;

        /// <summary>
        /// Prnom du patient
        /// </summary>
        protected string vPrenom;

        /// <summary>
        /// DataGridView qu'il faut remplire
        /// </summary>
        protected DataGridView vDgv;

        /// <summary>
        /// Handler de ma mise  jour de la DataGridView
        /// </summary>
        /// <param name="_ds">DataSet contenant les informations  afficher</param>
        protected delegate void UpdateGuiHandler(DataSet _ds);

        /// <summary>
        /// Handler de mise  jour
        /// </summary>
        protected UpdateGuiHandler UpdateGui;

        /// <summary>
        /// ProgressBar pour le travail sur le DB
        /// </summary>
        protected ToolStripProgressBar vProgressBar;

        /// <summary>
        /// Rference sur la fentre pour les traitement de la GUI
        /// </summary>
        protected fr_AddMeeting vFrame;

        /// <summary>
        /// Sql Manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        public ThreadHandle() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_ID">Identifiant du patient  trouver</param>
        ///<param name="_frame">Fentre dans laquelle se font les traitements GUI</param>        
        public ThreadHandle(int _ID, fr_AddMeeting _frame)
        {
            vID = _ID;
            vFrame = _frame;
            vProgressBar = _frame.tsPB_ProgressBar;
            vDgv = _frame.dgv_Patient;
        }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Nom">Nom (ou partie) du patient</param>
        /// <param name="_Prenom">Prnom (ou partie) du patient</param>
        ///<param name="_frame">Fentre dans laquelle se font les traitements GUI</param>        
        public ThreadHandle(string _Nom, string _Prenom, fr_AddMeeting _frame)
        {
            vNom = _Nom;
            vPrenom = _Prenom;
            vDgv = _frame.dgv_Patient;
            vProgressBar = _frame.tsPB_ProgressBar;
            vFrame = _frame;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Recherche d'un patient sur son nom et prnom
        /// </summary>
        public void ProcessOnName()
        {
            SqlManager.PrepareProcedure("SELECT_Patient");
            SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = vNom + "%";
            SqlManager.Parameters.Add("[_Prenom]", OleDbType.VarChar).Value = vPrenom + "%";

            DataSet ds = new DataSet();
            try
            {
                SqlManager.ExecuteSqlProcedure(ref ds);
                UpdateGui = new UpdateGuiHandler(this.FillDataGridView);
                try { vDgv.Invoke(UpdateGui, ds); }
                catch (InvalidOperationException ex)
                { Log.GetInstance().Write(ex.Message, "ThreadHandle.ProcessOnName()", LogType.Erreur); }
            }
            catch (ThreadAbortException) { }
            catch (OleDbException ex)
            {
                MessageBox.Show("Une erreur dans la base de donnes est survenue."
                + "\nVrifier l'emplacement de la base de donnes dans les options", "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.GetInstance().Write(ex.Message, "GUI.ThreadHandle.ProcessOnName", LogType.Erreur);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue. Veuillez redmarrer l'application",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.GetInstance().Write(ex.Message, "GUI.ThreadHandle.ProcessOnName", LogType.Erreur);
            }

        }
        /// <summary>
        /// Fait une recherche de patient sur son Identifiant
        /// </summary>
        public void ProcessOnID()
        {
            DataSet ds = new DataSet();
            SqlManager.PrepareProcedure("SELECT_PatientOnID");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vID;
            try
            {
                //oDa.Fill(ds);
                SqlManager.ExecuteSqlProcedure(ref ds);


                UpdateGui = new UpdateGuiHandler(this.FillDataGridView);
                try { vDgv.Invoke(UpdateGui, ds); }
                catch (InvalidOperationException ex)
                { Log.GetInstance().Write(ex.Message, "ThreadHandle.ProcessOnName()", LogType.Erreur); }
            }
            catch (ThreadAbortException) { }
            catch (OleDbException ex)
            {
                MessageBox.Show("Une erreur dans la base de donnes est survenue."
                + "\nVrifier l'emplacement de la base de donnes dans les options", "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.GetInstance().Write(ex.Message, "GUI.ThreadHandle.ProcessOnName", LogType.Erreur);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue. Veuillez redmarrer l'application",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.GetInstance().Write(ex.Message, "GUI.ThreadHandle.ProcessOnName", LogType.Erreur);
            }
        }
        /// <summary>
        /// Mthode de traitement du DataGridView
        /// </summary>
        /// <param name="_ds">DataSet qui doit tre DataSource du DataGridView</param>
        private void FillDataGridView(DataSet _ds)
        {
            #region Si les TextBoxes sont vides => c'est une demande de modification
            if (vFrame.tb_Nom.Text == "" && vFrame.tb_Prenom.Text == "" && _ds.Tables[0].Rows.Count == 1)
            {
                vFrame.tb_Nom.Text = (string)_ds.Tables[0].Rows[0]["Nom"];
                vFrame.tb_Prenom.Text = (string)_ds.Tables[0].Rows[0]["Prenom"];
            }
            #endregion
            vDgv.DataSource = _ds.Tables[0];

            for (int i = 0; i < vDgv.Columns.Count; i++)
                vDgv.Columns[i].Visible = false;

            vDgv.Columns["Nom"].Visible = true;
            vDgv.Columns["Nom"].HeaderText = "Nom";

            vDgv.Columns["Prenom"].Visible = true;
            vDgv.Columns["Prenom"].HeaderText = "Prnom";

            vDgv.Columns["DateNaissance"].Visible = true;
            vDgv.Columns["DateNaissance"].HeaderText = "Date de naissance";

            vProgressBar.Style = ProgressBarStyle.Blocks;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le nom du patient
        /// </summary>
        public string Nom
        {
            get { return vNom; }
            set { vNom = value; }
        }

        /// <summary>
        /// Retourne ou initialise le prnom tu patient
        /// </summary>
        public string Prenom
        {
            get { return vPrenom; }
            set { vPrenom = value; }
        }

        /// <summary>
        /// Retourne ou initialise l'identifiant du patient  trouver
        /// </summary>
        public int ID
        {
            get { return vID; }
            set { vID = value; }
        }

        /// <summary>
        /// Retourn ou initialise le DataGrigView dans lequel on fait les affichages
        /// </summary>
        public fr_AddMeeting Frame
        {
            get { return vFrame; }
            protected set { vFrame = value; }
        }
        #endregion
    }
}