using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Utils.Log;
using PhoenixDLL;
using Components.Agenda.GUI;
using Components.Agenda.Management;
using Components.Agenda.Management.StructsEnums;
using System.Data.OleDb;
using Microsoft.Win32;
using Utils.Misc;
using PhoenixDLL.Class;
using PhoenixDLL.GUI;
using Components.SplashBar;
using PhoenixDLL.GUI.Seekers;
using Components.Agenda.GUI.EventArgsObj;

namespace PhoenixAgenda.GUI
{
    /// <summary>
    /// Fentre principale
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Title of the frame
        /// </summary>
        private const string vApplicName = "Phoenix - Cala";
        /// <summary>
        /// Point which is out of frame
        /// </summary>
        private static Point vOutOfFrame = new Point(15000, 15000);
        /// <summary>
        /// fichier de vLog
        /// </summary>
        protected Log vLog = Log.GetInstance();
        /// <summary>
        /// Options de la fentre
        /// </summary>
        protected Options vOpt = Options.GetInstance();
        /// <summary>
        /// MeetingSet contenant toutes les information sur les rendez-vous
        /// </summary>
        protected MeetingSet vMeetingSet = new MeetingSet();
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            ma_Month.DateToDisplay = DateTime.Today;
            DrawComponents();
            ShowMonth();
            this.Text = vApplicName;
            this.MouseWheel += new MouseEventHandler(Event_MouseWheel);
            Options.Instance.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);
            this.tsMenu_Option.Visible = Options.Instance.OptionsMenuVisible;
            da_Day.FirstHourToDisplay = Options.Instance.FirstHourToDisplayDateTime;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Montre les mois
        /// </summary>
        protected void ShowMonth()
        {
            da_Day.Visible = false;
            ma_Month.Visible = true;

            tsmi_Month.Visible = false;
            da_Day.Dock = DockStyle.None;
            da_Day.Location = vOutOfFrame;
            ma_Month.Location = new Point(0, 0);
            ma_Month.Dock = DockStyle.Fill;
            ma_Month.Focus();
            this.Text = vApplicName;
        }
        /// <summary>
        /// Montre les jours
        /// </summary>
        protected void ShowDay()
        {
            da_Day.Visible = false;
            ma_Month.Visible = false;
            ma_Month.Dock = DockStyle.None;
            ma_Month.Location = vOutOfFrame;

            tsmi_Month.Visible = true;            
            da_Day.DisplayFirstHour();

            this.Text += " (" + da_Day.DateToDisplay.ToShortDateString() + ")";
            da_Day.Dock = DockStyle.Fill;
            da_Day.Visible = true;
        }
        /// <summary>
        /// Dessine les composant sur base des Options
        /// </summary>
        protected void DrawComponents()
        {
            #region MonthAgenda
            ma_Month.DaysBackColor = vOpt.DaysBackColor;
            ma_Month.DaysBorderColor = vOpt.DaysBorderColor;
            ma_Month.DaysTextColor = vOpt.DaysTextColor;
            ma_Month.DaysTextFont = vOpt.DaysTextFont;
            ma_Month.DaysNonInMonthColor = vOpt.DaysNonInMonthColor;

            ma_Month.HeaderTextFont = vOpt.HeaderTextFont;
            ma_Month.HeaderTextColor = vOpt.HeaderTextColor;
            ma_Month.HeaderBackColor = vOpt.HeaderBackColor;

            ma_Month.HeaderDayBackColor = vOpt.HeaderDayBackColor;
            ma_Month.HeaderDayTextFont = vOpt.HeaderDayTextFont;
            ma_Month.HeaderDayTextColor = vOpt.HeaderDayTextColor;

            ma_Month.TodayBackColor = vOpt.TodayBackColor;
            #endregion

            #region DayAgenda
            da_Day.MeetingFadeColorOne = vOpt.FirstMeetingFadeColor;
            da_Day.MeetingFadeColorTwo = vOpt.SecondMeetingFadeColor;
            da_Day.MeetingTextColor = vOpt.MeetingTextColor;
            da_Day.MeetingTextFont = vOpt.MeetingTextFont;
            da_Day.NonMeetingBackColor = vOpt.NonMeetingBackColor;
            da_Day.RefreshComponent();
            #endregion
        }
        #endregion

        #region Event

        #region Menu
        /// <summary>
        /// Clique sur le bouton de demande d'affichage des mois
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_MonthVisible(object sender, EventArgs e)
        {
            ShowMonth();
            tsmi_Today.Visible = true;
        }
        /// <summary>
        /// Displays the current month
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_TodayVisible(object sender, EventArgs e)
        {
            ma_Month.DisplayToday();
        }
        /// <summary>
        /// Clique sur le bouton d'vOpt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            DialogResult dr = fr.ShowDialog();

            //Demande de mise--jour des vOpt
            if (dr == DialogResult.OK)
            {
                DrawComponents();
                da_Day.RefreshComponent();
                da_Day.FirstHourToDisplay = Options.Instance.FirstHourToDisplayDateTime;

                OnPaint(new PaintEventArgs(da_Day.CreateGraphics(), new Rectangle(da_Day.Location, da_Day.Size)));
            }

        }
        /// <summary>
        /// Clique sur une demande d'affichage de la fentre "A propos..."
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_About(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur une demande de slection des rendez-vous d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SelectMeetings(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.Text = "Recherche du patient...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;

            fr_MeetingsViewer fm = new fr_MeetingsViewer();
            fm.PatientManager = fr.SelectedItem;
            fm.ShowDialog();

        }
        #endregion

        #region MonthAgenda
        /// <summary>
        /// Clique sur un jour du MonthAgenda
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MonthAgenda_ClickedOnDay(object sender, DayEventArgs e)
        {
            da_Day.DateToDisplay = e.Date;
            da_Day.MeetingList = ma_Month.MeetingList;
            tsmi_Today.Visible = false;
            ShowDay();
        }
        #endregion

        #region DayAgenda
        /// <summary>
        /// Demande de rendez-vous
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MeetingAsked(object sender, MeetingEventArgs e)
        {
            fr_AddMeeting fr = new fr_AddMeeting();
            fr.MeetingInfo = e.MeetingInfo;
            fr.MeetingSet.MeetingList = ma_Month.MeetingList;
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
                da_Day.MeetingList = ma_Month.MeetingList;
            da_Day.Refresh();
        }
        /// <summary>
        /// Demande de modification d'un rendez-vous
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MeetingModificationAsked(object sender, MeetingModificationEventArgs e)
        {
            fr_AddMeeting fr = new fr_AddMeeting();
            fr.IsUpdate = true;
            ResultMeetingRow rmr = vMeetingSet.Seek(e.IDMeeting);
            if (rmr.IsResult)
            {

                fr.IdPatient = rmr.Result.IDPatient;
                fr.Title = rmr.Result.Title;
                fr.Message = rmr.Result.Message;
                fr.MeetingInfo = e.MeetingInfo;
                fr.MeetingSet.MeetingList = ma_Month.MeetingList;
                DialogResult dr = fr.ShowDialog();

                if (dr == DialogResult.OK)
                {
                    da_Day.MeetingList = ma_Month.MeetingList;
                    //ma_Month.MeetingList = fr.MeetingSet;
                }
                da_Day.Refresh();
            }
        }
        #endregion

        #region Mouse
        /// <summary>
        /// Event on MouseWheel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseWheel(object sender, MouseEventArgs e)
        {
            if (da_Day.Location != vOutOfFrame)
            {
                int Lines = 0;
                if (e.Delta < 0)
                    Lines = -(Options.Instance.LinesToWheel);
                else
                    Lines = Options.Instance.LinesToWheel;
                da_Day.Wheel(Lines);
            }
            else
            {
                if (e.Delta > 0)
                    ma_Month.MoveMonth(1);
                else
                    ma_Month.MoveMonth(-1);
            }
        }
        #endregion

        #region Misc
        /// <summary>
        /// Options changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            da_Day.FirstHourToDisplay = Options.Instance.FirstHourToDisplayDateTime;
        }
        #endregion

        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous
        /// </summary>
        public MeetingSet MeetingSet
        {
            get { return vMeetingSet; }
            set
            {
                vMeetingSet = value;
                ma_Month.MeetingList = vMeetingSet.MeetingList;
            }
        }
        #endregion
    }
}