using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL;
using PhoenixDLL.GUI;
using System.Data.OleDb;
using PhoenixDLL.Class;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.GUI.Managers;
using PhoenixCards.GUI;
using Components.SplashBar;
using System.IO;
using System.Drawing.Imaging;
using Utils.Misc;
using PhoenixDLL.Class.Exceptions;
using Utils.Log;

namespace PhoenixCards.GUI
{
    /// <summary>
    /// Fentre principale de la gestion des fiches mdicale
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        private const string vFrameTxr = "Phoenix - Pata";
        /// <summary>
        /// Vrifie si on  sauver les dernires modifications
        /// </summary>
        private bool vIsSaved = true;
        /// <summary>
        /// String de connexion
        /// </summary>
        private string vConStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
        /// <summary>
        /// Path de la base de donnes
        /// </summary>
        private string vPath = Options.GetInstance().Path;
        /// <summary>
        /// Information sur le patient courrant
        /// </summary>
        private PatientManager vPatientManager;
        /// <summary>
        /// Fichier d'vOptions
        /// </summary>
        private Options opt = Options.GetInstance();
        /// <summary>
        /// Collection des onglets 
        /// </summary>
        private List<TabPage> vTabPage = new List<TabPage>();
        /// <summary>
        /// Sauvegarde pour revenir  l'tat prcdent
        /// </summary>
        private string[] vPreviousState = new string[8];
        /// <summary>
        /// Bool pour viter que le premier remplissage ne soit considr comme une modification
        /// </summary>
        private bool vIsLoading = true;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            SetToolStrips();
            opt.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);

            foreach (TabPage tp in tc_General.TabPages)
                vTabPage.Add(tp);

            SetTabs();

            #region Visibilit des menus
            tsMenu_Modification.Enabled = false;
            tsMenu_Supprimer.Enabled = false;

            ts_SeekCancel.Visible = Options.Instance.PataMenu.SeekCancelVisible;
            ts_SeekCancel.Location = Options.Instance.PataMenu.SeekCancelLocation;
            tsMenu_SeekCancel.Checked = Options.Instance.PataMenu.PageStyleVisible;

            ts_General.Visible = Options.Instance.PataMenu.GeneralVisible;
            ts_General.Location = Options.Instance.PataMenu.GeneralLocation;
            tsMenu_General.Checked = Options.Instance.PataMenu.GeneralVisible;

            ts_PageStyle.Visible = Options.Instance.PataMenu.PageStyleVisible;
            ts_PageStyle.Location = Options.Instance.PataMenu.PageStyleLocation;
            tsMenu_PageStyle.Checked = Options.Instance.PataMenu.PageStyleVisible;

            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
            #endregion
        }
        #endregion

        #region Event
        /// <summary>
        /// Clique sur le menu de recherche
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK && fr.SelectedItem.IDPatient > 0)//&& fr.PatientInfo.IsFilled)
            {
                tssl_Name.Text = fr.SelectedItem.Surname + " " + fr.SelectedItem.FamillyName;
                tssl_Updated.Text = "Dernire mj: " + fr.SelectedItem.LastUpdate.ToShortDateString();

                tsMenu_Modification.Enabled = true;
                tsMenu_Modification.Text = "&Modification de " + tssl_Name.Text;
                tsMenu_Supprimer.Enabled = true;
                tsMenu_Supprimer.Text = "&Suppression de " + tssl_Name.Text;
                tsMenu_Previous.Enabled = true;
                tsb_Cancel.Enabled = true;

                #region Remplissage des fiches
                vPatientManager = fr.SelectedItem;
                CardsInfo c = fr.CardsInfo;
                tb_1.Text = c.Tab1;
                vPreviousState[0] = c.Tab1;

                tb_2.Text = c.Tab2;
                vPreviousState[1] = c.Tab2;

                tb_3.Text = c.Tab3;
                vPreviousState[2] = c.Tab3;

                tb_4.Text = c.Tab4;
                vPreviousState[3] = c.Tab4;

                tb_Biodynamics.Text = c.Biodynamics;
                vPreviousState[4] = c.Biodynamics;

                tb_FicheMed.Text = c.Cards;
                vPreviousState[5] = c.Cards;

                tb_Kinergetique.Text = c.Kinergetique;
                vPreviousState[6] = c.Kinergetique;

                tb_HE.Text = c.HE;
                vPreviousState[7] = c.HE;
                #endregion
            }
        }
        /// <summary>
        /// Clique sur le menu Options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// Menu de demande de retour  l'tat prcdent
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Previous(object sender, EventArgs e)
        {
            #region Demande de confirmation
            DialogResult dr = MessageBox.Show("Voulez-vous revenir  la version "
                + "prcdente des fiches mdicales? Si vous cliquez sur oui toutes"
                + " les modifications seront perdues!", "Question", 
                MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr != DialogResult.Yes) return;
            #endregion

            tb_1.Text = vPreviousState[0];
            tb_2.Text = vPreviousState[1];
            tb_3.Text = vPreviousState[2];
            tb_4.Text = vPreviousState[3];
            tb_Biodynamics.Text = vPreviousState[4];
            tb_FicheMed.Text = vPreviousState[5];
            tb_Kinergetique.Text = vPreviousState[6];
            tb_HE.Text = vPreviousState[7];
            UpdateCards();
            vIsSaved = false;
        }
        /// <summary>
        /// Menu Quitter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Sauve et quitte Pata
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SaveNQuit_Click(object sender, EventArgs e)
        {
            SaveCards();
            Close();
        }
        /// <summary>
        /// Demande une modification du patient courrant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PatMod(object sender, EventArgs e)
        {
            if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
            {
                fr_PatientManager fr = new fr_PatientManager();
                fr.Size = new Size(fr.Size.Width, 600);
                fr.Manager = vPatientManager;
                fr.Use = Use.Update;
                fr.Text = "Gestion de patient...";
                fr.ShowDialog();
                tssl_Name.Text = fr.Manager.ToString();

            }
        }
        /// <summary>
        /// Suppression du patient courrant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelPatient(object sender, EventArgs e)
        {
            try
            {
                if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
                {
                    DialogResult dr = MessageBox.Show("Voulez-vous rellement supprimer "
                        + tssl_Name.Text + "?",
                        "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    PatientManager pat = new PatientManager();
                    pat.IDPatient = vPatientManager.IDPatient;

                    if (dr == DialogResult.Yes)
                    {
                        pat.Delete();
                        tssl_Name.Text = "Session ferme";
                        tsMenu_Supprimer.Enabled = false;
                        tsMenu_Modification.Enabled = false;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de supprimer le patient.\r\n" + ex.Message,
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }
        /// <summary>
        /// Evnement sur une demande de sauvegarde
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Save(object sender, EventArgs e)
        {
            SaveCards();
        }
        /// <summary>
        /// Clique sur A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur la barre de status du patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_StatusName(object sender, EventArgs e)
        {
            if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
            {
                fr_PatientManager fr = new fr_PatientManager();
                fr.Size = new Size(fr.Size.Width, 600);
                fr.Manager = vPatientManager;
                fr.Use = Use.Update;
                fr.Text = "Gestion de patient...";
                fr.ShowDialog();
                tssl_Name.Text = fr.Manager.ToString();
                tssl_Updated.Text = "Dernire mj " + fr.Manager.LastUpdate.ToShortDateString();
            }
        }
        /// <summary>
        /// Evnement sur changement de texte
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_TextChanged(object sender, EventArgs e)
        {
            if (vIsLoading)
                vIsLoading = false;
            else
                vIsSaved = false;
        }
        /// <summary>
        /// Evnement sur fermeture de la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FormClosing(object sender, FormClosingEventArgs e)
        {
            #region Location of the menus
            Options.Instance.PataMenu.GeneralLocation = ts_General.Location;
            Options.Instance.PataMenu.PageStyleLocation = ts_PageStyle.Location;
            Options.Instance.PataMenu.SeekCancelLocation = ts_SeekCancel.Location;
            #endregion

            if (!vIsSaved && vPatientManager != null)
            {
                DialogResult dr = MessageBox.Show("Voulez-vous sauver les modifications ralises?",
                     "Attention", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

                if (dr == DialogResult.Yes)
                {
                    try { UpdateCards(); }
                    catch (NotSelectedException ex)
                    {
                        MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else if (dr == DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Load(object sender, EventArgs e)
        {
            ts_General.Location = Options.Instance.PataMenu.GeneralLocation;
            ts_PageStyle.Location = Options.Instance.PataMenu.PageStyleLocation;
            ts_SeekCancel.Location = Options.Instance.PataMenu.SeekCancelLocation;
        }
        /// <summary>
        /// Evnement sur un changement d'vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetTabs();
            SetToolStrips();
        }

        #region Mise en page
        /// <summary>
        /// Ajout la date dans le fentre courrante
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDate(object sender, EventArgs e)
        {
            string AddDate = "\r\n" + DateTime.Today.ToShortDateString()
                + "\r\n==============================\r\n\r\n";
            AddInTab(AddDate);
        }
        /// <summary>
        /// Ajoute un sparateur dans la entre courrante
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddSeparator(object sender, EventArgs e)
        {
            string Separator = "\r\n=====================================" +
                "===========================================\r\n\r\n";
            AddInTab(Separator);
        }
        /// <summary>
        /// Adds text in one of the TextBoxes
        /// </summary>
        /// <param name="_TextToAppend">Text to insert</param>
        private void AddInTab(string _TextToAppend)
        {
            if (tc_General.SelectedTab == tab_Kinergetique)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_Kinergetique.SelectionLength = 0;
                this.tb_Kinergetique.SelectionStart -= this.tb_Kinergetique.SelectionLength;
                this.tb_Kinergetique.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_HE)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_HE.SelectionLength = 0;
                this.tb_HE.SelectionStart -= this.tb_HE.SelectionLength;
                this.tb_HE.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_FicheMed)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_FicheMed.SelectionLength = 0;
                this.tb_FicheMed.SelectionStart -= this.tb_FicheMed.SelectionLength;
                this.tb_FicheMed.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_Biodynamics)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_Biodynamics.SelectionLength = 0;
                this.tb_Biodynamics.SelectionStart -= this.tb_Biodynamics.SelectionLength;
                this.tb_Biodynamics.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_4)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_4.SelectionLength = 0;
                this.tb_4.SelectionStart -= this.tb_4.SelectionLength;
                this.tb_4.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_3)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_3.SelectionLength = 0;
                this.tb_3.SelectionStart -= this.tb_3.SelectionLength;
                this.tb_3.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_2)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_2.SelectionLength = 0;
                this.tb_2.SelectionStart -= this.tb_2.SelectionLength;
                this.tb_2.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
            else if (tc_General.SelectedTab == tab_1)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_1.SelectionLength = 0;
                this.tb_1.SelectionStart -= this.tb_1.SelectionLength;
                this.tb_1.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
        }
        #endregion

        #region Menu Gestion
        /// <summary>
        /// Clique sur une demande de menu de gesion
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekCancel(object sender, EventArgs e)
        {
            if (!tsMenu_SeekCancel.Checked)
            {
                tsMenu_SeekCancel.Checked = true;
                ts_SeekCancel.Visible = true;
            }
            else
            {
                tsMenu_SeekCancel.Checked = false;
                ts_SeekCancel.Visible = false;
            }
            SetIconMenus();
        }
        /// <summary>
        /// Demande sur une demande de menu gnral       
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_General(object sender, EventArgs e)
        {
            if (!tsMenu_General.Checked)
            {
                tsMenu_General.Checked = true;
                ts_General.Visible = true;
            }
            else
            {
                tsMenu_General.Checked = false;
                ts_General.Visible = false;
            }
            SetIconMenus();
        }
        /// <summary>
        /// Clique sur une demande de menu de mise en page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PageStyle(object sender, EventArgs e)
        {
            if (!tsMenu_PageStyle.Checked)
            {
                tsMenu_PageStyle.Checked = true;
                ts_PageStyle.Visible = true;
            }
            else
            {
                tsMenu_PageStyle.Checked = false;
                ts_PageStyle.Visible = false;
            }
            SetIconMenus();
        } 

        #region Ajout
        /// <summary>
        /// Menu d'ajout de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddReput(object sender, EventArgs e)
        {
            fr_ReputationManager fr = new fr_ReputationManager();
            fr.Manager = new ReputationManager();
            fr.Text = "Gestion des rputations...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleManager fr = new fr_MutuelleManager();
            fr.Manager = new MutuelleManager();
            fr.Text = "Gestion des mutuelles...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajoute un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDoctor(object sender, EventArgs e)
        {
            fr_DoctorManager fr = new fr_DoctorManager();
            fr.Manager = new DoctorManager();
            fr.Text = "Gestion des mdecins...";
            fr.Use = Use.Add;
            fr.Size = new Size(327, 381);
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrug(object sender, EventArgs e)
        {
            fr_DrugsManager fr = new fr_DrugsManager();
            fr.Manager = new DrugsManager();
            fr.Text = "Gestion des mdicaments";
            fr.Use = Use.Add;
            fr.ShowDialog();
        } 
        /// <summary>
        /// Ajout d'un type de mdicament        
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeManager fr = new fr_DrugsTypeManager();
            fr.Manager = new DrugsTypeManager();
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Evnement sur une demande d'ajotu de patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPatient(object sender, EventArgs e)
        {
            fr_PatientManager fr = new fr_PatientManager();
            fr.Use = Use.Add;
            fr.Size = new Size(fr.Size.Width, 600);
            fr.Text = "Gestion de patient...";
            fr.ShowDialog();
        }
        #endregion

        #region Modification
        /// <summary>
        /// Modification d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle...";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_MutuelleManager hm = new fr_MutuelleManager();
                hm.Manager = fr.SelectedItem;
                hm.Text = "Modification d'une mutuelle...";
                hm.Use = Use.Update;
                hm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_ReputationManager rm = new fr_ReputationManager();
                rm.Manager = fr.SelectedItem;
                rm.Text = "Modification d'une rputation...";
                rm.Use = Use.Update;
                rm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDoctor(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_DoctorManager dm = new fr_DoctorManager();
                dm.Manager = fr.SelectedItem;
                dm.Text = "Modification d'un mdecin...";
                dm.Use = Use.Update;
                dm.Size = new Size(327, 381);

                dm.ShowDialog();
            }

        }
        /// <summary>
        /// Modification of a drug
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Manager = fr.SelectedItem;
            dm.Text = "Modification d'un mdicament";
            dm.Use = Use.Update;
            dm.Size = new Size(327, 350);
            dm.ShowDialog();
        }
        /// <summary>
        /// Modification of a drug type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Manager = fr.SelectedItem;
            dtm.Text = "Modification d'un type de mdicament";
            dtm.Use = Use.Update;
            dtm.Size = new Size(327, 350);
            dtm.ShowDialog();
        }
        /// <summary>
        /// Moodification of a A patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModAPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.Text = "Recherche d'un patient...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_PatientManager pm = new fr_PatientManager();
            pm.Manager = fr.SelectedItem;
            pm.Text = "Modification d'un patient...";
            pm.Use = Use.Update;
            pm.Size = new Size(pm.Size.Width, 600);
            pm.ShowDialog();

        }
        #endregion

        #region Suppression
        /// <summary>
        /// Suppression de mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDoc(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DoctorManager dm = new fr_DoctorManager();
            dm.Text = "Suppression de ce mdecin";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 400);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_ReputationManager rm = new fr_ReputationManager();
            rm.Text = "Suppression de cette rputation";
            rm.Manager = fr.SelectedItem;
            rm.Use = Use.Delete;
            rm.Size = new Size(rm.Size.Width, 250);
            rm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche d'un mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Text = "Suppression de ce mdicament";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 250);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une mutuelle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_MutuelleManager mm = new fr_MutuelleManager();
            mm.Text = "Suppression de cette mutuelle";
            mm.Manager = fr.SelectedItem;
            mm.Use = Use.Delete;
            mm.Size = new Size(mm.Size.Width, 250);
            mm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un type de mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Text = "Suppression de ce type de mdicament";
            dtm.Manager = fr.SelectedItem;
            dtm.Use = Use.Delete;
            dtm.Size = new Size(dtm.Size.Width, 250);
            dtm.ShowDialog();
        }
        #endregion

        #endregion
        #endregion

        #region Methods
        /// <summary>
        /// Mise  jour des fiches mdicales
        /// </summary>
        protected bool UpdateCards()
        {
            if (vPatientManager.IDPatient > 0)//(vPatientInfo.IsFilled)
            {
                string CoSt = vConStr + vPath;
                CoSt += "\"";

                OleDbConnection oConn = new OleDbConnection(CoSt);
                OleDbCommand oCom = new OleDbCommand("UPDATE_FicheMedicale", oConn);
                oCom.CommandType = CommandType.StoredProcedure;

                oCom.Parameters.Add("_ID", OleDbType.Integer).Value = vPatientManager.IDPatient;
                oCom.Parameters.Add("_FicheMedicale", OleDbType.Variant).Value = tb_FicheMed.Text;
                oCom.Parameters.Add("_Biodynamics", OleDbType.Variant).Value = tb_Biodynamics.Text;
                oCom.Parameters.Add("_HE", OleDbType.Variant).Value = tb_HE.Text;
                oCom.Parameters.Add("_Kinergetique", OleDbType.Variant).Value = tb_Kinergetique.Text;
                oCom.Parameters.Add("_FICHE_SUPPL_1", OleDbType.Variant).Value = tb_1.Text;
                oCom.Parameters.Add("_FICHE_SUPPL_2", OleDbType.Variant).Value = tb_2.Text;
                oCom.Parameters.Add("_FICHE_SUPPL_3", OleDbType.Variant).Value = tb_3.Text;
                oCom.Parameters.Add("_FICHE_SUPPL_4", OleDbType.Variant).Value = tb_4.Text;
                oCom.Parameters.Add("_MAJ", OleDbType.Date).Value = DateTime.Now;


                oConn.Open();
                oCom.ExecuteNonQuery();
                oConn.Close();
                return true;
            }
            else
                throw new NotSelectedException("Aucun patient de selectionn");
        }
        /// <summary>
        /// Initialise les onglets  afficher
        /// </summary>
        protected void SetTabs()
        {
            try
            {
                tc_General.TabPages.Clear();

                foreach (TabPage tp in vTabPage)
                {
                    tc_General.TabPages.Add(tp);
                }

                if (!opt.IsVisible1)
                    tc_General.TabPages.RemoveByKey("tab_1");
                else
                    tc_General.TabPages["tab_1"].Text = opt.AdditionnalTitle1;

                if (!opt.IsVisible2)
                    tc_General.TabPages.RemoveByKey("tab_2");
                else
                    tc_General.TabPages["tab_2"].Text = opt.AdditionnalTitle2;

                if (!opt.IsVisible3)
                    tc_General.TabPages.RemoveByKey("tab_3");
                else
                    tc_General.TabPages["tab_3"].Text = opt.AdditionnalTitle3;

                if (!opt.IsVisible4)
                    tc_General.TabPages.RemoveByKey("tab_4");
                else
                    tc_General.TabPages["tab_4"].Text = opt.AdditionnalTitle4;
            }
            catch (ObjectDisposedException ex)
            { Log.Instance.Write(ex.Message, "PhoenixCars.fr_Main.SetTabs()", LogType.Erreur); }
        }
        /// <summary>
        /// Initialise les valeurs pour les menu  icones
        /// </summary>
        protected void SetIconMenus()
        {
            PataMenu pm = Options.Instance.PataMenu;
            pm.GeneralVisible = tsMenu_General.Checked;
            pm.PageStyleVisible = tsMenu_PageStyle.Checked;
            pm.SeekCancelVisible = tsMenu_SeekCancel.Checked;
            Options.Instance.PataMenu = pm;
        }
        /// <summary>
        /// Saves the cards
        /// </summary>
        private void SaveCards()
        {
            try
            {
                if (vPatientManager != null)
                {
                    UpdateCards();
                    vIsSaved = true;
                    tssl_Updated.Text = "Dernire maj: " + DateTime.Now.ToShortDateString();
                    MessageBox.Show("Enregistrement effectu", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                    MessageBox.Show("Veuillez selectionnner un patient avant d'enregistrer!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (NotSelectedException ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Sets the size of the image of the menus
        /// </summary>
        private void SetToolStrips()
        {
            ts_General.AutoSize = false;
            ts_PageStyle.AutoSize = false;
            ts_SeekCancel.AutoSize = false;
            ts_General.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_PageStyle.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_SeekCancel.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_General.AutoSize = true;
            ts_PageStyle.AutoSize = true;
            ts_SeekCancel.AutoSize = true;
        }
        #endregion

        #region Debugging
        /// <summary>
        /// Clique sur une recherche de mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DoctorSeeking(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur recherche de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Reputation(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
                MessageBox.Show(fr.SelectedItem.ToString());
        }
        /// <summary>
        /// Clique sur une demande de listing d'honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Honoraire(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
                MessageBox.Show(fr.SelectedItem.ToString());
        }
        /// <summary>
        /// Menu pour voir les photos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Photos(object sender, EventArgs e)
        {
            MessageBox.Show("Non, il n'y a plus de menu d'ajout de photo dans le mode DEBUG!",
                "Debug Folie", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }
        /// <summary>
        /// Viewer de photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ViewPicture(object sender, EventArgs e)
        {
            MessageBox.Show("Non, il n'y a plus de menu d'ajout de photo dans le mode DEBUG!",
                "Debug Folie", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }
        /// <summary>
        /// Affiche le nouveau fr_PatientSeeker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewPatientSeeker(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Lancement du nouveau DoctorSeeker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewDoctorSeeker(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau selectionneur de type de mdicaments
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewTypeDrugSeeker(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.ShowDialog();

        }
        /// <summary>
        /// Nouveau selectionneur de mutuelle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau selectionneur de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewReputation(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau menu de recherche de Mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekDrugs(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// LongTextEditor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_LongTextEditor_Click(object sender, EventArgs e)
        {
            Components.TypeEditors.GUI.fr_LongTextEditor fr = new Components.TypeEditors.GUI.fr_LongTextEditor();

            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;

            MessageBox.Show("Txt: " + fr.SelectedText + "\r\nFont: " + fr.SelectedFont.ToString());

        }
        #endregion

        #region Properties
        /// <summary>
        /// Retourne une valeur booleenne qui indique si on a modifier le contenu des fiches
        /// </summary>
        public bool Saved
        {
            get
            {
                if (this.vIsLoading) return true;
                else return this.vIsSaved;
            }
        }
        #endregion
    }
}