using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.GUI;
using PhoenixDLL.Class;
using PhoenixStatistics.Class;
using System.IO;
using Utils.Log;

namespace PhoenixCommander.GUI
{
    /// <summary>
    /// Phoenix Suite
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Informations sur les rendez-vous
        /// </summary>
        private MeetingSet vMeetingSet = new MeetingSet();
        /// <summary>
        /// Gestionnaire de statistiques
        /// </summary>
        private StatManager vStatManager = new StatManager();
        /// <summary>
        /// Agenda
        /// </summary>
        private PhoenixAgenda.GUI.fr_Main Cala;
        /// <summary>
        /// Fiches mdicales
        /// </summary>
        private PhoenixCards.GUI.fr_Main Pata;
        /// <summary>
        /// Photos
        /// </summary>
        private PhoenixPics.GUI.fr_Main Poto;
        /// <summary>
        /// Prescriptions
        /// </summary>
        private PhoenixPrescription.GUI.fr_Main Pega;
        /// <summary>
        /// Honoraires
        /// </summary>
        private PhoenixMoney.GUI.fr_Main Paga;
        /// <summary>
        /// Statistiques
        /// </summary>
        private PhoenixStatistics.GUI.fr_Main Stata;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            Options.Instance.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);
            SetupDeck();
        }
        #endregion

        #region Events
        /// <summary>
        /// Chargement de la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            BringToFront();
        }
        /// <summary>
        /// Clique sur une demande de Calendrier
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Cala(object sender, EventArgs e)
        {
            if (Cala == null || Cala.IsDisposed)
            {
                Cala = new PhoenixAgenda.GUI.fr_Main();
                Cala.MdiParent = this;
                Cala.MeetingSet = vMeetingSet;
                Cala.Show();
            }
            else
                Cala.BringToFront();
        }
        /// <summary>
        /// Clique sur une demande de fiche mdicale
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Pata(object sender, EventArgs e)
        {
            if (Pata == null || Pata.IsDisposed)
            {
                Pata = new PhoenixCards.GUI.fr_Main();
                Pata.MdiParent = this;
                Pata.Show();
            }
            else
                Pata.BringToFront();
        }
        /// <summary>
        /// Clique sur une demande de Photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Poto(object sender, EventArgs e)
        {
            if (Poto == null || Poto.IsDisposed)
            {
                Poto = new PhoenixPics.GUI.fr_Main();
                Poto.MdiParent = this;
                Poto.Show();
            }
            else
                Poto.BringToFront();
        }
        /// <summary>
        /// Clique sur une demande de prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Pega(object sender, EventArgs e)
        {
            if (Pega == null || Pega.IsDisposed)
            {
                Pega = new PhoenixPrescription.GUI.fr_Main();
                Pega.MdiParent = this;
                Pega.Show();
            }
            else
                Pega.BringToFront();
        }
        /// <summary>
        /// Clique sur une demande d'honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Paga(object sender, EventArgs e)
        {
            if (Paga == null || Paga.IsDisposed)
            {
                Paga = new PhoenixMoney.GUI.fr_Main();
                Paga.MdiParent = this;
                Paga.Show();
            }
            else
                Paga.BringToFront();
        }
        /// <summary>
        /// Clique sur une demande de statistiques
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Stata(object sender, EventArgs e)
        {
            if (Stata == null || Stata.IsDisposed)
            {
                Stata = new PhoenixStatistics.GUI.fr_Main();
                Stata.MdiParent = this;
                Stata.StatManager = vStatManager;
                Stata.Show();
            }
            else
                Stata.BringToFront();
        }
        /// <summary>
        /// Menu quitter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Save the DataBase
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Save(object sender, EventArgs e)
        {
            try
            {
                DateTime dt = DateTime.Today;
                sfd_SaveDB.FileName = "" + dt.Year + "-" + dt.Month + "-" + dt.Day + "-" + "GestionPatient.mdb";
                DialogResult dr = sfd_SaveDB.ShowDialog();
                if (dr != DialogResult.OK) return;

                #region Traitement du nom du fichier
                string FileName = sfd_SaveDB.FileName;
                if (!FileName.EndsWith(".mdb")) FileName += ".mdb";
                #endregion

                #region Copie de la BD
                FileInfo fi = new FileInfo(Options.Instance.Path);
                fi.CopyTo(FileName);
                MessageBox.Show("La base de donnes a t sauvegare!", "Information",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                #endregion
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de faire la sauvegarde de la base de donnes!\r\n" + ex.Message,
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                Log.Instance.Write(ex.Message, "PhoenixCommander.fr_Main.MenuClick_Save()", LogType.Erreur);
            }
        }
        /// <summary>
        /// Demande d'vOpt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// Options changed Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetupDeck();            
        }
        /// <summary>
        /// The mouse hoves on a button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ToolTips(object sender, EventArgs e)
        {
            tt_Tips.AutoPopDelay = 5000;
            tt_Tips.InitialDelay = 250;
            tt_Tips.ReshowDelay = 250;
            tt_Tips.ShowAlways = true;

            if (sender == btn_Cala)
            {
                tt_Tips.ToolTipTitle = "Cala";
                tt_Tips.SetToolTip(this.btn_Cala, "Agenda pour la gestion des rendez-vous");
            }
            else if (sender == btn_Paga)
            {
                tt_Tips.ToolTipTitle = "Paga";
                tt_Tips.SetToolTip(this.btn_Paga, "Gestionnaire des honoraires");
            }
            else if (sender == btn_Pata)
            {
                tt_Tips.ToolTipTitle = "Pata";
                tt_Tips.SetToolTip(this.btn_Pata, "Gestionnaire des fiches mdicales");
            }
            else if (sender == btn_Pega)
            {
                tt_Tips.ToolTipTitle = "Pega";
                tt_Tips.SetToolTip(this.btn_Pega, "Gestionnaire des prescriptions");
            }
            else if (sender == btn_Poto)
            {
                tt_Tips.ToolTipTitle = "Poto";
                tt_Tips.SetToolTip(this.btn_Poto, "Gestionnaire des photos");
            }
            else if (sender == btn_Stata)
            {
                tt_Tips.ToolTipTitle = "Stata";
                tt_Tips.SetToolTip(this.btn_Stata, "Gestionnaire des statistiques");
            }
            else if (sender == btn_Quit)
            {
                tt_Tips.ToolTipTitle = "Quitter";
                tt_Tips.SetToolTip(this.btn_Quit, "Quitter Phoenix Suite");
            }
            else if (sender == btn_Options)
            {
                tt_Tips.ToolTipTitle = "Options";
                tt_Tips.SetToolTip(this.btn_Options, "Pour la gestion des options de la Phoenix Suite");
            }
            else if (sender == btn_Backup)
            {
                tt_Tips.ToolTipTitle = "Backup de la base de donnes";
                tt_Tips.SetToolTip(this.btn_Backup, "Pour faire un backup complet de la base de donnes");
            }
        }
        #endregion

        #region Mthods
        /// <summary>
        /// Sets up the visibility and the position of the menu deck
        /// </summary>
        private void SetupDeck()
        {
            #region Position du menu latral
            if (Options.Instance.DeckVisible)
            {
                tsmi_Menu.Visible = false;
                tlp_MenuLR.Visible = Options.Instance.DeckVisible;
                tsmi_Menu.Visible = (Options.Instance.DeckVisible ^ true);

                if (Options.Instance.DeckPosition == DeckPosition.Gauche ||
                    Options.Instance.DeckPosition == DeckPosition.Droite)
                {
                    tlp_MenuLR.Visible = true;
                    tlp_MenuUD.Visible = false;

                    tlp_MenuLR.Controls.Add(btn_Cala, 0, 0);
                    tlp_MenuLR.Controls.Add(btn_Pata, 0, 1);                                       
                    tlp_MenuLR.Controls.Add(btn_Pega, 0, 2);
                    tlp_MenuLR.Controls.Add(btn_Poto, 0, 3);
                    tlp_MenuLR.Controls.Add(btn_Paga, 0, 4); 
                    tlp_MenuLR.Controls.Add(btn_Stata, 0, 5);
                    tlp_MenuLR.Controls.Add(btn_Options, 0, 6);
                    tlp_MenuLR.Controls.Add(btn_Backup, 0, 7);
                    tlp_MenuLR.Controls.Add(btn_Quit, 0, 8);

                    if (Options.Instance.DeckPosition == DeckPosition.Gauche)
                        tlp_MenuLR.Dock = DockStyle.Left;
                    else
                        tlp_MenuLR.Dock = DockStyle.Right;
                }
                else
                {
                    tlp_MenuUD.Visible = true;
                    tlp_MenuLR.Visible = false;

                    tlp_MenuUD.Controls.Add(btn_Cala, 0, 0);
                    tlp_MenuUD.Controls.Add(btn_Pata, 1, 0);                                        
                    tlp_MenuUD.Controls.Add(btn_Pega, 2, 0);
                    tlp_MenuUD.Controls.Add(btn_Poto, 3, 0);
                    tlp_MenuUD.Controls.Add(btn_Paga, 4, 0);
                    tlp_MenuUD.Controls.Add(btn_Stata, 5, 0);
                    tlp_MenuUD.Controls.Add(btn_Options, 6, 0);
                    tlp_MenuUD.Controls.Add(btn_Backup, 7, 0);
                    tlp_MenuUD.Controls.Add(btn_Quit, 8, 0);

                    if (Options.Instance.DeckPosition == DeckPosition.Haut)
                        tlp_MenuUD.Dock = DockStyle.Top;
                    else
                        tlp_MenuUD.Dock = DockStyle.Bottom;
                }
            }
            #endregion

            else
            {
                tlp_MenuUD.Visible = false;
                tlp_MenuLR.Visible = false;
                tsmi_Menu.Visible = true;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le MeetingSet des rendez-vous
        /// </summary>
        public MeetingSet MeetingSet { set { vMeetingSet = value; } get { return vMeetingSet; } }
        /// <summary>
        /// Retourne ou initialise le gestionnaire de statistiques
        /// </summary>
        public StatManager StatManager { set { vStatManager = value; } get { return vStatManager; } }
        #endregion
    }
}