using System;
using System.Collections.Generic;
using System.Windows.Forms;
using PhoenixCommander.GUI;
using Components.SplashBar;
using PhoenixDLL.Class;
using System.IO;
using System.Data.OleDb;
using PhoenixDLL.GUI;
using PhoenixStatistics.Class;
using Utils.Log;
using Utils.DataBase;

namespace PhoenixCommander
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            MeetingSet ms = new MeetingSet();

            try
            {
                if (!Options.Instance.IsFirstInstance)
                {
                    #region Sauvegarde de la base de donnes
                    TimeSpan ts = DateTime.Today - Options.Instance.LastDBSave;
                    if (ts.Days > 30 && Options.Instance.AutomaticDBSave)
                    {
                        DialogResult dr = MessageBox.Show("Cela fait plus de 30 jours que vous n'avez pas sauvegard"
                            + " la base de donnes et il serait raisonnable de faire"
                            + " une sauvegarde de la base de donnes.\r\nVoulez-vous faire une sauvegarde?", "Attention",
                               MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk);

                        if (dr == DialogResult.Yes)
                        {
                            #region SaveFileDialog setup
                            SaveFileDialog sfd = new SaveFileDialog();
                            sfd.Filter = @"""Base de donnes"" (*.mdb)| ""Tous les fichiers"" (*.*)";
                            DateTime dt = DateTime.Today;
                            sfd.FileName = "" + dt.Year + "-" + dt.Month + "-" + dt.Day + "-" + "GestionPatient.mdb";
                            dr = sfd.ShowDialog();
                            if (dr != DialogResult.OK) return;
                            #endregion

                            #region Traitement du nom du fichier
                            string FileName = sfd.FileName;
                            if (!FileName.EndsWith(".mdb")) FileName += ".mdb";
                            #endregion

                            #region Copie de la BD
                            FileInfo fi = new FileInfo(Options.Instance.Path);
                            fi.CopyTo(FileName, true);
                            Options.Instance.LastDBSave = DateTime.Today;
                            #endregion
                        }
                        if (dr == DialogResult.No)
                        {
                            dr = MessageBox.Show("Voulez-vous annuler le systme de vrification de sauvegarde automatique?", "Question",
                                 MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                            if (dr == DialogResult.No)
                                Options.Instance.AutomaticDBSave = true;
                            else
                                Options.Instance.AutomaticDBSave = false;
                        }
                    }
                    #endregion
                }
                else
                    Options.Instance.IsFirstInstance = false;
                RegistryManager.ProcessRegistry();
                ms.Path = Options.GetInstance().Path;
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                SqlManager.ExecuteNonQuery("SELECT COUNT(*) FROM Honoraire;");
            }
            #region Exception
            catch (FileNotFoundException)
            {
                MessageBox.Show("La base de donnes n'a pas t trouve. Veuillez indiquer son emplacement...",
                    "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                OpenFileDialog ofd = new OpenFileDialog();
                ofd.Title = "Slection de la base de donnes...";
                DialogResult dr = ofd.ShowDialog();

                if (dr == DialogResult.OK)
                {
                    Options.GetInstance().Path = ofd.FileName;
                    ms.Path = Options.Instance.Path;
                }
                else
                    return;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "PhoenixCommander.Program()", LogType.Erreur);
                DialogResult dr = MessageBox.Show("Une erreur est intervenue.\r\n" + ex.Message
                    + "\r\nEst-ce une erreur en rapport avec l'emplacement de la base de donnes?",
                    "Erreur", MessageBoxButtons.YesNo, MessageBoxIcon.Error);
                if (dr == DialogResult.Yes)
                {
                    OpenFileDialog ofd = new OpenFileDialog();
                    ofd.ShowDialog();
                    Options.Instance.Path = ofd.FileName;
                }
                MessageBox.Show("L'application va maintenant se fermer!", "Information",
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            #endregion

            #region Chargement des rendez-vous
            try
            {
                StatManager sm = new StatManager();
                SplashBar.ShowSplash(9, 0, 1);
                SplashBar.PerformStep();
                fr_Main fr = new fr_Main();

                ms.Fill();
                SplashBar.PerformStep();

                sm.CalculateMeetingOnMonth();
                SplashBar.PerformStep();

                sm.CalculateAges();
                SplashBar.PerformStep();

                sm.CalculateWomenAges();
                SplashBar.PerformStep();

                sm.CalculateMenAges();
                SplashBar.PerformStep();

                sm.CalculateAllCount();
                SplashBar.PerformStep();

                sm.CalculateAllAverageAges();
                SplashBar.PerformStep();

                SplashBar.CloseSplash();

                fr.StatManager = sm;
                fr.MeetingSet = ms;
                Options.Instance.IsEventable = true;
                Application.Run(fr);
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex.Message, "PhoenixCommander.Program.Main()", LogType.Erreur);
                SplashBar.CloseSplash();
                DialogResult dr = MessageBox.Show("Une erreur est apparue au niveau de la base de donnes. La plus part du temps c'est" +
                    " parce que ce n'est pas la bonne base de donnes qui a t selectionne dans les options\r\n(" + ex.Message
                    + ")\r\nVoulez-vous ouvrir le menu d'options?", "Attention",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk);
                if (dr == DialogResult.No) return;
                else
                {
                    try
                    {
                        fr_Options fro = new fr_Options();
                        fro.StartPosition = FormStartPosition.CenterScreen;
                        fro.ShowDialog();
                    }
                    catch (Exception exc)
                    {
                        MessageBox.Show("Une erreur est survenue...\r\n" + exc.Message
                            + "\r\nSi le problme persiste veuillez contacter votre administrateur",
                            "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est intervenue.\r\n" + ex.Message + "\r\nRedmarrer et si cela se reproduit, contactez votre administrateur", "Erreur",
                     MessageBoxButtons.OK, MessageBoxIcon.Error);
                SplashBar.CloseSplash();
            }
            #endregion
        }
    }
}