using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using System.ComponentModel;
using Utils.Log;
using Utils.DataBase;
using PhoenixDLL.Class.Interface;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Gestionnaire de mdecins
    /// </summary>
    public class DoctorManager : IManager<DoctorManager>
    {
        #region Variables
        /// <summary>
        /// Identifiant du mdecin
        /// </summary>
        private int vID = 0;
        /// <summary>
        /// Nom du mdecin
        /// </summary>
        private string vNom = "";
        /// <summary>
        /// Prnom du mdecin
        /// </summary>
        private string vPrenom = "";
        /// <summary>
        /// Adresse du mdecin
        /// </summary>
        private string vAdresse = "";
        /// <summary>
        /// Ville du mdecin
        /// </summary>
        private string vVille = "";
        /// <summary>
        /// Code postal du mdecin
        /// </summary>
        private string vCodePostal = "";
        /// <summary>
        /// Code postal du mdecin
        /// </summary>
        private string vTelephone = "";
        /// <summary>
        /// Fax du mdecin
        /// </summary>
        private string vFax = "";
        /// <summary>
        /// Est-ce un homopathe
        /// </summary>
        private bool vIsHomeo = false;
        /// <summary>
        /// Est-ce un mdecin traitant
        /// </summary>
        private bool vIsMedecin = false;
        /// <summary>
        /// Gestionnaire SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public DoctorManager() { }
        #endregion

        #region Methodes
        /// <summary>
        /// Surcharge le mthode de convertion en chaine de caractre
        /// </summary>
        /// <returns>Le nom du mdecin</returns>
        public override string ToString()
        {
            return this.Prenom + " " + this.Nom;
        }
        /// <summary>
        /// Sur base d'un identifiant, on cre une instance de DoctorManager
        /// </summary>
        /// <param name="_ID">Identifiant du mdecin</param>
        /// <returns>Une instance de DoctorManager</returns>
        public static DoctorManager Create(int _ID)
        {
            DoctorManager dm = new DoctorManager();
            dm.ID = _ID;
            return dm.Create();
        }
        /// <summary>
        /// Sur base d'un identifiant, on cre une instance de DoctorManager
        /// </summary>        
        /// <returns>Une instance de DoctorManager</returns>
        public DoctorManager Create()
        {
            OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
            SqlManager.PrepareProcedure("SELECT_MedecinOnID");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vID;
            DataSet ds = new DataSet();
            int cpt = SqlManager.ExecuteSqlProcedure(ref ds);

            DoctorManager doc = new DoctorManager();
            if (cpt == 1)
            {
                if (!ds.Tables[0].Rows[0]["Adresse"].Equals(DBNull.Value))
                    doc.Adresse = (string)ds.Tables[0].Rows[0]["Adresse"];

                if (!ds.Tables[0].Rows[0]["CodePostal"].Equals(DBNull.Value))
                    doc.CodePostal = (string)ds.Tables[0].Rows[0]["CodePostal"];

                if (!ds.Tables[0].Rows[0]["Fax"].Equals(DBNull.Value))
                    doc.Fax = (string)ds.Tables[0].Rows[0]["Fax"];

                if (!ds.Tables[0].Rows[0]["id_medecin"].Equals(DBNull.Value))
                    doc.ID = (int)ds.Tables[0].Rows[0]["id_medecin"];

                if (!ds.Tables[0].Rows[0]["Homeopathe"].Equals(DBNull.Value))
                    doc.IsHomeo = (bool)ds.Tables[0].Rows[0]["Homeopathe"];

                if (!ds.Tables[0].Rows[0]["MedecinTraitant"].Equals(DBNull.Value))
                    doc.IsMedecin = (bool)ds.Tables[0].Rows[0]["MedecinTraitant"];

                if (!ds.Tables[0].Rows[0]["Nom"].Equals(DBNull.Value))
                    doc.Nom = (string)ds.Tables[0].Rows[0]["Nom"];

                if (!ds.Tables[0].Rows[0]["Prenom"].Equals(DBNull.Value))
                    doc.Prenom = (string)ds.Tables[0].Rows[0]["Prenom"];

                if (!ds.Tables[0].Rows[0]["Telephone"].Equals(DBNull.Value))
                    doc.Telephone = (string)ds.Tables[0].Rows[0]["Telephone"];

                if (!ds.Tables[0].Rows[0]["Ville"].Equals(DBNull.Value))
                    doc.Ville = (string)ds.Tables[0].Rows[0]["Ville"];
                doc.vEmpty = false;
            }
            return doc;
        }
        /// <summary>
        /// Ajoute un mdecin sur base des informations contenues dans l'objet
        /// </summary>
        public void Add()
        {
            try
            {
                #region Check if doctor with same name exists
                string SqlQuery = "SELECT COUNT(*) FROM Medecin WHERE UCASE(Nom) LIKE \""
                    + this.Nom.ToUpper() + "\" AND UCASE(Prenom) LIKE \"" + this.Prenom.ToUpper() +"\";";
                int cnt = (int)SqlManager.ExecuteScalar(SqlQuery);
                if (cnt > 0)
                {
                    DialogResult dr = MessageBox.Show("Un mdecin avec le mme nom et prnom existe. "
                        + "Voulez-vous faire l'ajout malgr tout?", "Attention", MessageBoxButtons.YesNo,
                        MessageBoxIcon.Exclamation);
                    if (DialogResult.No == dr) return;
                }
                #endregion

                SqlManager.PrepareProcedure("INSERT_Medecin");
                SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = this.vNom.ToUpper();
                SqlManager.Parameters.Add("[_Prenom]", OleDbType.VarChar).Value = this.vPrenom;
                SqlManager.Parameters.Add("[_Adresse]", OleDbType.VarChar).Value = this.vAdresse;
                SqlManager.Parameters.Add("[_Ville]", OleDbType.VarChar).Value = this.vVille;
                SqlManager.Parameters.Add("[_CodePostal]", OleDbType.VarChar).Value = this.vCodePostal;
                SqlManager.Parameters.Add("[_Telephone]", OleDbType.VarChar).Value = this.vTelephone;
                if (vFax == "")
                    SqlManager.Parameters.Add("[_Fax]", OleDbType.Integer).Value = DBNull.Value;
                else
                    SqlManager.Parameters.Add("[_Fax]", OleDbType.VarChar).Value = this.vFax;
                SqlManager.Parameters.Add("[_Homeopathe]", OleDbType.Boolean).Value = this.vIsHomeo;
                SqlManager.Parameters.Add("[_MedecinTraitant]", OleDbType.Boolean).Value = this.vIsMedecin;

                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                this.ID = (int)LastID;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DoctorManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Fait une suppression de mdecin
        /// </summary>
        public void Delete()
        {
            try
            {
                SqlManager.ExecuteNonQuery("DELETE * FROM Medecin WHERE id_Medecin = " + vID);
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DoctorManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Fait une mise  jour du mdecin
        /// </summary>
        public void Update()
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_Medecin");
                SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = this.vNom.ToUpper();
                SqlManager.Parameters.Add("[_Prenom]", OleDbType.VarChar).Value = this.vPrenom;
                SqlManager.Parameters.Add("[_Adresse]", OleDbType.VarChar).Value = this.vAdresse;
                SqlManager.Parameters.Add("[_Ville]", OleDbType.VarChar).Value = this.vVille;
                SqlManager.Parameters.Add("[_CodePostal]", OleDbType.VarChar).Value = this.vCodePostal;
                SqlManager.Parameters.Add("[_Telephone]", OleDbType.VarChar).Value = this.vTelephone;
                SqlManager.Parameters.Add("[_Fax]", OleDbType.VarChar).Value = this.vFax;
                SqlManager.Parameters.Add("[_Homeopathe]", OleDbType.Boolean).Value = this.vIsHomeo;
                SqlManager.Parameters.Add("[_MedecinTraitant]", OleDbType.Boolean).Value = this.vIsMedecin;
                SqlManager.Parameters.Add("[id_medecin]", OleDbType.Integer).Value = this.vID;

                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DoctorManager.Update()", LogType.Erreur);
                throw;
            }
        }
        #endregion

        #region Operateurs
        /// <summary>
        /// Casting explicit du type DoctorManager vers String
        /// </summary>
        /// <param name="_DoctorManager">DoctorManager  convertir</param>
        /// <returns>Le nom prnom du mdecin</returns>
        public static implicit operator string(DoctorManager _DoctorManager)
        {
            return _DoctorManager.ToString();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'identifiant du mdecin
        /// </summary>
        [Browsable(false)]
        public int ID { get { return vID; } set { vID = value; } }
        /// <summary>
        /// retourne ou initialise le nom du mdecin
        /// </summary>
        [Category("1- Donnes privs")]
        [Description("Nom du mdecin")]
        [DisplayName("1.1- Nom")]
        public string Nom { get { return vNom; } set { vNom = value; } }
        /// <summary>
        /// Retourne ou initialise le prnom du mdecin
        /// </summary>
        [Category("1- Donnes privs")]
        [Description("Prnom du mdecin")]
        [DisplayName("1.2- Prnom")]
        public string Prenom { get { return vPrenom; } set { vPrenom = value; } }
        /// <summary>
        /// Retourne ou initialise l'adresse du mdecin
        /// </summary>
        [Category("2- Adresse")]
        [Description("Rue et numro")]
        [DisplayName("2.1- Rue et numro")]
        public string Adresse { get { return vAdresse; } set { vAdresse = value; } }
        /// <summary>
        /// Retourne ou initialise la ville du mdecin
        /// </summary>
        [Category("2- Adresse")]
        [Description("Ville")]
        [DisplayName("2.2- Ville")]
        public string Ville { get { return vVille; } set { vVille = value; } }
        /// <summary>
        /// Retourne ou initialise le code postal du mdecin
        /// </summary>
        [Category("2- Adresse")]
        [Description("Code postal")]
        [DisplayName("2.3- Code postal")]
        public string CodePostal { get { return vCodePostal; } set { vCodePostal = value; } }
        /// <summary>
        /// Retourne ou initialise le numro de tlphone du mdecin
        /// </summary>
        [Category("3- Tlphone Fax")]
        [Description("Tlphone")]
        [DisplayName("3.1- Tlphone")]
        public string Telephone { get { return vTelephone; } set { vTelephone = value; } }
        /// <summary>
        /// Retourn ou initialise le fax du mdecin
        /// </summary>
        [Category("3- Tlphone Fax")]
        [Description("Fax")]
        [DisplayName("3.2- Fax")]
        public string Fax { get { return vFax; } set { vFax = value; } }
        /// <summary>
        /// Indique si le mdecin est homopathe
        /// </summary>
        [Category("4- Spcialit")]
        [Description("Est-ce un homeopathe?")]
        [DisplayName("4.1- Homopathe")]
        public bool IsHomeo { get { return vIsHomeo; } set { vIsHomeo = value; } }
        /// <summary>
        /// Indique si le mdecin est mdecin traitant
        /// </summary>
        [Category("4- Spcialit")]
        [Description("Est-ce un mdecin traitant?")]
        [DisplayName("4.2- Mdecin traitant")]
        public bool IsMedecin { get { return vIsMedecin; } set { vIsMedecin = value; } }
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }
        #endregion
    }
}
