using System;
using System.Collections.Generic;
using System.Text;
using PhoenixDLL.Class.Interface;
using System.ComponentModel;
using Utils.DataBase;
using System.Data.OleDb;
using Utils.Log;
using System.Data;
using PhoenixDLL.Editors;
using System.Drawing.Design;
using Components.TypeEditors.GUI;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Gestionnaire de mdicament
    /// </summary>
    public class DrugsManager : IManager<DrugsManager>
    {
        #region Variables
        /// <summary>
        /// ID du mdicament
        /// </summary>
        private int vIDMedic = -1;
        /// <summary>
        /// Remarques sur le mdicament
        /// </summary>
        private string vRemark = "";
        /// <summary>
        /// Nom du mdicament
        /// </summary>
        private string vNom;
        /// <summary>
        /// Manager de mdicament
        /// </summary>
        private DrugsTypeManager vDrugTypeManager = new DrugsTypeManager();
        /// <summary>
        /// Gestionnaire SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public DrugsManager() { }
        #endregion

        #region Mthodes
        /// <summary>
        /// Mise  jour du mdicament sur base du contenu de cet objet
        /// </summary>
        public void Update()
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_Medicament");
                SqlManager.Parameters.Add("[_IDType]", OleDbType.Integer).Value = this.DrugsType.ID;
                SqlManager.Parameters.Add("[_Rmq]", OleDbType.VarChar).Value = this.Remark;
                SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = this.Name;
                SqlManager.Parameters.Add("[__ID]", OleDbType.Integer).Value = this.ID;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DrugManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Ajoute un nouveau mdicament sur base des informations contenue dans l'objet
        /// </summary>
        public void Add()
        {
            try
            {
                #region Test si mdicament avec ce nom existe
                string SQLQuery = "SELECT COUNT(*) FROM Medicament WHERE Nom LIKE \"" + this.Name + "\";";
                int cnt = (int)SqlManager.ExecuteScalar(SQLQuery);
                if (cnt > 0)
                {
                    DialogResult dr = MessageBox.Show("Une mdicament portant ce nom existe dj."
                        + " Faut-il l'ajouter malgr tout?",
                        "Attention", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
                    if (dr == DialogResult.No) return;
                }
                #endregion
                SqlManager.PrepareProcedure("INSERT_Medicament");
                SqlManager.Parameters.Add("[_IDType]", OleDbType.Integer).Value = this.DrugsType.ID;
                SqlManager.Parameters.Add("[_Rmq]", OleDbType.VarChar).Value = this.Remark;
                SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = this.Name;

                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                this.ID = (int)LastID;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DrugManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Supprime le mdicament selon l'identifiant contenu dans l'objet
        /// </summary>
        /// <exception cref="OleDbException">En cas d'erreur de la base de donnes</exception>
        /// <exception cref="Exception">En cas d'erreur inatendue</exception>
        public void Delete()
        {
            try
            {
                SqlManager.PrepareProcedure("DELETE_Medicament");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.ID;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex.Message, "DrugManager.Delete()", LogType.Erreur);
                throw;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DrugManager.Delete()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Cre un DrugsManager selon l'identifiant contenu dans l'objet
        /// </summary>
        /// <returns></returns>
        public DrugsManager Create()
        {
            try
            {
                SqlManager.PrepareProcedure("SELECT_MedicamentOnID");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.ID;
                DataSet ds = new DataSet();
                int nbr = SqlManager.ExecuteSqlProcedure(ref ds);

                if (nbr > 0)
                {
                    DrugsManager hm = new DrugsManager();
                    hm.vEmpty = false;
                    hm.ID = (int)ds.Tables[0].Rows[0]["id_Medicament"];
                    hm.Name = (string)ds.Tables[0].Rows[0]["Nom"];
                    hm.Remark = (string)ds.Tables[0].Rows[0]["Rmq"];
                    return hm;
                }
                return new DrugsManager();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DrugManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Retourne un DrugsManager sur base d'un identifiant
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        public static DrugsManager Create(int _ID)
        {
            DrugsManager dm = new DrugsManager();
            dm.ID = _ID;
            return dm.Create();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'id du mdicament
        /// </summary>
        [Browsable(false)]
        public int ID { get { return vIDMedic; } set { vIDMedic = value; } }
        /// <summary>
        /// Retourne ou initialise les remarques sur le mdicament
        /// </summary>
        [Category("Mdicament")]
        [Description("Donne une remarque sur de mdicaments")]
        [DisplayName("Remarque")]
        [Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public string Remark { get { return vRemark; } set { vRemark = value; } }
        /// <summary>
        /// Retourne ou initialise le nom du mdicament
        /// </summary>
        [Category("Mdicament")]
        [Description("Donne le nom du mdicaments")]
        [DisplayName("Nom")]
        public string Name { get { return vNom; } set { vNom = value; } }
        /// <summary>
        /// Retourne ou initialise le type de mdicament        
        /// </summary>
        [Category("Mdicament")]
        [Description("Donne le type du mdicaments")]
        [DisplayName("Type de mdicament")]
        [Editor(typeof(UIDrugsTypeEditor), typeof(UITypeEditor))]
        public DrugsTypeManager DrugsType { get { return vDrugTypeManager; } set { vDrugTypeManager = value; } }
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }
        #endregion

        #region Overriden
        /// <summary>
        /// Donne le nom du mdicament
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Name;
        }
        /// <summary>
        /// En faisant un cast d'un DrugsManager vrs un string, on donne le nom du mdicament
        /// </summary>
        /// <param name="_Value"></param>
        /// <returns></returns>
        public static implicit operator string(DrugsManager _Value)
        { return _Value.ToString(); }
        #endregion
    }
}
