using System;
using System.Collections.Generic;
using System.Text;
using Utils.DataBase;
using System.ComponentModel;
using PhoenixDLL.Class.Interface;
using System.Data.OleDb;
using System.Data;
using Utils.Log;
using Components.TypeEditors.GUI;
using System.Drawing.Design;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Gestionnaire de type de mdicament
    /// </summary>
    public class DrugsTypeManager : IManager<DrugsTypeManager>
    {
        #region Variables
        /// <summary>
        /// Identifiant du mdicament
        /// </summary>
        private int vID = -1;
        /// <summary>
        /// Nom du type de mdicament
        /// </summary>
        private string vType = "";
        /// <summary>
        /// Remarques sur le type
        /// </summary>
        private string vRmq = "";
        /// <summary>
        /// Manager de SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'id du type de mdicament
        /// </summary>
        [Browsable(false)]
        public int ID { get { return vID; } set { vID = value; } }

        /// <summary>
        /// Retourne ou initialise le nom du type de mdicament
        /// </summary>
        [Category("Type de mdicament")]
        [Description("Donne le nom du type de mdicaments")]
        [DisplayName("Nom du type")]
        public string Type { get { return vType; } set { vType = value; } }

        /// <summary>
        /// Retourne ou initialise la remarque sur le type de mdicament
        /// </summary>
        [Category("Type de mdicament")]
        [Description("Donne la remarque sur le type de mdicaments")]
        [DisplayName("Remarques")]
        [Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public string Remark { get { return vRmq; } set { vRmq = value; } }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait une mise  jour du type de mdicament sur base des informations contenue dans l'objet
        /// </summary>
        public void Update()
        {
            SqlManager.PrepareProcedure("UPDATE_TypeMedic");
            SqlManager.Parameters.Add("[_Type]", OleDbType.VarChar).Value = this.vType;
            SqlManager.Parameters.Add("[Rmq]", OleDbType.Variant).Value = this.vRmq;
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.ID;
            SqlManager.ExecuteNonQueryProcedure();
        }
        /// <summary>
        /// Ajoute un type de mdicament sur base des informations contenue dans l'objet
        /// </summary>
        public void Add()
        {
            #region Check if type exists
            string SqlQuery = "SELECT COUNT(*) FROM Type_Medicament WHERE UCASE(Type) LIKE \"" + this.Type.ToUpper() + "\"";
            int cnt = (int)SqlManager.ExecuteScalar(SqlQuery);
            if (cnt > 0)
            {
                DialogResult dr = MessageBox.Show("Un type de mdicament avec le mme nom existe. "
                    + "Voulez-vous faire l'ajout malgr tout?", "Attention", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Exclamation);
                if (DialogResult.No == dr) return;
            }
            #endregion
            SqlManager.PrepareProcedure("INSERT_TypeMedic");
            SqlManager.Parameters.Add("[_Type]", OleDbType.VarChar).Value = this.vType;
            SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = this.Remark;

            object LastID = new object();
            SqlManager.ExecuteNonQueryProcedure(ref LastID);
            this.ID = (int)LastID;
        }
        /// <summary>
        /// Fait une suppression de type de mdicament sur base de l'identifiant contenu dans l'objet
        /// </summary>
        public void Delete()
        {
            SqlManager.PrepareProcedure("DELETE_TypeMedic");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vID;
            SqlManager.ExecuteNonQueryProcedure();
        }
        /// <summary>
        /// Rempli une instance sur base des informations contenues dans le objet
        /// </summary>
        /// <returns>Une instance de DrugsTypeManager</returns>
        public DrugsTypeManager Create()
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
                DataSet ds = new DataSet();
                int nbr = SqlManager.ExecuteSql("SELECT * FROM Type_Medicament WHERE id_Type =" + vID, ref ds);


                if (nbr > 0)
                {
                    DrugsTypeManager hm = new DrugsTypeManager();
                    hm.vEmpty = false;
                    hm.ID = (int)ds.Tables[0].Rows[0]["id_Type"];
                    hm.Type = (string)ds.Tables[0].Rows[0]["Type"];
                    hm.Remark = (string)ds.Tables[0].Rows[0]["Rmq"];
                    return hm;
                }
                return new DrugsTypeManager();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "DrugsTypeManager.Create()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Cre une instance de DrugsTypeManager sur base d'un identifiant
        /// </summary>
        /// <param name="_ID">IDentifiant  trouver</param>
        /// <returns>Une instance de DrugsTypeManager</returns>
        public static DrugsTypeManager Create(int _ID)
        {
            DrugsTypeManager dm = new DrugsTypeManager();
            dm.ID = _ID;
            return dm.Create();
        }
        #endregion

        #region Overriden
        /// <summary>
        /// Donne le nom de la rputation
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Type;
        }

        /// <summary>
        /// Casting d'un DrugsTypeManager en sting
        /// </summary>
        /// <param name="_drugs"></param>
        /// <returns></returns>
        public static implicit operator string(DrugsTypeManager _drugs)
        {
            return _drugs.ToString();
        }
        #endregion

        #region IManager<DrugsTypeManager> Members
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }

        #endregion
    }
}
