using System;
using System.Collections.Generic;
using System.Text;
using PhoenixDLL.Class.Interface;
using Utils.DataBase;
using System.Data;
using System.Data.OleDb;
using System.ComponentModel;
using PhoenixDLL.Editors;
using System.Drawing.Design;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Manager for the meetings
    /// </summary>
    public class MeetingManager : IManager<MeetingManager>
    {
        #region Variables
        /// <summary>
        /// Meeting identification
        /// </summary>
        private int vIDRdv = -1;
        /// <summary>
        /// Indicates whether the patient came to the rdv
        /// </summary>
        private bool vCame = false;
        /// <summary>
        /// Indicates whether the patient payed the rdv
        /// </summary>
        private bool vPayed = false;
        /// <summary>
        /// Begin date of the rdv
        /// </summary>
        private DateTime vStart = new DateTime();
        /// <summary>
        /// End date of the rdv
        /// </summary>
        private DateTime vEnd = new DateTime();
        /// <summary>
        /// Title of the rdv
        /// </summary>
        private string vTitle = "";
        /// <summary>
        /// Message of the rdv
        /// </summary>
        private string vMessage = "";
        /// <summary>
        /// Indicates whether the object is filled
        /// </summary>
        private bool vEmpty = true;
        /// <summary>
        /// For the SQL Queries
        /// </summary>
        protected OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// The patient of the rendez-vous
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        #endregion

        #region IManager<MeetingManager> Members
        /// <summary>
        /// Update the meeting with the IDMeeting of this object with the information contains in this object
        /// </summary>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public void Update()
        {
            SqlManager.PrepareProcedure("UPDATE_RendezVous");
            SqlManager.Parameters.Add("[_Debut]", OleDbType.Date).Value = this.Start;
            SqlManager.Parameters.Add("[_Fin]", OleDbType.Date).Value = this.End;
            SqlManager.Parameters.Add("[Message]", OleDbType.VarChar).Value = this.Message;
            SqlManager.Parameters.Add("[_Titre]", OleDbType.VarChar).Value = this.Title;
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.IDMeeting;
            SqlManager.Parameters.Add("[_Came]", OleDbType.Boolean).Value = this.Came;
            SqlManager.Parameters.Add("[_Payed]", OleDbType.Boolean).Value = this.Payed;
            SqlManager.ExecuteNonQueryProcedure();
        }
        /// <summary>
        /// Add the data in this object in the Database
        /// </summary>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public void Add()
        {
            SqlManager.PrepareProcedure("INSERT_RendezVous");
            SqlManager.Parameters.Add("[_IDPatient]", OleDbType.Integer).Value = this.Patient.IDPatient;
            SqlManager.Parameters.Add("[_Debut]", OleDbType.Date).Value = this.Start;
            SqlManager.Parameters.Add("[_Fin]", OleDbType.Date).Value = this.End;
            SqlManager.Parameters.Add("[_Titre]", OleDbType.VarChar).Value = this.Title;
            SqlManager.Parameters.Add("[Msg]", OleDbType.VarChar).Value = this.Message;
            SqlManager.ExecuteNonQueryProcedure();
        }
        /// <summary>
        /// Delete the meeting with the IDMeeting of this object
        /// </summary>
        /// <exception cref="OleDbException">If there is an error in the Database</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public void Delete()
        {
            string SQLQuery = "DELETE * FROM RendezVous WHERE id_RendezVous = " + this.IDMeeting;
            SqlManager.ExecuteNonQuery(SQLQuery);
        }
        /// <summary>
        /// Create a new instance of MeetingManager with the IDMeeting of this object
        /// </summary>
        /// <returns></returns>
        public MeetingManager Create()
        {
            string SqlQuery = "SELECT * FROM RendezVous WHERE id_rdv = " + vIDRdv;
            DataSet ds = new DataSet();
            int res = SqlManager.ExecuteSql(SqlQuery, ref ds);
            if (res < 1) throw new Exception("Aucun rendez-vous n'existe pour cet identifiant!");

            #region Fill the object
            MeetingManager mm = new MeetingManager();
            mm.IDMeeting = (int)ds.Tables[0].Rows[0]["id_Rdv"];
            mm.Patient = PatientManager.Create((int)ds.Tables[0].Rows[0]["id_Patient"]);
            mm.Came = (bool)ds.Tables[0].Rows[0]["came_to_rdv"];
            mm.Payed = (bool)ds.Tables[0].Rows[0]["a_paye"];
            mm.Start = (DateTime)ds.Tables[0].Rows[0]["Debut"];
            mm.End = (DateTime)ds.Tables[0].Rows[0]["Fin"];
            mm.Title = (string)ds.Tables[0].Rows[0]["Titre"];
            mm.Message = (string)ds.Tables[0].Rows[0]["Message"];
            #endregion

            return mm;
        }
        /// <summary>
        /// Create a filled instance of MeetingManager
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        public static MeetingManager Create(int _ID)
        {
            MeetingManager mm = new MeetingManager();
            mm.IDMeeting = _ID;
            return mm.Create();

        }
        /// <summary>
        /// Indicates of the object is filled
        /// </summary>
        [Browsable(false)]
        public bool Empty
        {
            get { return vEmpty; }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the meeting ID
        /// </summary>
        [Browsable(false)]
        public int IDMeeting { get { return vIDRdv; } set { vIDRdv = value; } }
        /// <summary>
        /// Gets or sets the ID of the patient
        /// </summary>
        [Editor(typeof(UIPatientEditor), typeof(UITypeEditor))]
        [Category("1- Patient")]
        [Description("Selectionne le patient du rendez-vous")]
        [DisplayName("Patient")]
        public PatientManager Patient { get { return vPatientManager; } set { vPatientManager = value; } }
        /// <summary>
        /// Gets or sets the begining date of the rendez-vous
        /// </summary>
        [Category("2- Dates")]
        [Description("Indique la date de dbut du rendez-vous")]
        [DisplayName("Dbut du rendez-vous")]
        public DateTime Start { get { return vStart; } set { vStart = value; } }
        /// <summary>
        /// Gets or sets the ending date of the rendez-vous
        /// </summary>
        [Category("2- Dates")]
        [Description("Indique la date de fin du rendez-vous")]
        [DisplayName("Fin du rendez-vous")]
        public DateTime End { get { return vEnd; } set { vEnd = value; } }
        /// <summary>
        /// Gets or sets the title of the rendez-vous
        /// </summary>
        [Category("3- Texte")]
        [Description("Indique le titre du rendez-vous")]
        [DisplayName("Titre du rendez-vous")]
        public string Title { get { return vTitle; } set { vTitle = value; } }
        /// <summary>
        /// Gets or sets the message of the rendez-vous
        /// </summary>
        [Category("3- Texte")]
        [Description("Indique le message du rendez-vous")]
        [DisplayName("Message du rendez-vous")]
        public string Message { get { return vMessage; } set { vMessage = value; } }
        /// <summary>
        /// Indicates whether the patient came to this rendez-vous
        /// </summary>
        [Category("4- Binaire")]
        [Description("Indique si le patient est venu  ce rendez-vous")]
        [DisplayName("Est venu  ce rendez-vous")]
        public bool Came { get { return vCame; } set { vCame = value; } }
        /// <summary>
        /// Indicates whether the patient payed this meeting
        /// </summary>
        [Category("4- Binaire")]
        [Description("Indique si le patient a pay ce rendez-vous")]
        [DisplayName("A pay ce rendez-vous")]
        public bool Payed { get { return vPayed; } set { vPayed = value; } }
        #endregion
    }
}
