using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using System.ComponentModel;
using Utils.Log;
using Utils.DataBase;
using PhoenixDLL.Class.Interface;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Classe de getion des honoraires    
    /// </summary>
    public class MutuelleManager : IManager<MutuelleManager>
    {
        #region Variables
        /// <summary>
        /// Identifiant d'honoraire
        /// </summary>
        private int vID = -1;
        /// <summary>
        /// Prix d'honoraire
        /// </summary>
        private int vPrix = 0;
        /// <summary>
        /// Nom de l'honoraire
        /// </summary>
        private string vType = "";
        /// <summary>
        /// Manager de code SQL
        /// </summary>
        private OleDbSql SqlManager;
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public MutuelleManager()
        {
            SqlManager = new OleDbSql(Options.GetInstance().Path);
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait une mise  jour d'un honoraire
        /// </summary>
        public void Update()
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_Honoraire");
                SqlManager.Parameters.Add("[_Prix]", OleDbType.Integer).Value = this.vPrix;
                SqlManager.Parameters.Add("[_Type]", OleDbType.VarChar).Value = this.vType;
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.vID;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "HonoraireManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Ajoute un honoraire.
        /// </summary>
        public void Add()
        {
            try
            {
                #region Check if Mutuelle exists
                string SqlQuery = "SELECT COUNT(*) FROM Honoraire WHERE UCASE(Type) LIKE \"" + this.Type.ToUpper() + "\"";
                int cnt = (int)SqlManager.ExecuteScalar(SqlQuery);
                if (cnt > 0)
                {
                    DialogResult dr = MessageBox.Show("Une mutuelle avec le mme nom existe. "
                        + "Voulez-vous faire l'ajout malgr tout?", "Attention", MessageBoxButtons.YesNo,
                        MessageBoxIcon.Exclamation);
                    if (DialogResult.No == dr) return;
                }
                #endregion
                SqlManager.PrepareProcedure("INSERT_Honoraire");
                SqlManager.Parameters.Add("[_Prix]", OleDbType.Integer).Value = this.vPrix;
                SqlManager.Parameters.Add("[_Type]", OleDbType.VarChar).Value = this.vType;

                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                this.ID = (int)LastID;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "HonoraireManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Supprime l'honoraire
        /// </summary>
        public void Delete()
        {
            try
            {
                SqlManager.PrepareProcedure("DELETE_Honoraire");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.vID;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "HonoraireManager.Delete()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Rempli le manager sur base d'un identifiant
        /// </summary>
        public MutuelleManager Create()
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
                DataSet ds = new DataSet();
                int nbr = SqlManager.ExecuteSql("SELECT * FROM Honoraire WHERE id_honoraire =" + vID, ref ds);


                if (nbr > 0)
                {
                    MutuelleManager hm = new MutuelleManager();
                    hm.vEmpty = false;
                    hm.ID = (int)ds.Tables[0].Rows[0]["id_honoraire"];
                    hm.Type = (string)ds.Tables[0].Rows[0]["Type"];
                    return hm;
                }
                return new MutuelleManager();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "HonoraireManager.Create()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Rempli le manager sur base d'un identifiant
        /// </summary>
        /// <param name="_ID">ID de l'honoraire  rechercher</param>
        public static MutuelleManager Create(int _ID)
        {
            MutuelleManager mu = new MutuelleManager();
            mu.ID = _ID;
            return mu.Create();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Identifiant de l'honoraire
        /// </summary>
        [Browsable(false)]
        public int ID { get { return vID; } set { vID = value; } }
        /// <summary>
        /// Type de la mutuelle
        /// </summary>
        [Category("Mutuelle")]
        [Description("Donne le nom de la mutuelle")]
        [DisplayName("Nom de la mutuelle")]
        public string Type { get { return vType; } set { vType = value; } }
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }
        #endregion

        #region Overriden
        /// <summary>
        /// Retourne le nom de l'honoraire
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.vType;
        }
        #endregion
    }
}
