using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using PhoenixDLL.Editors;
using System.Drawing.Design;
using System.Data;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// MeetingManager with less Update possibilities
    /// </summary>
    public class PartialMeetingManager : MeetingManager
    {
        #region Methods
        /// <summary>
        /// Create a new instance of MeetingManager with the IDMeeting of this object
        /// </summary>
        /// <returns></returns>
        public new PartialMeetingManager Create()
        {
            string SqlQuery = "SELECT * FROM RendezVous WHERE id_rdv = " + base.IDMeeting;
            DataSet ds = new DataSet();
            int res = SqlManager.ExecuteSql(SqlQuery, ref ds);
            if (res < 1) throw new Exception("Aucun rendez-vous n'existe pour cet identifiant!");

            #region Fill the object
            PartialMeetingManager mm = new PartialMeetingManager();
            mm.IDMeeting = (int)ds.Tables[0].Rows[0]["id_Rdv"];
            mm.Patient = PatientManager.Create((int)ds.Tables[0].Rows[0]["id_Patient"]);
            mm.Came = (bool)ds.Tables[0].Rows[0]["came_to_rdv"];
            mm.Payed = (bool)ds.Tables[0].Rows[0]["a_paye"];
            mm.Start = (DateTime)ds.Tables[0].Rows[0]["Debut"];
            mm.End = (DateTime)ds.Tables[0].Rows[0]["Fin"];
            mm.Title = (string)ds.Tables[0].Rows[0]["Titre"];
            mm.Message = (string)ds.Tables[0].Rows[0]["Message"];
            #endregion

            return mm;
        }
        /// <summary>
        /// Create a filled instance of MeetingManager
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        public static new PartialMeetingManager Create(int _ID)
        {
            PartialMeetingManager mm = new PartialMeetingManager();
            mm.IDMeeting = _ID;
            return mm.Create();
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the the patient
        /// </summary>        
        [Category("1- Patient")]
        [Description("Selectionne le patient du rendez-vous")]
        [DisplayName("Patient")]
        [ReadOnly(true)]
        public PatientManager vPatient { get { return base.Patient; } private set { base.Patient = value; } }
        /// <summary>
        /// Gets or sets the the patient
        /// </summary>  
        [Browsable(false)]
        public new PatientManager Patient { get { return this.vPatient; } set { this.vPatient = value; } }
        /// <summary>
        /// Gets or sets the begining date of the rendez-vous
        /// </summary>
        [Category("2- Dates")]
        [Description("Indique la date de dbut du rendez-vous")]
        [DisplayName("Dbut du rendez-vous")]
        [ReadOnly(true)]
        public new DateTime Start { get { return base.Start; } set { base.Start = value; } }
        /// <summary>
        /// Gets or sets the ending date of the rendez-vous
        /// </summary>
        [Category("2- Dates")]
        [Description("Indique la date de fin du rendez-vous")]
        [DisplayName("Fin du rendez-vous")]
        [ReadOnly(true)]
        public new DateTime End { get { return base.End; } set { base.End = value; } }
        /// <summary>
        /// Gets or sets the title of the rendez-vous
        /// </summary>
        [Category("3- Texte")]
        [Description("Indique le titre du rendez-vous")]
        [DisplayName("Titre du rendez-vous")]
        [ReadOnly(true)]
        public new string Title { get { return base.Title; } set { base.Title = value; } }
        /// <summary>
        /// Gets or sets the message of the rendez-vous
        /// </summary>
        [Category("3- Texte")]
        [Description("Indique le message du rendez-vous")]
        [DisplayName("Message du rendez-vous")]
        [ReadOnly(true)]
        public new string Message { get { return base.Message; } set { base.Message = value; } }
        #endregion
    }
}
