using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;
using PhoenixDLL.Editors;
using System.Data.OleDb;
using System.Data;
using PhoenixDLL.GUI;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.Class.Interface;
using Utils.Log;
using Utils.DataBase;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Gestionnaire de patient
    /// </summary>
    public class PatientManager : IManager<PatientManager>
    {
        #region Variables
        /// <summary>
        /// Identifiant du patient
        /// </summary>
        private int vIDPatient;
        /// <summary>
        /// Nom de famille du paient
        /// </summary>
        private string vFamilyName = "";
        /// <summary>
        /// Nom du patient
        /// </summary>
        private string vName = "";
        /// <summary>
        /// Lieu de naissance du patient
        /// </summary>
        private string vPlaceOfBirth = "";
        /// <summary>
        /// Date de naissance du patient
        /// </summary>
        private DateTime vDateOfBirth = new DateTime();
        /// <summary>
        /// Sexe du patient
        /// </summary>
        private Sex vSex = Sex.Femme;
        /// <summary>
        /// Profession du patient
        /// </summary>
        private string vProfession = "";
        /// <summary>
        /// Tl priv du patient
        /// </summary>
        private string vPrivatePhone = "";
        /// <summary>
        /// Adresse du patient
        /// </summary>
        private string vAddress = "";
        /// <summary>
        /// Ville du patient
        /// </summary>
        private string vCity = "";
        /// <summary>
        /// Code postal du patient
        /// </summary>
        private string vZipCode = "";
        /// <summary>
        /// Tl pro du patient
        /// </summary>
        private string vProPhone = "";
        /// <summary>
        /// Mail du patient
        /// </summary>
        private string vMail = "";
        /// <summary>
        /// Gestionnaire de mdecin du patient
        /// </summary>
        private DoctorManager vMedicManager = new DoctorManager();
        /// <summary>
        /// Gestionnaire d'homopathe du patient
        /// </summary>
        private DoctorManager vHomeoManager = new DoctorManager();
        /// <summary>
        /// Gestionnaire de rputation du patient
        /// </summary>
        /// 
        private ReputationManager vReputation = new ReputationManager();
        /// <summary>
        /// Gestionnaire d'honoraire du patient
        /// </summary>
        private MutuelleManager vMutuelle = new MutuelleManager();
        /// <summary>
        /// Motif de premire venue
        /// </summary>
        private string vMotif1 = "";
        /// <summary>
        /// Motif de premire venue
        /// </summary>
        private string vMotif2 = "";
        /// <summary>
        /// Date d'inscription
        /// </summary>
        private DateTime vInscription = DateTime.Today;
        /// <summary>
        /// Date de dernire mise  jour
        /// </summary>
        private DateTime vLastUpdate = new DateTime();
        /// <summary>
        /// Dette du patient
        /// </summary>
        private float vDette = 0;
        /// <summary>
        /// Honoraire que doit payer le patient
        /// </summary>
        private int vHonoraire = 0;
        /// <summary>
        /// Manager de code SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates if the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public PatientManager() { }

        /// <summary>
        /// Constructeur de copie
        /// </summary>
        /// <param name="_value">PatientManager  dupliquer</param>
        public PatientManager(PatientManager _value)
        {
            this.Manager = _value;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Identifiant du patient
        /// </summary>
        [Browsable(false)]
        public int IDPatient
        {
            get { return vIDPatient; }
            set { vIDPatient = value; }
        }
        /// <summary>
        /// Gets or sets the debt of the patient
        /// </summary>
        [Category("6- Donnes Sensibles")]
        [Description("Initialise la dette du paitent")]
        [DisplayName("6.1- Dette")]
        public float Debt
        {
            get { return vDette; }
            set
            {
                if (vDette != value)
                {
                    DialogResult dr = MessageBox.Show("Vous tes sur le point de changer la dette du patient."
                        + " Voulez-vous vraiment faire cela?", "Attention", MessageBoxButtons.YesNo,
                        MessageBoxIcon.Asterisk);
                    if (dr != DialogResult.Yes) return;
                    vDette = value;
                }
            }
        }
        /// <summary>
        /// Retourne ou initialise le nom de famille
        /// </summary>
        [Category("1- Donnes personnelles")]
        [Description("Initialise le nom de famille")]
        [DisplayName("1.1- Nom de famille")]
        public string FamillyName
        {
            get { return vFamilyName; }
            set { vFamilyName = value; }
        }
        /// <summary>
        /// Retourne ou initialise le prnom
        /// </summary>
        [Category("1- Donnes personnelles")]
        [Description("Initialise le prnom")]
        [DisplayName("1.2- Prnom")]
        public string Surname
        {
            get { return vName; }
            set { vName = value; }
        }
        /// <summary>
        /// Retourne ou initialise le lieu de naissance
        /// </summary>
        [Category("1- Donnes personnelles")]
        [Description("Initialise le lieu de naissance")]
        [DisplayName("1.3- Lieu de naissance")]
        public string PlaceOfBirth
        {
            get { return vPlaceOfBirth; }
            set { vPlaceOfBirth = value; }
        }
        /// <summary>
        /// Retourne ou initialise la date de naissance
        /// </summary>
        [Category("1- Donnes personnelles")]
        [Description("Initialise la date de naissance")]
        [DisplayName("1.4- Date de naissance")]
        public DateTime DateOfBirth
        {
            get { return vDateOfBirth; }
            set
            {
                if (value >= DateTime.Today)
                    throw new ArgumentException("La date de naissance ne peut pas tre dans le futur");
                vDateOfBirth = value;
            }
        }
        /// <summary>
        /// Retourne ou initialise le sexe
        /// </summary>
        [Category("1- Donnes personnelles")]
        [Description("Initialise le sexe")]
        [DisplayName("1.5- Sexe")]
        public Sex Sex
        {
            get { return vSex; }
            set { vSex = value; }
        }
        /// <summary>
        /// Retourne ou initialise la profession
        /// </summary>
        [Category("3- Donnes professionnelles")]
        [Description("Initialise la profession")]
        [DisplayName("3.1- Profession")]
        public string Profession
        {
            get { return vProfession; }
            set { vProfession = value; }
        }
        /// <summary>
        /// Retourne ou initialise le numro de tl professionel
        /// </summary>
        [Category("3- Donnes professionnelles")]
        [Description("Initialise le numro de tl professionel")]
        [DisplayName("3.2- Tl. professionel")]
        public string ProPhone
        {
            get { return vProPhone; }
            set { vProPhone = value; }
        }
        /// <summary>
        /// Retourne ou initialise le numro de tl priv
        /// </summary>
        [Category("2- Donnes prives")]
        [Description("Initialise le numro de tl priv")]
        [DisplayName("2.3- Tl. priv")]
        public string PrivatePhone
        {
            get { return vPrivatePhone; }
            set { vPrivatePhone = value; }
        }
        /// <summary>
        /// Retourne ou initialise l'adresse
        /// </summary>
        [Category("2- Donnes prives")]
        [Description("Iitialise l'adresse")]
        [DisplayName("2.1- Adresse")]
        public string Address
        {
            get { return vAddress; }
            set { vAddress = value; }
        }
        /// <summary>
        /// Retourne ou initialise la ville
        /// </summary>
        [Category("2- Donnes prives")]
        [Description("Initialise la ville")]
        [DisplayName("2.2- Ville")]
        public string City
        {
            get { return vCity; }
            set { vCity = value; }
        }
        /// <summary>
        /// Retourne ou initialise le code postal
        /// </summary>
        [Category("2- Donnes prives")]
        [Description("Iitialise le code postal")]
        [DisplayName("2.3- Code postal")]
        public string ZipCode
        {
            get { return vZipCode; }
            set { vZipCode = value; }
        }
        /// <summary>
        /// Retourne ou initialise le mail
        /// </summary>
        [Category("2- Donnes prives")]
        [Description("Iitialise le mail")]
        [DisplayName("2.4- Mail")]
        public string Mail
        {
            get { return vMail; }
            set { vMail = value; }
        }

        #region Mdecin
        /// <summary>
        /// Initialise le nom du mdecin
        /// </summary>
        [Category("4- Donnes externes")]
        [Description("Iitialise le mdecin")]
        [DisplayName("4.1- Mdecin traitant")]
        [Editor(typeof(UIDoctorEditor), typeof(UITypeEditor))]
        public DoctorManager MedecinTraitant
        {
            get { return this.vMedicManager; }
            set { this.vMedicManager = value; }
        }
        #endregion

        #region Homeopathe
        /// <summary>
        /// Initialise le nom de l'homopathe
        /// </summary>
        [Category("4- Donnes externes")]
        [Description("Initialise l'homopathe")]
        [DisplayName("4.2- Homopathe")]
        [Editor(typeof(UIDoctorEditor), typeof(UITypeEditor))]
        public DoctorManager Homeopathe
        {
            get { return this.vHomeoManager; }
            set { this.vHomeoManager = value; }
        }
        #endregion

        #region Rputation
        ///// <summary>
        ///// Retourne ou initialise l'identifiant de la rputation
        ///// </summary>
        //[Browsable(false)]
        //public int IdReput
        //{
        //    get { return vIdReput; }
        //    set { vIdReput = value; }
        //}

        /// <summary>
        /// Initialise le nom de la rputation
        /// </summary>       
        [Category("4- Donnes externes")]
        [Description("Initialise la rputation")]
        [DisplayName("4.3- Rputation")]
        [Editor(typeof(UIReputationEditor), typeof(UITypeEditor))]
        public ReputationManager Reputation
        {
            get { return vReputation; }
            set { vReputation = value; }
        }
        #endregion

        #region Honoraire
        /// <summary>
        /// Retourne ou initialise l'honoraire du patient
        /// </summary>
        [Category("4- Donnes externes")]
        [Description("Initialise la mutuelle du patient")]
        [DisplayName("4.4- Mutuelle")]
        [Editor(typeof(UIMutuelleEditor), typeof(UITypeEditor))]
        public MutuelleManager Mutuelle
        {
            get { return vMutuelle; }
            set { vMutuelle = value; }
        }
        #endregion

        /// <summary>
        /// Retourne ou initialise le motif 1
        /// </summary>
        [Category("5- Donnes diverses")]
        [Description("Initialise le motif 1")]
        [DisplayName("5.1- Motif 1")]
        public string Motif1
        {
            get { return vMotif1; }
            set { vMotif1 = value; }
        }
        /// <summary>
        /// Retourne ou initialise le motif 2
        /// </summary>
        [Category("5- Donnes diverses")]
        [Description("Initialise le motif 2")]
        [DisplayName("5.2- Motif 2")]
        public string Motif2
        {
            get { return vMotif2; }
            set { vMotif2 = value; }
        }
        /// <summary>
        /// Retourne ou initialise les honoraires que le patient doit payer
        /// </summary>
        [Category("5- Donnes diverses")]
        [Description("Initialise les honoraires pour ce patient")]
        [DisplayName("5.3- Honoraire")]
        public int Honoraire { get { return vHonoraire; } set { vHonoraire = value; } }
        /// <summary>
        /// Retourne ou initialise la date d'inscription
        /// </summary>
        [Category("5- Donnes diverses")]
        [Description("Initialise la date d'inscription")]
        [DisplayName("5.4- Date d'inscription")]
        public DateTime DateInscription { get { return vInscription; } set { vInscription = value; } }
        /// <summary>
        /// Retourne ou initialise la date de dernire mise  jour
        /// </summary>
        [Category("5- Donnes diverses")]
        [Description("Initialise la dernire mise  jour")]
        [DisplayName("5.5- Dernire mise  jour")]
        [Browsable(false)]
        public DateTime LastUpdate { get { return vLastUpdate; } set { vLastUpdate = value; } }
        /// <summary>
        /// Retourne ou initialise cet instance de PatientManager
        /// </summary>
        [Browsable(false)]
        public PatientManager Manager
        {
            get { return this; }
            set
            {
                this.Address = value.Address;
                this.City = value.City;
                this.DateInscription = value.DateInscription;
                this.DateOfBirth = value.DateOfBirth;
                this.FamillyName = value.FamillyName;
                this.Homeopathe = value.Homeopathe;
                this.Mutuelle = value.Mutuelle;
                this.IDPatient = value.IDPatient;
                this.Mail = value.Mail;
                this.MedecinTraitant = value.MedecinTraitant;
                this.Motif1 = value.Motif1;
                this.Motif2 = value.Motif2;
                this.Surname = value.Surname;
                this.Manager = value.Manager;
                this.PlaceOfBirth = value.PlaceOfBirth;
                this.PrivatePhone = value.PrivatePhone;
                this.Profession = value.Profession;
                this.ProPhone = value.ProPhone;
                this.Reputation = value.Reputation;
                this.Sex = value.Sex;
                this.ZipCode = value.ZipCode;
                this.LastUpdate = value.LastUpdate;
            }
        }
        /// <summary>
        /// Indicates if the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty
        {
            get { return vEmpty; }
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Donne le nom de la rputation
        /// </summary>
        /// <param name="_ID">Identifiant de la rputation</param>
        /// <returns>Le nom de la rputation</returns>
        public ReputationManager GetReputation(int _ID)
        {
            int nbr = 0;
            DataSet ds = new DataSet();
            try
            {
                SqlManager.PrepareProcedure("SELECT_Reputation");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
                nbr = SqlManager.ExecuteSqlProcedure(ref ds);
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Update()", LogType.Erreur);
                throw;
            }
            if (nbr > 0)
            {
                ReputationManager rep = new ReputationManager();
                rep.Label = (string)ds.Tables[0].Rows[0]["Label"];
                rep.ID = (int)ds.Tables[0].Rows[0]["ID_Reputation"];
                return rep;
            }
            else
                return new ReputationManager();
        }
        /// <summary>
        /// Donne le nom du mdecin sur son ID
        /// </summary>
        /// <param name="_ID">IDentifiant du mdecin</param>
        /// <returns>Nom du mdecin</returns>
        public DoctorManager GetDoctor(int _ID)
        {
            SqlManager.PrepareProcedure("SELECT_MedecinOnID");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
            DataSet ds = new DataSet();
            int nbr = SqlManager.ExecuteSqlProcedure(ref ds);

            if (nbr > 0)
            {
                DoctorManager doc = new DoctorManager();
                if (!ds.Tables[0].Rows[0]["Adresse"].Equals(DBNull.Value))
                    doc.Adresse = (string)ds.Tables[0].Rows[0]["Adresse"];

                if (!ds.Tables[0].Rows[0]["CodePostal"].Equals(DBNull.Value))
                    doc.CodePostal = (string)ds.Tables[0].Rows[0]["CodePostal"];

                if (!ds.Tables[0].Rows[0]["Fax"].Equals(DBNull.Value))
                    doc.Fax = (string)ds.Tables[0].Rows[0]["Fax"];

                if (!ds.Tables[0].Rows[0]["id_medecin"].Equals(DBNull.Value))
                    doc.ID = (int)ds.Tables[0].Rows[0]["id_medecin"];

                if (!ds.Tables[0].Rows[0]["Homeopathe"].Equals(DBNull.Value))
                    doc.IsHomeo = (bool)ds.Tables[0].Rows[0]["Homeopathe"];

                if (!ds.Tables[0].Rows[0]["MedecinTraitant"].Equals(DBNull.Value))
                    doc.IsMedecin = (bool)ds.Tables[0].Rows[0]["MedecinTraitant"];

                if (!ds.Tables[0].Rows[0]["Nom"].Equals(DBNull.Value))
                    doc.Nom = (string)ds.Tables[0].Rows[0]["Nom"];

                if (!ds.Tables[0].Rows[0]["Prenom"].Equals(DBNull.Value))
                    doc.Prenom = (string)ds.Tables[0].Rows[0]["Prenom"];

                if (!ds.Tables[0].Rows[0]["Telephone"].Equals(DBNull.Value))
                    doc.Telephone = (string)ds.Tables[0].Rows[0]["Telephone"];

                if (!ds.Tables[0].Rows[0]["Ville"].Equals(DBNull.Value))
                    doc.Ville = (string)ds.Tables[0].Rows[0]["Ville"];
                return doc;
            }
            else
                return new DoctorManager();
        }
        /// <summary>
        /// Donne les information d'un honoraire sur base de son identifiant
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        public MutuelleManager GetHonoraire(int _ID)
        {
            int nbr = 0;
            DataSet ds = new DataSet();
            try
            {
                SqlManager.PrepareProcedure("SELECT_Honoraire");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
                nbr = SqlManager.ExecuteSqlProcedure(ref ds);
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Update()", LogType.Erreur);
                throw;
            }

            if (nbr > 0)
            {
                MutuelleManager rep = new MutuelleManager();
                rep.Type = (string)ds.Tables[0].Rows[0]["Type"];
                rep.ID = (int)ds.Tables[0].Rows[0]["id_honoraire"];
                return rep;
            }
            else
                return new MutuelleManager();
        }
        /// <summary>
        /// Fait une mise  jour du patient pour changer de docteur
        /// </summary>
        /// <param name="_IDDoctor">Identifiant du mdecin traitant</param>
        public void UpdateDoctor(int _IDDoctor)
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_MedecinOfPatient");
                SqlManager.Parameters.Add("[_Med]", OleDbType.Integer).Value = _IDDoctor;
                SqlManager.Parameters.Add("[_Pat]", OleDbType.Integer).Value = this.IDPatient;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Fait une mise  jour du patient pour changer d'homopathe
        /// </summary>
        /// <param name="_IDDoctor">Identifiant de l'homopathe</param>
        public void UpdateHomeopath(int _IDDoctor)
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_HomeoOfPatient");
                SqlManager.Parameters.Add("[_Med]", OleDbType.Integer).Value = _IDDoctor;
                SqlManager.Parameters.Add("[_Pat]", OleDbType.Integer).Value = this.IDPatient;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Mise  jour d'un patient selon les informations contenue dans l'objet
        /// </summary>
        /// <exception cref="OleDbException"></exception>
        /// <exception cref="InvalidOperationException"></exception>
        public void Update()
        {
            if (this.Reputation == null) this.Reputation = new ReputationManager();
            if (this.Mutuelle == null) this.Mutuelle = new MutuelleManager();
            if (this.MedecinTraitant == null) this.MedecinTraitant = new DoctorManager();
            if (this.Homeopathe == null) this.Homeopathe = new DoctorManager();
            SqlManager.PrepareProcedure("UPDATE_Patient");

            SqlManager.Parameters.Add("[_Id]", OleDbType.Integer).Value = this.IDPatient;
            SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar, 50).Value = this.FamillyName.ToUpper();
            SqlManager.Parameters.Add("[_Prenom]", OleDbType.VarChar, 50).Value = this.Surname;
            SqlManager.Parameters.Add("[_LieuNaissance]", OleDbType.VarChar, 50).Value = this.PlaceOfBirth;
            SqlManager.Parameters.Add("[_DateNaissance]", OleDbType.Date).Value = this.DateOfBirth;
            SqlManager.Parameters.Add("[_Adresse]", OleDbType.VarChar, 100).Value = this.Address;
            SqlManager.Parameters.Add("[_CodePostal]", OleDbType.VarChar, 50).Value = this.ZipCode;
            SqlManager.Parameters.Add("[_Profession]", OleDbType.VarChar, 50).Value = this.Profession;
            SqlManager.Parameters.Add("[_Ville]", OleDbType.VarChar, 50).Value = this.City;
            SqlManager.Parameters.Add("[_TelephonePrive]", OleDbType.VarChar, 20).Value = this.PrivatePhone;
            SqlManager.Parameters.Add("[_TelephonePro]", OleDbType.VarChar, 20).Value = this.ProPhone;

            string _Sex = "M";
            if (this.Sex == Sex.Femme) _Sex = "F";
            SqlManager.Parameters.Add("[_Sexe]", OleDbType.Variant, 1).Value = _Sex;

            SqlManager.Parameters.Add("[_Mail]", OleDbType.VarChar, 50).Value = this.Mail;

            if (this.Reputation.ID == -1 || this.Reputation.ID == 0 || this.Reputation.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idr]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idr]", OleDbType.Integer).Value = this.vReputation.ID;//this.IdReput;

            if (this.Homeopathe.ID == -1 || this.Homeopathe.ID == 0 || this.Homeopathe.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idh]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idh]", OleDbType.Integer).Value = this.Homeopathe.ID;

            if (this.MedecinTraitant.ID == -1 || this.MedecinTraitant.ID == 0 || this.MedecinTraitant.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idm]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idm]", OleDbType.Integer).Value = this.MedecinTraitant.ID;

            SqlManager.Parameters.Add("[_Motif1]", OleDbType.Variant).Value = this.Motif1;
            SqlManager.Parameters.Add("[_Motif2]", OleDbType.Variant).Value = this.Motif2;
            SqlManager.Parameters.Add("[INSC]", OleDbType.Date).Value = this.DateInscription;

            if (this.vMutuelle.ID == -1 || this.vMutuelle.ID == 0)
                SqlManager.Parameters.Add("[_idHO]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_idHO]", OleDbType.Integer).Value = this.vMutuelle.ID;

            SqlManager.Parameters.Add("[_MAJ]", OleDbType.Date).Value = this.LastUpdate;
            SqlManager.Parameters.Add("[_Honoraire]", OleDbType.Integer).Value = this.Honoraire;
            SqlManager.Parameters.Add("[_Dette]", OleDbType.Integer).Value = this.Debt;


            try
            {
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Update()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Met  jour la dette du patient
        /// </summary>
        /// <param name="_Sum">Total a ajouter pour la dette</param>
        public void UpdateDebt(int _Sum)
        {
            if (this.IDPatient != 0 && this.IDPatient != -1)
            {
                vDette = _Sum;
                SqlManager.ExecuteNonQuery("UPDATE Patient SET dette_Patient = " + _Sum + " WHERE id_Patient = " + this.IDPatient);
            }
            else
            {
                throw new Exception("Erreur identifiant du patient incorrect");
            }

        }
        /// <summary>
        /// Ajoute un patient selon les informations contenue dans l'objet
        /// </summary>
        /// <exception cref="OleDbException"></exception>
        /// <exception cref="InvalidOperationException"></exception>
        public void Add()
        {
            #region Check if patient exists
            string SqlQuery = "SELECT COUNT(*) FROM Patient WHERE UCASE(Nom) LIKE \"" + this.FamillyName.ToUpper()
                + "\" AND UCASE(Prenom) LIKE \"" + this.Surname.ToUpper() + "\"";
            int cnt = (int)SqlManager.ExecuteScalar(SqlQuery);
            if (cnt > 0)
            {
                DialogResult dr = MessageBox.Show("Un patient avec le mme nom et prnom existe. "
                    + "Voulez-vous faire l'ajout malgr tout?", "Attention", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Exclamation);
                if (DialogResult.No == dr) return;
            }
            #endregion

            SqlManager.PrepareProcedure("INSERT_Patient");

            //SqlManager.Parameters.Add("[_Id]", OleDbType.Integer).Value = this.IDPatient;
            SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar, 50).Value = this.FamillyName.ToUpper();
            SqlManager.Parameters.Add("[_Prenom]", OleDbType.VarChar, 50).Value = this.Surname;
            SqlManager.Parameters.Add("[_LieuNaissance]", OleDbType.VarChar, 50).Value = this.PlaceOfBirth;
            SqlManager.Parameters.Add("[_DateNaissance]", OleDbType.Date).Value = this.DateOfBirth;
            SqlManager.Parameters.Add("[_Adresse]", OleDbType.VarChar, 100).Value = this.Address;
            SqlManager.Parameters.Add("[_CodePostal]", OleDbType.VarChar, 50).Value = this.ZipCode;
            SqlManager.Parameters.Add("[_Profession]", OleDbType.VarChar, 50).Value = this.Profession;
            SqlManager.Parameters.Add("[_Ville]", OleDbType.VarChar, 50).Value = this.City;
            SqlManager.Parameters.Add("[_TelephonePrive]", OleDbType.VarChar, 20).Value = this.PrivatePhone;
            SqlManager.Parameters.Add("[_TelephonePro]", OleDbType.VarChar, 20).Value = this.ProPhone;

            string _Sex = "M";
            if (this.Sex == Sex.Femme) _Sex = "F";
            SqlManager.Parameters.Add("[_Sexe]", OleDbType.Variant, 1).Value = _Sex;

            SqlManager.Parameters.Add("[_Mail]", OleDbType.VarChar, 50).Value = this.Mail;

            if (this.Reputation.ID == -1 || this.Reputation.ID == 0 || this.Homeopathe.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idr]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idr]", OleDbType.Integer).Value = this.vReputation.ID;//this.IdReput;

            if (this.Homeopathe.ID == -1 || this.Homeopathe.ID == 0 || this.Homeopathe.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idh]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idh]", OleDbType.Integer).Value = this.Homeopathe.ID;

            if (this.MedecinTraitant.ID == -1 || this.MedecinTraitant.ID == 0 || this.Homeopathe.ID == Value.NULL)
                SqlManager.Parameters.Add("[_Idm]", OleDbType.Integer).Value = DBNull.Value;
            else
                SqlManager.Parameters.Add("[_Idm]", OleDbType.Integer).Value = this.MedecinTraitant.ID;

            SqlManager.Parameters.Add("[_Motif1]", OleDbType.Variant).Value = this.Motif1;
            SqlManager.Parameters.Add("[_Motif2]", OleDbType.Variant).Value = this.Motif2;
            SqlManager.Parameters.Add("[INSC]", OleDbType.Date).Value = this.DateInscription;
            SqlManager.Parameters.Add("[_idho]", OleDbType.Integer).Value = this.Mutuelle.ID;

            SqlManager.Parameters.Add("[_MAJ]", OleDbType.Date).Value = DateTime.Today;
            SqlManager.Parameters.Add("[_Honoraire]", OleDbType.Integer).Value = this.Honoraire;
            SqlManager.Parameters.Add("[_Dette]", OleDbType.Integer).Value = this.Debt;

            try
            {
                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                this.IDPatient = (int)LastID;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex.Message, "PatientManager.Add()", LogType.Erreur);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Supprime un patient selon les informations contenue dans l'objet
        /// </summary>
        /// <exception cref="OleDbException"></exception>
        /// <exception cref="InvalidOperationException"></exception>
        public void Delete()
        {
            SqlManager.PrepareProcedure("DELETE_Patient");
            SqlManager.Parameters.Add("[_Id]", OleDbType.Integer).Value = this.IDPatient;

            try
            {
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException ex)
            {
                Log.GetInstance().Write(ex.Message, "PatientManager.Delete()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Cre une instance de PatientManager sur base de l'identifiant d'un patient
        /// </summary>
        /// <returns>Une instance de PatientManager</returns>
        public PatientManager Create()
        {
            DataSet ds = new DataSet();
            OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
            int i = SqlManager.ExecuteSql("SELECT * FROM Patient WHERE id_Patient = " + vIDPatient + ";", ref ds);

            if (i > 0)
            {
                PatientManager pm = new PatientManager();
                pm.vEmpty = false;
                pm.IDPatient = (int)ds.Tables[0].Rows[0]["id_patient"];

                if (!ds.Tables[0].Rows[0]["DateNaissance"].Equals(DBNull.Value))
                    pm.DateOfBirth = (DateTime)ds.Tables[0].Rows[0]["DateNaissance"];

                if (!ds.Tables[0].Rows[0]["Nom"].Equals(DBNull.Value))
                    pm.FamillyName = (string)ds.Tables[0].Rows[0]["Nom"];

                if (!ds.Tables[0].Rows[0]["Prenom"].Equals(DBNull.Value))
                    pm.Surname = (string)ds.Tables[0].Rows[0]["Prenom"];

                if (!ds.Tables[0].Rows[0]["Lieu_Naissance"].Equals(DBNull.Value))
                    pm.PlaceOfBirth = (string)ds.Tables[0].Rows[0]["Lieu_Naissance"];

                if (!ds.Tables[0].Rows[0]["Adresse"].Equals(DBNull.Value))
                    pm.Address = (string)ds.Tables[0].Rows[0]["Adresse"];

                if (!ds.Tables[0].Rows[0]["Code_Postal"].Equals(DBNull.Value))
                    pm.ZipCode = (string)ds.Tables[0].Rows[0]["Code_Postal"];

                if (!ds.Tables[0].Rows[0]["Profession"].Equals(DBNull.Value))
                    pm.Profession = (string)ds.Tables[0].Rows[0]["Profession"];

                if (!ds.Tables[0].Rows[0]["Ville"].Equals(DBNull.Value))
                    pm.City = (string)ds.Tables[0].Rows[0]["Ville"];

                if (!ds.Tables[0].Rows[0]["Telephone_Professionel"].Equals(DBNull.Value))
                    pm.ProPhone = (string)ds.Tables[0].Rows[0]["Telephone_Professionel"];

                if (!ds.Tables[0].Rows[0]["Telephone_Prive"].Equals(DBNull.Value))
                    pm.PrivatePhone = (string)ds.Tables[0].Rows[0]["Telephone_Prive"];

                Sex sx = Sex.Femme;
                if (!ds.Tables[0].Rows[0]["Sexe"].Equals(DBNull.Value) &&
                    ((string)ds.Tables[0].Rows[0]["Sexe"]).ToUpper() == "M")
                    sx = Sex.Homme;

                pm.Sex = sx;

                if (!ds.Tables[0].Rows[0]["Mail"].Equals(DBNull.Value))
                    pm.Mail = (string)ds.Tables[0].Rows[0]["Mail"];

                if (!ds.Tables[0].Rows[0]["id_Homeopathe"].Equals(DBNull.Value))
                    pm.Homeopathe = DoctorManager.Create((int)ds.Tables[0].Rows[0]["id_Homeopathe"]);
                else
                    pm.Homeopathe = null;

                if (!ds.Tables[0].Rows[0]["id_Medecin"].Equals(DBNull.Value))
                    pm.MedecinTraitant = DoctorManager.Create((int)ds.Tables[0].Rows[0]["id_Medecin"]);
                else
                    pm.MedecinTraitant = null;

                if (!ds.Tables[0].Rows[0]["id_Reputation"].Equals(DBNull.Value))
                    pm.Reputation = ReputationManager.Create((int)ds.Tables[0].Rows[0]["id_Reputation"]);
                else
                    pm.Reputation = null;

                if (!ds.Tables[0].Rows[0]["id_honoraire"].Equals(DBNull.Value))
                    pm.Mutuelle = MutuelleManager.Create((int)ds.Tables[0].Rows[0]["id_honoraire"]);
                else
                    pm.Mutuelle = null;

                pm.DateInscription = (DateTime)ds.Tables[0].Rows[0]["Prem_Consult"];

                if (!ds.Tables[0].Rows[0]["Motif_1"].Equals(DBNull.Value))
                    pm.Motif1 = (string)ds.Tables[0].Rows[0]["Motif_1"];

                if (!ds.Tables[0].Rows[0]["Motif_2"].Equals(DBNull.Value))
                    pm.Motif2 = (string)ds.Tables[0].Rows[0]["Motif_2"];

                if (!ds.Tables[0].Rows[0]["Derniere_MAJ"].Equals(DBNull.Value))
                    pm.LastUpdate = (DateTime)ds.Tables[0].Rows[0]["Derniere_MAJ"];

                object temp = ds.Tables[0].Rows[0]["Dette_Patient"];
                pm.vDette = (int)temp;

                temp = ds.Tables[0].Rows[0]["Honoraire"];
                pm.Honoraire = (int)temp;

                return pm;
            }
            else
                return new PatientManager();
        }
        /// <summary>
        /// Cre une instance de PatientManager sur base de l'identifiant d'un patient
        /// </summary>
        /// <param name="_ID">Identifiant du patient</param>
        /// <returns>Une instance de PatientManager</returns>
        public static PatientManager Create(int _ID)
        {
            PatientManager pm = new PatientManager();
            pm.IDPatient = _ID;
            return pm.Create();
        }
        /// <summary>
        /// Substract the _Sum from the patient's debt
        /// </summary>
        /// <param name="_Sum">_Sum to substract</param>
        public void SubDebt(int _Sum)
        {
            vDette -= _Sum;
            if (vDette < 0)
                vDette = 0;
        }
        /// <summary>
        /// Set the Debt to a new value
        /// </summary>
        /// <param name="_Debt"></param>
        public void SetDebt(int _Debt) { vDette = _Debt; }
        #endregion

        #region Overriden
        /// <summary>
        /// Donne le nom du patient
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.vName + " " + this.vFamilyName.ToUpper();
        }
        #endregion
    }

    /// <summary>
    /// Donne le sexe du patient
    /// </summary>
    public enum Sex
    {
        /// <summary>
        /// Homme
        /// </summary>
        Homme,
        /// <summary>
        /// Femme
        /// </summary>
        Femme,
    }
}
