using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using PhoenixDLL.Class.Interface;
using Utils.DataBase;
using System.Data.OleDb;
using Utils.Log;
using System.Data;
using PhoenixDLL.Editors;
using System.Drawing.Design;
using Components.TypeEditors;
using Components.TypeEditors.GUI;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Gestionnaire de prescriptions
    /// </summary>
    public class PrescriptionManager : IManager<PrescriptionManager>
    {
        #region Variables
        /// <summary>
        /// Identifiant de la prescription
        /// </summary>
        private int vIDPrescription = -1;
        /// <summary>
        /// IDentifiant du patient
        /// </summary>
        private int vIDPatient = -1;
        /// <summary>
        /// Informations sur le patient
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// IDentifiant du mdicament
        /// </summary>
        private int vIDMedic = -1;
        /// <summary>
        /// Informations sur le Manager
        /// </summary>
        private DrugsManager vDrugsManager = new DrugsManager();
        /// <summary>
        /// Remarques sur la prescription
        /// </summary>
        private string vRmq = "";
        /// <summary>
        /// Date de la prescription
        /// </summary>
        private DateTime vDate = new DateTime();
        /// <summary>
        /// Gestionnaire de SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'identifiant du patient
        /// </summary>
        [Browsable(false)]
        public int IDPatient { get { return vIDPatient; } set { vIDPatient = value; } }
        /// <summary>
        /// Retourne ou initialise l'identifiant du mdicament
        /// </summary>
        [Browsable(false)]
        public int IDDrug { get { return vIDMedic; } set { vIDMedic = value; } }
        /// <summary>
        /// Retourne ou initialise l'identifiant de la prescription
        /// </summary>
        [Browsable(false)]
        public int IDPrescription { get { return vIDPrescription; } set { vIDPrescription = value; } }
        /// <summary>
        /// Retourne ou initialise le patient de la prescription
        /// </summary>
        [Category("Prescription")]
        [Description("Donne le nom du patient")]
        [DisplayName("Patient")]
        public PatientManager PatientManager
        {
            get { return vPatientManager; }
            set
            {
                vPatientManager = value;
                vIDPatient = vPatientManager.IDPatient;
            }
        }
        /// <summary>
        /// Retourne ou initialise le mdicament de la prescription
        /// </summary>
        [Category("Prescription")]
        [Description("Donne le nom du mdicament")]
        [DisplayName("Mdicament")]
        [Editor(typeof(UIDrugsEditor), typeof(UITypeEditor))]
        public DrugsManager DrugsManager
        {
            get { return vDrugsManager; }
            set
            {
                vDrugsManager = value;
                vIDMedic = vDrugsManager.ID;
            }
        }
        /// <summary>
        /// Retourne ou initialise la remarque pour la prescription
        /// </summary>
        [Category("Prescription")]
        [Description("Donne une remarque sur la prescription de ce mdicament")]
        [DisplayName("Remarque")]
        [Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public string Remark { get { return vRmq; } set { vRmq = value; } }
        /// <summary>
        /// Retourne ou initialise la Date de la prescription
        /// </summary>
        [Category("Prescription")]
        [Description("Donne la date de la prescription")]
        [DisplayName("Date de la prescription")]
        public DateTime Date { get { return vDate; } set { vDate = value; } }
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait une mise  jour de la prescription
        /// </summary>
        public void Update()
        {
            try
            {
                SqlManager.PrepareProcedure("UPDATE_Prescription");
                SqlManager.Parameters.Add("_IDP", OleDbType.Integer).Value = this.IDPatient;
                SqlManager.Parameters.Add("_IDM", OleDbType.Integer).Value = this.IDDrug;
                SqlManager.Parameters.Add("_RMQ", OleDbType.VarChar).Value = this.Remark;
                SqlManager.Parameters.Add("_Date", OleDbType.Date).Value = this.Date;
                SqlManager.Parameters.Add("_IDPR", OleDbType.Integer).Value = this.IDPrescription;
                SqlManager.ExecuteScalarProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "PrescriptionManager.Add()", LogType.Erreur);
            }

        }
        /// <summary>
        /// Fait un ajout de la prescription
        /// </summary>
        public void Add()
        {
            try
            {
                SqlManager.PrepareProcedure("INSERT_Prescription");
                SqlManager.Parameters.Add("_IDP", OleDbType.Integer).Value = this.IDPatient;
                SqlManager.Parameters.Add("_IDM", OleDbType.Integer).Value = this.IDDrug;
                SqlManager.Parameters.Add("_RMQ", OleDbType.VarChar).Value = this.Remark;
                SqlManager.Parameters.Add("_DP", OleDbType.Date).Value = this.Date;
                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                //this.IDPatient = (int)LastID;
                this.IDPrescription = (int)LastID;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "PrescriptionManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Fait une suppression d'une prescription sur base de l'identifiant compris dans cet objet
        /// </summary>
        public void Delete()
        {
            try
            {
                SqlManager.PrepareProcedure("DELETE_Prescription");
                SqlManager.Parameters.Add("_ID", OleDbType.Integer).Value = this.IDPrescription;
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "PrescriptionManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Cre une instance d'un PrescriptionManager sur base d'un identifiant
        /// </summary>
        /// <returns>Une instance d'un prescription manager</returns>
        public PrescriptionManager Create()
        {
            PrescriptionManager pm = new PrescriptionManager();
            DataSet ds = new DataSet();
            int nbr = SqlManager.ExecuteSql("SELECT * FROM Prescription WHERE  id_prescription = "
                + this.IDPrescription, ref ds);
            if (nbr < 1) return new PrescriptionManager();
            pm.vEmpty = false;
            pm.Date = (DateTime)ds.Tables[0].Rows[0]["Date_P"];
            pm.IDPrescription = (int)ds.Tables[0].Rows[0]["id_Prescription"];
            pm.IDDrug = (int)ds.Tables[0].Rows[0]["Id_Medicament"];
            pm.IDPatient = (int)ds.Tables[0].Rows[0]["Id_patient"];
            pm.Remark = (string)ds.Tables[0].Rows[0]["Rmq"];
            pm.PatientManager = PatientManager.Create(this.IDPatient);
            pm.DrugsManager = DrugsManager.Create(this.IDDrug);
            return pm;
        }
        /// <summary>
        /// Cre une instance d'une Prescription sur base d'un identifiant
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        public static PrescriptionManager Create(int _ID)
        {
            try
            {
                PrescriptionManager pm = new PrescriptionManager();
                pm.IDPrescription = _ID;
                return pm.Create();
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "PrescriptionManager.Create()", LogType.Erreur);
                throw;
            }
        }
        #endregion

        #region Overriden
        /// <summary>
        /// Retourne une prescription sour forme de texte
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return "Prescription de " + this.DrugsManager.Name
                + " pour " + this.PatientManager.ToString()
                + " le " + this.Date.ToShortDateString();
        }

        /// <summary>
        /// Retourne une prescription sour forme de texte
        /// </summary>
        /// <param name="_value"></param>
        /// <returns></returns>
        public static implicit operator string(PrescriptionManager _value)
        {
            return _value.ToString();
        }
        #endregion
    }
}
