using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using Utils.Log;
using System.ComponentModel;
using Utils.DataBase;
using PhoenixDLL.Class.Interface;
using System.Windows.Forms;

namespace PhoenixDLL.Class.Managers
{
    /// <summary>
    /// Contient les informations sur une rputation
    /// </summary>
    public class ReputationManager : IManager<ReputationManager>
    {
        #region variables
        /// <summary>
        /// Identifiant de la rputation
        /// </summary>
        private int vID = 0;
        /// <summary>
        /// Explication de la rputation
        /// </summary>
        private string vLabel = "";
        /// <summary>
        /// Manager SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        private bool vEmpty = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public ReputationManager() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_ID">Identifiant de la rputation</param>
        /// <param name="_Label">Label de la rputation</param>
        public ReputationManager(int _ID, string _Label)
        {
            vID = _ID;
            vLabel = _Label;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Converti l'objet Rputation en un string
        /// </summary>
        /// <returns>Le nom de la rputation</returns>
        public override string ToString()
        {
            return Label;
        }
        /// <summary>
        /// Converti l'objet Rputation en un string
        /// </summary>
        /// <param name="_Reputation">L'objet rputation  convertir</param>
        /// <returns>Le nom de la rputation</returns>
        public static explicit operator string(ReputationManager _Reputation)
        {
            return _Reputation.ToString();
        }
        /// <summary>
        /// Ajout de rputation
        /// </summary>
        public virtual void Add()
        {
            #region Check if the Reputations exists
            string SqlQuery = "SELECT COUNT(*) FROM Reputation WHERE UCASE(Label) LIKE \"" + this.Label.ToUpper() + "\"";
            int cnt = (int)SqlManager.ExecuteScalar(SqlQuery);
            if (cnt > 0)
            {
                DialogResult dr = MessageBox.Show("Une rputation avec le mme nom existe. "
                    + "Voulez-vous faire l'ajout malgr tout?", "Attention", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Exclamation);
                if (DialogResult.No == dr) return;
            }
            #endregion
            SqlManager.PrepareProcedure("INSERT_Reputation");
            SqlManager.Parameters.Add("[_Txt]", OleDbType.Variant).Value = this.vLabel;

            try
            {
                object LastID = new object();
                SqlManager.ExecuteNonQueryProcedure(ref LastID);
                this.ID = (int)LastID;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message,
                    "ReputationSManager.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Mise  jour d'une rputation
        /// </summary>
        public virtual void Update()
        {
            SqlManager.PrepareProcedure("Update_Reputation");
            SqlManager.Parameters.Add("[_Txt]", OleDbType.Variant).Value = this.Label;
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.ID;

            try
            {
                SqlManager.ExecuteNonQueryProcedure();
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message,
                    "ReputationManager.Update", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Fait une suppression de la rputation courrante
        /// </summary>
        public void Delete()
        {
            try
            {
                SqlManager.ExecuteNonQuery("DELETE * FROM Reputation WHERE id_Reputation = " + ID);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Cre une instance de rputation sur base de son identifiant
        /// </summary>        
        /// <exception cref="SystemException"></exception>
        /// <exception cref="Exception"></exception>
        public ReputationManager Create()
        {
            try
            {
                DataSet ds = new DataSet();
                OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
                int i = SqlManager.ExecuteSql("SELECT * FROM Patient WHERE id_Patient = " + vID + ";", ref ds);
                if (i > 0)
                {
                    ReputationManager rm = new ReputationManager();
                    rm.vEmpty = false;
                    rm.ID = (int)ds.Tables[0].Rows[0]["id_reputation"];
                    rm.Label = (string)ds.Tables[0].Rows[0]["Label"];
                    return rm;
                }
                return new ReputationManager();
            }
            catch (SystemException) { throw; }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Cre une instance de rputation sur base de son identifiant
        /// </summary>
        /// <param name="_ID">Identifiant de la rputation</param>
        /// <exception cref="SystemException"></exception>
        /// <exception cref="Exception"></exception>
        public static ReputationManager Create(int _ID)
        {
            ReputationManager rm = new ReputationManager();
            rm.ID = _ID;
            return rm.Create();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'identifiant de la rputation
        /// </summary>
        [Browsable(false)]
        public int ID { get { return vID; } set { vID = value; } }
        /// <summary>
        /// Retourne ou initialise l'explication de la rputation
        /// </summary>
        [Category("Rputation")]
        [Description("Nom de la rputation")]
        [DisplayName("Nom de la rputation")]
        public string Label { get { return vLabel; } set { vLabel = value; } }
        /// <summary>
        /// Indicates whether the object is empty
        /// </summary>
        [Browsable(false)]
        public bool Empty { get { return vEmpty; } }
        #endregion
    }
}
