using System;
using System.Collections.Generic;
using System.Text;
using Components.Agenda.Management;
using System.Data.OleDb;
using System.Data;
using System.Configuration;
using Utils.Misc;
using System.IO;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using Utils.Log;
using Components.Agenda.Management.StructsEnums;

namespace PhoenixDLL.Class
{
    /// <summary>
    /// Surcharge du RootMeetingSet
    /// </summary>
    public class MeetingSet : RootMeetingSet
    {
        #region Variables
        /// <summary>
        /// String de connexion
        /// </summary>
        protected string conStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
        /// <summary>
        /// Manager SQL
        /// </summary>
        protected OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        #endregion

        #region Implmentation des virtual et abstract
        /// <summary>
        /// Ajout d'un rendez-vous et sauvegard de celui-ci dans la BD
        /// </summary>
        /// <param name="_Value">Les informations sur le rendez-vous</param>
        /// <exception cref="Exception">En cas de rendez-vous non ajout</exception>        
        public override void Add(MeetingRow _Value)
        {
            try
            {
                SqlManager.PrepareProcedure("INSERT_Rendezvous");
                SqlManager.Parameters.Add("_IDPatient", OleDbType.Integer).Value = _Value.IDPatient;
                SqlManager.Parameters.Add("_Debut", OleDbType.Date).Value = _Value.StartDate;
                SqlManager.Parameters.Add("_Fin", OleDbType.Date).Value = _Value.EndDate;
                SqlManager.Parameters.Add("_Titre", OleDbType.Variant).Value = _Value.Title;
                SqlManager.Parameters.Add("_Msg", OleDbType.Variant).Value = _Value.Message;
                SqlManager.Parameters.Add("_ID", OleDbType.Integer).Value = _Value.IDMeeting;
                object LastID = 0;
                SqlManager.ExecuteNonQueryProcedure(ref LastID);

                //Je met l'id du rendez-vous dans la MeetingList
                _Value.IDMeeting = (int)LastID;

                #region Ajout de la dette
                PatientManager pm = PatientManager.Create(_Value.IDPatient);
                pm.UpdateDebt(_Value.Honoraire);
                #endregion

                base.Add(_Value);
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "MeetingSet.Add()", LogType.Erreur);
                throw;
            }
        }
        /// <summary>
        /// Supprime un rendez-vous
        /// </summary>
        /// <param name="_Value">Informations sur le rendez-vous  supprimer</param>
        /// <exception cref="Exception">En cas de rendez-vous non supprimer</exception>        
        public override void Remove(MeetingRow _Value)
        {
            try
            {
                SqlManager.PrepareProcedure("DELETE_RendezVous");
                SqlManager.Parameters.Add("_ID", OleDbType.Integer).Value = _Value.IDMeeting;
                SqlManager.ExecuteNonQueryProcedure();
                base.Remove(_Value);

                #region On retranche le prix de la dette
                DataSet ds = new DataSet();
                int i = SqlManager.ExecuteSql("SELECT * FROM Patient WHERE id_patient = " + _Value.IDPatient, ref ds);

                if (i > 0)
                {
                    int SubDebt = (int)ds.Tables[0].Rows[0]["Honoraire"];
                    PatientManager pm = PatientManager.Create(_Value.IDPatient);
                    pm.SubDebt(SubDebt);                    
                    pm.Update();
                }
                #endregion
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "MeetingSet.Remove()", LogType.Erreur);
                throw new Exception("Impossible to remove the meeting from the data base:\n" + ex.Message);
            }
        }
        /// <summary>
        /// Remplissage des rendez-vous qui sont dans la base de donne
        /// </summary>
        /// <exception cref="Exception">En cas d'erreur dans le traitement sur la base de donne</exception>
        /// <exception cref="OleDbException">Une erreur au niveau de la base de donnes...</exception>
        public override void Fill()
        {
            try
            {
                base.vMeetingList.Clear();

                DataSet ds = new DataSet();
                SqlManager.PrepareProcedure("SELECT_RendezVous");
                SqlManager.ExecuteSqlProcedure(ref ds);

                MeetingRow mr = new MeetingRow();

                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    mr.StartDate = (DateTime)dr["Debut"];
                    mr.EndDate = (DateTime)dr["Fin"];

                    if (dr["Titre"].Equals(DBNull.Value))
                        mr.Title = "";
                    else
                        mr.Title = (string)dr["Titre"];

                    if (dr["Message"].Equals(DBNull.Value))
                        mr.Message = "";
                    else
                        mr.Message = (string)dr["Message"];//(string)dr["Prenom"] + " " + (string)dr["Nom"];

                    mr.IDMeeting = (int)dr["ID_Rdv"];
                    mr.IDPatient = (int)dr["p.ID_Patient"];
                    this.vMeetingList.Add(mr);
                }
            }
            #region Exceptions
            catch (ArgumentException ex)
            {
                Log.Instance.Write("ArgumentException: " + ex.Message, "MeetingSet.Fill()", LogType.Erreur);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write("InvalidOperationException: " + ex.Message, "MeetingSet.Fill()", LogType.Erreur);
                throw;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write("OleDbException: " + ex.Message, "MeetingSet.Fill()", LogType.Erreur);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write("Exception: " + ex.Message, "MeetingSet.Fill()", LogType.Erreur); throw;
            }
            #endregion
        }
        /// <summary>
        /// Modifie les dates de dbut, de fin, le message et le titre d'un rendez-vous
        /// </summary>
        /// <param name="_Value">Les nouvelles valeurs du rendez-vous</param>
        public override void Modify(MeetingRow _Value)
        {
            try
            {
                MeetingManager mm = MeetingManager.Create(_Value.IDMeeting);
                SqlManager.PrepareProcedure("UPDATE_RendezVous");
                SqlManager.Parameters.Add("_Debut", OleDbType.Date).Value = _Value.StartDate;
                SqlManager.Parameters.Add("_Fin", OleDbType.Date).Value = _Value.EndDate;
                SqlManager.Parameters.Add("_Message", OleDbType.Variant).Value = _Value.Message;
                SqlManager.Parameters.Add("_Titre", OleDbType.Variant).Value = _Value.Title;
                SqlManager.Parameters.Add("_ID", OleDbType.Integer).Value = _Value.IDMeeting;
                SqlManager.Parameters.Add("_Came", OleDbType.Boolean).Value = mm.Came;
                SqlManager.Parameters.Add("_Payed", OleDbType.Boolean).Value = mm.Payed;
                SqlManager.ExecuteNonQueryProcedure();

                base.Modify(_Value);
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "MeetingSet.Modify()", LogType.Erreur);
                throw;
            }

        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Mthode de remplissage de la liste de rendez-vous
        /// </summary>
        public void Fill(int _ID)
        {
            try
            {
                vMeetingList.Clear();

                SqlManager.PrepareProcedure("SELECT_RendezVousOnPatient");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSqlProcedure(ref ds);

                MeetingRow mr = new MeetingRow();

                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    mr.StartDate = (DateTime)dr["Debut"];
                    mr.EndDate = (DateTime)dr["Fin"];

                    if (dr["Titre"].Equals(DBNull.Value))
                        mr.Title = "";
                    else
                        mr.Title = (string)dr["Titre"];

                    if (dr["Message"].Equals(DBNull.Value))
                        mr.Message = "";
                    else
                        mr.Message = (string)dr["Message"];//(string)dr["Prenom"] + " " + (string)dr["Nom"];

                    mr.IDMeeting = (int)dr["ID_Rdv"];
                    mr.IDPatient = (int)dr["ID_Patient"];
                    this.vMeetingList.Add(mr);
                }
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, "MeetingSet.Fill(int _ID)", LogType.Erreur);
                throw;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le path de la base de donne
        /// </summary>
        /// <exception cref="FileNotFoundException">Si la path donne ne pointe pas sur un fichier</exception>
        public string Path
        {
            get { return Options.GetInstance().Path; }
            set
            {
                if (System.IO.File.Exists(value))
                {
                    Options.GetInstance().Path = value;
                    SqlManager.DbPath = value;
                }
                else
                    throw new FileNotFoundException("La base de donnes n'a pas t trouve", value);
            }
        }
        /// <summary>
        /// Gets a list of formatted meetings ready to display (De [LongDate]  [ShortTime]"
        /// </summary>
        public List<String> DisplayMeetingSet
        {
            get
            {
                List<String> ls = new List<String>();
                foreach (MeetingRow mr in base.vMeetingList)
                {
                    String s = TextManager.FirstUpper(mr.StartDate.ToLongDateString()) + " de "
                        + mr.StartDate.ToShortTimeString() + "  " + mr.EndDate.ToShortTimeString();
                    ls.Add(s);
                }
                return ls;
            }
        }
        #endregion
    }
}
