using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.ComponentModel;
using Utils.Misc;
using System.Runtime.Serialization;
using System.IO;
using Components.TypeEditors;
using System.Drawing.Design;
using System.Data.OleDb;
using System.Data;
using System.Windows.Forms;
using PhoenixDLL.Class;
using Components.TypeEditors.GUI;
using System.Drawing.Printing;
using Utils.Log;

namespace PhoenixDLL.Class
{
    #region Delegate
    /// <summary>
    /// Handler d'vnement de changement d'vOptions
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>    
    public delegate void OnOptionChangedHandler(object sender, EventArgs e);
    /// <summary>
    /// Event Handler for a restart of the application
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void OnRestardNeededHandler(object sender, EventArgs e);
    #endregion

    /// <summary>
    /// Classe d'Options
    /// </summary>
    [Serializable]
    public class Options : Serializer, IDisposable
    {
        #region Variables
        /// <summary>
        /// Premier dmarrage des vOptions
        /// </summary>
        private bool vFirstInstance = true;
        /// <summary>
        /// Indicates whether Options can throw events
        /// </summary>
        private bool vEventable = true;

        #region Singletton
        /// <summary>
        /// Rfrence sur le singletton
        /// </summary>        
        private static Options refOptions = null;
        #endregion

        #region MonthAgenda
        /// <summary>
        /// Couleur de fond du header
        /// </summary>
        protected Color vHeaderBackColor = Color.White;
        /// <summary>
        /// Couleur du texte du header
        /// </summary>
        protected Color vHeaderTextColor = Color.Black;
        /// <summary>
        /// Police d'criture du header
        /// </summary>
        protected Font vHeaderTextFont = new Font("Times New Roman", 12);

        /// <summary>
        /// Couleur de fond du HeaderDay
        /// </summary>
        protected Color vHeaderDayBackColor = Color.White;
        /// <summary>
        /// Couleur d'criture du HeaderDay
        /// </summary>
        protected Color vHeaderDayTextColor = Color.Black;
        /// <summary>
        /// Police d'criture du HeaderDay
        /// </summary>
        protected Font vHeaderDayFont = new Font("Times New Roman", 12);

        /// <summary>
        /// Couleur de font des jours
        /// </summary>
        protected Color vDaysBackColor = Color.White;
        /// <summary>
        /// Couleur d'criture des jours
        /// </summary>
        protected Color vDaysTextColor = Color.Black;
        /// <summary>
        /// Couleur de la bordure qui spare les jours
        /// </summary>
        protected Color vDaysBorderColor = Color.Silver;
        /// <summary>
        /// Couleur des jours qui sont hors du mois
        /// </summary>
        protected Color vDaysNonInMonthColor = Color.Silver;
        /// <summary>
        /// Police d'criture des jours
        /// </summary>
        protected Font vDaysFont = new Font("Times New Roman", 12);

        /// <summary>
        /// Couleur de aujourd'hui
        /// </summary>
        protected Color vToday = Color.Tomato;
        #endregion

        #region DayAgenda
        /// <summary>
        /// Couleur de dgrad
        /// </summary>
        protected Color vFadeColor1 = SystemColors.ActiveCaption;//Color.Silver;
        /// <summary>
        /// Couleur de fond d'un rendez-vous
        /// </summary>
        protected Color vFadeColor2 = SystemColors.ActiveBorder;//Color.PeachPuff;
        /// <summary>
        /// Couleur du texte des rendez-vous
        /// </summary>
        protected Color vMeetingTextColor = Color.Black;
        /// <summary>
        /// Couleur de la police d'criture 
        /// </summary>
        protected Font vMeetingTextFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur de fond des non rendez-vous
        /// </summary>
        protected Color vNonMeetingBackColor = Color.White;
        /// <summary>
        /// Indicates the first hour to display in the DayAgenda
        /// </summary>
        protected DateTime vFirstHourToDisplay = new DateTime(1, 1, 1, 8, 30, 0);
        /// <summary>
        /// Nombre de lignes qu'il faut scroller  chaque clique de la molette
        /// </summary>
        protected int vLineToWheel = 1;
        #endregion

        #region Base de donnes
        /// <summary>
        /// Chemin de la base de donnes
        /// </summary>
        protected string vBDPath = ApplicationPath.CommandPath + "\\GestionPatient.mdb";
        /// <summary>
        /// Indicates if the DataBase was saved
        /// </summary>
        protected DateTime vLastSave = DateTime.Today;//new DateTime(1830, 7, 21);
        /// <summary>
        /// Indicates if the program checks for new DataBase updates
        /// </summary>
        protected bool vAutomaticSave = true;
        /// <summary>
        /// Indicates whether a test is useful before add
        /// </summary>
        protected bool vTestBeforeAdd = true;
        #endregion

        #region SMTP
        /// <summary>
        /// Mot de passe du serveur SMTP pour l'envois d'erreur
        /// </summary>
        protected string vSMTPPassword = "";

        /// <summary>
        /// Addresse du server SMTP pour l'envoi des messages d'erreurs
        /// </summary>
        protected string vSMTPServer = "";

        /// <summary>
        /// Port de communication du serveur SMTP
        /// </summary>
        protected int vSMTPPort = 25;
        #endregion

        #region Impression
        /// <summary>
        /// Entte pour l'impression des prescriptions
        /// </summary>
        private string vPrintPrescriptionHeader = "";
        /// <summary>
        /// Police de caractre pour le header de prescriptions
        /// </summary>
        private Font vPrintPrescriptionFont = new Font("Times New Roman", 8);
        /// <summary>
        /// Informations sur l'impression des prescriptions
        /// </summary>
        private PrinterSettings vPrinterSettingsPrescriptions = new PrinterSettings();
        /// <summary>
        /// Informations sur l'impression des rendez-vous d'un patient
        /// </summary>
        private PrinterSettings vPrinterSettingsMeetings = new PrinterSettings();
        #endregion

        #region Fiche mdicale
        /// <summary>
        /// Titre de la fiche supplmentaire 1
        /// </summary>
        protected string vAdditionnalTitle1 = "Fiche 1";
        /// <summary>
        /// Titre de la fiche supplmentaire 2
        /// </summary>
        protected string vAdictionnalTitle2 = "Fiche 2";
        /// <summary>
        /// Titre de la fiche supplmentaire 3
        /// </summary>
        protected string vAdditionnalTitle3 = "Fiche 3";
        /// <summary>
        /// Titre de la fiche supplmentaire 4
        /// </summary>
        protected string vAdditionnalTitle4 = "Fiche 4";
        /// <summary>
        /// Vivibilit de la fiche 1
        /// </summary>
        protected bool vIsVisible1 = false;
        /// <summary>
        /// Vivibilit de la fiche 2
        /// </summary>
        protected bool vIsVisible2 = false;
        /// <summary>
        /// Vivibilit de la fiche 3
        /// </summary>
        protected bool vIsVisible3 = false;
        /// <summary>
        /// Vivibilit de la fiche 4
        /// </summary>
        protected bool vIsVisible4 = false;
        #endregion

        #region Versions
        /// <summary>
        /// Indicates the version of the software
        /// </summary>
        public const string VERSION = "BETA 0.9.10";
        #endregion

        #region DeskSave
        /// <summary>
        /// Informations sur les menus de Pata
        /// </summary>
        private PataMenu vPataMenu = new PataMenu();
        /// <summary>
        /// Informations sur les menus de Pega
        /// </summary>
        private PegaMenu vPegaMenu = new PegaMenu();
        /// <summary>
        /// Indicates if the lateral menu is visible
        /// </summary>
        private bool vDeckVisible = true;
        /// <summary>
        /// Indicates the lateral deck position
        /// </summary>
        private DeckPosition vDeckPosition = DeckPosition.Bas;
        /// <summary>
        /// Indicates if the vOptions menu is visible on each soft
        /// </summary>
        private bool vOptionsOnEachSoft = false;
        /// <summary>
        /// Indicates the size of the menu's images
        /// </summary>
        private Size vMenuImageScalingSize = new Size(20, 20);
        #endregion
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        protected Options()
            : base(ApplicationPath.CommandPath + "\\Options.srz") { RegistryManager.ProcessRegistry(); }

        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        /// <param name="_Path">Path du fichier d'vOptions</param>
        protected Options(string _Path) : base(_Path) { }

        /// <summary>
        /// Destructeur
        /// </summary>
        ~Options() { base.Serialize(this); }

        /// <summary>
        /// Mthode factory qui retourne une rfrence sur l'objet Options
        /// </summary>
        /// <returns>Une rfrence sur l'objet Options</returns>
        public static Options GetInstance()
        {
            string path = RegistryManager.Option;
            if (refOptions == null)
            {
                try
                { refOptions = (Options)Deserialize(path); }

                catch (FileNotFoundException) { refOptions = new Options(path); }
                catch (Exception) { refOptions = new Options(); }
            }
            return refOptions;
        }
        #endregion

        #region Accesseurs
        #region MonthAgenda
        /// <summary>
        /// Retourne ou initialise la couleur de font du header
        /// </summary>
        [Category("Affichage du mois - Header")]
        [Description("Initialise la couleur de font du header")]
        [DisplayName("Couleur de fond du header")]
        public Color HeaderBackColor
        {
            get { return vHeaderBackColor; }
            set { vHeaderBackColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur du texte du header
        /// </summary>
        [Category("Affichage du mois - Header")]
        [Description("Initialise la couleur du texte du header")]
        [DisplayName("Couleur du texte du header")]
        public Color HeaderTextColor
        {
            get { return vHeaderTextColor; }
            set { vHeaderTextColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la police de caractre du header
        /// </summary>
        [Category("Affichage du mois - Header")]
        [Description("Initialise la police du texte du header")]
        [DisplayName("Police du header")]
        public Font HeaderTextFont
        {
            get { return vHeaderTextFont; }
            set { vHeaderTextFont = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fond du header des jours
        /// </summary>
        [Category("Affichage du mois - Header Jours")]
        [Description("Initialise la couleur de fond du header des jours")]
        [DisplayName("Couleur de fond du header de jour")]
        public Color HeaderDayBackColor
        {
            get { return vHeaderDayBackColor; }
            set { vHeaderDayBackColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur du texte du header des jours
        /// </summary>
        [Category("Affichage du mois - Header Jours")]
        [Description("Initialise la couleur du texte du header des jours")]
        [DisplayName("Couleur du texte du header de jour")]
        public Color HeaderDayTextColor
        {
            get { return vHeaderDayTextColor; }
            set { vHeaderDayTextColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la police d'criture du header des jours
        /// </summary>
        [Category("Affichage du mois - Header Jours")]
        [Description("Initialise la police d'criture du header des jours")]
        [DisplayName("Police du header de jour")]
        public Font HeaderDayTextFont
        {
            get { return vHeaderDayFont; }
            set { vHeaderDayFont = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours
        /// </summary>
        [Category("Affichage du mois - Jours")]
        [Description("Initialise la couleur de fond des jours")]
        [DisplayName("Couleur de fond des jours")]
        public Color DaysBackColor
        {
            get { return vDaysBackColor; }
            set { vDaysBackColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fond des jours
        /// </summary>
        [Category("Affichage du mois - Jours")]
        [Description("Initialise la couleur de fond des jours")]
        [DisplayName("Couleur du texte de jours")]
        public Color DaysTextColor
        {
            get { return vDaysTextColor; }
            set { vDaysTextColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de la bordure des jours
        /// </summary>
        [Category("Affichage du mois - Jours")]
        [Description("Initialise la couleur de la bordure des jours")]
        [DisplayName("Couleur de la bordure des jours")]
        public Color DaysBorderColor
        {
            get { return vDaysBorderColor; }
            set { vDaysBorderColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise le couleur de fond des jours hors du mois
        /// </summary>
        [Category("Affichage du mois - Jours")]
        [Description("Initialise le couleur de fond des jours hors du mois")]
        [DisplayName("Couleurs des jours hors mois")]
        public Color DaysNonInMonthColor
        {
            get { return vDaysNonInMonthColor; }
            set { vDaysNonInMonthColor = value; }
        }

        /// <summary>
        /// Retourne ou initialise la police d'affichage des jours
        /// </summary>
        [Category("Affichage du mois - Jours")]
        [Description("Initialise la police d'affichage des jours")]
        [DisplayName("Police des jours")]
        public Font DaysTextFont
        {
            get { return vDaysFont; }
            set { vDaysFont = value; }
        }


        /// <summary>
        /// Retourne ou initialise la couleur du jour d'aujourd'hui
        /// </summary>
        [Category("Affichage du mois - Aujourd'hui")]
        [Description("Initialise la couleur du jour d'aujourd'hui")]
        [DisplayName("Couleur d'aujourd'hui")]
        public Color TodayBackColor
        {
            get { return vToday; }
            set { vToday = value; }
        }
        #endregion

        #region DayAgenda
        /// <summary>
        /// Retourne ou initialise la premire couleur de fondu pour le rendez-vous
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise le couleur de fondu pour le rendez-vous")]
        [DisplayName("Couleur 1 de fondu d'un rendez-vous")]
        public Color FirstMeetingFadeColor
        {
            get { return vFadeColor1; }
            set { vFadeColor1 = value; }
        }
        /// <summary>
        /// Retourne ou initialise la deuxime couleur de fond d'un rendez-vous
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise la deuxime couleur de fond d'un rendez-vous")]
        [DisplayName("Couleur 2 de fondu d'un rendez-vous")]
        public Color SecondMeetingFadeColor
        {
            get { return vFadeColor2; }
            set { vFadeColor2 = value; }
        }
        /// <summary>
        /// Retourne ou initialise la couleur d'criture dans un rendez-vous
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise la couleur d'criture dans un rendez-vous")]
        [DisplayName("Couleur d'criture dans un rendez-vous")]
        public Color MeetingTextColor
        {
            get { return vMeetingTextColor; }
            set { vMeetingTextColor = value; }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture d'un rendez-vous
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise la police d'criture d'un rendez-vous")]
        [DisplayName("Police d'criture dans un rendez-vous")]
        public Font MeetingTextFont
        {
            get { return vMeetingTextFont; }
            set { vMeetingTextFont = value; }
        }
        /// <summary>
        /// Retourne ou initialise la couleur d'affichage d'une heure hors rendez-vous
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise la couleur d'affichage d'une heure hors rendez-vous")]
        [DisplayName("Couleur de fond d'une heure sans rendez-vous")]
        public Color NonMeetingBackColor
        {
            get { return vNonMeetingBackColor; }
            set { vNonMeetingBackColor = value; }
        }
        /// <summary>
        /// Gets or stes the first hour to display in the DayAgenda
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Initialise la premire heure  afficher dans la jour")]
        [DisplayName("Premire heure  afficher dans le jour")]
        [Editor(typeof(UIHourEditor), typeof(UITypeEditor))]
        public string FirstHourToDisplayString
        {
            get { return vFirstHourToDisplay.ToShortTimeString(); ; }
            set
            {
                try
                {
                    vFirstHourToDisplay = DateTime.Parse(value);
                    OnOptionChanged();
                }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, "Options.FirstHourToDisplayString", LogType.Erreur);
                    throw;
                }
            }
        }
        /// <summary>
        /// Indique le nombre de lignes qui dfilent au roulement de la molette de la souris
        /// </summary>
        [Category("Affichage du jour")]
        [Description("Indique le nombre de lignes qui dfilent au roulement de la molette de la souris")]
        [DisplayName("Sensibilit de la molette de la souris")]
        public int LinesToWheel
        {
            get { return vLineToWheel; }
            set { vLineToWheel = value; }
        }
        /// <summary>
        /// Gets the DateTime object of the first hour to display
        /// </summary>
        [Browsable(false)]
        public DateTime FirstHourToDisplayDateTime { get { return vFirstHourToDisplay; } }
        #endregion

        #region Base de donnes
        /// <summary>
        /// Retourne ou initialise le path de la base de donnes.
        /// </summary>
        /// <exception cref="FileNotFoundException">Si le fichier n'existe pas</exception>
        [Category("Base de donnes")]
        [DisplayName("[Expert] Chemin de la base de donnes")]
        [Description("[Expert] Initialise le path de la base de donnes")]
        [Editor(typeof(UIPathEditor), typeof(UITypeEditor))]
        public string Path
        {
            get { return vBDPath; }
            set
            {
                if (File.Exists(value))
                {
                    if (vBDPath != value)
                    {
                        vBDPath = value;

                        #region Ecriture dans la base de registre
                        RegistryManager.ProcessRegistry();
                        #endregion

                        if (vEventable)
                            OnRestardNeeded();
                    }
                }
                else
                    throw new FileNotFoundException("Le fichier indiqu n'existe pas!", value);
            }
        }
        /// <summary>
        /// Indicates if the database was saved
        /// </summary>
        [Category("Base de donnes")]
        [DisplayName("Dernire sauvegarde de la base de donnes")]
        [Description("Dernire sauvegarde de la base de donnes")]
        [ReadOnly(true)]
        public DateTime LastDBSave { get { return vLastSave; } set { vLastSave = value; } }
        /// <summary>
        /// Indicates if the program checks for new DataBase updates
        /// </summary>
        [Category("Base de donnes")]
        [DisplayName("[Expert] Vrification de dernire sauvegarde")]
        [Description("[Expert] Vrifie s'il faut faire une nouvelle sauvegarde de la base de donnes")]
        public bool AutomaticDBSave { get { return vAutomaticSave; } set { vAutomaticSave = value; } }
        /// <summary>
        /// Indicates whether a test is useful before add
        /// </summary>
        [Category("Base de donnes")]
        [DisplayName("[Expert] Test avant de faire un ajout dans le base de donnes")]
        [Description("[Expert] Indique s'il faut faire un test avant un ajout dans la base de donnes")]
        public bool TestBeforeAdd { get { return vTestBeforeAdd; } set { vTestBeforeAdd = value; } }
        #endregion

        #region Miscelanious
        /// <summary>
        /// Retourne ou initialise le mot de passe du serveur SMTP
        /// </summary>
        [Category("SMTP")]
        [DisplayName("[Expert] Mot de passe du serveur SMTP")]
        [Description("[Expert] Indique le mot de passe du serveur SMTP pour l'envois de mails en cas d'erreur")]

        [PasswordPropertyText(true)]
        public string SMTPPassword
        {
            get { return vSMTPPassword; }
            set { vSMTPPassword = value; }
        }

        /// <summary>
        /// Retourne ou initialise l'adresse du serveur SMTP
        /// </summary>
        [Category("SMTP")]
        [DisplayName("[Expert] Serveur SMTP")]
        [Description("[Expert] Indique le serveur SMTP pour l'envois de mails en cas d'erreur")]
        public string SMTPServer
        {
            get { return vSMTPServer; }
            set { vSMTPServer = value; }
        }

        /// <summary>
        /// retourne ou initialise le port de communication du serveur SMTP
        /// </summary>
        [Category("SMTP")]
        [DisplayName("[Expert] Port de communication du serveur SMTP")]
        [Description("Initialise le port de communicaiton du serveur SMTP")]
        [DefaultValue(25)]
        public int SMTPPort
        {
            get { return vSMTPPort; }
            set { vSMTPPort = value; }// 25; }
        }
        #endregion

        #region Fiches mdicales
        /// <summary>
        /// Retourne ou initialise la titre de la fiche supplmentaire 1
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Titre de la fiche 1")]
        [Description("Initialise le titre de la fiche optionnelle 1")]
        public string AdditionnalTitle1
        {
            get { return vAdditionnalTitle1; }
            set
            {
                vAdditionnalTitle1 = value;
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la titre de la fiche supplmentaire 2
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Titre de la fiche 2")]
        [Description("Initialise le titre de la fiche optionnelle 2")]
        public string AdditionnalTitle2
        {
            get { return vAdictionnalTitle2; }
            set
            {
                vAdictionnalTitle2 = value;
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la titre de la fiche supplmentaire 3
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Titre de la fiche 3")]
        [Description("Initialise le titre de la fiche optionnelle 3")]
        public string AdditionnalTitle3
        {
            get { return vAdditionnalTitle3; }
            set
            {

                vAdditionnalTitle3 = value;
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la titre de la fiche supplmentaire 4
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Titre de la fiche 4")]
        [Description("Initialise le titre de la fiche optionnelle 4")]
        public string AdditionnalTitle4
        {
            get { return vAdditionnalTitle4; }
            set
            {
                vAdditionnalTitle4 = value;
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la fait que le fiche supplmentaire 1 est visible ou pas
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Visibilit de la fiche supplmentaire 1")]
        [Description("Indique si cette fiche supplmentaire doit tre visible ou pas")]
        public bool IsVisible1
        {
            get { return vIsVisible1; }
            set
            {
                if (CountCardNumber(1) == 0)
                    vIsVisible1 = value;
                else
                {
                    ClearCardNumber(1, value);
                    vIsVisible1 = value;
                }
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la fait que le fiche supplmentaire 2 est visible ou pas
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Visibilit de la fiche supplmentaire 2")]
        [Description("Indique si cette fiche supplmentaire doit tre visible ou pas")]
        public bool IsVisible2
        {
            get { return vIsVisible2; }
            set
            {
                if (CountCardNumber(2) == 0)
                    vIsVisible2 = value;
                else
                {
                    ClearCardNumber(2, value);
                    vIsVisible2 = value;
                }
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la fait que le fiche supplmentaire 3 est visible ou pas
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Visibilit de la fiche supplmentaire 3")]
        [Description("Indique si cette fiche supplmentaire doit tre visible ou pas")]
        public bool IsVisible3
        {
            get { return vIsVisible3; }
            set
            {
                if (CountCardNumber(3) == 0)
                    vIsVisible3 = value;
                else
                {
                    ClearCardNumber(3, value);
                    IsVisible3 = value;
                }
                if (vEventable)
                    OnOptionChanged();
            }
        }

        /// <summary>
        /// Retourne ou initialise la fait que le fiche supplmentaire 4 est visible ou pas
        /// </summary>
        [Category("Fiches supplmentaires")]
        [DisplayName("Visibilit de la fiche supplmentaire 4")]
        [Description("Indique si cette fiche supplmentaire doit tre visible ou pas")]
        public bool IsVisible4
        {
            get { return vIsVisible4; }
            set
            {
                if (CountCardNumber(4) == 0)

                    vIsVisible4 = value;
                else
                {
                    ClearCardNumber(4, value);
                    vIsVisible4 = value;
                }
                if (vEventable)
                    OnOptionChanged();
            }
        }
        #endregion

        #region Impression
        /// <summary>
        /// Retourne ou initialise le texte pour l'entte d'impression des prescriptions
        /// </summary>
        [Category("Impressions - Prescriptions")]
        [DisplayName("Texte d'entte des prescriptions")]
        [Description("Initialise le texte pour l'entte d'impression des prescriptions")]
        [Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public string PrintPrescriptionHeader
        {
            get { return vPrintPrescriptionHeader; }
            set { vPrintPrescriptionHeader = value; }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture du headr de prescription
        /// </summary>
        [Category("Impressions - Prescriptions")]
        [DisplayName("Police de caractre d'entte des prescriptions")]
        [Description("Initialise la police d'criture du headr de prescription")]
        //[Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public Font FontPrescriptionHeader
        {
            get { return vPrintPrescriptionFont; }
            set { vPrintPrescriptionFont = value; }
        }
        /// <summary>
        /// Retourne ou initialise les vOptions d'impressions d'une prescription
        /// </summary>
        [Browsable(false)]
        public PrinterSettings PrinterSettingsPrescritions
        {
            get { return vPrinterSettingsPrescriptions; }
            set { vPrinterSettingsPrescriptions = value; }
        }
        ///<summary>
        /// Informations sur l'impression des rendez-vous d'un patient
        /// </summary>
        [Browsable(false)]
        public PrinterSettings PrinterSettingsMeetings
        {
            get { return vPrinterSettingsMeetings; }
            set { vPrinterSettingsMeetings = value; }
        }
        #endregion

        #region Versions
        /// <summary>
        /// Retourne ou initialise la valeur qui indique si c'est la premire fois que l'on instancie le fichier d'Options
        /// </summary>
        [Browsable(false)]
        public bool IsFirstInstance { get { return this.vFirstInstance; } set { vFirstInstance = value; } }
        /// <summary>
        /// Retourne une instance de Options
        /// </summary>
        [Browsable(false)]
        public static Options Instance { get { return GetInstance(); } }
        /// <summary>
        /// Gets or sets the value which indicates whether Options can throw events
        /// </summary>
        [Browsable(false)]
        public bool IsEventable { get { return vEventable; } set { vEventable = value; } }
        #endregion

        #region DeskSave
        /// <summary>
        /// Retourne ou initialise les informations de visibilit des menus de Pata
        /// </summary>
        [Browsable(false)]
        public PataMenu PataMenu { get { return vPataMenu; } set { vPataMenu = value; } }
        /// <summary>
        /// Retourne ou initialise les informations de visibilit des menus de Pega
        /// </summary>
        [Browsable(false)]
        public PegaMenu PegaMenu { get { return vPegaMenu; } set { vPegaMenu = value; } }
        /// <summary>
        /// Indicates if the lateral menu is visible
        /// </summary>
        [Category("Interface utilisateur")]
        [DisplayName("Visibilit du menu latral")]
        [Description("Indique si le menu latral est visible ou non")]
        public bool DeckVisible
        {
            get { return vDeckVisible; }
            set
            {
                if (vDeckVisible != value)
                {
                    vDeckVisible = value;
                    if (vEventable)
                        OnOptionChanged();
                }
            }
        }
        /// <summary>
        /// Gets or sets the visibility if the vOptions menu in Phoenix Suite       
        /// </summary>
        [Category("Interface utilisateur")]
        [DisplayName("Visibilit du menu options")]
        [Description("Indique si le menu d'options est visible dans chaque fentre")]
        [Browsable(false)]
        public bool OptionsMenuVisible
        {
            get { return vOptionsOnEachSoft; }
            set { vOptionsOnEachSoft = value; }
        }
        /// <summary>
        /// Indicates the position of the lateral menu
        /// </summary>
        [Category("Interface utilisateur")]
        [DisplayName("Position du menu latral")]
        [Description("Indique la position du menu latral")]
        public DeckPosition DeckPosition
        {
            get { return vDeckPosition; }
            set
            {
                if (vDeckPosition != value)
                {
                    vDeckPosition = value;
                    if (vEventable)
                        OnOptionChanged();
                }
            }
        }
        /// <summary>
        /// Gets or sets the size of the images of the nomade menus
        /// </summary>
        [Category("Interface utilisateur")]
        [DisplayName("Taille des images des menus nomades")]
        [Description("Indique la taille des images des menus nomades")]
        public Size MenuImageScalingSize
        {
            get { return vMenuImageScalingSize; }
            set
            {
                vMenuImageScalingSize = value;
                if (vEventable)
                    OnOptionChanged();
            }
        }
        #endregion
        #endregion

        #region IDisposable Members
        /// <summary>
        /// Fait une destruction de l'objet
        /// </summary>
        public void Dispose()
        {
            base.Serialize(this);
        }
        #endregion

        #region Events
        [NonSerialized]
        private OnOptionChangedHandler DelegateOptionChangedHandler;
        /// <summary>
        /// Se produit lorsqu'il y a eu un changement d'option  traiter
        /// </summary>
        [Browsable(false)]
        public event OnOptionChangedHandler OptionChanged
        {
            add { DelegateOptionChangedHandler += value; }
            remove { DelegateOptionChangedHandler -= value; }
        }
        /// <summary>
        /// Notifie une demande de notify pour l'vnement d'vOptions change
        /// </summary>                       
        protected virtual void OnOptionChanged()
        {
            if (DelegateOptionChangedHandler != null)
                DelegateOptionChangedHandler(this, new EventArgs());

        }
        [NonSerialized]
        private OnRestardNeededHandler DelegateOnRestardNeeded;
        /// <summary>
        /// Occures when a vOptions which needs a restart changes
        /// </summary>
        [Browsable(false)]
        public event OnRestardNeededHandler RestartNeeded
        {
            add { DelegateOnRestardNeeded += value; }
            remove { DelegateOnRestardNeeded -= value; }
        }
        /// <summary>
        /// Notify a restart needing
        /// </summary>
        protected virtual void OnRestardNeeded()
        {
            if (DelegateOnRestardNeeded != null)
                DelegateOnRestardNeeded(this, new EventArgs());
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Compte le nombre de fiches optionnelles qui ont des informations...
        /// </summary>
        /// <param name="_CardNumber">Numro de la fiche optionnelle</param>
        /// <returns>le nombre de fiche remplis</returns>
        protected int CountCardNumber(int _CardNumber)
        {
            try
            {
                string conStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
                string vPath = Options.GetInstance().Path;
                string st = conStr + vPath + "\"";

                OleDbConnection oConn = new OleDbConnection(st);
                OleDbCommand oCom;

                switch (_CardNumber)
                {
                    case 1:
                        oCom = new OleDbCommand("COUNT_FicheSupp1", oConn);
                        break;
                    case 2:
                        oCom = new OleDbCommand("COUNT_FicheSupp2", oConn);
                        break;
                    case 3:
                        oCom = new OleDbCommand("COUNT_FicheSupp3", oConn);
                        break;
                    case 4:
                        oCom = new OleDbCommand("COUNT_FicheSupp4", oConn);
                        break;
                    default:
                        throw new ArgumentException("Veuillez choisir un nombre entre 1 et 4 inclus", "_CardNumber");
                }

                oCom.CommandType = CommandType.StoredProcedure;
                OleDbDataAdapter oDa = new OleDbDataAdapter(oCom);

                oConn.Open();
                int i = (int)oCom.ExecuteScalar(); //ExecuteNonQuery();
                oConn.Close();

                return i;
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Supprime le contenu de la fiche de numro donn en argument
        /// </summary>
        /// <param name="_CardNumber">Numro de la fiche  nettoyer</param>
        /// <param name="_IsSkipped">Faut-il faire le traitement?</param>
        protected void ClearCardNumber(int _CardNumber, bool _IsSkipped)
        {
            if (_IsSkipped)
            {
                DialogResult dr = MessageBox.Show("Cette fiche contient dj des informations."
                + "Faut-il toutes les effacer?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (dr == DialogResult.Yes)
                {
                    string conStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
                    string vPath = Options.GetInstance().Path;
                    string st = conStr + vPath + "\"";

                    OleDbConnection oConn = new OleDbConnection(st);
                    OleDbCommand oCom;

                    switch (_CardNumber)
                    {
                        case 1:
                            oCom = new OleDbCommand("CLEAR_FicheSupp1", oConn);
                            break;
                        case 2:
                            oCom = new OleDbCommand("CLEAR_FicheSupp2", oConn);
                            break;
                        case 3:
                            oCom = new OleDbCommand("CLEAR_FicheSupp3", oConn);
                            break;
                        case 4:
                            oCom = new OleDbCommand("CLEAR_FicheSupp4", oConn);
                            break;
                        default:
                            throw new ArgumentException("Veuillez choisir un nombre entre 1 et 4 inclus", "_CardNumber");
                    }

                    oCom.CommandType = CommandType.StoredProcedure;
                    OleDbDataAdapter oDa = new OleDbDataAdapter(oCom);

                    oConn.Open();
                    oCom.ExecuteNonQuery();
                    oConn.Close();
                }
            }
        }
        #endregion
    }

    /// <summary>
    /// Use to have a copy of the vOptions
    /// </summary>
    [Serializable]
    public class SavedOptions
    {
        #region Variables
        private string vAdditionnalTitle1;
        private string vAdditionnalTitle2;
        private string vAdditionnalTitle3;
        private string vAdditionnalTitle4;
        private Color vDaysBackColor;
        private Color vDaysBorderColor;
        private Color vDaysNonInMonthColor;
        private Color vDaysTextColor;
        private Font vDaysTextFont;
        private Color vFirstMeetingFadeColor;
        private Font vFontPrescriptionHeader;
        private Color vHeaderBackColor;
        private Color vHeaderDayBackColor;
        private Color vHeaderDayTextColor;
        private Font vHeaderDayTextFont;
        private Color vHeaderTextColor;
        private Font vHeaderTextFont;
        private bool vIsFirstInstance;
        private bool vIsVisible1;
        private bool vIsVisible2;
        private bool vIsVisible3;
        private bool vIsVisible4;
        private Color vMeetingTextColor;
        private Font vMeetingTextFont;
        private Color vNonMeetingBackColor;
        private PataMenu vPataMenu;
        private string vPath;
        private PegaMenu vPegaMenu;
        private PrinterSettings vPrinterSettingsPrescritions;
        private string vPrintPrescriptionHeader;
        private Color vSecondMeetingFadeColor;
        private string vSMTPPassword;
        private int vSMTPPort;
        private string vSMTPServer;
        private Color vTodayBackColor;
        #endregion

        #region Constructor
        /// <summary>
        /// Save the Options.Instance state
        /// </summary>
        public SavedOptions() { Save(); }
        /// <summary>
        /// Destructor
        /// </summary>
        ~SavedOptions() { Options.Instance.IsEventable = true; }
        #endregion

        #region Methods
        /// <summary>
        /// Save the state of Options
        /// </summary>
        public void Save()
        {
            vAdditionnalTitle1 = Options.Instance.AdditionnalTitle1;
            vAdditionnalTitle2 = Options.Instance.AdditionnalTitle2;
            vAdditionnalTitle3 = Options.Instance.AdditionnalTitle3;
            vAdditionnalTitle4 = Options.Instance.AdditionnalTitle4;
            vDaysBackColor = Options.Instance.DaysBackColor;
            vDaysBorderColor = Options.Instance.DaysBorderColor;
            vDaysNonInMonthColor = Options.Instance.DaysNonInMonthColor;
            vDaysTextColor = Options.Instance.DaysTextColor;
            vDaysTextFont = Options.Instance.DaysTextFont;
            vFirstMeetingFadeColor = Options.Instance.FirstMeetingFadeColor;
            vFontPrescriptionHeader = Options.Instance.FontPrescriptionHeader;
            vHeaderBackColor = Options.Instance.HeaderBackColor;
            vHeaderDayBackColor = Options.Instance.HeaderDayBackColor;
            vHeaderDayTextColor = Options.Instance.HeaderDayTextColor;
            vHeaderDayTextFont = Options.Instance.HeaderDayTextFont;
            vHeaderTextColor = Options.Instance.HeaderTextColor;
            vHeaderTextFont = Options.Instance.HeaderTextFont;
            vIsFirstInstance = Options.Instance.IsFirstInstance;
            vIsVisible1 = Options.Instance.IsVisible1;
            vIsVisible2 = Options.Instance.IsVisible2;
            vIsVisible3 = Options.Instance.IsVisible3;
            vIsVisible4 = Options.Instance.IsVisible4;
            vMeetingTextColor = Options.Instance.MeetingTextColor;
            vMeetingTextFont = Options.Instance.MeetingTextFont;
            vNonMeetingBackColor = Options.Instance.NonMeetingBackColor;
            vPataMenu = Options.Instance.PataMenu;
            vPath = Options.Instance.Path;
            vPegaMenu = Options.Instance.PegaMenu;
            vPrinterSettingsPrescritions = Options.Instance.PrinterSettingsPrescritions;
            vPrintPrescriptionHeader = Options.Instance.PrintPrescriptionHeader;
            vSecondMeetingFadeColor = Options.Instance.SecondMeetingFadeColor;
            vSMTPPassword = Options.Instance.SMTPPassword;
            vSMTPPort = Options.Instance.SMTPPort;
            vSMTPServer = Options.Instance.SMTPServer;
            vTodayBackColor = Options.Instance.TodayBackColor;
        }
        /// <summary>
        /// Undo the changes made in the vOptions
        /// </summary>
        /// <returns></returns>
        public void Undo()
        {
            Options.Instance.IsEventable = false;
            Options.Instance.AdditionnalTitle1 = vAdditionnalTitle1;
            Options.Instance.AdditionnalTitle2 = vAdditionnalTitle2;
            Options.Instance.AdditionnalTitle3 = vAdditionnalTitle3;
            Options.Instance.AdditionnalTitle4 = vAdditionnalTitle4;
            Options.Instance.DaysBackColor = vDaysBackColor;
            Options.Instance.DaysBorderColor = vDaysBorderColor;
            Options.Instance.DaysNonInMonthColor = vDaysNonInMonthColor;
            Options.Instance.DaysTextColor = vDaysTextColor;
            Options.Instance.DaysTextFont = vDaysTextFont;
            Options.Instance.FirstMeetingFadeColor = vFirstMeetingFadeColor;
            Options.Instance.FontPrescriptionHeader = vFontPrescriptionHeader;
            Options.Instance.HeaderBackColor = vHeaderBackColor;
            Options.Instance.HeaderDayBackColor = vHeaderDayBackColor;
            Options.Instance.HeaderDayTextColor = vHeaderDayTextColor;
            Options.Instance.HeaderDayTextFont = vHeaderDayTextFont;
            Options.Instance.HeaderTextColor = vHeaderTextColor;
            Options.Instance.HeaderTextFont = vHeaderTextFont;
            Options.Instance.IsFirstInstance = vIsFirstInstance;
            Options.Instance.IsVisible1 = vIsVisible1;
            Options.Instance.IsVisible2 = vIsVisible2;
            Options.Instance.IsVisible3 = vIsVisible3;
            Options.Instance.IsVisible4 = vIsVisible4;
            Options.Instance.MeetingTextColor = vMeetingTextColor;
            Options.Instance.MeetingTextFont = vMeetingTextFont;
            Options.Instance.NonMeetingBackColor = vNonMeetingBackColor;
            Options.Instance.PataMenu = vPataMenu;
            Options.Instance.Path = vPath;
            Options.Instance.PegaMenu = vPegaMenu;
            Options.Instance.PrinterSettingsPrescritions = vPrinterSettingsPrescritions;
            Options.Instance.PrintPrescriptionHeader = vPrintPrescriptionHeader;
            Options.Instance.SecondMeetingFadeColor = vSecondMeetingFadeColor;
            Options.Instance.SMTPPassword = vSMTPPassword;
            Options.Instance.SMTPPort = vSMTPPort;
            Options.Instance.SMTPServer = vSMTPServer;
            Options.Instance.TodayBackColor = vTodayBackColor;
            Options.Instance.IsEventable = true;
        }
        #endregion
    }

    #region DeckData
    /// <summary>
    /// Indique les menu qui sont actif dans Pata
    /// </summary>
    [Serializable]
    public class PataMenu
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public PataMenu() { }
        /// <summary>
        /// Indique si le menu PageStyle est visible
        /// </summary>
        public bool PageStyleVisible = false;
        /// <summary>
        /// Indique su le menu SeekCancel est visible
        /// </summary>
        public bool SeekCancelVisible = false;
        /// <summary>
        /// Indique si le menu General est visible
        /// </summary>
        public bool GeneralVisible = false;
        /// <summary>
        /// Location of the PageStyle menu
        /// </summary>
        public Point PageStyleLocation = new Point();
        /// <summary>
        /// Loction of the SeekCancel menu
        /// </summary>
        public Point SeekCancelLocation = new Point();
        /// <summary>
        /// Location of the General menu
        /// </summary>
        public Point GeneralLocation = new Point();
    }
    /// <summary>
    /// Indique les menu qui sont actif dans Pega
    /// </summary>
    [Serializable]
    public class PegaMenu
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public PegaMenu() { }
        /// <summary>
        /// Indique si le menu de gestion doit tre visible ou non
        /// </summary>
        public bool ManagmentVisible = false;
        /// <summary>
        /// Location of the menu
        /// </summary>
        public Point ManagmentLocation = new Point();
    }
    /// <summary>
    /// Indicates the position of the lateral menu
    /// </summary>
    [Serializable]
    public enum DeckPosition
    {
        /// <summary>
        /// On the left
        /// </summary>
        Gauche,
        /// <summary>
        /// On the right
        /// </summary>
        Droite,
        /// <summary>
        /// Up the window
        /// </summary>
        Haut,
        /// <summary>
        /// Down the window
        /// </summary>
        Bas,
    }
    #endregion
}
