using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Windows.Forms;
using Utils.Misc;
using PhoenixDLL.Class;

namespace PhoenixDLL.Class
{
    /// <summary>
    /// Gestionnaire des vOptions dans la base de registre
    /// </summary>
    public static class RegistryManager
    {
        #region Variables
        /// <summary>
        /// Path de base dans la base de registre pour le rpertoire d'installation
        /// </summary>
        private static readonly string RegistryPath = @"Software\Phoenix\";
        /// <summary>
        /// Clef CURRENT_USER
        /// </summary>
        private static RegistryKey RKey = Registry.CurrentUser.OpenSubKey(RegistryPath, true);
        #endregion

        #region Mthodes
        /// <summary>
        /// Rinitialise la path de l'application
        /// </summary>
        private static void ResetPath()
        {
            RKey.SetValue("Path", ApplicationPath.CommandPath);
        }
        /// <summary>
        /// Sets the path of the DataBase
        /// </summary>
        private static void ResetDataBase()
        {
            try
            {
                RKey = Registry.CurrentUser.CreateSubKey(RegistryPath, RegistryKeyPermissionCheck.Default);
                string path = Options.Instance.Path;
                RKey.SetValue("DataBase", path);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Sets the path of the vOptions file
        /// </summary>
        private static void ResetOptions()
        {
            RKey.SetValue("Options", Path + "\\Options.srz");
        }
        /// <summary>
        /// Sets the Registry for the Phoenix Suite
        /// </summary>
        /// <returns></returns>
        public static void ProcessRegistry()
        {
            ResetPath();
            ResetDataBase();
            ResetOptions();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Gets or sets the DataBase path in the Registry
        /// </summary>
        public static string DataBase
        {
            get { return Options.Instance.Path; }
        }
        /// <summary>
        /// Gets or sets the path of the Phoenix Suite
        /// </summary>
        public static string Path
        {
            get
            {
                try
                {
                    return (string)RKey.GetValue("Path");
                }
                catch (Exception) { return ""; }
            }
        }
        /// <summary>
        /// Retourne le path du fichier d'option
        /// </summary>
        /// <returns></returns>
        public static string Option
        {
            get
            {
                RegistryKey CUkey = Registry.CurrentUser;
                RegistryKey key = CUkey.OpenSubKey(RegistryPath, true);
                string Path;

                if (key == null)
                {
                    #region Path des vOptions
                    key = Registry.CurrentUser.CreateSubKey(RegistryPath, RegistryKeyPermissionCheck.Default);
                    Path = ApplicationPath.CommandPath + "\\Options.srz";
                    key.SetValue("Options", ApplicationPath.CommandPath + "\\Options.srz");
                    #endregion
                    ResetDataBase();
                    ResetPath();

                    return Path;
                }
                else
                {
                    Path = (string)key.GetValue("Options");

                    if (Path == null)
                    {
                        key.SetValue("Options", ApplicationPath.CommandPath + "\\Options.srz");
                        Path = ApplicationPath.CommandPath + "\\Options.srz";
                    }
                }
                return Path;
            }
        }
        #endregion
    }
}
