using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.Data.OleDb;
using Utils.Log;
using PhoenixDLL.GUI.Managers.Root;

namespace PhoenixDLL.GUI.Managers
{
    /// <summary>
    /// Gestionnaire graphique de patients
    /// </summary>
    public partial class fr_PatientManager : fr_RootManager
    {
        #region Variables
        /// <summary>
        /// Informations sur le patient
        /// </summary>
        protected PatientManager vPatient = new PatientManager();
        /// <summary>
        /// Identifiant du patient
        /// </summary>
        protected int vIDPatient = -1;
        /// <summary>
        /// Gestionnaire de SQL
        /// </summary>
        protected OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_PatientManager()
        {
            InitializeComponent();
            pg_PropertyGrid.SelectedObject = vPatient;
            base.vUse = Use.Update;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Donne le nom de la rputation
        /// </summary>
        /// <param name="_ID">Identifiant de la rputation</param>
        /// <returns>Le nom de la rputation</returns>
        private ReputationManager GetReputation(int _ID)
        {
            SqlManager.PrepareProcedure("SELECT_Reputation");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
            DataSet ds = new DataSet();
            int nbr = SqlManager.ExecuteSqlProcedure(ref ds);

            if (nbr > 0)
            {
                ReputationManager rep = new ReputationManager();
                rep.Label = (string)ds.Tables[0].Rows[0]["Label"];
                rep.ID = (int)ds.Tables[0].Rows[0]["ID_Reputation"];
                return rep;
            }
            else
                return new ReputationManager();
        }
        /// <summary>
        /// Donne le nom du mdecin sur son ID
        /// </summary>
        /// <param name="_ID">IDentifiant du mdecin</param>
        /// <returns>Nom du mdecin</returns>
        private DoctorManager GetDoctor(int _ID)
        {
            SqlManager.PrepareProcedure("SELECT_MedecinOnID");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
            DataSet ds = new DataSet();
            int nbr = SqlManager.ExecuteSqlProcedure(ref ds);


            if (nbr > 0)
            {
                DoctorManager doc = new DoctorManager();
                if (!ds.Tables[0].Rows[0]["Adresse"].Equals(DBNull.Value))
                    doc.Adresse = (string)ds.Tables[0].Rows[0]["Adresse"];

                if (!ds.Tables[0].Rows[0]["CodePostal"].Equals(DBNull.Value))
                    doc.CodePostal = (string)ds.Tables[0].Rows[0]["CodePostal"];

                if (!ds.Tables[0].Rows[0]["Fax"].Equals(DBNull.Value))
                    doc.Fax = (string)ds.Tables[0].Rows[0]["Fax"];

                if (!ds.Tables[0].Rows[0]["id_medecin"].Equals(DBNull.Value))
                    doc.ID = (int)ds.Tables[0].Rows[0]["id_medecin"];

                if (!ds.Tables[0].Rows[0]["Homeopathe"].Equals(DBNull.Value))
                    doc.IsHomeo = (bool)ds.Tables[0].Rows[0]["Homeopathe"];

                if (!ds.Tables[0].Rows[0]["MedecinTraitant"].Equals(DBNull.Value))
                    doc.IsMedecin = (bool)ds.Tables[0].Rows[0]["MedecinTraitant"];

                if (!ds.Tables[0].Rows[0]["Nom"].Equals(DBNull.Value))
                    doc.Nom = (string)ds.Tables[0].Rows[0]["Nom"];

                if (!ds.Tables[0].Rows[0]["Prenom"].Equals(DBNull.Value))
                    doc.Prenom = (string)ds.Tables[0].Rows[0]["Prenom"];

                if (!ds.Tables[0].Rows[0]["Telephone"].Equals(DBNull.Value))
                    doc.Telephone = (string)ds.Tables[0].Rows[0]["Telephone"];

                if (!ds.Tables[0].Rows[0]["Ville"].Equals(DBNull.Value))
                    doc.Ville = (string)ds.Tables[0].Rows[0]["Ville"];
                return doc;
            }
            else
                return new DoctorManager();
        }
        /// <summary>
        /// Donne les information d'un honoraire sur base de son identifiant
        /// </summary>
        /// <param name="_ID"></param>
        /// <returns></returns>
        private MutuelleManager GetHonoraire(int _ID)
        {
            SqlManager.PrepareProcedure("SELECT_Honoraire");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = _ID;
            DataSet ds = new DataSet();
            int nbr = SqlManager.ExecuteSqlProcedure(ref ds);

            if (nbr > 0)
            {
                MutuelleManager rep = new MutuelleManager();
                rep.Type = (string)ds.Tables[0].Rows[0]["Type"];
                rep.ID = (int)ds.Tables[0].Rows[0]["id_honoraire"];
                return rep;
            }
            else
                return new MutuelleManager();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise l'identifiant du patient  traiter
        /// </summary>
        [Browsable(false)]
        public int IdPatient
        {
            get { return vIDPatient; }
            set
            {
                vIDPatient = value;

                //Selection du patient
                string conStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
                string vPath = Options.GetInstance().Path;
                string st = conStr + vPath + "\"";

                OleDbConnection oConn = new OleDbConnection(st);

                OleDbCommand oCom = new OleDbCommand("SELECT_PatientOnID", oConn);
                oCom.CommandType = CommandType.StoredProcedure;
                OleDbDataAdapter oDa = new OleDbDataAdapter(oCom);

                oCom.Parameters.Add("[_ID]", OleDbType.Integer).Value = value;

                DataSet ds = new DataSet();
                oDa.Fill(ds);

                vPatient.IDPatient = (int)ds.Tables[0].Rows[0]["Id_Patient"];

                if (!ds.Tables[0].Rows[0]["Adresse"].Equals(DBNull.Value))
                    vPatient.Address = (string)ds.Tables[0].Rows[0]["Adresse"];

                if (!ds.Tables[0].Rows[0]["Ville"].Equals(DBNull.Value))
                    vPatient.City = (string)ds.Tables[0].Rows[0]["Ville"];

                if (!ds.Tables[0].Rows[0]["DateNaissance"].Equals(DBNull.Value))
                    vPatient.DateOfBirth = (DateTime)ds.Tables[0].Rows[0]["DateNaissance"];

                if (!ds.Tables[0].Rows[0]["Nom"].Equals(DBNull.Value))
                    vPatient.FamillyName = (string)ds.Tables[0].Rows[0]["Nom"];

                if (!ds.Tables[0].Rows[0]["ID_Homeopathe"].Equals(DBNull.Value))
                {
                    vPatient.Homeopathe.ID = (int)ds.Tables[0].Rows[0]["ID_Homeopathe"];
                    vPatient.Homeopathe = GetDoctor(vPatient.Homeopathe.ID);
                }

                if (!ds.Tables[0].Rows[0]["id_medecin"].Equals(DBNull.Value))
                {
                    vPatient.MedecinTraitant.ID = (int)ds.Tables[0].Rows[0]["id_medecin"];
                    vPatient.MedecinTraitant = GetDoctor(vPatient.MedecinTraitant.ID);
                }

                if (!ds.Tables[0].Rows[0]["id_Reputation"].Equals(DBNull.Value))
                {
                    vPatient.Reputation.ID = (int)ds.Tables[0].Rows[0]["id_Reputation"];
                    vPatient.Reputation = GetReputation(vPatient.Reputation.ID);
                }

                if (!ds.Tables[0].Rows[0]["Motif_1"].Equals(DBNull.Value))
                    vPatient.Motif1 = (string)ds.Tables[0].Rows[0]["Motif_1"];

                if (!ds.Tables[0].Rows[0]["Motif_2"].Equals(DBNull.Value))
                    vPatient.Motif2 = (string)ds.Tables[0].Rows[0]["Motif_2"];

                if (!ds.Tables[0].Rows[0]["Prenom"].Equals(DBNull.Value))
                    vPatient.Surname = (string)ds.Tables[0].Rows[0]["Prenom"];

                if (!ds.Tables[0].Rows[0]["Lieu_Naissance"].Equals(DBNull.Value))
                    vPatient.PlaceOfBirth = (string)ds.Tables[0].Rows[0]["Lieu_Naissance"];

                if (!ds.Tables[0].Rows[0]["Telephone_prive"].Equals(DBNull.Value))
                    vPatient.PrivatePhone = (string)ds.Tables[0].Rows[0]["Telephone_prive"];

                if (!ds.Tables[0].Rows[0]["Profession"].Equals(DBNull.Value))
                    vPatient.Profession = (string)ds.Tables[0].Rows[0]["Profession"];

                if (!ds.Tables[0].Rows[0]["Telephone_professionel"].Equals(DBNull.Value))
                    vPatient.ProPhone = (string)ds.Tables[0].Rows[0]["Telephone_professionel"];

                if (!ds.Tables[0].Rows[0]["Mail"].Equals(DBNull.Value))
                    vPatient.Mail = (string)ds.Tables[0].Rows[0]["Mail"];

                if (!ds.Tables[0].Rows[0]["Code_Postal"].Equals(DBNull.Value))
                    vPatient.ZipCode = (string)ds.Tables[0].Rows[0]["Code_Postal"];

                if (!ds.Tables[0].Rows[0]["id_Honoraire"].Equals(DBNull.Value))
                {
                    vPatient.Mutuelle.ID = (int)ds.Tables[0].Rows[0]["id_Honoraire"];
                    vPatient.Mutuelle = GetHonoraire(vPatient.Mutuelle.ID);
                }

                if (ds.Tables[0].Rows[0]["Sexe"].Equals(DBNull.Value))
                    vPatient.Sex = Sex.Femme; //Parce qu'elles sont plus nombreuse ;)

                else if (((string)ds.Tables[0].Rows[0]["Sexe"]).ToLower() == "m")
                    vPatient.Sex = Sex.Homme;
                else
                    vPatient.Sex = Sex.Femme;
            }
        }
        /// <summary>
        /// Retourne ou initialise les informations sur le patient
        /// </summary>
        [Browsable(false)]
        public PatientManager Manager
        {
            set
            {
                vPatient = value;
                pg_PropertyGrid.SelectedObject = value;
            }
            get { return vPatient; }
        }
        #endregion

        #region Events
        /// <summary>
        /// Clique sur OK
        /// </summary>               
        protected override void ProcessOK()
        {
            bool vClose = true;
            try
            {
                if (vUse == Use.Update)
                {
                    vPatient.Update();
                    base.DialogResult = DialogResult.OK;
                }
                if (vUse == Use.Add)
                {
                    if (vPatient.FamillyName == "" || vPatient.Surname == "")
                    {
                        MessageBox.Show("Un patient ne peut tre sans nom ou sans prnom!",
                            "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        vClose = false;
                        return;
                    }
                    if (vPatient.Mutuelle.ID < 1)
                    {
                        MessageBox.Show("Veuillez choisir une mutuelle car un patient ne peut tre sans mutuelle!",
                            "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        vClose = false;
                        return;
                    }
                    vPatient.Add();
                    base.DialogResult = DialogResult.OK;
                }
            }
            catch (Exception ex)
            {
                #region Deleting error
                if (vUse == Use.Delete)
                {
                    MessageBox.Show("Il est impossible d'effacer cette entre...\r\n("
                        + ex.Message + ")", "Asterisk", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                #endregion
                else
                {
                    MessageBox.Show("Une erreur est survenue (" + ex.Message + ")",
                        "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Log.Instance.Write(ex.Message, "fr_ReputationManager.ProcessOK()", LogType.Erreur);
                }
            }
            finally
            {
                if (vClose)
                    this.Close();
            }
        }
        #endregion
    }

    /// <summary>
    /// Diffrents type d'utilisation de la fentre 
    /// </summary>
    public enum Use
    {
        /// <summary>
        /// Ajout de patient
        /// </summary>
        Add,
        /// <summary>
        /// Modification de patient
        /// </summary>
        Update,
        /// <summary>
        /// Suppression d'un patient
        /// </summary>
        Delete,
    }
}

