using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.Data.OleDb;
using Utils.Log;
using System.Threading;
using PhoenixDLL.GUI.Seekers.Root;
using PhoenixDLL.GUI.Managers;

namespace PhoenixDLL.GUI.Seekers
{
    /// <summary>
    /// Chercheur de mdicament
    /// </summary>
    public partial class fr_DrugsSeeker : fr_RootSeeker
    {
        #region Variables
        /// <summary>
        /// Mdicament selectionn
        /// </summary>
        private DrugsManager vDrugsManager = new DrugsManager();
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_DrugsSeeker()
        {
            InitializeComponent();
            base.Text = "Recherche de mdicaments...";
            base.DeleteVisible = true;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Refreshes the DataGridView
        /// </summary>
        protected override void RefreshDGV()
        {
            base.tsPB_ProgressBar.Style = ProgressBarStyle.Marquee;
            lbl_Type.Text = "";

            //Cration et lancement du thread
            if (th_Seeker != null && !th_Seeker.IsAlive)
            {
                ThreadDrugs th = new ThreadDrugs(this);
                th.DrugName = tb_Nom.Text;

                th_Seeker = new Thread(new ThreadStart(th.ProcessOnName));
                th_Seeker.Name = "DataBase Thread";
                th_Seeker.Start();
            }
            else
            {
                if (th_Seeker != null)
                    th_Seeker.Abort();

                ThreadDrugs th = new ThreadDrugs(this);
                th.DrugName = tb_Nom.Text;

                th_Seeker = new Thread(new ThreadStart(th.ProcessOnName));
                th_Seeker.Name = "DataBase Thread";
                th_Seeker.Start();
            }
        }
        #endregion

        #region Events
        /// <summary>
        /// Evnement sur demande de type de rendez-vous
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Type(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.btn_AddReputation.Visible = false;

            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                lbl_Type.Text = fr.SelectedItem.Type;
                base.tsPB_ProgressBar.Style = ProgressBarStyle.Marquee;

                //Cration et lancement du thread
                if (th_Seeker != null && !th_Seeker.IsAlive)
                {
                    ThreadDrugs th = new ThreadDrugs(this);
                    th.DrugsTypeManager = fr.SelectedItem;

                    th_Seeker = new Thread(new ThreadStart(th.ProcessOnType));
                    th_Seeker.Name = "DataBase Thread";
                    th_Seeker.Start();
                }
                else
                {
                    if (th_Seeker != null)
                        th_Seeker.Abort();

                    ThreadDrugs th = new ThreadDrugs(this);
                    th.DrugsTypeManager = fr.SelectedItem;
                    th_Seeker = new Thread(new ThreadStart(th.ProcessOnType));
                    th_Seeker.Name = "DataBase Thread";
                    th_Seeker.Start();
                }

            }
        }
        /// <summary>
        /// Evnement sur un changement de texte
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_TextChanged(object sender, EventArgs e) { RefreshDGV(); }
        /// <summary>
        /// Traitement sur un double clique dans le DataGridView
        /// </summary>
        protected override void ExecuteDoubleClick()
        {
            if (dgv_Results.SelectedRows.Count < 1) return;
            vDrugsManager.DrugsType.ID = (int)dgv_Results.SelectedRows[0].Cells["id_Type"].Value;
            vDrugsManager.DrugsType = DrugsTypeManager.Create(vDrugsManager.DrugsType.ID);
            vDrugsManager.ID = (int)dgv_Results.SelectedRows[0].Cells["id_Medicament"].Value;
            vDrugsManager.Name = (string)dgv_Results.SelectedRows[0].Cells["Nom"].Value;
            vDrugsManager.Remark = (string)dgv_Results.SelectedRows[0].Cells["Rmq"].Value;
            base.ExecuteDoubleClick();
        }
        /// <summary>
        /// Excute un ajout de mdicament
        /// </summary>
        protected override void ExecuteAdd()
        {
            fr_DrugsManager fr = new fr_DrugsManager();
            fr.Manager = new DrugsManager();
            fr.Use = Use.Add;
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            RefreshDGV();
            
        }
        /// <summary>
        /// Excute une suppression d'un mdicament
        /// </summary>
        protected override void ExecuteDel()
        {
            try
            {
                DrugsManager dm = DrugsManager.Create((int)dgv_Results.SelectedRows[0].Cells["id_Medicament"].Value);
                DialogResult dr = MessageBox.Show("Voulez-vous supprimer ce mdicament (" + dm.ToString()
                    + ")?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr != DialogResult.Yes) return;
                dm.Delete();
                RefreshDGV();
            }
            catch (OleDbException ex)
            {
                MessageBox.Show("Impossible de supprimer le mdicament.\r\n" + ex.Message,
                    "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue.\r\n" + ex.Message,
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne le mdicament slectionn
        /// </summary>
        public DrugsManager SelectedItem { get { return vDrugsManager; } }
        #endregion
    }

    /// <summary>
    /// Thread pour la recherche des mdicaments
    /// </summary>
    internal class ThreadDrugs
    {
        #region Variables
        /// <summary>
        /// Gestionnaire de type de mdicament pour une recherche par type
        /// </summary>
        private DrugsTypeManager vDrugsTypeManager = new DrugsTypeManager();

        /// <summary>
        /// Nom du mdicament  trouver
        /// </summary>
        private string vDrugName = "";

        /// <summary>
        /// Gestionnaire de SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);

        /// <summary>
        /// Delegate pour le traitement GUI inter threads
        /// </summary>
        /// <param name="_ds"></param>
        private delegate void UpdateGuiHandler(DataSet _ds);

        /// <summary>
        /// Mthode de tratement inter threads
        /// </summary>
        private UpdateGuiHandler UpdateGui;

        /// <summary>
        /// Rfrence sur la fentre d'appel du thread pour le travail sur GUI
        /// </summary>
        private fr_DrugsSeeker vFrame;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_frame">Rfrence sur la frame sur laquelle se fait le traitement GUI</param>
        public ThreadDrugs(fr_DrugsSeeker _frame)
        {
            vFrame = _frame;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait une recherche de mdicament sur son type
        /// </summary>
        public void ProcessOnType()
        {
            SqlManager.PrepareProcedure("SELECT_MedicamentOnType");
            SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = this.DrugsTypeManager.ID;
            DataSet ds = new DataSet();

            try
            {
                SqlManager.ExecuteSqlProcedure(ref ds);

                UpdateGui = new UpdateGuiHandler(this.FillDataGridView);
                try { vFrame.dgv_Results.Invoke(UpdateGui, ds); }
                catch (InvalidOperationException ex)
                { Log.GetInstance().Write(ex.Message, "ThreadDrugs.ProcessOnType()", LogType.Erreur); }
            }
            catch (OleDbException ex)
            {
                Log.GetInstance().Write(ex.Message, "fr_DrugsSeeker()", LogType.Erreur);
            }
        }
        /// <summary>
        /// Fait une recherche de mdicament sur son nom
        /// </summary>
        public void ProcessOnName()
        {
            string Nom = vDrugName + "%";
            SqlManager.PrepareProcedure("SELECT_MedicamentOnName");
            SqlManager.Parameters.Add("[_Nom]", OleDbType.VarChar).Value = Nom;
            DataSet ds = new DataSet();

            try
            {
                int i = SqlManager.ExecuteSqlProcedure(ref ds);
                UpdateGui = new UpdateGuiHandler(this.FillDataGridView);
                try { vFrame.dgv_Results.Invoke(UpdateGui, ds); }
                catch (InvalidOperationException ex)
                { Log.GetInstance().Write(ex.Message, "ThreadDrugs.ProcessOnName()", LogType.Erreur); }
            }
            catch (OleDbException ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Rempli le DataGridView avec les informations issues de la recherche
        /// </summary>
        /// <param name="ds"></param>
        private void FillDataGridView(DataSet ds)
        {
            try
            {
                this.vFrame.dgv_Results.DataSource = ds.Tables[0];


                for (int i = 0; i < this.vFrame.dgv_Results.Columns.Count; i++)
                    this.vFrame.dgv_Results.Columns[i].Visible = false;

                this.vFrame.dgv_Results.Columns["Nom"].Visible = true;
                this.vFrame.dgv_Results.Columns["Nom"].HeaderText = "Mdicament";

                this.vFrame.tsPB_ProgressBar.Style = ProgressBarStyle.Blocks;
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "FillDataGridView", LogType.Erreur);
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le DrugsTypeManager pour la recherche par type
        /// </summary>
        public DrugsTypeManager DrugsTypeManager { get { return vDrugsTypeManager; } set { vDrugsTypeManager = value; } }

        /// <summary>
        /// Retourne ou initialise le nom du mdicament sur lequel on fait une recherche
        /// </summary>
        public string DrugName { get { return vDrugName; } set { vDrugName = value; } }
        #endregion
    }
}

