using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.Data.OleDb;
using Utils.Misc;
using PhoenixDLL.Print;
using Components.Agenda.Management.StructsEnums;

namespace PhoenixDLL.GUI
{
    /// <summary>
    /// Visionneuse de rendez-vous pour un patient
    /// </summary>
    public partial class fr_MeetingsViewer : Form
    {
        #region Variables
        /// <summary>
        /// Patient pour lequel on recherche les rendez-vous
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// Gestionnaire de code SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Manager d'impression de rendez-vous
        /// </summary>
        private MeetingPrint vMeetingPrint = new MeetingPrint();
        /// <summary>
        /// Informations sur les rendez-vous
        /// </summary>
        private MeetingSet ms = new MeetingSet();
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_MeetingsViewer()
        {
            InitializeComponent();
            lbl_Nom.Text = "";            
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le patient pour lequel on recherche les rendez-vous
        /// </summary>
        public PatientManager PatientManager
        {
            get { return vPatientManager; }
            set
            {
                vPatientManager = value;
                lbl_Nom.Text = value.ToString();

                ms = new MeetingSet();
                ms.Fill(vPatientManager.IDPatient);
                if (ms.MeetingList.Count < 1) return;
                ms.Sort();

                vMeetingPrint.PatientManager = vPatientManager;
                vMeetingPrint.MeetingSet = ms;

                foreach (MeetingRow mr in ms.MeetingList)
                {
                    DataGridViewRow row = new DataGridViewRow();
                    dgv_Results.Rows.Add(mr.ToString());
                }                  
            }
        }
        #endregion

        #region Events
        /// <summary>
        /// Quitter la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e) { this.Close(); }
        /// <summary>
        /// Demande d'impression du relev de rendez-vous
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PrintPreview(object sender, EventArgs e)
        {
            vMeetingPrint.Document.PrinterSettings = Options.Instance.PrinterSettingsMeetings;
            ppd.Document = vMeetingPrint.Document;
            ppd.ShowDialog();
        }
        /// <summary>
        /// Menu de configuration de l'imprimante
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SetupPrinter(object sender, EventArgs e)
        {
            PrintDialog pd = new PrintDialog();
            pd.PrinterSettings = Options.Instance.PrinterSettingsMeetings;
            DialogResult dr = pd.ShowDialog();
            if (dr != DialogResult.OK) return;

            Options.Instance.PrinterSettingsMeetings = pd.PrinterSettings;
            vMeetingPrint.Document.PrinterSettings = pd.PrinterSettings;
        }
        #endregion
    }
}