using System;
using System.Collections.Generic;
using System.Text;
using PhoenixDLL.Class;
using PhoenixDLL.Print.Root;
using System.Drawing.Printing;
using PhoenixDLL.Class.Managers;
using System.Drawing;
using Utils.Misc;
using Components.Agenda.Management;
using System.Windows.Forms;
using PhoenixDLL.Class.Exceptions;

namespace PhoenixDLL.Print
{
    /// <summary>
    /// Rendez-vous d'un patient  imprimer
    /// </summary>
    public class MeetingPrint : RootPrint
    {
        #region Variables
        /// <summary>
        /// Informations sur les rendez-vous
        /// </summary>
        private MeetingSet vMeetingSet = new MeetingSet();
        /// <summary>
        /// Informations sur le patient
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// Bool qui dit si c'est la premire page d'impression
        /// </summary>
        private bool vIsFirstPage = true;
        /// <summary>
        /// Index du passage en revue du MeetingSet
        /// </summary>
        private int Index = 0;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>   
        public MeetingPrint() : base() { }
        #endregion

        #region Mthodes
        /// <summary>
        /// Excute le travail d'impression
        /// </summary>
        /// <param name="e"></param>
        protected override void ProcessPrint(PrintPageEventArgs e)
        {
            if (vMeetingSet.MeetingList.Count < 1) return;

            Font font = new Font("Times New Roman", 12);
            PointF point;
            try
            {
                #region Premiere page
                if (vIsFirstPage)
                {
                    string s = "madame ";
                    if (PatientManager.Sex == Sex.Homme)
                        s = "monsieur ";
                    string Txt = "Rendez-vous de " + s + " ";

                    point = new PointF(e.PageSettings.Margins.Left, e.PageSettings.Margins.Top);

                    #region Date
                    string Date = TextManager.FirstUpper(DateTime.Today.ToLongDateString());
                    SizeF size = e.Graphics.MeasureString(Date, font);
                    e.Graphics.DrawString(Date, font, new SolidBrush(Color.Black),
                        new PointF(e.PageBounds.Width - (e.PageSettings.Margins.Right + size.Width), point.Y));
                    #endregion

                    #region Patient
                    point = new PointF(e.PageSettings.Margins.Left, point.Y + (font.Height * 2));
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Bold);
                    e.Graphics.DrawString(Txt + vPatientManager.ToString(), font, new SolidBrush(Color.Black), point);
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);
                    #endregion

                    #region Ligne
                    point = new PointF(e.PageSettings.Margins.Left, point.Y + (2 * font.Height));
                    e.Graphics.DrawLine(new Pen(Color.Gray), point, new PointF(point.X + e.PageSettings.Margins.Right, point.Y));
                    #endregion
                }
                #endregion

                #region Autre page
                else
                    point = new PointF(e.PageSettings.Margins.Left, e.PageSettings.Margins.Top);
                #endregion

                #region Rendez-vous
                //foreach (MeetingRow mr in vMeetingSet)
                for (; Index < vMeetingSet.Count; Index++)
                {
                    point = new PointF(e.PageSettings.Margins.Left, point.Y + (font.Height * 2));

                    #region Vrifiaction si fin de page
                    if (point.Y + (font.Height * 3) > e.MarginBounds.Height)
                    {
                        e.HasMorePages = true;
                        return;
                    }
                    #endregion

                    e.Graphics.DrawString((Index + 1) + "- " + vMeetingSet[Index].ToString(),
                        font, new SolidBrush(Color.Black), point);
                }
                #endregion
            }
            catch (OutOfPageException ex)
            {
                e.Graphics.Clear(Color.White);
                MessageBox.Show(ex.Message, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le MeetingSet contenant les rendez-vous d'un patient donn par PatientManager
        /// </summary>
        public MeetingSet MeetingSet { get { return vMeetingSet; } set { vMeetingSet = value; } }
        /// <summary>
        /// Retourne ou initialise le patient pour lequel il faut trouver les rendez-vous
        /// </summary>
        public PatientManager PatientManager { get { return vPatientManager; } set { vPatientManager = value; } }
        #endregion
    }

}
