using System;
using System.Collections.Generic;
using System.Text;
using PhoenixDLL.Class.Managers;
using System.Drawing.Printing;
using System.Drawing;
using System.Windows.Forms;
using PhoenixDLL.Class;
using PhoenixDLL.Print.Root;
using Utils.Misc;
using PhoenixDLL.Class.Exceptions;

namespace PhoenixDLL.Print
{
    /// <summary>
    /// Prescriptions  imprimer
    /// </summary>
    public class PrescriptionPrint : RootPrint
    {
        #region Variables
        /// <summary>
        /// Liste des prescriptions
        /// </summary>
        private List<PrescriptionManager> vList = new List<PrescriptionManager>();
        /// <summary>
        /// Header de la prescription
        /// </summary>
        private string vHeader = Options.Instance.PrintPrescriptionHeader;

        #region Impression
        /// <summary>
        /// Bool qui dit si le Header a t dj fait
        /// </summary>
        private bool vHeaderDone = false;
        /// <summary>
        /// Buffer de prescriptions
        /// </summary>
        private string[,] Buffer;
        /// <summary>
        /// Largeur maximum des mdicaments
        /// </summary>
        SizeF MaxWidth;
        #endregion
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public PrescriptionPrint() : base() { }
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Prescriptions">Liste des prescriptions  imprimer</param>
        public PrescriptionPrint(List<PrescriptionManager> _Prescriptions)
            : base() { vList = _Prescriptions; }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait le traitement d'un vnement d'impression
        /// </summary>
        /// <param name="e"></param>
        protected override void ProcessPrint(PrintPageEventArgs e)
        {
            try
            {
                Font font = new Font("Times New Roman", 8);
                //PointF point = new PointF(e.PageSettings.Margins.Left, e.PageSettings.Margins.Top);
                PointF point = new PointF(10, 10);

                if (!vHeaderDone)
                {
                    #region Variables
                    Buffer = new string[vList.Count, 2];
                    MaxWidth = new SizeF(e.PageSettings.Margins.Left, 0);
                    #endregion

                    #region Header
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Bold);

                    #region Date

                    string Date;
                    if (vList.Count > 0)
                        Date = vList[0].Date.ToLongDateString();
                    else
                        Date = DateTime.Today.ToLongDateString();

                    SizeF size = e.Graphics.MeasureString(Date, font);
                    e.Graphics.DrawString(Date, font, new SolidBrush(Color.Black),
                        new PointF(e.PageBounds.Width - size.Width - 10, point.Y));
                    //new PointF(e.PageBounds.Width - (e.PageSettings.Margins.Right + size.Width), point.Y));
                    #endregion

                    #region Header
                    e.Graphics.DrawString(vHeader, font, new SolidBrush(Color.Black),
                        new PointF(point.X, point.Y + size.Height));
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);

                    string[] _Args = { "\r\n", "\n" };
                    int Lines = TextManager.CountIn(vHeader, _Args);
                    //point = new PointF(e.PageSettings.Margins.Left, point.Y + font.Height * Lines + 20);
                    point = new PointF(10, point.Y + font.Height * Lines + 20);
                    e.Graphics.DrawLine(new Pen(Color.Gray), point, new PointF(point.X + e.PageSettings.Margins.Right, point.Y));
                    #endregion

                    #region Pour
                    if (vList.Count < 1) return;
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Italic);
                    point = new PointF(point.X, point.Y + font.Height);
                    string Sexe = "madame ";
                    if (vList[0].PatientManager.Sex == Sex.Homme)
                        Sexe = "monsieur ";
                    e.Graphics.DrawString("Pour " + Sexe + " " + vList[0].PatientManager.ToString(), font, new SolidBrush(Color.Black), point);
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);
                    point = new PointF(point.X, point.Y + (2 * font.Height));
                    e.Graphics.DrawLine(new Pen(Color.Gray), point, new PointF(point.X + e.PageSettings.Margins.Right, point.Y));
                    //point = new PointF(point.X, point.Y + font.Height);// * 2);
                    #endregion
                    #endregion

                    #region Liste des prescriptions
                    //Remplissage d'un buffer qui est la liste de toutes les presctions du jour de ce patient
                    for (int i = 0; i < vList.Count; i++)
                    {
                        Buffer[i, 0] = vList[i].DrugsManager.Name + " (" + vList[i].DrugsManager.DrugsType.Type + ")";
                        Buffer[i, 1] = vList[i].Remark.Replace("\r\n", " "); ;
                    }
                    #endregion
                }

                #region Prescription
                int index = -1;
                while (++index < (Buffer.Length / 2))
                {
                    SizeF StringSize = e.Graphics.MeasureString(Buffer[index, 1], font);
                    point = new PointF(point.X, point.Y + (2 * font.Height));

                    #region Nombre de lignes
                    int Lines = 1;
                    if (StringSize.Width > (e.PageBounds.Width - 20))
                    {
                        Lines = (int)(StringSize.Width / (e.PageBounds.Width - 20));

                        if ((StringSize.Width % (e.PageBounds.Width - 20)) != 0)
                            Lines++;
                    }
                    #endregion

                    #region Une remarque est trop grande pour une feuille
                    if (e.PageBounds.Height < ((/*e.PageBounds.Height -*/ 20) + (font.Height * Lines + 1)))
                        throw new OutOfPageException();
                    #endregion

                    #region Test fin de page
                    //Si on est en fin de page, on retourne en dbut de boucle pour continuer l'impression
                    if (point.Y + (font.Height * Lines) + 60 > e.MarginBounds.Height)
                    {
                        string[,] BBuf = new string[(Buffer.Length / 2) - index, 2];
                        int k = 0;
                        for (int j = index; j < Buffer.Length / 2; j++, k++)
                        {
                            BBuf[k, 0] = Buffer[j, 0];
                            BBuf[k, 1] = Buffer[j, 1];
                        }
                        Buffer = BBuf;
                        index = 0;

                        e.HasMorePages = true;
                        vHeaderDone = true;
                        return;
                    }
                    #endregion

                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Underline);
                    e.Graphics.DrawString(Buffer[index, 0], font, new SolidBrush(Color.Black), point);
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);
                    point = new PointF(point.X + 20, point.Y + (font.Height));// * 2));

                    e.Graphics.DrawString(Buffer[index, 1], font, new SolidBrush(Color.Black),
                        new RectangleF(point, new SizeF(e.PageBounds.Width - 20 - 20, Lines * font.Height)));

                    point = new PointF(point.X - 20, point.Y + (Lines * font.Height));
                }

                #endregion

                #region Signature
                e.HasMorePages = false;
                vHeaderDone = false;
                font = new Font(font, FontStyle.Bold);
                e.Graphics.DrawString("Signature:", font, new SolidBrush(Color.Black),
                     new PointF(10 /*+ point.X*/, point.Y + 2 * (font.Height)));// + 10)));
                #endregion
            }
            catch (OutOfPageException ex)
            {
                e.Graphics.Clear(Color.White);
                MessageBox.Show(ex.Message, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }
        /// <summary>
        /// Refresh the header of prescriptions
        /// </summary>
        public void RefreshHeader() { vHeader = Options.Instance.PrintPrescriptionHeader; }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la liste des prescriptions  imprimer
        /// </summary>
        public List<PrescriptionManager> PrescriptionList { get { return vList; } set { vList = value; } }        
        #endregion
    }
}
