using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace PhoenixDLL.Print.Root
{
    /// <summary>
    /// Classe de base pour la gestion des impressions
    /// </summary>
    public class RootPrint
    {
        #region Variables
        /// <summary>
        /// Document  imprimer
        /// </summary>
        private PrintDocument vDocument = new PrintDocument();
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public RootPrint() { vDocument.PrintPage += new PrintPageEventHandler(Event_PrintPage); }
        #endregion

        #region Mthodes
        /// <summary>
        /// Demande une impression
        /// </summary>
        public void Print() { Document.Print(); }

        /// <summary>
        /// Fait une prview de l'impression
        /// </summary>
        public void Preview()
        {
            PrintPreviewDialog ppd = new PrintPreviewDialog();
            ppd.ShowIcon = false;
            ppd.Document = Document;
            ppd.Text = "Visualisation";
            ppd.ShowDialog();
        }

        /// <summary>
        /// Fait le traitement de l'vnement d'impression
        /// </summary>
        /// <param name="e"></param>
        protected virtual void ProcessPrint(PrintPageEventArgs e) { }
        #endregion

        #region Events
        /// <summary>
        /// Evnemtent sur demande d'impression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PrintPage(object sender, PrintPageEventArgs e)
        {
            ProcessPrint(e);
        }


        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le document  imprimer
        /// </summary>
        public PrintDocument Document { get { return vDocument; } set { vDocument = value; } }

        /// <summary>
        /// Retourne ou initialise la mise en page du document
        /// </summary>
        public PageSettings PageSettings
        {
            get { return vDocument.DefaultPageSettings; }
            set { vDocument.DefaultPageSettings = value; }
        }
        #endregion
    }
}
