using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.GUI.Managers;

namespace PhoenixMoney.GUI
{
    /// <summary>
    /// Evaluation of the debt of a patient
    /// </summary>
    public partial class fr_Evaluate : Form
    {
        #region Variables
        /// <summary>
        /// The patient.
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// The debt
        /// </summary>
        private int vTempDebt = 0;
        /// <summary>
        /// Number of meetings unpayed
        /// </summary>
        private int vTempMeetings = 0;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Evaluate()
        {
            InitializeComponent();
            lbl_Debt.Visible = false;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Refreshes the data of the menu
        /// </summary>
        private void RefreshData()
        {
            #region Patient
            lbl_Nom.Text = vPatientManager.ToString();
            lbl_DateOfBirth.Text = "Date de naissance: " + vPatientManager.DateOfBirth.ToShortDateString();
            #endregion

            #region Meetings
            OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
            SqlManager.SqlQuery = "SELECT COUNT(*) FROM RendezVous WHERE id_Patient = " + vPatientManager.IDPatient
                + " AND a_paye = false AND Debut < DATE();";
            vTempMeetings = (int)SqlManager.ExecuteScalar();
            lbl_MissedMeetings.Text = "Rendez-vous non pays: " + vTempMeetings;
            #endregion

            #region Debt
            vTempDebt = (vTempMeetings * vPatientManager.Honoraire);
            lbl_Debt.Text = "Evaluation de la dette: " + vTempDebt + "";
            lbl_Debt.Visible = true;
            lbl_HonorairePatient.Text = "Honoraire: " + vPatientManager.Honoraire + "";
            lbl_ActualDebt.Text = "Dette actuelle: " + vPatientManager.Debt + "";
            #endregion
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the patient
        /// </summary>
        public PatientManager PatientManager
        {
            get { return vPatientManager; }
            set
            {
                vPatientManager = value;
                RefreshData();
            }

        }
        /// <summary>
        /// gets or sets the id of the patient
        /// </summary>
        public int IDPatient
        {
            get { return vPatientManager.IDPatient; }
            set
            {
                vPatientManager = PatientManager.Create(value);
                RefreshData();
            }
        }
        #endregion

        #region Event
        /// <summary>
        /// Modify the patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Modify(object sender, EventArgs e)
        {
            fr_PatientManager fr = new fr_PatientManager();
            fr.Manager = vPatientManager;
            fr.Size = new Size(fr.Size.Width, 600);
            fr.Use = Use.Update;
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            vPatientManager = fr.Manager;
            RefreshData();
        }
        /// <summary>
        /// Recalculate the Debt        
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_RecalculateDebt(object sender, EventArgs e)
        {
            int Price = rb_Automatic.Checked ? vPatientManager.Honoraire : (int)nud_Tempo.Value;
            vTempDebt = Price * vTempMeetings;
            lbl_Debt.Text = "Evaluation de la dette: " + (int)vTempDebt + "";
        }
        /// <summary>
        /// Update the debt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Update(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Voulez-vous mettre  jour la dette du patient?", "Question", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (dr == DialogResult.Cancel) return;
            else if (dr == DialogResult.No) Close();
            else
            {
                vPatientManager.SetDebt(vTempDebt);
                vPatientManager.Update();
                Close();
            }
        }
        /// <summary>
        /// RadioButton checked changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_Automatic.Checked)
            {
                nud_Tempo.Value = vPatientManager.Honoraire;
                nud_Tempo.Enabled = false;
            }
            else
            {
                nud_Tempo.Enabled = true;
            }
        }
        /// <summary>
        /// NumericUpDown Changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_NumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            lbl_HonorairePatient.Text = "Honoraire: " + nud_Tempo.Value + "";
        }
        #endregion
    }
}