using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.DataBase;
using PhoenixDLL.Class;
using Utils.Log;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Managers;
using PhoenixMoney.GUI;

namespace PhoenixMoney.GUI
{
    /// <summary>
    /// History of the patients which did not come to meetings
    /// </summary>
    public partial class fr_HistoryNonCameMeeting : Form
    {
        #region Variables
        /// <summary>
        /// The type of meetings the frame needs to process
        /// </summary>
        private FrameUse vUse = FrameUse.UnpayedMeetings;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_HistoryNonCameMeeting()
        {
            InitializeComponent();
        }
        #endregion

        #region Events
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e) { RefreshFrame(); }
        /// <summary>
        /// Right click on the DataGridView select the row
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseDown_Dgv(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo HitTestInfo = this.dgv_NonMeetings.HitTest(e.X, e.Y);
                if (HitTestInfo.RowIndex > -1)
                {
                    this.dgv_NonMeetings.SelectedRows[0].Selected = false;
                    this.dgv_NonMeetings.Rows[HitTestInfo.RowIndex].Selected = true;
                }
            }
        }
        /// <summary>
        /// Put the meeting as "Came to meeting"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_CameToMeeting(object sender, EventArgs e)
        {
            int IDMeeting = (int)dgv_NonMeetings.SelectedRows[0].Cells["id_rdv"].Value;
            PartialMeetingManager mm = PartialMeetingManager.Create(IDMeeting);
            fr_MeetingManager fr = new fr_MeetingManager();
            fr.Use = Use.Update;
            fr.MeetingManager = mm;
            DialogResult dr =fr.ShowDialog();
            if (dr != DialogResult.OK) return;

            RefreshFrame();
        }
        /// <summary>
        /// Modify Patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Modify(object sender, EventArgs e)
        {
            int IDPatient = (int)dgv_NonMeetings.SelectedRows[0].Cells["id_Patient"].Value;
            fr_PatientManager fr = new fr_PatientManager();
            fr.Manager = PatientManager.Create(IDPatient);
            fr.Use = Use.Update;
            fr.Size = new Size(fr.Size.Width, 600);
            string Name = fr.Manager.FamillyName;
            string Surname = fr.Manager.Surname;
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            if (fr.Manager.Surname != Surname || fr.Manager.FamillyName != Name)
                RefreshFrame();
        }
        /// <summary>
        /// Manage the debt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Dette(object sender, EventArgs e)
        {
            int IDPatient = (int)dgv_NonMeetings.SelectedRows[0].Cells["id_Patient"].Value;
            fr_Evaluate fr = new fr_Evaluate();
            fr.IDPatient = IDPatient;
            fr.ShowDialog();
        }
        #endregion

        #region Methods
        /// <summary>
        /// Refresh the database data
        /// </summary>
        private void RefreshFrame()
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                switch (vUse)
                {
                    case FrameUse.UnpayedMeetings:
                        SqlManager.SqlQuery = "SELECT p.id_Patient,p.Nom, p.Prenom, p.DateNaissance, r.Debut, r.id_rdv,"
                       + " FORMAT(r.Fin, \"hh:nn\")as Fin, p.Honoraire"
                       + " FROM Patient p, RendezVous r WHERE r.id_Patient = p.id_Patient"
                       + " AND r.a_paye = false AND Debut < DATE();";
                        break;
                    case FrameUse.UncameMeetings:
                        SqlManager.SqlQuery = "SELECT p.id_Patient,p.Nom, p.Prenom, p.DateNaissance, r.Debut, r.id_rdv,"
                       + " FORMAT(r.Fin, \"hh:nn\")as Fin, p.Honoraire"
                       + " FROM Patient p, RendezVous r WHERE r.id_Patient = p.id_Patient"
                       + " AND r.came_to_rdv = false AND Debut < DATE();";
                        break;
                }
                DataSet ds = new DataSet();
                int res = SqlManager.ExecuteSql(ref ds);
                if (res < 1)
                {
                    MessageBox.Show("Il n'y a aucun rendez-vous non pay!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                }

                #region DataGridView displaying
                dgv_NonMeetings.DataSource = ds.Tables[0];
                dgv_NonMeetings.Columns["id_Patient"].Visible = false;
                dgv_NonMeetings.Columns["id_rdv"].Visible = false;
                dgv_NonMeetings.Columns["Debut"].DisplayIndex = 0;
                dgv_NonMeetings.Columns["Debut"].HeaderText = "Dbut du rendez-vous";
                dgv_NonMeetings.Columns["Fin"].DisplayIndex = 1;

                dgv_NonMeetings.Columns["Nom"].DisplayIndex = 2;
                dgv_NonMeetings.Columns["Prenom"].DisplayIndex = 3;
                dgv_NonMeetings.Columns["DateNaissance"].DisplayIndex = 4;
                dgv_NonMeetings.Columns["DateNaissance"].HeaderText = "Date de naissance";

                dgv_NonMeetings.Columns["Fin"].HeaderText = "Fin du rendez-vous";
                dgv_NonMeetings.Columns["Honoraire"].DisplayIndex = 5;
                #endregion

            }
            #region Exception
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de faire le listing des rendez-vous non pays!\r\n(" + ex.Message + ")",
                    "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                Log.Instance.Write(ex.Message, "fr_HistoriqueNonCameMeeting.Ctor", LogType.Erreur);
            }
            #endregion
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the use of the frame
        /// </summary>
        public FrameUse FrameUse
        {
            get { return vUse; }
            set
            {
                vUse = value;
                switch (value)
                {
                    case FrameUse.UnpayedMeetings:
                        lbl_Explain.Text = "    Voici la liste des rendez-vous ou le patient s'est prsent mais qui ne sont pas marqus comme pays. L'honoraire indiqu dans la liste est le dernier honoraire li au patient et non l'honoraire en cours au moment de la prise de rendez-vous. Vous pouvez choisir de marquer un ou plusieurs de ces rendez-vous comme prests. Vous pouvez galement choisir de modifier les honoraires demand au patient ou demander une valuation pour recalculer la somme due de ce patient.";
                        break;
                    case FrameUse.UncameMeetings:
                        lbl_Explain.Text = "    Voici la liste des rendez-vous ou le patient NE s'est PAS prsent. L'honoraire indiqu dans la liste est le dernier honoraire li au patient et non l'honoraire en cours au moment de la prise de rendez-vous. Vous pouvez choisir de marquer un ou plusieurs de ces rendez-vous comme prests. Vous pouvez galement choisir de modifier les honoraires demand au patient ou demander une valuation pour recalculer la somme due de ce patient.";
                        break;
                }
            }
        }
        #endregion
    }
    /// <summary>
    /// Use of the frame
    /// </summary>
    public enum FrameUse
    {
        /// <summary>
        /// Meetings the patient did not pay
        /// </summary>
        UnpayedMeetings,
        /// <summary>
        /// Meetings the patient did not came
        /// </summary>
        UncameMeetings,

    }
}