using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using PhoenixDLL.Class;
using PhoenixDLL.GUI;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using Utils.Log;

namespace PhoenixMoney.GUI
{
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Identifiant du patient  grer
        /// </summary>
        private int vIDPatient = -1;
        /// <summary>
        /// Dette lors de la selection d'un patient endett
        /// </summary>
        private int vSelectedDebts = 0;
        /// <summary>
        /// ID du payeur de dette
        /// </summary>
        private int vSelectedDebtsSolder = -1;
        /// <summary>
        /// SqlManager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
        }
        #endregion

        #region Events
        #region Frame
        /// <summary>
        /// Chargement de la frame
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Frame_Load(object sender, EventArgs e)
        {
            RefreshToday();
            RefreshDettes();
        }
        /// <summary>
        /// The frame is focused
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Frame_Enter(object sender, EventArgs e)
        {
            RefreshDettes();
            RefreshToday();
        }
        #endregion

        #region Menu
        /// <summary>
        /// Menu d'vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// A Propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Non payed meetings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NonPayedHistory(object sender, EventArgs e)
        {
            fr_HistoryNonCameMeeting fr = new fr_HistoryNonCameMeeting();
            fr.FrameUse = FrameUse.UnpayedMeetings;
            fr.Text = "Rendez-vous non pays...";
            fr.ShowDialog();
            RefreshData();
        }
        /// <summary>
        /// Uncame meetings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_UncameHistory(object sender, EventArgs e)
        {
            fr_HistoryNonCameMeeting fr = new fr_HistoryNonCameMeeting();
            fr.FrameUse = FrameUse.UncameMeetings;
            fr.Text = "Rendez-vous non prests...";
            fr.ShowDialog();
            RefreshData();
        }
        #endregion

        #region Onglet rendez-vous aujourd'hui
        /// <summary>
        /// Selection de OUI pour a payer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventRadio_PayedYES(object sender, EventArgs e)
        {
            tb_APayer.Enabled = true;
        }
        /// <summary>
        /// Selection de NON pour a payer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventRadio_PayedNO(object sender, EventArgs e)
        {
            tb_APayer.Enabled = false;
        }
        /// <summary>
        /// Double clique sur le DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventDoubleClick_DGV_Today(object sender, EventArgs e)
        {
            if (dgv_Today.Rows.Count < 1 || dgv_Today.SelectedRows.Count < 1) return;

            #region Selection de l'honoraire
            MutuelleManager hm = new MutuelleManager();
            if (!dgv_Today.SelectedRows[0].Cells["id_honoraire"].Value.Equals(DBNull.Value))
                hm = MutuelleManager.Create((int)dgv_Today.SelectedRows[0].Cells["id_honoraire"].Value);
            #endregion

            try
            {
                int value = (int)dgv_Today.SelectedRows[0].Cells["Dette_Patient"].Value;
                lbl_Sur.Text = "Dette: " + value + " ";

                bool boolean = (bool)dgv_Today.SelectedRows[0].Cells["came_To_rdv"].Value;
                rb_Came_Yes.Checked = boolean;
                rb_Came_No.Checked = !boolean;
                if (value == 0)
                {
                    MessageBox.Show("Le patient n'a aucune dette. Il a dj pay ses rendez-vous!",
                        "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    rb_Payed_Yes.Checked = true;
                    rb_Payed_Yes.Enabled = false;
                    //rb_Payed_No.Checked = false;
                    rb_Payed_No.Enabled = false;
                    tb_APayer.Enabled = false;
                }
                else
                {
                    boolean = (bool)dgv_Today.SelectedRows[0].Cells["a_paye"].Value;
                    rb_Payed_Yes.Checked = boolean;
                    rb_Payed_No.Checked = !boolean;
                }

                object Hon = dgv_Today.SelectedRows[0].Cells["Honoraire"].Value.ToString();
                lbl_Sur.Text += " Hon: " + Hon + "";


                string nom = (string)dgv_Today.SelectedRows[0].Cells["Prenom"].Value + " "
                    + (string)dgv_Today.SelectedRows[0].Cells["Nom"].Value;
                lbl_Nom_Prenom.Text = "Gestion de " + nom;

                boolean = (bool)dgv_Today.SelectedRows[0].Cells["Mauvais_Payeur"].Value;
                rb_BadPayeur_Yes.Checked = boolean;
                rb_BadPayeur_No.Checked = !boolean;


                vIDPatient = (int)dgv_Today.SelectedRows[0].Cells["id_patient"].Value;
            }
            #region Exception
            catch (ArgumentOutOfRangeException) { }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de slectionner ce patient!\r\n" + ex.Message, "Asterisk",
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            #endregion
        }
        /// <summary>
        /// Boutton quitter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Cancel(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Clique sur OK
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_OK(object sender, EventArgs e)
        {
            if (vIDPatient != -1)
            {
                try
                {
                    #region ProgressBar
                    tspb_ProgressBar.Maximum = 4;
                    tspb_ProgressBar.Minimum = 0;
                    tspb_ProgressBar.Step = 1;
                    #endregion

                    int Dette = (int)dgv_Today.SelectedRows[0].Cells["Dette_Patient"].Value;
                    int Payed = 0;
                    bool Parsed = Int32.TryParse(tb_APayer.Text, out Payed);

                    Dette -= Payed;
                    Dette = Dette < 0 ? 0 : Dette;


                    if (!Parsed)
                    {
                        MessageBox.Show("Veuillez n'entrer que des chiffre pour le payemenet de la dette!",
                            "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                        return;
                    }

                    #region Mise  jour du patient
                    tspb_ProgressBar.PerformStep();
                    SqlManager.PrepareProcedure("UPDATE_PatientRDV");
                    SqlManager.Parameters.Add("[_Dette]", OleDbType.Integer).Value = Dette;
                    SqlManager.Parameters.Add("[_mp]", OleDbType.Boolean).Value = rb_BadPayeur_Yes.Checked;
                    SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vIDPatient;
                    SqlManager.ExecuteNonQueryProcedure();
                    tspb_ProgressBar.PerformStep();

                    #endregion

                    #region Mise  jour du rendez-vous
                    tspb_ProgressBar.PerformStep();

                    SqlManager.PrepareProcedure("UPDATE_RendezVousPosteriori");
                    SqlManager.Parameters.Add("[_Came]", OleDbType.Boolean).Value = rb_Came_Yes.Checked;
                    SqlManager.Parameters.Add("[_Payed]", OleDbType.Boolean).Value = rb_Payed_Yes.Checked;
                    SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value =
                        (int)dgv_Today.SelectedRows[0].Cells["id_Rdv"].Value;

                    SqlManager.ExecuteNonQueryProcedure();
                    tspb_ProgressBar.PerformStep();
                    #endregion

                    MessageBox.Show("Mise  jour effectue!", "Information", MessageBoxButtons.OK,
                        MessageBoxIcon.Information);

                    #region Reset Display
                    lbl_Sur.Text = "Dette: xx  Hon: xx";
                    rb_BadPayeur_No.Checked = false;
                    rb_BadPayeur_Yes.Checked = false;
                    rb_Came_No.Checked = false;
                    rb_Came_Yes.Checked = false;
                    rb_Payed_No.Checked = false;
                    rb_Payed_Yes.Checked = false;
                    lbl_Nom_Prenom.Text = "Gestion de ...";
                    #endregion
                }
                catch (OleDbException ex)
                {
                    MessageBox.Show("Erreur au niveau de la base de donnes: "
                        + ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally { tspb_ProgressBar.Value = 0; }

                RefreshToday();
                RefreshDettes();
                PatientManager pm = PatientManager.Create(vIDPatient);
                lbl_Sur.Text = "Dette: " + pm.Debt + "  Hon: " + pm.Honoraire + "";
            }
        }
        /// <summary>
        /// Rafraichir les donnes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Refresh(object sender, EventArgs e)
        {
            RefreshDettes();
            RefreshToday();
        }
        #endregion

        #region Onglet Dettes
        /// <summary>
        /// Double clique dans le DataGridView des dettes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventDoubleClick_DGV_Dette(object sender, EventArgs e)
        {
            try
            {
                btn_Pay.Text = (string)dgv_Dettes.SelectedRows[0].Cells["Prenom"].Value + " "
                    + (string)dgv_Dettes.SelectedRows[0].Cells["Nom"].Value + " paye!";
                btn_Pay.Enabled = true;
                tb_DebtPayed.Enabled = true;

                vSelectedDebts = (int)dgv_Dettes.SelectedRows[0].Cells["dette_patient"].Value;
                vSelectedDebtsSolder = (int)dgv_Dettes.SelectedRows[0].Cells["id_Patient"].Value;
            }
            catch (ArgumentOutOfRangeException) { }
        }
        /// <summary>
        /// Payement de la dette
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Pay(object sender, EventArgs e)
        {
            int Payed;
            bool OK = Int32.TryParse(tb_DebtPayed.Text, out Payed);
            if (!OK)
            {
                MessageBox.Show("Veuillez n'entrer que des chiffres dans la case de payement!",
                    "Information", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            else
            {
                try
                {
                    if ((vSelectedDebts - Payed) < 0)
                    {
                        MessageBox.Show("Le patient paye plus qu'il ne doit.\r\nIl doit " + vSelectedDebts + " et il"
                            + " paye " + Payed + "", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    vSelectedDebts -= Payed;
                    string SQLQuery = "UPDATE PATIENT SET Dette_Patient = " + vSelectedDebts
                        + " WHERE id_Patient = " + this.vSelectedDebtsSolder + ";";
                    SqlManager.ExecuteNonQuery(SQLQuery);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Une erreur est intervenue pendant la mise  jour.\r\n" + ex.Message, "Erreur",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Log.Instance.Write(ex.Message, "PhoenixMoney.GUI.fr_Main.btn_Pay_Click()", LogType.Erreur);
                }

                tb_DebtPayed.Text = "";
                tb_DebtPayed.Enabled = false;
                btn_Pay.Text = "Accepter";
                btn_Pay.Enabled = false;

                RefreshDettes();
                RefreshToday();
            }
        }
        #endregion
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait un rafraichissement du DataGridView
        /// </summary>
        private void RefreshToday()
        {
            DataSet ds = new DataSet();
            SqlManager.PrepareProcedure("SELECT_RendezVousOnDate");
            SqlManager.Parameters.Add("[_Date]", OleDbType.Date).Value = DateTime.Today;
            try
            {
                SqlManager.ExecuteSqlProcedure(ref ds);
                dgv_Today.DataSource = ds.Tables[0];

                #region Affichage
                for (int i = 0; i < dgv_Today.Columns.Count; i++)
                    dgv_Today.Columns[i].Visible = false;

                dgv_Today.Columns["Nom"].Visible = true;
                dgv_Today.Columns["Nom"].HeaderText = "Nom";

                dgv_Today.Columns["Prenom"].Visible = true;
                dgv_Today.Columns["Prenom"].HeaderText = "Prnom";

                dgv_Today.Columns["Debut"].Visible = true;
                dgv_Today.Columns["Debut"].HeaderText = "Dbut";

                dgv_Today.Columns["Fin"].Visible = true;
                dgv_Today.Columns["Fin"].HeaderText = "Fin";
                #endregion
            }
            catch (OleDbException ex)
            {
                MessageBox.Show("Erreur au niveau de la base de donnes: "
                    + ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Rrefresh les dettes des patients
        /// </summary>
        private void RefreshDettes()
        {
            DataSet ds = new DataSet();


            try
            {
                SqlManager.ExecuteSql("SELECT * FROM Patient WHERE dette_patient > 0;", ref ds);
                dgv_Dettes.DataSource = ds.Tables[0];

                #region Affichage
                for (int i = 0; i < dgv_Dettes.Columns.Count; i++)
                    dgv_Dettes.Columns[i].Visible = false;

                dgv_Dettes.Columns["Nom"].Visible = true;
                dgv_Dettes.Columns["Nom"].HeaderText = "Nom";

                dgv_Dettes.Columns["Prenom"].Visible = true;
                dgv_Dettes.Columns["Prenom"].HeaderText = "Prnom";

                dgv_Dettes.Columns["Dette_Patient"].Visible = true;
                dgv_Dettes.Columns["Dette_Patient"].HeaderText = "Dette";
                #endregion
            }
            catch (OleDbException ex)
            {
                MessageBox.Show("Erreur au niveau de la base de donnes: "
                    + ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Refresh all the data
        /// </summary>
        private void RefreshData()
        {
            RefreshToday();
            RefreshDettes();
        }
        #endregion
    }
}