using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.IO;
using System.Data.OleDb;
using System.Drawing.Imaging;
using System.Threading;
using Utils.Log;

namespace PhoenixPics.GUI
{
    /// <summary>
    /// Gestion des photos
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Thread
        /// <summary>
        /// Thread de traitement
        /// </summary>
        private Thread vThread;
        #endregion

        #region Variables
        /// <summary>
        /// Manager SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);

        /// <summary>
        /// Gestionnaire de Patient
        /// </summary>
        private PatientManager pat = new PatientManager();
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
        }
        #endregion

        #region Events
        /// <summary>
        /// Clique sur la selection d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectionPatient(object sender, EventArgs e)
        {
            ThumbList.Images.Clear();

            #region Selection du patient
            try
            {
                this.pb_Picture.Image = null;
                stl_NomPatient.Text = "Aucun patient";
                fr_PatientSeeker fr = new fr_PatientSeeker();
                DialogResult dr = fr.ShowDialog();
                if (dr == DialogResult.OK)
                {
                    pat = fr.SelectedItem;
                    ThumbList.Images.Clear();
                    stl_NomPatient.Text = pat.ToString();
                    lb_Thumb.Items.Clear();

                    ThreadPicture th = new ThreadPicture(this);
                    th.PatientManager = this.pat;
                    this.vThread = new Thread(new ThreadStart(th.ProcessPicsSelection));
                    this.vThread.Start();
                }
            }
            catch (Exception ex)
            {
                Log.GetInstance().Write(ex.Message, "Picture.fr_Main.MenuClick_SelectionPatient()", LogType.Erreur);
                MessageBox.Show(ex.Message, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            #endregion
        }
        /// <summary>
        /// Choix des vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
            SqlManager.DbPath = Options.GetInstance().Path;
        }
        /// <summary>
        /// Ajout de photos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPic(object sender, EventArgs e)
        {
            if (pat.IDPatient == -1) return;

            DialogResult dr = ofd_PicturesSelection.ShowDialog();

            if (dr == DialogResult.OK)
            {
                Image bmp;
                MemoryStream ms;
                SqlManager.Open();
                tspb_ProgressBar.Maximum = ofd_PicturesSelection.FileNames.Length * 5;
                tspb_ProgressBar.Minimum = 0;
                tspb_ProgressBar.Step = 1;

                try
                {
                    foreach (string file in ofd_PicturesSelection.FileNames)
                    {
                        ms = new MemoryStream();
                        SqlManager.PrepareProcedure("INSERT_Photo", false);
                        SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = pat.IDPatient;
                        tspb_ProgressBar.PerformStep();

                        bmp = new Bitmap(file);
                        bmp = new Bitmap(bmp, new Size(bmp.Width, bmp.Height));
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Pic]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        bmp = new Bitmap(bmp, new Size(100, 100));
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Thumb]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        fr_Remarks fr = new fr_Remarks();
                        fr.Image = bmp;
                        DialogResult drr = fr.ShowDialog();

                        SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = fr.Remark;
                        tspb_ProgressBar.PerformStep();

                        object LastId = 0;
                        SqlManager.ExecuteNonQueryProcedure(ref LastId);
                        SqlManager.Parameters.Clear();
                        ThumbList.Images.Add(bmp);
                        lb_Thumb.Items.Add(((object)LastId.ToString()));
                        tspb_ProgressBar.PerformStep();
                    }
                }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, "PhoenixPics.fr_Main.MenuClick_AddPic()", LogType.Erreur);
                    MessageBox.Show("Une erreur est survenue lors de l'ajout d'une photo.\r\n" + ex.Message,
                        "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                SqlManager.Close();
                tspb_ProgressBar.Value = 0;
            }
        }
        /// <summary>
        /// Evnement de de mande de redessinage de la ListBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ListBoxDrawItem(object sender, DrawItemEventArgs e)
        {
            if (e.Index < 0 || ThumbList.Images.Count <= e.Index) return;
            Point p = e.Bounds.Location;
            ThumbList.Draw(e.Graphics, p, e.Index);
        }
        /// <summary>
        /// On  slectionne une image dans la ThumbBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ListBoxIndexChanged(object sender, EventArgs e)
        {
            if (lb_Thumb.SelectedIndex == -1) return;
            int idp = Int32.Parse(lb_Thumb.Items[lb_Thumb.SelectedIndex].ToString());

            DataSet ds = new DataSet();
            int i = SqlManager.ExecuteSql("SELECT * FROM PHOTO WHERE id_Photo = " + idp, ref ds);

            Bitmap bmp = new Bitmap(new MemoryStream((byte[])ds.Tables[0].Rows[0]["BinPic"]));
            string Rmq = (string)ds.Tables[0].Rows[0]["Remarques"];
            pb_Picture.Image = bmp;
            tb_Remarks.Text = Rmq;
            btn_Del.Enabled = btn_Modify.Enabled = true;
        }
        /// <summary>
        /// Double clique sur la PictureBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DoubleClick_PictureBox(object sender, EventArgs e)
        {
            if (pb_Picture.Image == null) return;

            fr_FullScreenPicture fr = new fr_FullScreenPicture();
            fr.Image = pb_Picture.Image;
            fr.ShowDialog();
        }
        /// <summary>
        /// Bouton modifier une photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_Modify_Click(object sender, EventArgs e)
        {
            if (btn_Modify.Text.ToLower() == "Activer Modifier".ToLower())
            {
                tb_Remarks.ReadOnly = false;
                btn_Modify.Text = "Modifier";
            }
            else
                if (lb_Thumb.SelectedItem != null)
                {
                    SqlManager.ExecuteNonQuery("UPDATE Photo SET Remarques = \""
                            + tb_Remarks.Text + "\" WHERE id_Photo = " + (string)this.lb_Thumb.SelectedItem);
                    tb_Remarks.ReadOnly = true;
                    btn_Modify.Enabled = false;
                    btn_Modify.Text = "Activer Modifier";
                }
        }
        /// <summary>
        /// Suppression d'une photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_Del_Click(object sender, EventArgs e)
        {
            if (lb_Thumb.SelectedItem != null)
            {
                DialogResult dr =
                MessageBox.Show("Voulez-vous supprimer cette photo?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (dr == DialogResult.Yes)
                {
                    SqlManager.ExecuteNonQuery("DELETE * FROM Photo WHERE id_Photo = " + (string)this.lb_Thumb.SelectedItem);
                    pb_Picture.Image = null;
                    int index = lb_Thumb.SelectedIndex;
                    lb_Thumb.Items.RemoveAt(index);
                    ThumbList.Images.RemoveAt(index);
                    btn_Modify.Enabled = false;
                    btn_Del.Enabled = false;
                }
            }
        }
        /// <summary>
        /// A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsMenu_APropos_Click(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        #endregion
    }

    /// <summary>
    /// Thread de traitement des photos
    /// </summary>
    internal class ThreadPicture
    {
        #region Variables
        /// <summary>
        /// DateSet pour la gestion des donnes
        /// </summary>
        private DataSet vDataSet = new DataSet();

        /// <summary>
        /// Manager de SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);

        /// <summary>
        /// Patient selectionn
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();

        /// <summary>
        /// Frame dans laquelle les traitements GUI se feront
        /// </summary>
        private fr_Main vFrame;

        /// <summary>
        /// Nombre de tuple dans le DataSet
        /// </summary>
        private int vRowsInDataSet;
        #endregion

        #region Delegates
        /// <summary>
        /// Handler pour une mise  jour des miniatures
        /// </summary>        
        private delegate void UpdateThumbHandler();

        /// <summary>
        /// Delegate pour une mise  jour des miniatures
        /// </summary>
        private UpdateThumbHandler UpdateThumb;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_fr">Frame dans laquelle les traitements GUI se feront</param>
        public ThreadPicture(fr_Main _fr) { vFrame = _fr; }

        #endregion

        #region Mthodes
        /// <summary>
        /// Fait une recherche de photos sur un patient
        /// </summary>
        public void ProcessPicsSelection()
        {
            #region Selection des Thumbs
            SqlManager.PrepareProcedure("SELECT_PhotoOnPatient");
            SqlManager.Parameters.Add("[_ID]", System.Data.OleDb.OleDbType.Integer).Value = vPatientManager.IDPatient;
            vDataSet = new DataSet();
            vRowsInDataSet = SqlManager.ExecuteSqlProcedure(ref vDataSet);
            #endregion

            #region Traitement de la GUI
            this.UpdateThumb = new UpdateThumbHandler(GUIUpdateThumb);
            try { vFrame.Invoke(UpdateThumb); }
            catch (Exception ex) { Log.GetInstance().Write(ex.Message, "ThreadPicture.ProcessPicsSelection()", LogType.Erreur); }
            #endregion
        }

        /// <summary>
        /// Cration des minitatures
        /// </summary>        
        private void GUIUpdateThumb()
        {
            Bitmap bmp;
            int IdPic = -1;
            vFrame.tspb_ProgressBar.Maximum = vDataSet.Tables[0].Rows.Count;
            vFrame.tspb_ProgressBar.Minimum = 0;
            vFrame.tspb_ProgressBar.Step = 1;


            foreach (DataRow dr in vDataSet.Tables[0].Rows)
            {
                DataColumnCollection dc = vDataSet.Tables[0].Columns;

                bmp = new Bitmap(new MemoryStream((byte[])dr["Thumb"]));
                vFrame.ThumbList.Images.Add(bmp);

                IdPic = (int)dr["id_Photo"];
                vFrame.lb_Thumb.Items.Add(IdPic.ToString());
                vFrame.tspb_ProgressBar.PerformStep();
            }
            vFrame.tspb_ProgressBar.Value = 0;
            vFrame.stl_NomPatient.Text = vPatientManager.ToString();
            vFrame.tsMenu_AjoutPhoto.Enabled = true;

            if (vRowsInDataSet > 1)
                vFrame.stl_Photos.Text = vRowsInDataSet + " photos";
            else
                vFrame.stl_Photos.Text = vRowsInDataSet + " photo";
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le patient sur lequel les recherches se font
        /// </summary>
        public PatientManager PatientManager { get { return vPatientManager; } set { vPatientManager = value; } }
        #endregion
    }
}