using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.Data.OleDb;
using Utils.Log;

namespace PhoenixPrescription.GUI
{
    /// <summary>
    /// Visionneuse des prescriptions d'un patient
    /// </summary>
    public partial class fr_HistoriquePatient : Form
    {
        #region Variables
        /// <summary>
        /// Informations sur un patient
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// Manager de Sql
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Indicates whether a delete occured
        /// </summary>
        private bool vDeleted = false;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_HistoriquePatient()
        {
            InitializeComponent();
        }
        #endregion

        #region Accesseur
        /// <summary>
        /// Retourne ou initialise la patient sur lequel on recherche l'historique
        /// </summary>
        public PatientManager PatientManager
        {
            get { return vPatientManager; }
            set
            {
                vPatientManager = value;
                SqlManager.PrepareProcedure("SELECT_PrescriptionOnPatient");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vPatientManager.IDPatient;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSqlProcedure(ref ds);
                dgv_History.DataSource = ds.Tables[0];

                for (int i = 0; i < dgv_History.Columns.Count; i++)
                    dgv_History.Columns[i].Visible = false;

                dgv_History.Columns["Nom"].Visible = true;
                dgv_History.Columns["Nom"].HeaderText = "Mdicament";
                dgv_History.Columns["Nom"].DisplayIndex = 0;

                dgv_History.Columns["p.Rmq"].Visible = true;
                dgv_History.Columns["p.Rmq"].HeaderText = "Remarques";
                dgv_History.Columns["p.Rmq"].DisplayIndex = 1;

                dgv_History.Columns["Date_P"].Visible = true;
                dgv_History.Columns["Date_P"].HeaderText = "Date de prescription";
                dgv_History.Columns["Date_P"].DisplayIndex = 2;


            }
        }
        /// <summary>
        /// Indicates whether a delete occured
        /// </summary>
        public bool Deleted { get { return vDeleted; } }
        #endregion

        #region Events
        /// <summary>
        /// Demande une suppression de cette prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Delete(object sender, EventArgs e)
        {

            DialogResult dr = MessageBox.Show("Voulez-vous supprimer cette prescription?", 
                "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.No) return;
            try
            {
                if (dgv_History.Rows.Count < 1) return;
                int idP = (int)dgv_History.SelectedRows[0].Cells["id_Prescription"].Value;
                PrescriptionManager pm = PrescriptionManager.Create(idP);
                pm.Delete();
                dgv_History.Rows.RemoveAt(dgv_History.SelectedRows[0].Index);
                vDeleted = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue pendant la suppression", "Erreur",
                     MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex.Message, "MenuClick_Delete()", LogType.Erreur);
            }
        }
        /// <summary>
        /// Event sur un clique droit sur le DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_DGV_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo HitTestInfo = this.dgv_History.HitTest(e.X, e.Y);
                if (HitTestInfo.RowIndex > -1)
                {
                    this.dgv_History.SelectedRows[0].Selected = false;
                    this.dgv_History.Rows[HitTestInfo.RowIndex].Selected = true;
                }
            }
        }
        #endregion


    }
}