using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.GUI.Managers;
using PhoenixDLL.Print;
using System.Data.OleDb;

namespace PhoenixPrescription.GUI
{
    /// <summary>
    /// Fentre principale des prescriptions
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Manager de patient
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// Gestionnaire de prescription
        /// </summary>
        private PrescriptionManager vPrescriptionManager = new PrescriptionManager();
        /// <summary>
        /// Liste des mdicament prescrit cette session
        /// </summary>
        private List<PrescriptionManager> vPrescriptionList = new List<PrescriptionManager>();
        /// <summary>
        /// Gestionnair de requte SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Gestionnaire d'impression de prescriptions
        /// </summary>
        private PrescriptionPrint vPrescriptionPrint = new PrescriptionPrint();
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            Options.Instance.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);
            SetLabelsVisible(false);
            lbl_Medicament.Text = "";
            ts_Managment.AutoSize = false;
            ts_Managment.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_Managment.AutoSize = true;

            #region Visibilit des menus
            tsMenu_Managment.Checked = Options.Instance.PegaMenu.ManagmentVisible;
            ts_Managment.Visible = Options.Instance.PegaMenu.ManagmentVisible;
            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
            #endregion
        }
        #endregion

        #region Event
        /// <summary>
        /// Clique sur le menu d'vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur selection d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;

            vPatientManager = fr.SelectedItem;
            tsMenu_SelectionMedic.Enabled = true;
            SetLabelsVisible(true);

            vPrescriptionList.Clear();
            dgv_Prescription.DataSource = null;

            #region Affichage Patient
            if (fr.SelectedItem.Sex == Sex.Homme)
                lbl_Nom.Text = "Monsieur ";
            else
                lbl_Nom.Text = "Madame ";

            lbl_Nom.Text += fr.SelectedItem.ToString();

            lbl_DateOfBirth.Text = "Date de naissance: " + fr.SelectedItem.DateOfBirth.ToLongDateString();
            lbl_Address.Text = "Adresse: " + fr.SelectedItem.Address;
            lbl_City.Text = "Ville: " + fr.SelectedItem.ZipCode + " " + fr.SelectedItem.City;
            #endregion

            #region Affichage Mdicale info
            int nbr = CountPrescriptions(fr.SelectedItem);
            lbl_Prescriptions.Text = "Total des prescriptions: " + nbr;
            lbl_LastUpdate.Text = "Dernire visite: " + fr.SelectedItem.LastUpdate.ToShortDateString();
            lbl_Medic.Text = "Mdecin traitant: " + fr.SelectedItem.MedecinTraitant.ToString();
            lbl_Homeo.Text = "Homopathe: " + fr.SelectedItem.Homeopathe.ToString();
            #endregion
        }
        /// <summary>
        /// Selection d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectDrug(object sender, EventArgs e)
        {
            tb_Remarks.Text = "";
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            btn_Accepter.Enabled = true;
            tsMenu_AddPrescription.Enabled = true;
            vPrescriptionManager = new PrescriptionManager();
            vPrescriptionManager.PatientManager = vPatientManager;
            vPrescriptionManager.DrugsManager = fr.SelectedItem;
            vPrescriptionManager.Date = DateTime.Today;

            lbl_Medicament.Text = vPrescriptionManager.DrugsManager.ToString();
        }
        /// <summary>
        /// Fait la prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPrescription(object sender, EventArgs e)
        {
            vPrescriptionManager.Remark = tb_Remarks.Text;
            vPrescriptionManager.Add();
            vPrescriptionList.Add(vPrescriptionManager);

            dgv_Prescription.DataSource = null;
            dgv_Prescription.Rows.Clear();
            dgv_Prescription.DataSource = vPrescriptionList;

            for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                dgv_Prescription.Columns[i].Visible = false;

            dgv_Prescription.Columns["DrugsManager"].Visible = true;
            dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

            dgv_Prescription.Columns["Remark"].Visible = true;
            dgv_Prescription.Columns["Remark"].DisplayIndex = 1;

            lbl_Medicament.Text = "";
            btn_Accepter.Enabled = false;
            tsMenu_AddPrescription.Enabled = false;
            int nbr = CountPrescriptions(this.vPatientManager);
            lbl_Prescriptions.Text = "Total des prescriptions: " + nbr;
        }        
        /// <summary>
        /// Menu de demande d'impression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PrintPreview(object sender, EventArgs e)
        {
            vPrescriptionPrint.PrescriptionList = vPrescriptionList;
            vPrescriptionPrint.RefreshHeader();
            vPrescriptionPrint.Document.PrinterSettings = Options.Instance.PrinterSettingsPrescritions;
            ppd_PrintPreview.Document = vPrescriptionPrint.Document;
            ppd_PrintPreview.ShowDialog();
        }
        /// <summary>
        /// Prview de l'impression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SetupPrinter(object sender, EventArgs e)
        {
            PrintDialog pd = new PrintDialog();
            pd.PrinterSettings = Options.Instance.PrinterSettingsPrescritions;
            DialogResult dr = pd.ShowDialog();
            if (dr != DialogResult.OK) return;

            Options.Instance.PrinterSettingsPrescritions = pd.PrinterSettings;
            vPrescriptionPrint.PrescriptionList = vPrescriptionList;
            vPrescriptionPrint.Document.PrinterSettings = pd.PrinterSettings;
        }
        /// <summary>
        /// Evnement sur une demande de modification de prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModificationPrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.Rows.Count < 1 || dgv_Prescription.SelectedRows.Count < 1) return;

            PrescriptionManager pm = vPrescriptionList[(int)dgv_Prescription.SelectedRows[0].Index];

            fr_PrescriptionManager fr = new fr_PrescriptionManager();
            fr.Manager = pm;
            fr.Use = Use.Update;
            fr.Text = "Modification d'une rputation";
            fr.Size = new Size(fr.Size.Width, 250);
            fr.ShowDialog();
        }
        /// <summary>
        /// Demande de suppression de la prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SuppressionPrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.Rows.Count < 1 || dgv_Prescription.SelectedRows.Count < 1) return;
            int idx = (int)dgv_Prescription.SelectedRows[0].Index;

            PrescriptionManager pm = vPrescriptionList[idx];
            DialogResult dr = MessageBox.Show("Voulez-vous supprimer cette prescription?\r\n"
                + vPrescriptionList[idx].ToString(), "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (dr == DialogResult.Yes)
            {
                dgv_Prescription.DataSource = null;
                vPrescriptionList[idx].Delete();
                vPrescriptionList.RemoveAt(idx);
                dgv_Prescription.DataSource = vPrescriptionList;

                for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                    dgv_Prescription.Columns[i].Visible = false;

                dgv_Prescription.Columns["DrugsManager"].Visible = true;
                dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

                dgv_Prescription.Columns["Remark"].Visible = true;
                dgv_Prescription.Columns["Remark"].DisplayIndex = 1;
                int nbr = CountPrescriptions(this.vPatientManager);
                lbl_Prescriptions.Text = "Total des prescriptions: " + nbr;
            }
        }
        /// <summary>
        /// A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Donne l'historique d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_HistoriquePatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;

            fr_HistoriquePatient hp = new fr_HistoriquePatient();
            hp.PatientManager = fr.SelectedItem;
            hp.ShowDialog();

            //We deleted prescriptions. Refresh needed
            if (hp.Deleted)
            {
                PrescriptionManager p = new PrescriptionManager();
                for (int i = 0; i < vPrescriptionList.Count; i++)
                {
                    p = PrescriptionManager.Create(vPrescriptionList[i].IDPrescription);
                    if (p.Empty)
                        vPrescriptionList.Remove(vPrescriptionList[i]);
                }

                if (vPrescriptionList.Count > 0)
                {
                    dgv_Prescription.DataSource = null;
                    dgv_Prescription.DataSource = vPrescriptionList;

                    for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                        dgv_Prescription.Columns[i].Visible = false;

                    dgv_Prescription.Columns["DrugsManager"].Visible = true;
                    dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

                    dgv_Prescription.Columns["Remark"].Visible = true;
                    dgv_Prescription.Columns["Remark"].DisplayIndex = 1;
                }
                else
                {
                    //dgv_Prescription.Rows.Clear();
                    dgv_Prescription.DataSource = null;
                }
                int nbr = CountPrescriptions(fr.SelectedItem);
                lbl_Prescriptions.Text = "Total des prescriptions: " + nbr;
            }
        }
        /// <summary>
        /// Clique sur la gestion d'affichage des menus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Managment(object sender, EventArgs e)
        {
            if (!tsMenu_Managment.Checked)
            {
                tsMenu_Managment.Checked = true;
                ts_Managment.Visible = true;
            }
            else
            {
                tsMenu_Managment.Checked = false;
                ts_Managment.Visible = false;
            }
            PegaMenu p = new PegaMenu();
            p.ManagmentVisible = ts_Managment.Visible;
            Options.Instance.PegaMenu = p;
        }
        /// <summary>
        /// On vOptions changes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            ts_Managment.AutoSize = false;
            ts_Managment.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_Managment.AutoSize = true;
        }
        /// <summary>
        /// Frame is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FormClosing(object sender, FormClosingEventArgs e)
        {
            Options.Instance.PegaMenu.ManagmentLocation = ts_Managment.Location;
        }
        /// <summary>
        /// The form is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Load(object sender, EventArgs e)
        {
            ts_Managment.Location = Options.Instance.PegaMenu.ManagmentLocation;
        }  
        /// <summary>
        /// Mouse Down on the DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_DGV_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo HitTestInfo = this.dgv_Prescription.HitTest(e.X, e.Y);
                if (HitTestInfo.RowIndex > -1)
                {
                    this.dgv_Prescription.SelectedRows[0].Selected = false;
                    this.dgv_Prescription.Rows[HitTestInfo.RowIndex].Selected = true;
                }
            }
        }

        #region Menu Gestion
        #region Ajout
        /// <summary>
        /// Menu d'ajout de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddReput(object sender, EventArgs e)
        {
            fr_ReputationManager fr = new fr_ReputationManager();
            fr.Manager = new ReputationManager();
            fr.Text = "Gestion des rputations...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleManager fr = new fr_MutuelleManager();
            fr.Manager = new MutuelleManager();
            fr.Text = "Gestion des mutuelles...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajoute un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDoctor(object sender, EventArgs e)
        {
            fr_DoctorManager fr = new fr_DoctorManager();
            fr.Manager = new DoctorManager();
            fr.Text = "Gestion des mdecins...";
            fr.Use = Use.Add;
            fr.Size = new Size(327, 381);
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrug(object sender, EventArgs e)
        {
            fr_DrugsManager fr = new fr_DrugsManager();
            fr.Manager = new DrugsManager();
            fr.Text = "Gestion des mdicaments";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un type de mdicament        
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeManager fr = new fr_DrugsTypeManager();
            fr.Manager = new DrugsTypeManager();
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Evnement sur une demande d'ajotu de patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPatient(object sender, EventArgs e)
        {
            fr_PatientManager fr = new fr_PatientManager();
            fr.Use = Use.Add;
            fr.Size = new Size(fr.Size.Width, 600);
            fr.Text = "Gestion de patient...";
            fr.ShowDialog();
        }
        #endregion

        #region Modification
        /// <summary>
        /// Modification d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle...";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_MutuelleManager hm = new fr_MutuelleManager();
                hm.Manager = fr.SelectedItem;
                hm.Text = "Modification d'une mutuelle...";
                hm.Use = Use.Update;
                hm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_ReputationManager rm = new fr_ReputationManager();
                rm.Manager = fr.SelectedItem;
                rm.Text = "Modification d'une rputation...";
                rm.Use = Use.Update;
                rm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDoctor(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_DoctorManager dm = new fr_DoctorManager();
                dm.Manager = fr.SelectedItem;
                dm.Text = "Modification d'un mdecin...";
                dm.Use = Use.Update;
                dm.Size = new Size(327, 381);

                dm.ShowDialog();
            }

        }
        /// <summary>
        /// Modification of a drug
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Manager = fr.SelectedItem;
            dm.Text = "Modification d'un mdicament";
            dm.Use = Use.Update;
            dm.Size = new Size(327, 350);
            dm.ShowDialog();
        }
        /// <summary>
        /// Modification of a drug type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Manager = fr.SelectedItem;
            dtm.Text = "Modification d'un type de mdicament";
            dtm.Use = Use.Update;
            dtm.Size = new Size(327, 350);
            dtm.ShowDialog();
        }
        /// <summary>
        /// Moodification of a A patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModAPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.Text = "Recherche d'un patient...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_PatientManager pm = new fr_PatientManager();
            pm.Manager = fr.SelectedItem;
            pm.Text = "Modification d'un patient...";
            pm.Use = Use.Update;
            pm.Size = new Size(pm.Size.Width, 600);
            pm.ShowDialog();

        }
        #endregion

        #region Suppression
        /// <summary>
        /// Suppression de mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDoc(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DoctorManager dm = new fr_DoctorManager();
            dm.Text = "Suppression de ce mdecin";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 400);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_ReputationManager rm = new fr_ReputationManager();
            rm.Text = "Suppression de cette rputation";
            rm.Manager = fr.SelectedItem;
            rm.Use = Use.Delete;
            rm.Size = new Size(rm.Size.Width, 250);
            rm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche d'un mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Text = "Suppression de ce mdicament";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 250);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une mutuelle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_MutuelleManager mm = new fr_MutuelleManager();
            mm.Text = "Suppression de cette mutuelle";
            mm.Manager = fr.SelectedItem;
            mm.Use = Use.Delete;
            mm.Size = new Size(mm.Size.Width, 250);
            mm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un type de mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Text = "Suppression de ce type de mdicament";
            dtm.Manager = fr.SelectedItem;
            dtm.Use = Use.Delete;
            dtm.Size = new Size(dtm.Size.Width, 250);
            dtm.ShowDialog();
        }
        #endregion

        #endregion
        #endregion

        #region Mthodes
        /// <summary>
        /// Rends les diffrents labels de la fentre visible ou pas
        /// </summary>
        /// <param name="_IsVisible">A true tout est visible,  false tout est invisible</param>
        private void SetLabelsVisible(bool _IsVisible)
        {
            #region Misc
            btn_SelectMedic.Enabled = _IsVisible;
            tsMenu_SelectionMedic.Enabled = _IsVisible;
            #endregion

            lbl_Address.Visible = _IsVisible;
            lbl_City.Visible = _IsVisible;
            lbl_DateOfBirth.Visible = _IsVisible;
            lbl_Homeo.Visible = _IsVisible;
            lbl_LastUpdate.Visible = _IsVisible; ;
            lbl_Medic.Visible = _IsVisible;
            lbl_Medicament.Visible = _IsVisible;
            lbl_Nom.Visible = _IsVisible;
            lbl_Prescriptions.Visible = _IsVisible;
        }
        /// <summary>
        /// count the number of prescriptions for the patient
        /// </summary>
        /// <param name="_patient">Manager of the patient</param>
        /// <returns>Number of presctiptions for this patient</returns>
        private int CountPrescriptions(PatientManager _patient)
        {
            return (int)SqlManager.ExecuteScalar("SELECT COUNT(*) FROM Prescription WHERE id_Patient = "
                + _patient.IDPatient);
        }
        #endregion
    }
}