using System;
using System.Collections.Generic;
using System.Text;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.Data.OleDb;
using System.Data;
using PhoenixDLL.Class.Managers;

namespace PhoenixStatistics.Class
{
    /// <summary>
    /// Gestionnaire de statistique
    /// </summary>
    public class StatManager
    {
        #region Variables
        /// <summary>
        /// Gestionnaire de SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// contient les rendez-vous par mois
        /// </summary>
        private double[] vMeetingOnMonth = new double[12];
        /// <summary>
        /// Age moyen des patients
        /// </summary>
        private double vAverageAge = 0;
        /// <summary>
        /// Age moyen des femems
        /// </summary>
        private double vAverageWomenAge = 0;
        /// <summary>
        /// Age moyen des hommes
        /// </summary>
        private double vAverageMenAge = 0;
        /// <summary>
        /// Liste des ages
        /// </summary>
        private double[] vAges = new double[10];
        /// <summary>
        /// Liste des ages des femmes
        /// </summary>
        private double[] vWomenAges = new double[10];
        /// <summary>
        /// Liste des ages des hommes
        /// </summary>
        private double[] vMenAges = new double[10];
        /// <summary>
        /// Age maximum des patient
        /// </summary>
        private double vMaxAgeMan = 0, vMaxAgeWoman = 0, vMaxAge = 0;
        /// <summary>
        /// Compte des hommes
        /// </summary>
        private int vCountMen = 0;
        /// <summary>
        /// Compte des femmes
        /// </summary>
        private int vCountWomen = 0;
        /// <summary>
        /// Compte de tous les patients
        /// </summary>
        private int vCountAll = 0;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public StatManager() { }
        #endregion

        #region Mthodes
        /// <summary>
        /// Calcule le nombre de rendez-vous par mois
        /// </summary>
        /// <returns>Un tableau de 12 lments qui comprend le nombre de rendez-vous par mois</returns>
        public double[] CalculateMeetingOnMonth()
        {
            try
            {
                string sql = "SELECT COUNT(*) FROM RendezVous WHERE FORMAT(Debut, \"mm\") = ";
                SqlManager.Open();
                for (int i = 1; i <= 12; i++)
                {
                    int res = (int)SqlManager.ExecuteScalar(sql + i.ToString());
                    vMeetingOnMonth[i - 1] = (double)res;
                }
                SqlManager.Close();
                return vMeetingOnMonth;
            }
            catch (Exception) { throw; }
        }

        /// <summary>
        /// Calcule le nombre de rendez-vous pour ce mois...
        /// </summary>
        /// <param name="_Month">Numro du moins compris entre 1 et 12</param>
        /// <returns>Le nombre de rendez-vous pour ce mois</returns>
        public int CalculateMeetingOf(int _Month)
        {
            if (_Month < 1 || _Month > 12) return 0;
            string sql = "SELECT COUNT(*) FROM RendezVous WHERE FORMAT(Debut, \"mm\") = " + _Month.ToString();
            return (int)SqlManager.ExecuteScalar(sql);
        }

        #region Ages
        /// <summary>
        /// Age moyen 
        /// </summary>
        /// <returns>L'age moyen</returns>
        public double CalculateAverageAge()
        {
            SqlManager.PrepareProcedure("STAT_AgeMoyen");
            object ret = SqlManager.ExecuteScalarProcedure();

            if (ret.Equals(DBNull.Value)) return 0.0;
            vAverageAge = (double)ret;
            return vAverageAge;
        }
        /// <summary>
        /// age moyen des femmes
        /// </summary>
        /// <returns>L'age moyen des femmes</returns>
        public double CalculateAverageWomenAge()
        {
            SqlManager.PrepareProcedure("STAT_AgeMoyenFemme");
            object ret = SqlManager.ExecuteScalarProcedure();

            if (ret.Equals(DBNull.Value)) return 0.0;
            vAverageWomenAge = (double)ret;
            return vAverageWomenAge;
        }
        /// <summary>
        /// Age moyen des hommes
        /// </summary>
        /// <returns>L'age moyen des hommes</returns>
        private double CalculateAverageMenAge()
        {
            SqlManager.PrepareProcedure("STAT_AgeMoyenHomme");
            object ret = SqlManager.ExecuteScalarProcedure();

            if (ret.Equals(DBNull.Value)) return 0.0;
            vAverageMenAge = (double)ret;
            return vAverageMenAge;
        }
        /// <summary>
        /// Calcule tous les ages moyens
        /// </summary>
        public void CalculateAllAverageAges()
        {
            this.CalculateAverageAge();
            this.CalculateAverageMenAge();
            this.CalculateAverageWomenAge();
        }
        /// <summary>
        /// Calcule les ages maximums
        /// </summary>
        public void CalculateMaxAges()
        {
            SqlManager.PrepareProcedure("STAT_MaxAge");
            object res = SqlManager.ExecuteScalarProcedure();
            if (!res.Equals(DBNull.Value))
            {
                vMaxAge = (double)res;

                SqlManager.PrepareProcedure("STAT_MaxAgeFemme");
                res = SqlManager.ExecuteScalarProcedure();
                if (res.Equals(DBNull.Value)) vMaxAgeWoman = 0;
                else vMaxAgeWoman = (double)res;

                SqlManager.PrepareProcedure("STAT_MaxAgeHomme");
                res = SqlManager.ExecuteScalarProcedure();
                if (res.Equals(DBNull.Value)) vMaxAgeMan = 0;
                else vMaxAgeMan = (double)res;
            }
        }
        /// <summary>
        /// Donne une liste de tous les ages
        /// </summary>
        /// <param name="_ProcName">Nom de la procdure de calcul des ages</param>
        /// <returns>Une liste des tous les ages</returns>
        private double[] pCalculatesAges(string _ProcName)
        {
            SqlManager.PrepareProcedure(_ProcName);
            DataSet ds = new DataSet();
            SqlManager.ExecuteSqlProcedure(ref ds);
            CalculateMaxAges();
            double[] Buf = new double[this.AgeList.Length];

            if (vMaxAge > 0)
            {
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    object obj = dr[0];
                    try
                    {
                        if ((double)obj > 0)
                            Buf[(int)((double)obj)]++;
                    }
                    catch (InvalidCastException) { }
                }
                return Buf;
            }
            double[] ret = { 0.0 };
            return ret;
        }
        /// <summary>
        /// Fait un calcul de l'age des tous les patients
        /// </summary>
        /// <returns></returns>
        public double[] CalculateAges()
        {
            vAges = pCalculatesAges("STAT_Ages");
            return vAges;
        }
        /// <summary>
        /// Calcul de l'age de tout les patients hommes
        /// </summary>
        /// <returns></returns>
        public double[] CalculateWomenAges()
        {
            vWomenAges = pCalculatesAges("STAT_AgesFemmes");
            return vWomenAges;
        }
        /// <summary>
        /// Calcul de l'age de toutes les patientes femmes
        /// </summary>
        /// <returns></returns>
        public double[] CalculateMenAges()
        {
            vMenAges = pCalculatesAges("STAT_AgesHommes");
            return vMenAges;
        }
        #endregion

        #region Sex
        /// <summary>
        /// Fait le compte des patients
        /// </summary>
        /// <param name="_sex">Sexe sur lequel on fait le compte</param>
        /// <returns>Le nombre de patient</returns>
        public int CalculateCountPatients(SSex _sex)
        {
            string Proc = "";
            switch (_sex)
            {
                case SSex.Female:
                    Proc = "STAT_CountPatientFemme";
                    break;
                case SSex.Male:
                    Proc = "STAT_CountPatientHomme";
                    break;
                case SSex.Neutral:
                    Proc = "STAT_CountPatient";
                    break;
            }

            SqlManager.PrepareProcedure(Proc);
            object ret = SqlManager.ExecuteScalarProcedure();

            switch (_sex)
            {
                case SSex.Female:
                    vCountWomen = (int)ret;
                    break;
                case SSex.Male:
                    vCountMen = (int)ret;
                    break;
                case SSex.Neutral:
                    vCountAll = (int)ret;
                    break;
            }
            return (int)ret;
        }

        /// <summary>
        /// Calcule et enregistre le comtpe des tous les patient selon le sexe.
        /// </summary>
        public void CalculateAllCount()
        {
            vCountWomen = CalculateCountPatients(SSex.Female);
            vCountMen = CalculateCountPatients(SSex.Male);
            vCountAll = CalculateCountPatients(SSex.Neutral);
        }
        #endregion

        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne le nombre de rendez-vous par mois
        /// </summary>
        public double[] MeetingOnMonth { get { return vMeetingOnMonth; } }
        /// <summary>
        /// Retourne l'age moyen des patients
        /// </summary>
        public double AverageAge { get { return vAverageAge; } }
        /// <summary>
        /// Retourne l'age moyen des hommes
        /// </summary>
        public double AverageMenAge { get { return vAverageMenAge; } }
        /// <summary>
        /// Retourne l'age moyen des femmes
        /// </summary>
        public double AverageWomenAge { get { return vAverageWomenAge; } }
        /// <summary>
        /// Retourne la liste des ages des patients
        /// </summary>
        public double[] Ages { get { return vAges; } }
        /// <summary>
        /// Retourne la liste des ages des patientes femmes
        /// </summary>
        public double[] WomenAges { get { return vWomenAges; } }
        /// <summary>
        /// Retourne la liste des patients hommes
        /// </summary>
        public double[] MenAges { get { return vMenAges; } }
        /// <summary>
        /// Retourne la liste des ages de 0 ans  l'age le plus lev de la base de donnes
        /// </summary>
        public double[] AgeList
        {
            get
            {
                double[] buf = new double[(int)vMaxAge + 1];
                for (int i = 0; i < (int)vMaxAge + 1; i++)
                {
                    buf[i] += (i + 1);
                }
                return buf;
            }
        }
        /// <summary>
        /// Donne le nombre d'hommes dans les patients
        /// </summary>
        public int CountMen { get { return vCountMen; } }
        /// <summary>
        /// Donne le nombre de femmes dans les patients
        /// </summary>
        public int CountWomen { get { return vCountWomen; } }
        /// <summary>
        /// Donne le nombre de patients
        /// </summary>
        public int CountAll { get { return vCountAll; } }
        #endregion
    }

    /// <summary>
    /// Donne le sexe sur lequel on doit faire la statistique
    /// </summary>
    public enum SSex
    {
        /// <summary>
        /// Fait la requte SQL sur les hommes
        /// </summary>
        Male,
        /// <summary>
        /// Fait le requte SQL sur les femmes
        /// </summary>
        Female,
        /// <summary>
        /// Fait le requte SQL sur les deux sexes.
        /// </summary>
        Neutral,
    }
}
