using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Managers;

namespace PhoenixStatistics.GUI
{
    public partial class fr_AgeCorrector : Form
    {
        #region Variables
        /// <summary>
        /// Sql manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Indicates whether the ages was modified
        /// </summary>
        private bool vModified = false;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>        
        public fr_AgeCorrector()
        {
            InitializeComponent();

            #region Corrector
            nud_Age.Value = 100;
            #endregion
        }
        #endregion

        #region Events
        /// <summary>
        /// Click on search
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SearchOlderThan(object sender, EventArgs e) { RefreshOlderThan(); }
        /// <summary>
        /// ContextMenuStrip modify this old patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModifyOlderThan(object sender, EventArgs e)
        {
            int IDPatient = (int)dgv_Age.SelectedRows[0].Cells["ID_Patient"].Value;
            PatientManager pm = PatientManager.Create(IDPatient);
            fr_PatientManager fr = new fr_PatientManager();
            fr.Manager = pm;
            fr.Use = Use.Update;
            fr.Text = "Modification d'un patient";
            fr.Size = new Size(fr.Size.Width, 600);
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
            {
                RefreshOlderThan();
                vModified = true;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Fill the DataGridView of the patient older than.
        /// </summary>
        private void RefreshOlderThan()
        {
            #region SQL Query
            dgv_Age.DataSource = null;
            DateTime dt = new DateTime((int)(DateTime.Today.Year - nud_Age.Value), 1, 1);
            string SQLQuery = "SELECT * FROM Patient WHERE DateNaissance < FORMAT(\"" + dt.ToString("d")
                + "\",\"dd/mm/yyyy\")";
            DataSet ds = new DataSet();
            int res = SqlManager.ExecuteSql(SQLQuery, ref ds);
            #endregion

            #region Displaying
            if (res < 1)
                MessageBox.Show("Aucun patient n'est plus vieux que " + nud_Age.Value + " ans!", "Information",
                     MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
            {
                dgv_Age.DataSource = ds.Tables[0];
                for (int i = 0; i < dgv_Age.Columns.Count; i++)
                    dgv_Age.Columns[i].Visible = false;

                dgv_Age.Columns["Nom"].Visible = true;
                dgv_Age.Columns["Prenom"].Visible = true;
                dgv_Age.Columns["DateNaissance"].Visible = true;
                dgv_Age.Columns["DateNaissance"].HeaderText = "Date de naissance";
            }
            #endregion
        }
        #endregion

        #region Properties
        /// <summary>
        /// Indicates whether the user modified data
        /// </summary>
        public bool Modified { get { return vModified; } }
        #endregion
    }
}