using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixStatistics.Class;
using ZedGraph;
using Utils.Misc;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Managers;

namespace PhoenixStatistics.GUI
{
    /// <summary>
    /// Fentre principale pour les statistiques
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Manager for the statistics
        /// </summary>
        private StatManager vStatManager = new StatManager();
        /// <summary>
        /// Sql manager for queries
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
        }
        #endregion

        #region Event
        /// <summary>
        /// Chargement de la page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void fr_Main_Load(object sender, EventArgs e)
        {
            InitializeGraphMeetingPerSeason();
            InitializeGraphAges();
            InitializeGraphSex();
            InitializeAdditionalAge();
        }
        /// <summary>
        /// Clique sur A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_About(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Ask the Age Corrector frame
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        private void MenuClick_AgeCorrector(object sender, EventArgs e)
        {
            fr_AgeCorrector fr = new fr_AgeCorrector();
            fr.ShowDialog();
            if (!fr.Modified) return;

            DialogResult dr = MessageBox.Show("Vous avez modifi un patient. Les graphiques ne sont plus  jour. "
                + "Voulez-vous redmarrer l'application?", "Question", MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
                Application.Restart();
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Calcule les statistiques
        /// </summary>
        public void CalculateStatistice()
        {
            vStatManager.CalculateMeetingOnMonth();
            vStatManager.CalculateAges();
            vStatManager.CalculateWomenAges();
            vStatManager.CalculateMenAges();
            vStatManager.CalculateAllCount();
            vStatManager.CalculateAllAverageAges();
        }
        /// <summary>
        /// Initialise la graphe des rendez-vous par saisons
        /// </summary>
        private void InitializeGraphMeetingPerSeason()
        {
            GraphPane Panel = zgc_RdvOnMonth.GraphPane;
            Panel.Title.Text = "Rendez-vous par mois";
            Panel.XAxis.Title.Text = "Mois";
            Panel.XAxis.Type = AxisType.Text;
            Panel.YAxis.Title.Text = "Nombre de rendez-vous";
            Panel.CurveList.Clear();

            BarItem[] BarItems = new BarItem[12];
            Color[] color = {Color.SkyBlue, Color.SlateBlue, Color.SlateGray, Color.YellowGreen, Color.SpringGreen, Color.SteelBlue,
                    Color.Tan, Color.Teal, Color.Thistle, Color.Tomato, Color.Transparent, Color.Turquoise};

            for (int i = 0; i < 12; i++)
            {
                double[] buf = { vStatManager.MeetingOnMonth[i] };
                BarItems[i] = Panel.AddBar(FrenchMonth.GetNameOf(i + 1), null, buf, Color.Red);
                BarItems[i].Bar.Fill = new Fill(color[i], Color.White, color[i]);
            }


            zgc_RdvOnMonth.Invalidate();
            zgc_RdvOnMonth.AxisChange();
        }
        /// <summary>
        /// Initialise la graphe des ages des patients
        /// </summary>
        private void InitializeGraphAges()
        {
            GraphPane TotalPanel = zgc_Ages.GraphPane;
            TotalPanel.Title.Text = "Graphique des ages totaux.";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            TotalPanel.CurveList.Clear();

            LineItem Courbe = TotalPanel.AddCurve("Age des patients", vStatManager.AgeList, vStatManager.Ages,
                Color.Orange, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            Courbe = TotalPanel.AddCurve("Age des femmes", vStatManager.AgeList, vStatManager.WomenAges,
                Color.DeepPink, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            Courbe = TotalPanel.AddCurve("Age des hommes", vStatManager.AgeList, vStatManager.MenAges,
                Color.Navy, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            zgc_Ages.AxisChange();
            zgc_Ages.Invalidate();

        }
        /// <summary>
        /// Initialise le graph des sexe des patients
        /// </summary>
        private void InitializeGraphSex()
        {
            GraphPane TotalPanel = zgc_Sex.GraphPane;
            TotalPanel.Title.Text = "Graphique des sexes.";
            TotalPanel.CurveList.Clear();

            if (vStatManager.CountAll != 0)
            {
                GraphPane Panel = zgc_Sex.GraphPane;
                float pctHomme = ((float)vStatManager.CountMen / (float)vStatManager.CountAll) * 100;
                float pctFemme = (float)((float)vStatManager.CountWomen / (float)vStatManager.CountAll) * 100;

                string title = vStatManager.CountAll + " patients (" + vStatManager.CountMen
                    + " homme et " + vStatManager.CountWomen + " femmes).";

                PieItem PieMen = Panel.AddPieSlice(pctHomme, Color.Blue, Color.White, 45F, 0.1, "Hommes ( "
                    + vStatManager.CountMen + " - " + pctHomme.ToString("0.00") + "%)");
                PieItem PieWomen = Panel.AddPieSlice(pctFemme, Color.Pink, Color.White, 45F, 0.1, "Femmes ( "
                    + vStatManager.CountWomen + " - " + pctFemme.ToString("0.00") + "%)");

                // Additions des parts de tartes    
                CurveList curves = Panel.CurveList;
                double Total = 0;
                for (int i = 0; i < curves.Count; i++)
                    Total += ((PieItem)curves[i]).Value;

                zgc_Sex.AxisChange();
                zgc_Sex.Invalidate();
            }

        }
        /// <summary>
        /// Initialise les graphs additionnels
        /// </summary>
        private void InitializeAdditionalAge()
        {
            lbl_AverageAge.Text = "Age moyen des patients: " + vStatManager.AverageAge.ToString("0.00") + " ans";
            lbl_MaxAgeFemme.Text = "Age moyen des femmes: " + vStatManager.AverageWomenAge.ToString("0.00") + " ans";
            lbl_MaxAgeHomme.Text = "Age moyen des hommes: " + vStatManager.AverageMenAge.ToString("0.00") + " ans";


            GraphPane TotalPanel = zgc_AgeMoyen.GraphPane;
            TotalPanel.Title.Text = "Age des patients";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            LineItem Courbe = TotalPanel.AddCurve("Age des patients", vStatManager.AgeList, vStatManager.Ages,
                Color.Orange, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            TotalPanel = zgc_AgeMoyenFemmes.GraphPane;
            TotalPanel.Title.Text = "Age des femmes";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            Courbe = TotalPanel.AddCurve("Age des femmes", vStatManager.AgeList, vStatManager.WomenAges,
                Color.DeepPink, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            TotalPanel = zgc_AgeMoyenHommes.GraphPane;
            TotalPanel.Title.Text = "Age des hommes";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            Courbe = TotalPanel.AddCurve("Age des hommes", vStatManager.AgeList, vStatManager.MenAges,
                Color.Navy, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            zgc_AgeMoyen.AxisChange();
            zgc_AgeMoyenFemmes.AxisChange();
            zgc_AgeMoyenHommes.AxisChange();

            zgc_AgeMoyen.Invalidate();
            zgc_AgeMoyenFemmes.Invalidate();
            zgc_AgeMoyenHommes.Invalidate();
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la gestionnaire de statistiques
        /// </summary>
        public StatManager StatManager { get { return vStatManager; } set { vStatManager = value; } }
        #endregion
    }
}