using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;

namespace Utils.DataBase
{
    /// <summary>
    /// Classe de gestion de requtes SQL pour ACCESS
    /// </summary>
    public class OleDbSql
    {
        #region Variables
        /// <summary>
        /// Chaine de connexion  une base de donnes Access
        /// </summary>
        protected static string vConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
        /// <summary>
        /// Emplacement de la base de donnes
        /// </summary>
        protected string vDBPath = "";
        /// <summary>
        /// ConnectionString complte
        /// </summary>
        protected string vFullString = "";
        /// <summary>
        /// Requte SQL
        /// </summary>
        protected string vSqlQuery = "";

        #region Variables prives
        private OleDbConnection oConn = new OleDbConnection();
        private OleDbCommand oCom;
        private OleDbDataAdapter oDa;
        private bool vIsPreparedProcedure = false;
        private Exception vOpenError;
        private bool vIsManagedPrepared = true;
        #endregion
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Path">Emplacement de la base de donnes</param>
        public OleDbSql(string _Path) { DbPath = _Path; }
        #endregion

        #region Mthodes
        /// <summary>
        /// Retourne la valeur numrique de l'identifiant de la dernire entre dans la base de donnes
        /// </summary>
        /// <returns>Un objet reprsentant la dernire clef primaire</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        private object LastID()
        {
            try
            {
                vSqlQuery = "SELECT @@IDENTITY";
                //oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                //oConn.Open();
                object LastKey = oCom.ExecuteScalar();
                //oConn.Close();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute une requte de selection
        /// </summary>
        /// <param name="_SqlQuery">Le requte SQL de selection</param>
        /// <param name="_ds">DataSet rempli par le requte SQL</param>
        /// <param name="_IsManagedConnection">A true, la connexion est cre ouverte et ferme automatiquement
        ///  false, il faut faire appel  la commande Open() et Close();</param>
        /// <returns>Un DataSet contenant les informations sur le SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteSql(string _SqlQuery, ref DataSet _ds, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;

                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);

                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                oDa = new OleDbDataAdapter(oCom);

                int nbr = oDa.Fill(_ds);
                return nbr;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute une requte de selection
        /// </summary>
        /// <param name="_SqlQuery">Le requte SQL de selection</param>
        /// <param name="_ds">DataSet rempli par le requte SQL</param>
        /// <returns>Un DataSet contenant les informations sur le SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteSql(string _SqlQuery, ref DataSet _ds) { return ExecuteSql(_SqlQuery, ref _ds, true); }
        /// <summary>
        /// Excute une requte de selection
        /// </summary>        
        /// <param name="_ds">DataSet rempli par le requte SQL</param>
        /// <returns>Un DataSet contenant les informations sur le SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteSql(ref DataSet _ds)
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteSql(vSqlQuery, ref _ds, true);
        }

        /// <summary>
        /// Excute une commande qui ne retourne qu'une seule valeur (Par exemple SELECT MAX)
        /// </summary>
        /// <param name="_SqlQuery">La requte SQL</param>
        /// <param name="_IsManagedConnection">A true, la connexion est cre ouverte et ferme automatiquement
        ///  false, il faut faire appel  la commande Open() et Close();</param>
        /// <returns>Le valeur unique rsultant de la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public object ExecuteScalar(string _SqlQuery, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                if (_IsManagedConnection)
                    oConn.Open();

                object LastKey = oCom.ExecuteScalar();

                if (_IsManagedConnection)
                    oConn.Close();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute une commande qui ne retourne qu'une seule valeur (Par exemple SELECT MAX)
        /// </summary>
        /// <param name="_SqlQuery">La requt SQL</param>
        /// <returns>Le valeur unique rsultant de la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public object ExecuteScalar(string _SqlQuery) { return ExecuteScalar(_SqlQuery, true); }
        /// <summary>
        /// Excute une commande qui ne retourne qu'une seule valeur (Par exemple SELECT MAX)
        /// </summary>        
        /// <returns>Le valeur unique rsultant de la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public object ExecuteScalar()
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteScalar(vSqlQuery, true);
        }

        /// <summary>
        /// Fait une commande SQL qui ne renvois aucun rsultat (INSERT, UPDATE, DELETE,...)
        /// </summary>
        /// <param name="_SqlQuery">La requte SQL</param>
        /// <param name="_IsManagedConnection">A true, la connexion est cre ouverte et ferme automatiquement
        ///  false, il faut faire appel  la commande Open() et Close();</param>
        /// <returns>Le nombre de tuples impliqu dans la commande SQL</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteNonQuery(string _SqlQuery, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                if (_IsManagedConnection)
                    oConn.Open();

                int i = oCom.ExecuteNonQuery();

                if (_IsManagedConnection)
                    oConn.Close();

                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Fait une commande SQL qui ne renvois aucun rsultat (INSERT, UPDATE, DELETE,...)
        /// </summary>
        /// <param name="_SqlQuery">La requt SQL</param>
        /// <returns>Le nombre de tuples impliqu dans la commande SQL</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteNonQuery(string _SqlQuery) { return ExecuteNonQuery(_SqlQuery, true); }
        /// <summary>
        /// Fait une commande SQL qui ne renvois aucun rsultat (INSERT, UPDATE, DELETE,...)
        /// </summary>        
        /// <returns>Le nombre de tuples impliqu dans la commande SQL</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public int ExecuteNonQuery()
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteNonQuery(vSqlQuery, true);
        }

        /// <summary>
        /// Prpare une utilisation d'une procdure stocke
        /// </summary>
        /// <param name="_ProcedureName">Nom de la procdure</param>
        /// <param name="_IsManagedConnection">A true, la connexion est ouverte et ferme automatiquement. A false, c'est manuel</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public void PrepareProcedure(string _ProcedureName, bool _IsManagedConnection)
        {
            vIsManagedPrepared = _IsManagedConnection;
            try
            {
                vSqlQuery = _ProcedureName;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);

                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.StoredProcedure;
                vIsPreparedProcedure = true;
            }
            catch (OleDbException) { throw; }
            catch (ArgumentException) { throw; }

        }
        /// <summary>
        /// Prpare une utilisation d'une procdure stocke
        /// </summary>
        /// <param name="_ProcedureName">Nom de la procdure</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public void PrepareProcedure(string _ProcedureName) { PrepareProcedure(_ProcedureName, true); }

        /// <summary>
        /// Execute la procdure stocke de Selection pralablement prpare par PrepareProcedure
        /// </summary>
        /// <param name="_ds">DataSet qui sera rempli par le rsultat de la requte</param>
        /// <returns>Le nombre de tuples touch par le requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public int ExecuteSqlProcedure(ref DataSet _ds)
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oDa = new OleDbDataAdapter(oCom);

                int nbr = oDa.Fill(_ds);
                oCom.Parameters.Clear();
                return nbr;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute la procdure stocke Scalaire pralablement prpare par PrepareProcedure
        /// </summary>
        /// <returns>Le rsultat unique rsultant de la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public object ExecuteScalarProcedure()
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                object LastKey = oCom.ExecuteScalar();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute la procdure stocke sans rsultat (INSERT, UPDATE) pralablement prpare par PrepareProcedure
        /// </summary>
        /// <returns>Le nombre de tuple touch par la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public int ExecuteNonQueryProcedure()
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                int i = oCom.ExecuteNonQuery();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Excute la procdure stocke sans rsultat (INSERT, UPDATE) pralablement prpare par PrepareProcedure
        /// </summary>
        /// <param name="_LastID">Dernier ID modifi par le requte</param>
        /// <returns>Le nombre de tuple touch par la requte</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public int ExecuteNonQueryProcedure(ref object _LastID)
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                int i = oCom.ExecuteNonQuery();
                _LastID = this.LastID();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }

        /// <summary>
        /// Ouvre une connexion  la base de donnes
        /// </summary>
        public bool Open()
        {
            if (vFullString != "")
            {
                try
                {
                    oConn = new OleDbConnection(vFullString);
                    oConn.Open();
                    return true;
                }
                catch (Exception ex)
                {
                    vOpenError = ex;
                    return false;
                }
            }
            return false;
        }
        /// <summary>
        /// Ferme la connexion en cours
        /// </summary>
        public void Close()
        {
            if (oConn.State != ConnectionState.Closed)
            {
                oConn.Close();
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne la chaine de connexion pour une base de donnes Access
        /// </summary>
        public static string ConnectionString { get { return vConnectionString; } }
        /// <summary>
        /// Retourne ou initialise le chemin de la base de donnes
        /// </summary>
        public string DbPath
        {
            get { return vDBPath; }
            set
            {
                vDBPath = value;
                vFullString = vConnectionString + vDBPath + "\"";
            }
        }
        /// <summary>
        /// Retourne les paramtres de la procdure stocke 
        /// </summary>
        public OleDbParameterCollection Parameters { get { return oCom.Parameters; } }
        /// <summary>
        /// Retoure l'tat de la connexion  la base de donnes
        /// </summary>
        public ConnectionState ConnectionState { get { return oConn.State; } }
        /// <summary>
        /// Retourne l'exception qui  t lance lors de la connexion. NULL si aucune erreur. Une fois
        /// L'exception rcuprer, elle est remise  NULL
        /// </summary>
        public Exception OpenError
        {
            get
            {
                Exception buf = vOpenError;
                vOpenError = null;
                return buf;
            }
        }
        /// <summary>
        /// Retourne ou initialise la requte SQL du manager
        /// </summary>
        public string SqlQuery { get { return vSqlQuery; } set { vSqlQuery = value; } }
        #endregion
    }

    /// <summary>
    /// Classe d'exception dsignant une erreur d'excution de procadure non prpare
    /// </summary>
    public class ProcedureNonPreparedException : Exception
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public ProcedureNonPreparedException() : base("La procdure stocke n'est pas prpare et ne peut tre execute") { }
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="Message">Message d'erreur de l'exception</param>
        public ProcedureNonPreparedException(string Message) : base(Message) { }

    }
}
