using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;

namespace Utils.Log
{
    /// <summary>
    /// Classe d'criture de log
    /// </summary>      
    public class Log
    {
        #region Variables
        /// <summary>
        /// Fichier dans lequel il faut crire
        /// </summary>
        private FileInfo vFileToLog;

        /// <summary>
        /// Rfrence sur l'objet Log unique
        /// </summary>
        private static readonly Log refLog = new Log();
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        /// <exception cref="Exception">Une erreur est survenue et il est ompossible de traiter les logs</exception>
        protected Log() { }

        /// <summary>
        /// Donne la rfrence sur l'objet Log unique
        /// </summary>
        /// <returns>Une rfrence sur un objet Log</returns>
        public static Log GetInstance()
        {
            refLog.TestFile();
            return refLog;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Ecrit une entre de log sans titre de de type Divers
        /// </summary>
        /// <param name="_Msg">Message du log</param>
        public void Write(string _Msg)
        {
            try
            {
                Write(_Msg, "", LogType.Divers);
            }
            catch (Exception) { throw; }
        }

        /// <summary>
        /// Ecrit une entre de log de type Divers
        /// </summary>
        /// <param name="_Title">Titre du log</param>
        /// <param name="_Msg">Message du log</param>
        public void Write(string _Msg, string _Title)
        {
            try
            {
                Write(_Msg, _Title, LogType.Divers);
            }
            catch (Exception) { throw; }
        }

        /// <summary>
        /// Ecrit une entre de log
        /// </summary>
        /// <param name="_Type">Type de log</param>
        /// <param name="_Title">Titre du log</param>
        /// <param name="_Msg">Message du log</param>
        /// <exception cref="Exception">En cas d'erreur dans l'criture des logs</exception>
        public void Write(string _Msg, string _Title, LogType _Type)
        {
            #region Type
            string Type = _Type.ToString();
            int size = LogType.Information.ToString().Length - Type.Length;
            string wSpace = "";
            for (int i = 0; i < size; i++)
                wSpace += " ";
            Type += wSpace;
            #endregion

            #region Title
            wSpace = "";
            size = 50 - _Title.Length;
            for (int i = 0; i < size; i++)
                wSpace += " ";
            _Title += wSpace;
            #endregion

            #region Enlevers les Sparateurs
            _Msg = _Msg.Replace(this.CellSeparator, " ");
            _Msg = _Msg.Replace(this.LineSeparator, " ");
            #endregion

            #region Entre  logger
            string ToWrite = DateTime.Now.ToString() + CellSeparator
                + Type + CellSeparator + _Title + CellSeparator + _Msg + CellSeparator;

            try
            {
                StreamWriter sw = vFileToLog.AppendText();
                sw.WriteLine(ToWrite);
                sw.Close();
            }
            catch (ThreadStateException ex) { Log.Instance.Write(ex.Message, "Log.Write()", LogType.Erreur); }
            catch (ThreadAbortException) { }
            catch (Exception ex)
            {
                throw new Exception("Erreur dans l'criture des logs" + ex.Message);
            }

            #endregion
        }

        /// <summary>
        /// Teste la taille du fichier de log
        /// </summary>
        private void TestFile()
        {
            try
            {
                //Si le dosier n'existe pas
                if (!Directory.Exists(CommandPath + @"\Log"))
                {
                    string path = CommandPath + @"\Log";
                    Console.WriteLine(path);
                    Directory.CreateDirectory(path);
                }

                //Si le ficher est plus grand que 250k crer un nouveau
                if (Directory.GetFiles(CommandPath + @"\Log\").Length != 0)
                {
                    string[] NameOfFiles = Directory.GetFiles(CommandPath + @"\Log\");

                    FileInfo fi = new FileInfo(NameOfFiles[NameOfFiles.Length - 1]);

                    if (fi.Length > 250000)
                    {
                        string FilePath = CommandPath + @"\Log\Log_" + NameOfFiles.Length + ".wlg";
                        FileInfo newF = new FileInfo(FilePath);
                        newF.Create().Close();
                        StreamWriter sw = newF.AppendText();
                        sw.WriteLine(WLogTag);
                        sw.Close();
                        vFileToLog = newF;
                    }
                    else
                        vFileToLog = fi;
                }

                //cration d'un fichier s'il n'existe pas
                else
                {
                    string FilePath = CommandPath + @"\Log\Log.wlg";
                    FileInfo newF = new FileInfo(FilePath);
                    newF.Create().Close();
                    StreamWriter sw = newF.AppendText();
                    sw.WriteLine(WLogTag);
                    sw.WriteLine(DateTime.Now.ToString() + CellSeparator
                        + LogType.Information + CellSeparator + "First Log" + CellSeparator + "First Log" + CellSeparator);
                    sw.Close();
                    vFileToLog = newF;
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        #endregion

        #region Debug
        /// <summary>
        /// Donne la ligne de commande du programme en cours
        /// </summary>
        private string CommandLine { get { return Environment.CommandLine; } }

        /// <summary>
        /// Donne le path d'excution du programme en cours
        /// </summary>
        private string CommandPath
        {
            get
            {
                string Path = CommandLine;
                Path = Path.Remove(Path.LastIndexOf("\\"));
                Path = Path.Trim('\"');
                return Path;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Sparateur de cellule
        /// </summary>
        protected string CellSeparator { get { return "|"; } }

        /// <summary>
        /// Sparateur de colonne
        /// </summary>
        protected string LineSeparator { get { return "\r\n"; } }

        /// <summary>
        /// Tag de certification de WLoGFile
        /// </summary>
        private string WLogTag { get { return "<WLoG File>"; } }

        /// <summary>
        /// Path du fichier dans lequel on crit les logs
        /// </summary>
        public string FileToLog { get { return vFileToLog.FullName; } }

        /// <summary>
        /// Donne la rfrence sur l'objet Log unique
        /// </summary>        
        public static Log Instance { get { return GetInstance(); } }
        #endregion
    }

    /// <summary>
    /// Donne le type d'entre de log
    /// </summary>
    public enum LogType
    {
        /// <summary>
        /// Divers
        /// </summary>
        Divers,
        /// <summary>
        /// Debug
        /// </summary>
        Debug,
        /// <summary>
        /// Erreur
        /// </summary>
        Erreur,
        /// <summary>
        /// Information
        /// </summary>
        Information,
        /// <summary>
        /// Information
        /// </summary>
        Rapport
    }
}
