using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Mail;
using System.Net;

namespace Utils.Misc
{
    /// <summary>
    /// Classe d'envois de mail pour Gmail
    /// </summary>
    public class Mailer
    {
        #region Variables
        /// <summary>
        /// Login pour le SMTP
        /// </summary>
        protected string vAddress;

        /// <summary>
        /// Password pour le SMTP
        /// </summary>
        protected string vPassword;

        /// <summary>
        /// Message d'erreur du dernier envoi de mail
        /// </summary>
        protected string vSendingErrorMessage = "";

        /// <summary>
        /// Port de communication du serveur SMTP
        /// </summary>
        protected int vPort = 25;

        /// <summary>
        /// Adresse du serveur SMTP
        /// </summary>
        protected string vSMTPServer;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur d'initialisation
        /// </summary>
        /// <param name="_Address">Login pour le SMTP </param>
        /// <param name="_Password">Password pour le SMTP</param>
        /// <param name="_Port">Numro du port de communication du serveur SMTP</param>
        /// <param name="_SMTPServer">Adresse du serveur SMTP</param>
        public Mailer(string _Address, string _Password, int _Port, string _SMTPServer)
        {
            vPassword = _Password;
            vAddress = _Address;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Envois un mail 
        /// </summary>
        /// <param name="to">Adresse de destination</param>
        /// <param name="subject">Sujet du mail</param>
        /// <param name="message">Message du mail</param>        
        /// <returns>True si la mail est envoy sinon false</returns>
        public bool SendMail(string to, string subject, string message)
        {
            try
            {
                //SMTPClient
                SmtpClient smtp = new SmtpClient();
                smtp.Credentials = new NetworkCredential(vAddress, vPassword);
                smtp.Port = vPort; // 25;//587;
                smtp.Host = vSMTPServer; // "smtp.skynet.be";//"smtp.gmail.com";
                smtp.EnableSsl = false;//true;

                MailMessage mail = new MailMessage();
                mail.From = new MailAddress(vAddress, subject, Encoding.UTF8);
                mail.To.Add(to);
                mail.Subject = "Error Reporting";
                mail.Body = message;
                mail.DeliveryNotificationOptions = DeliveryNotificationOptions.OnFailure;
                mail.ReplyTo = new MailAddress(vAddress);

                smtp.Send(mail);
            }
            catch (Exception ex)
            {
                vSendingErrorMessage = ex.Message;
                return false;
            }
            return true;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le login pour le SMTP
        /// </summary>
        public string Address
        {
            get { return vAddress; }
            set { vAddress = value; }
        }

        /// <summary>
        /// Mot de passe pour le SMTP
        /// </summary>
        public string Password
        {
            get { return vPassword; }
            set { vPassword = value; }
        }

        /// <summary>
        /// Dernier message d'erreur pour l'envois d'un mail
        /// </summary>
        public string SendingError { get { return vSendingErrorMessage; } }

        /// <summary>
        /// Port de communication du serveur SMTP
        /// </summary>
        public int Port
        {
            get { return vPort; }
            set { vPort = value; }
        }

        /// <summary>
        /// Retourne ou initialise l'adresse du serveur SMTP
        /// </summary>
        public string SMTPServer
        {
            get { return vSMTPServer; }
            set { vSMTPServer = value; }
        }
        #endregion
    }
}
