using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.IO;
using Utils.Log;

namespace Utils.Misc
{
    /// <summary>
    /// Classe pour la srialisation et la dsrialisation
    /// </summary>
    [Serializable]
    public class Serializer
    {
        #region Variables
        /// <summary>
        /// Path du fichier srialis
        /// </summary>
        protected string vPath = ApplicationPath.CommandPath + "\\" + DateTime.Now.Ticks + ".srz";
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public Serializer() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Path">Donne le path du du fichier dans lequel on srialise</param>
        public Serializer(string _Path) { vPath = _Path; }
        #endregion

        #region Mthodes
        /// <summary>
        /// Serialise un objet dans le fichier indiqu dans _FilePath
        /// </summary>
        /// <param name="_FilePath">Path du fichier dans lequel on srialise</param>
        /// <param name="_ToSerialize">L'objet  srialiser</param>
        /// <exception cref="SerializationException">Pour une erreur dans la srialisation ou la dsrialisation</exception>
        /// <exception cref="ArgumentNullException">Le FileStream pour la dsrialisation </exception>
        public static void Serialize(object _ToSerialize, string _FilePath)
        {
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                FileStream fs = new FileStream(_FilePath, FileMode.Create, FileAccess.Write);
                bf.Serialize(fs, _ToSerialize);
                fs.Close();
            }
            catch (SerializationException ex)
            {
                Log.Log.Instance.Write(ex.Message, "Serializer.Serialize", LogType.Erreur);
                //throw;
            }
            catch (ArgumentNullException ex)
            {
                Log.Log.Instance.Write(ex.Message, "Serializer.Serialize", LogType.Erreur);
                //throw;
            }
            catch (Exception ex)
            {
                Log.Log.Instance.Write(ex.Message, "Serializer.Serialize", LogType.Erreur);
                //throw;
            }
        }

        /// <summary>
        /// Srialise un objet
        /// </summary>
        /// <param name="_ToSerialize">Objet  srialiser</param>
        /// <exception cref="SerializationException">Pour une erreur dans la srialisation ou la dsrialisation</exception>
        /// <exception cref="ArgumentNullException">Le FileStream pour la dsrialisation </exception>
        public void Serialize(object _ToSerialize)
        {
            try
            {
                Serialize(_ToSerialize, vPath);
            }
            catch (SerializationException) { throw; }
            catch (ArgumentNullException) { throw; }
        }

        /// <summary>
        /// Dserialise le fichier contenu dans le Path _FilePath
        /// </summary>
        /// <param name="_FilePath">Path du fichier  dsrialiser</param>
        /// <returns>L'objet dsrialis</returns>
        /// <exception cref="SerializationException">Pour une erreur dans la srialisation ou la dsrialisation</exception>
        /// <exception cref="ArgumentNullException">Le FileStream pour la dsrialisation </exception>
        /// <exception cref="ArgumentNullException">Le fichier  srialiser n'est pas trouv</exception>
        public static object Deserialize(string _FilePath)
        {
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                FileStream fs = new FileStream(_FilePath, FileMode.Open, FileAccess.Read);
                object ret = bf.Deserialize(fs);
                fs.Close();
                return ret;
            }
            catch (FileNotFoundException) { throw; }
            catch (SerializationException) { throw; }
            catch (ArgumentNullException) { throw; }

        }

        /// <summary>
        /// Dserialise le fichier.
        /// </summary>        
        /// <returns>L'objet dsrialis</returns>
        /// <exception cref="SerializationException">Pour une erreur dans la srialisation ou la dsrialisation</exception>
        /// <exception cref="ArgumentNullException">Le FileStream pour la dsrialisation </exception>
        public object Deserialize()
        {
            try { return Deserialize(vPath); }
            catch (SerializationException) { throw; }
            catch (ArgumentNullException) { throw; }
        }
        #endregion
    }
}
